@@ -182,14 +182,13 @@ typedef enum
 	GF_PIXEL_RGBD		=	GF_4CC('R', 'G', 'B', 'D'),
 	/*!RGB24 + depth plane (7 lower bits) + shape mask. Component ordering in bytes is R-G-B-(S+D).*/
 	GF_PIXEL_RGBDS		=	GF_4CC('3', 'C', 'D', 'S'),
-	/*!Stereo RGB24 */
-	GF_PIXEL_RGBS		=	GF_4CC('R', 'G', 'B', 'S'),
-	/*!Stereo RGBA. Component ordering in bytes is R-G-B-A. */
-	GF_PIXEL_RGBAS		=	GF_4CC('R', 'G', 'A', 'S'),
 
 	/*internal format for OpenGL using pachek RGB 24 bit plus planar depth plane at the end of the image*/
 	GF_PIXEL_RGB_DEPTH = GF_4CC('R', 'G', 'B', 'd'),
 
+	/*generic pixel format uncv from ISO/IEC 23001-17*/
+	GF_PIXEL_UNCV = GF_4CC('u', 'n', 'c', 'v'),
+
 	/*!YUV packed 422 format*/
 	GF_PIXEL_YUYV		=	GF_4CC('Y','U','Y','V'),
 	/*!YUV packed 422 format*/
@@ -237,11 +236,17 @@ typedef enum
 	/*!444 YUV, 10 bits, little endian*/
 	GF_PIXEL_YUV444_10	=	GF_4CC('Y','4','1','0'),
 	/*!444 YUV packed*/
-	GF_PIXEL_YUV444_PACK	=	GF_4CC('Y','4','4','p'),
+	GF_PIXEL_YUV444_PACK	=	GF_4CC('Y','U','V','4'),
+	/*!444 VYU packed (v308) */
+	GF_PIXEL_VYU444_PACK	=	GF_4CC('V','Y','U','4'),
 	/*!444 YUV+Alpha packed*/
 	GF_PIXEL_YUVA444_PACK	=	GF_4CC('Y','A','4','p'),
-	/*!444 YUV 10 bit packed*/
+	/*!444 UYVA packed (v408) */
+	GF_PIXEL_UYVA444_PACK	=	GF_4CC('U','Y','V','A'),
+	/*!444 YUV 10 bit packed little endian (v410)*/
 	GF_PIXEL_YUV444_10_PACK	=	GF_4CC('Y','4','1','p'),
+	/*!422 YUV 10 bit packed in v210 format*/
+	GF_PIXEL_V210			= GF_4CC('v','2','1','0'),
 
 	/*!Unknown format exposed a single OpenGL texture to be consumed using samplerExternalOES*/
 	GF_PIXEL_GL_EXTERNAL	=	GF_4CC('E','X','G','L')
@@ -349,7 +354,13 @@ u32 gf_pixel_fmt_to_qt_type(GF_PixelFormat pixfmt);
 /*!
 \brief Codec IDs
 
-Codec ID identifies the stream coding type. The enum is divided into values less than 255, which are equivalent to MPEG-4 systems ObjectTypeIndication. Other values are 4CCs, usually matching ISOMEDIA sample entry types*/
+Codec ID identifies the stream coding type. The enum is divided into values less than 255, which are equivalent to MPEG-4 systems ObjectTypeIndication. Other values are 4CCs, usually matching ISOMEDIA sample entry types.
+
+Unless specified otherwise the decoder configuration is:
+- the one specified in MPEG-4 systems if any
+- or the one defined in ISOBMFF and derived specs (3GPP, dolby, etc) if any
+- or NULL
+*/
 typedef enum
 {
 	/*!Never used by PID declarations, but used by filters caps*/
@@ -499,10 +510,15 @@ typedef enum
 	/*! codecid for subtitle/text streams in tx3g / apple text format*/
 	GF_CODECID_TX3G = GF_4CC( 't', 'x', '3', 'g' ),
 
+	/*! codecid for SSA / ASS text streams (only demux -> tx3f conv)*/
+	GF_CODECID_SUBS_SSA = GF_4CC( 'a', 's', 's', 'a' ),
+
+	GF_CODECID_DVB_SUBS = GF_4CC( 'd', 'v', 'b', 's' ),
+	GF_CODECID_DVB_TELETEXT = GF_4CC( 'd', 'v', 'b', 't' ),
 	/*!
 		\brief OGG DecoderConfig
 
-	 The DecoderConfig for theora, vorbis, flac and opus contains all intitialization ogg packets for the codec
+	 The DecoderConfig for theora, vorbis and speek contains all intitialization ogg packets for the codec
 	  and is formatted as follows:\n
 	 \code
 	  while (dsi_size) {
@@ -511,6 +527,11 @@ typedef enum
 			dsi_size -= packet_size;
 		}
 	 \endcode
+
+	 The DecoderConfig for FLAC is the full flac header without "fLaC" magic
+
+	 The DecoderConfig for OPUS is the full opus header without "OpusHead" magic
+
 	*/
 	/*! codecid for theora video streams*/
 	GF_CODECID_THEORA = GF_4CC('t','h','e','u'),
@@ -588,12 +609,13 @@ typedef enum
 	/*! codecid for USAC / xHE-AACv2 audio */
 	GF_CODECID_USAC = GF_4CC('u','s','a','c'),
 
-	GF_CODECID_V210 = GF_4CC('v','2','1','0'),
-
-
 	/*! codecid for MPEG-1 Audio streams, layer 1*/
 	GF_CODECID_MPEG_AUDIO_L1 = GF_4CC('m','p','a','1'),
 
+	GF_CODECID_MSPEG4_V3 = GF_4CC('D','I','V','3'),
+
+	GF_CODECID_ALAC = GF_4CC('A','L','A','C'),
+
 	//fake codec IDs for RTP
 	GF_CODECID_FAKE_MP2T = GF_4CC('M','P','2','T')
 } GF_CodecID;
@@ -643,6 +665,12 @@ GF_CodecID gf_codecid_from_oti(u32 stream_type, u32 oti);
 */
 u32 gf_codecid_4cc_type(GF_CodecID codecid);
 
+/*! Checks if reframer/unframer exists for this codec in gpac
+\param codecid target codec ID
+\return GF_TRUE if reframer/unframer exist (bitstream reparse is possible), GF_FALSE otherwise
+*/
+Bool gf_codecid_has_unframer(GF_CodecID codecid);
+
 /*! Gets the codecid given the associated short name
 \param cname target codec short name
 \return codecid codec ID
@@ -893,7 +921,7 @@ u32 gf_audio_fmt_to_isobmf(GF_AudioFormat afmt);
 GF_AudioFormat gf_audio_fmt_enum(u32 *idx, const char **name, const char **fileext, const char **desc);
 
 /*! get CICP layout code point from audio configuration
-\param nb_chan number of channels
+\param nb_chan total number of channels
 \param nb_surr number of surround channels
 \param nb_lfe number of LFE channels
 \return CICP layout code point format or 0 if unknown
@@ -1645,7 +1673,6 @@ enum
 	GF_PROJ360_MESH
 };
 
-
 /*! @} */
 
 #ifdef __cplusplus
