@@ -495,11 +495,12 @@ Bool gf_dash_group_enum_descriptor(GF_DashClient *dash, u32 group_idx, GF_DashDe
 \param has_next_segment set to GF_TRUE if next segment location is known (unthreaded mode) or next segment is downloaded (threaded mode) (optional, may be NULL)
 \param key_url set to the key URL of the next segment for MPEG-2 TS full segment encryption (optional, may be NULL). The URL is either a URN or a resolved URL
 \param key_IV set to the key initialization vector of the next segment for MPEG-2 TS full segment encryption (optional, may be NULL)
+\param utc set to UTC mapping for first sample of segment, 0 if none defined
 \return GF_BUFFER_TOO_SMALL if no segment found, GF_EOS if end of session, GF_URL_REMOVED if segment is disabled (but all output info is OK, this can be ignored and considered as GF_OK by the user) or error if any
 */
 GF_Err gf_dash_group_get_next_segment_location(GF_DashClient *dash, u32 group_idx, u32 dependent_representation_index, const char **url, u64 *start_range, u64 *end_range,
         s32 *switching_index, const char **switching_url, u64 *switching_start_range, u64 *switching_end_range,
-        const char **original_url, Bool *has_next_segment, const char **key_url, bin128 *key_IV);
+        const char **original_url, Bool *has_next_segment, const char **key_url, bin128 *key_IV, u64 *utc);
 
 /*! gets some info on the segment
 \param dash the target dash client
@@ -868,9 +869,10 @@ GF_Err gf_dash_group_get_quality_info(GF_DashClient *dash, u32 group_idx, u32 qu
 \param group_idx the 0-based index of the target group
 \param segment_timeline_timescale set to segment timeline timescale, or to 0 if no segment timeline
 \param init_url set to initialization URL (template, timeline or base URL for VoD) as indicated in manifest (no resolution to base URL)
+\param hls_variant set toHLS variant name or NULL
 \return segment template, NULL if no templates used. Memory must be freed by caller
 */
-char *gf_dash_group_get_template(GF_DashClient *dash, u32 group_idx, u32 *segment_timeline_timescale, const char **init_url);
+char *gf_dash_group_get_template(GF_DashClient *dash, u32 group_idx, u32 *segment_timeline_timescale, const char **init_url, const char **hls_variant);
 
 /*! checks automatic switching mode
 \param dash the target dash client
@@ -1065,12 +1067,18 @@ void gf_dash_set_group_download_state(GF_DashClient *dash, u32 group_idx, u32 de
 */
 void gf_dash_group_store_stats(GF_DashClient *dash, u32 group_idx, u32 dep_rep_idx, u32 bytes_per_sec, u64 file_size, Bool is_broadcast, u64 us_since_start);
 
+/*! Shifts the computed availabilityStartTime by offsetting (or resetting) the SuggestedPresentationDelay value parsed in the DASH manifest
+\param dash the target dash client
+\param spd availabilityStartTime shift in milliseconds. Positive values shift the clock in the future, negative ones in the past, "-I" will force the SuggestedPresentationDelay value to zero
+*/
+void gf_dash_set_suggested_presentation_delay(GF_DashClient *dash, s32 spd);
+
 /*! sets availabilityStartTime shift for ROUTE. By default the ROUTE tune-in is done by matching the last received segment name
 to the segment template and deriving the ROUTE UTC reference from that. The function allows shifting the computed value by a given amount.
 \param dash the target dash client
 \param ast_shift clock shift in milliseconds of the ROUTE receiver tune-in. Positive values shift the clock in the future, negative ones in the past
 */
-void gf_dash_set_route_ast_shift(GF_DashClient *dash, u32 ast_shift);
+void gf_dash_set_route_ast_shift(GF_DashClient *dash, s32 ast_shift);
 
 /*! gets the minimum wait time before calling \ref gf_dash_process again for unthreaded mode
 \param dash the target dash client
