@@ -308,6 +308,8 @@ typedef struct
 	char *service_name;
 	/*RTSP status code of the command as parsed. One of the above RTSP StatusCode*/
 	u32 StatusCode;
+
+	Bool is_resend;
 } GF_RTSPCommand;
 
 /*! creates an RTSP command
@@ -465,8 +467,9 @@ for other protocols)
 \param sess the target RTSP session
 \param ResetConnection if set, this will destroy the associated TCP socket. This is useful in case of timeouts, because
 some servers do not restart with the right CSeq.
+\return number of retries for reset happening before first server reply, 0 otherwise
 */
-void gf_rtsp_session_reset(GF_RTSPSession *sess, Bool ResetConnection);
+u32 gf_rtsp_session_reset(GF_RTSPSession *sess, Bool ResetConnection);
 
 /*! checks if an RTSP session matches an RTSP URL
 \param sess the target RTSP session
@@ -479,13 +482,33 @@ Bool gf_rtsp_is_my_session(GF_RTSPSession *sess, char *url);
 \param sess the target RTSP session
 \return the server name
 */
-char *gf_rtsp_get_server_name(GF_RTSPSession *sess);
+const char *gf_rtsp_get_server_name(GF_RTSPSession *sess);
+
+/*! gets user name of an RTSP session
+\param sess the target RTSP session
+\return the user name or NULL if none
+*/
+const char *gf_rtsp_get_user(GF_RTSPSession *sess);
+
+/*! gets password of an RTSP session
+\param sess the target RTSP session
+\return the password or NULL if none
+*/
+const char *gf_rtsp_get_password(GF_RTSPSession *sess);
+
+
 /*! gets server port of an RTSP session
 \param sess the target RTSP session
 \return the server port
 */
 u16 gf_rtsp_get_session_port(GF_RTSPSession *sess);
 
+/*! checks if RTSP connection is over TLS
+\param sess the target RTSP session
+\return GF_TRUE if connection is secured
+*/
+Bool gf_rtsp_use_tls(GF_RTSPSession *sess);
+
 /*! fetches an RTSP response from the server.  the GF_RTSPResponse will be reseted before fetch
 \param sess the target RTSP session
 \param rsp the response object to fill with the response. This will be reseted before TCP fetch
@@ -530,6 +553,12 @@ void gf_rtsp_reset_aggregation(GF_RTSPSession *sess);
 */
 GF_Err gf_rtsp_send_command(GF_RTSPSession *sess, GF_RTSPCommand *com);
 
+/*! checks connection status - should be called before processing any RTSP for non-blocking IO
+\param sess the target RTSP session
+\return GF_IP_NETWORK_EMPTY if connection is still pending or data cannot be flushed, GF_OK if connected and no more data to send or error if any
+*/
+GF_Err gf_rtsp_check_connection(GF_RTSPSession *sess);
+
 
 /*! callback function for interleaved RTSP/TCP transport
 \param sess the target RTSP session
@@ -573,14 +602,18 @@ u32 gf_rtsp_unregister_interleave(GF_RTSPSession *sess, u8 LowInterID);
 /*! creates a new RTSP session from an existing socket in listen state. If no pending connection
 	is detected, return NULL
 \param rtsp_listener the listening server socket
+\param allow_http_tunnel indicate if HTTP tunnel should be enabled
+\param ssl_ctx OpenSSL context
 \return the newly allocated RTSP session if any, NULL otherwise
 */
-GF_RTSPSession *gf_rtsp_session_new_server(GF_Socket *rtsp_listener);
+GF_RTSPSession *gf_rtsp_session_new_server(GF_Socket *rtsp_listener, Bool allow_http_tunnel, void *ssl_ctx);
 
+/*! special error code for \ref gf_rtsp_get_command*/
+#define GF_RTSP_TUNNEL_POST	-1000
 /*! fetches an RTSP request
 \param sess the target RTSP session
 \param com the RTSP command to fill with the command. This will be reseted before fetch
-\return error if any
+\return error if any or GF_RTSP_TUNNEL_POST if this is a POST on a HTPP tunnel
 */
 GF_Err gf_rtsp_get_command(GF_RTSPSession *sess, GF_RTSPCommand *com);
 
@@ -620,6 +653,19 @@ GF_Err gf_rtsp_get_remote_address(GF_RTSPSession *sess, char *buffer);
 */
 GF_Err gf_rtsp_session_write_interleaved(GF_RTSPSession *sess, u32 idx, u8 *pck, u32 pck_size);
 
+/*! gets sessioncookie for HTTP tunnel
+\param sess the target RTSP session
+\return cookie or NULL if none
+*/
+const char *gf_rtsp_get_session_cookie(GF_RTSPSession *sess);
+
+/*! move TCP connection of a POST HTTP tunnel link to main session
+\param sess the target RTSP session
+\param post_sess the target RTSP POST http tunnel session - the session is not destroyed, only its connection is detached
+\return error if any
+*/
+GF_Err gf_rtsp_merge_tunnel(GF_RTSPSession *sess, GF_RTSPSession *post_sess);
+
 /*
 		RTP LIB EXPORTS
 */
@@ -747,6 +793,16 @@ GF_Err gf_rtp_initialize(GF_RTPChannel *ch, u32 UDPBufferSize, Bool IsSource, u3
 */
 GF_Err gf_rtp_stop(GF_RTPChannel *ch);
 
+/*! sets source-specific IPs - this MUST be called before calling \ref gf_rtp_initialize
+\param ch the target RTP channel
+\param src_ip_inc IP of sources to receive from
+\param nb_src_ip_inc number of sources to receive from
+\param src_ip_exc IP of sources to exclude
+\param nb_src_ip_exc number of sources to exclude
+\return error if any
+*/
+GF_Err gf_rtp_set_ssm(GF_RTPChannel *ch, const char **src_ip_inc, u32 nb_src_ip_inc, const char **src_ip_exc, u32 nb_src_ip_exc);
+
 /*! inits the RTP info after a PLAY or PAUSE, rtp_time is the rtp TimeStamp of the RTP packet
 with seq_num sequence number. This info is needed to compute the CurrentTime of the RTP channel
 ssrc may not be known if sender hasn't indicated it (use 0 then)
@@ -838,6 +894,13 @@ this function at regular times
 */
 GF_Err gf_rtp_send_rtcp_report(GF_RTPChannel *ch);
 
+/*! forces loss rate for next Receiver report
+\param ch the target RTP channel
+\param loss_rate loss rate in per-thousand
+\return error if any
+*/
+void gf_rtp_set_loss_rate(GF_RTPChannel *ch, u32 loss_rate);
+
 /*! sends a BYE info (leaving the session)
 \param ch the target RTP channel
 \return error if any
@@ -1220,6 +1283,7 @@ typedef struct
 	/*config of the stream if carried in SDP*/
 	u8 *config;
 	u32 configSize;
+	u8 config_updated;
 	/* Stream Type*/
 	u8 StreamType;
 	/* stream profile and level indication - for AVC/H264, 0xPPCCLL, with PP:profile, CC:compatibility, LL:level*/
@@ -1399,6 +1463,8 @@ enum
 	GF_RTP_PAYT_LATM,
 	/*use AC3 audio format*/
 	GF_RTP_PAYT_AC3,
+	/*use EAC3 audio format*/
+	GF_RTP_PAYT_EAC3,
 	/*use H264-SVC transport*/
 	GF_RTP_PAYT_H264_SVC,
 	/*use HEVC/H265 transport (RFC 7798)*/
@@ -1410,6 +1476,8 @@ enum
 #endif
 	/*use VVC transport (no RFC yet)*/
 	GF_RTP_PAYT_VVC,
+	/*use opus audio format*/
+	GF_RTP_PAYT_OPUS,
 };
 
 
