@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2012
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / IETF RTP/RTSP/SDP sub-project
@@ -83,6 +83,17 @@ GF_Err gf_rtp_reorderer_add(GF_RTPReorder *po, const void * pck, u32 pck_size, u
 /*gets the output of the queue. Packet Data IS YOURS to delete*/
 void *gf_rtp_reorderer_get(GF_RTPReorder *po, u32 *pck_size, Bool force_flush);
 
+#define MAX_RTCP_RR	4
+
+typedef struct
+{
+	u32 ssrc;
+	u32 frac_lost;
+	u32 total_loss;
+	u32 last_rtp_sn;
+	u64 jitter;
+	u32 last_sr, delay_last_sr;
+} GF_RTCP_Report;
 
 /*the RTP channel with both RTP and RTCP sockets and buffers
 each channel is identified by a control string given in RTSP Describe
@@ -133,6 +144,7 @@ struct __tag_rtp_channel
 	u32 num_pck_sent, num_payload_bytes;
 	u32 forced_ntp_sec, forced_ntp_frac;
 
+	u32 force_loss_rate;
 	Bool no_auto_rtcp;
 	/*RTCP info*/
 	char *s_name, *s_email, *s_location, *s_phone, *s_tool, *s_note, *s_priv;
@@ -167,6 +179,13 @@ struct __tag_rtp_channel
 
 	gf_rtp_tcp_callback send_interleave;
 	void *interleave_cbk1, *interleave_cbk2;
+
+	GF_RTCP_Report rtcp_rr[MAX_RTCP_RR];
+	u32 nb_rctp_rr;
+
+
+	const char **ssm, **ssmx;
+	u32 nb_ssm, nb_ssmx;
 };
 
 /*gets UTC in the channel RTP timescale*/
@@ -213,7 +232,7 @@ void gf_rtp_get_next_report_time(GF_RTPChannel *ch);
 	if (sig < 0) { \
 		sprintf(temp, "%d", d);		\
 	} else { \
-		sprintf(temp, "%d", d);		\
+		sprintf(temp, "%u", d);		\
 	}	\
 	RTSP_WRITE_ALLOC_STR_WITHOUT_CHECK(buf, buf_size, pos, temp); \
 	}
@@ -244,6 +263,18 @@ typedef struct
 	void *ch_ptr;
 } GF_TCPChan;
 
+typedef enum
+{
+	RTSP_HTTP_NONE = 0,
+	RTSP_HTTP_CLIENT,
+	RTSP_HTTP_SERVER,
+	RTSP_HTTP_DISABLE
+} RTSP_HTTP_Tunnel;
+
+#ifdef GPAC_HAS_SSL
+#include <openssl/ssl.h>
+#endif
+
 /**************************************
 		RTSP Session
 ***************************************/
@@ -256,27 +287,29 @@ struct _tag_rtsp_session
 	/*server port (extracted from URL)*/
 	u16 Port;
 
+	char *User, *Pass;
+
 	/*if RTSP is on UDP*/
 	u8 ConnectionType;
 	/*TCP interleaving ID*/
 	u8 InterID;
 	/*http tunnel*/
-	Bool HasTunnel;
+	RTSP_HTTP_Tunnel tunnel_mode;
 	GF_Socket *http;
-	char HTTP_Cookie[30];
-	u32 CookieRadLen;
+	char *HTTP_Cookie;
+	u32 tunnel_state;
 
 	/*RTSP CHANNEL*/
 	GF_Socket *connection;
 	u32 SockBufferSize;
 	/*needs connection*/
 	u32 NeedConnection;
-
+	u32 timeout_in;
 	/*the RTSP sequence number*/
 	u32 CSeq;
 	/*this is for aggregated request in order to check SeqNum*/
 	u32 NbPending;
-
+	u32 nb_retry;
 	/*RTSP sessionID, arbitrary length, alpha-numeric*/
 	const char *last_session_id;
 
@@ -299,21 +332,31 @@ struct _tag_rtsp_session
 	/*all RTP channels in an interleaved RTP on RTSP session*/
 	GF_List *TCPChannels;
 	Bool interleaved;
+
+	u8 *async_buf;
+	u32 async_buf_size, async_buf_alloc;
+
+#ifdef GPAC_HAS_SSL
+	Bool use_ssl, ssl_connect_pending;
+	SSL_CTX *ssl_ctx;
+	SSL *ssl, *ssl_http;
+#endif
+
 };
 
 GF_RTSPSession *gf_rtsp_session_new(char *sURL, u16 DefaultPort);
 
-/*check connection status*/
-GF_Err gf_rtsp_check_connection(GF_RTSPSession *sess);
 /*send data on RTSP*/
 GF_Err gf_rtsp_send_data(GF_RTSPSession *sess, u8 *buffer, u32 Size);
 
+GF_Err gf_rstp_do_read_sock(GF_RTSPSession *sess, GF_Socket *sock, u8 *data, u32 data_size, u32 *out_read);
+
 /*
 			Common RTSP tools
 */
 
 /*locate body-start and body size in response/commands*/
-void gf_rtsp_get_body_info(GF_RTSPSession *sess, u32 *body_start, u32 *body_size);
+void gf_rtsp_get_body_info(GF_RTSPSession *sess, u32 *body_start, u32 *body_size, Bool skip_tunnel);
 /*read TCP until a full command/response is received*/
 GF_Err gf_rtsp_read_reply(GF_RTSPSession *sess);
 /*fill the TCP buffer*/
@@ -326,10 +369,6 @@ GF_RTSPTransport *gf_rtsp_transport_parse(u8 *buffer);
 GF_Err gf_rtsp_parse_header(u8 *buffer, u32 BufferSize, u32 BodyStart, GF_RTSPCommand *com, GF_RTSPResponse *rsp);
 void gf_rtsp_set_command_value(GF_RTSPCommand *com, char *Header, char *Value);
 void gf_rtsp_set_response_value(GF_RTSPResponse *rsp, char *Header, char *Value);
-/*deinterleave a data packet*/
-GF_Err gf_rtsp_set_deinterleave(GF_RTSPSession *sess);
-/*start session through HTTP tunnel (QTSS)*/
-GF_Err gf_rtsp_http_tunnel_start(GF_RTSPSession *sess, char *UserAgent);
 
 
 
@@ -447,6 +486,7 @@ GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, u8 *data, u32 data_size,
 GF_Err gp_rtp_builder_do_hevc(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize);
 GF_Err gp_rtp_builder_do_mp2t(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize);
 GF_Err gp_rtp_builder_do_vvc(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize);
+GF_Err gp_rtp_builder_do_opus(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize);
 
 #define RTP_VVC_AGG_NAL		0x1C //28
 #define RTP_VVC_FRAG_NAL	0x1D //29
@@ -477,6 +517,8 @@ struct __tag_rtp_depacketizer
 	GP_RTPSLMap sl_map;
 	/*! RTP clock rate*/
 	u32 clock_rate;
+	/*! audio channels from RTP map*/
+	u32 audio_channels;
 
 	//! clip rect X
 	u32 x;
