@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Ivica Arsov, Jean Le Feuvre
- *			Copyright (c) Mines-Telecom 2009-
+ *			Copyright (c) Mines-Telecom 2009-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / Wrapper
@@ -75,17 +75,13 @@ typedef struct
 #define RAW_OUT_PIXEL_FORMAT		GF_PIXEL_RGBA
 #define NBPP						4
 
-#define RAWCTX	AndroidContext *rc = (AndroidContext *)dr->opaque
+#define DROIDVOUTCTX	AndroidContext *rc = (AndroidContext *)dr->opaque
 
 
 
 //Functions specific to OpenGL ES2
 #ifdef GPAC_USE_GLES2
 
-#define GF_TRUE 1
-#define GF_FALSE 0
-
-
 //we custom-define these instead of importing gl_inc.h
 #define GL_COMPILE_STATUS 0x8B81
 #define GL_FRAGMENT_SHADER 0x8B30
@@ -93,8 +89,6 @@ typedef struct
 #define GL_LINK_STATUS 0x8B82
 #define GL_VERTEX_SHADER 0x8B31
 
-
-
 static char *glsl_vertex = "precision mediump float;\
 	attribute vec4 gfVertex;\
 	attribute vec4 gfTexCoord;\
@@ -119,9 +113,7 @@ static inline void gl_check_error()
 {
 	s32 res = glGetError();
 	if (res) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE,
-		       ("GL Error %d file %s line %d\n", res,
-				__FILE__, __LINE__));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[DroidVOUT] GL Error %d file %s line %d\n", res, __FILE__, __LINE__));
 	}
 }
 
@@ -130,7 +122,7 @@ static GLint gf_glGetUniformLocation(u32 glsl_program, const char *uniform_name)
 {
 	GLint loc = glGetUniformLocation(glsl_program, uniform_name);
 	if (loc<0) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[V3D:GLSL] Cannot find uniform \"%s\" in GLSL program\n", uniform_name));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[DroidVOUT] Cannot find uniform \"%s\" in GLSL program\n", uniform_name));
 	}
 	return loc;
 }
@@ -139,20 +131,21 @@ static GLint gf_glGetAttribLocation(u32 glsl_program, const char *attrib_name)
 {
 	GLint loc = glGetAttribLocation(glsl_program, attrib_name);
 	if (loc<0) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[V3D:GLSL] Cannot find attrib \"%s\" in GLSL program\n", attrib_name));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[DroidVOUT] Cannot find attrib \"%s\" in GLSL program\n", attrib_name));
 	}
 	return loc;
 }
 
 //modified version of visual_3d_compile_shader function
-Bool compile_shader(u32 shader_id, const char *name, const char *source) {
+static Bool compile_shader(u32 shader_id, const char *name, const char *source)
+{
 	GLint blen = 0;
 	GLsizei slen = 0;
 	u32 len;
 	GLint is_compiled = 0;
 
 
-	if(!source || !shader_id) return 0;
+	if (!source || !shader_id) return 0;
 	len = (u32) strlen(source);
 	glShaderSource(shader_id, 1, &source, &len);
 	glCompileShader(shader_id);
@@ -164,8 +157,7 @@ Bool compile_shader(u32 shader_id, const char *name, const char *source) {
 	if (blen > 1) {
 		char* compiler_log = (char*) gf_malloc(blen);
 		glGetShaderInfoLog(shader_id, blen, &slen, compiler_log);
-		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[GLSL] Failed to compile %s shader: %s\n", name, compiler_log));
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPOSE, ("[GLSL] ***** faulty shader code ****\n%s\n**********************\n", source));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_COMPOSE, ("[DroidVOUT] Failed to compile %s shader: %s\nShader code:\n%s\n", name, compiler_log, source));
 		gf_free (compiler_log);
 		return GF_FALSE;
 	}
@@ -174,29 +166,15 @@ Bool compile_shader(u32 shader_id, const char *name, const char *source) {
 }
 
 
-static Bool initGLES2(AndroidContext *rc) {
+static Bool initGLES2(AndroidContext *rc)
+{
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] InitGLES2: Version %s - Vendor %s - Renderer %s - Extensions %s",
+		glGetString(GL_VERSION),
+		glGetString(GL_VENDOR),
+		glGetString(GL_RENDERER),
+		glGetString(GL_EXTENSIONS)
+	));
 
-
-//PRINT OpengGL INFO
-	char* ext;
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android InitGLES2"));
-
-	ext = (char*)glGetString(GL_VENDOR);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Vendor: %s", ext));
-
-	ext = (char*)glGetString(GL_RENDERER);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Renderer: %s", ext));
-
-	ext = (char*)glGetString(GL_VERSION);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Version: %s", ext));
-
-	ext = (char*)glGetString(GL_EXTENSIONS);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Extensions: %s", ext));
-
-
-
-//Generic GL setup
 	/* Set the background black */
 	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
 
@@ -210,7 +188,7 @@ static Bool initGLES2(AndroidContext *rc) {
 	glDepthFunc(GL_LEQUAL);
 
 
-//Shaders setup
+	//Shaders setup
 	Bool res = GF_FALSE;
 	GLint linked;
 
@@ -220,11 +198,11 @@ static Bool initGLES2(AndroidContext *rc) {
 	rc->base_vertex = glCreateShader(GL_VERTEX_SHADER);
 	rc->base_fragment = glCreateShader(GL_FRAGMENT_SHADER);
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Compiling shaders"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] Compiling shaders"));
 	res = compile_shader(rc->base_vertex, "vertex", glsl_vertex);
-	if(!res) return GF_FALSE;
+	if (!res) return GF_FALSE;
 	res = compile_shader(rc->base_fragment, "fragment", glsl_fragment);
-	if(!res) return GF_FALSE;
+	if (!res) return GF_FALSE;
 
 	glAttachShader(rc->base_program, rc->base_vertex);
 	glAttachShader(rc->base_program, rc->base_fragment);
@@ -241,8 +219,6 @@ static Bool initGLES2(AndroidContext *rc) {
 	}
 	glUseProgram(rc->base_program);
 	gl_check_error();
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Shaders compiled"))
-
 	return GF_TRUE;
 }
 
@@ -255,14 +231,15 @@ static void load_matrix_shaders(GLuint program, Fixed *mat, const char *name)
 #endif
 	gl_check_error();
 	loc = glGetUniformLocation(program, name);
-	if(loc<0) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("GL Error (file %s line %d): Invalid matrix name", __FILE__, __LINE__));
+	if (loc<0) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DroidVOUT] GL Error (file %s line %d): Invalid matrix name", __FILE__, __LINE__));
 		return;
 	}
 	gl_check_error();
 
 #ifdef GPAC_FIXED_POINT
-	for (i=0; i<16; i++) _mat[i] = FIX2FLT(mat[i]);
+	for (i=0; i<16; i++)
+		_mat[i] = FIX2FLT(mat[i]);
 	glUniformMatrix4fv(loc, 1, GL_FALSE, (GLfloat *) _mat);
 #else
 	glUniformMatrix4fv(loc, 1, GL_FALSE, mat);
@@ -273,17 +250,16 @@ static void load_matrix_shaders(GLuint program, Fixed *mat, const char *name)
 
 //ES2 version of glOrthox() - resulting matrix is stored in rc->ortho
 //more info on Orthographic projection matrix at http://www.songho.ca/opengl/gl_projectionmatrix.html#ortho
-static void calculate_ortho(Fixed left, Fixed right, Fixed bottom, Fixed top, Fixed near, Fixed far,  AndroidContext *rc) {
-
-
-	if((left==right)|(bottom==top)|(near==far)) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("GL Error (file %s line %d): Invalid Orthogonal projection values", __FILE__, __LINE__));
+static void calculate_ortho(Fixed left, Fixed right, Fixed bottom, Fixed top, Fixed near, Fixed far,  AndroidContext *rc)
+{
+	if ((left==right)|(bottom==top)|(near==far)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("[DroidVOUT] GL Error (file %s line %d): Invalid Orthogonal projection values", __FILE__, __LINE__));
 		return;
 	}
 
 	gf_mx_init(rc->ortho);
 
-//For Orthographic Projection
+	//For Orthographic Projection
 	rc->ortho.m[0] = gf_divfix(2, (right-left));
 	rc->ortho.m[5] = gf_divfix(2, (top-bottom));
 	rc->ortho.m[10] = gf_divfix(-2, far-near);
@@ -293,39 +269,28 @@ static void calculate_ortho(Fixed left, Fixed right, Fixed bottom, Fixed top, Fi
 	rc->ortho.m[15] = FIX_ONE;
 }
 
-
-
 #endif	//Endof specifix for GLES2 (ifdef GPAC_USE_GLES2)
 
 
 #ifndef GPAC_USE_GLES2
-void initGL(AndroidContext *rc)
+static void initGL(AndroidContext *rc)
 {
-	char* ext;
+	char *exts = (char*)glGetString(GL_EXTENSIONS);
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android InitGL"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] InitGL: Version %s - Vendor %s - Renderer %s - Extensions %s",
+		glGetString(GL_VERSION),
+		glGetString(GL_VENDOR),
+		glGetString(GL_RENDERER),
+		exts
+	));
 
-	ext = (char*)glGetString(GL_VENDOR);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Vendor: %s", ext));
-
-	ext = (char*)glGetString(GL_RENDERER);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Renderer: %s", ext));
-
-	ext = (char*)glGetString(GL_VERSION);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Version: %s", ext));
-
-	ext = (char*)glGetString(GL_EXTENSIONS);
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("OpenGL ES Extensions: %s", ext));
-
-	if ( strstr(ext, "GL_OES_draw_texture") )
-	{
+	if (strstr(exts, "GL_OES_draw_texture")) {
 		rc->draw_texture = 1;
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("Using GL_OES_draw_texture"));
+		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DroidVOUT] Using GL_OES_draw_texture"));
 	}
-	if ( strstr(ext, "GL_ARB_texture_non_power_of_two") )
-	{
+	if (strstr(ext, "GL_ARB_texture_non_power_of_two")) {
 		rc->non_power_two = 0;
-		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("Using GL_ARB_texture_non_power_of_two"));
+		GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DroidVOUT] Using GL_ARB_texture_non_power_of_two"));
 	}
 
 	/* Enable smooth shading */
@@ -348,8 +313,7 @@ void initGL(AndroidContext *rc)
 }
 #endif
 
-void gluPerspective(GLfloat fovy, GLfloat aspect,
-                    GLfloat zNear, GLfloat zFar)
+static void gluPerspective(GLfloat fovy, GLfloat aspect, GLfloat zNear, GLfloat zFar)
 {
 #ifndef GPAC_USE_GLES2
 	GLfloat xmin, xmax, ymin, ymax;
@@ -364,9 +328,9 @@ void gluPerspective(GLfloat fovy, GLfloat aspect,
 #endif
 }
 
-void resizeWindow(AndroidContext *rc)
+static void resizeWindow(AndroidContext *rc)
 {
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("resizeWindow : start"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] resizing window to %u x %u\n", rc->width, rc->height));
 
 	/* Protect against a divide by zero */
 	if (rc->height==0)
@@ -395,21 +359,16 @@ void resizeWindow(AndroidContext *rc)
 	load_matrix_shaders(rc->base_program, (Fixed *) rc->identity.m, "gfModelViewMatrix");
 	gl_check_error();
 #endif
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("resizeWindow : end"));
 }
 
-void drawGLScene(AndroidContext *rc)
+static void drawGLScene(AndroidContext *rc)
 {
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("drawGLScene : start"));
-#endif /* DROID_EXTREME_LOGS */
 #ifdef GPAC_USE_GLES2
 	GLint loc_vertex_array, loc_texcoord_array;
 #endif
 
 	GLfloat vertices[4][3];
 	GLfloat texcoord[4][2];
-//	int i, j;
 
 #ifndef GPAC_USE_GLES2
 	float rgba[4];
@@ -445,27 +404,18 @@ void drawGLScene(AndroidContext *rc)
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
 
-//    for ( i = 0; i < rc->height/2; i++ )
-//    	for ( j = 0; j < rc->width; j++ )
-//    		rc->texData[ i*rc->width*NBPP + j*NBPP + 3] = 200;
-
-//    memset(rc->texData, 255, 4 * rc->width * rc->height );
-
 	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, rc->tex_width, rc->tex_height, 0,
 	              GL_RGBA, GL_UNSIGNED_BYTE, rc->texData );
 
 	gl_check_error();
-	if ( rc->draw_texture )
-	{
+	if ( rc->draw_texture ) {
 #ifndef GPAC_USE_GLES2
 		gl_check_error();
 		int cropRect[4] = {0,rc->height,rc->width,-rc->height};
 		glTexParameteriv(GL_TEXTURE_2D, GL_TEXTURE_CROP_RECT_OES, cropRect);
 		glDrawTexsOES(0, 0, 0, rc->width, rc->height);
 #endif
-	}
-	else
-	{
+	} else {
 		gl_check_error();
 
 #ifndef GPAC_USE_GLES2
@@ -494,8 +444,6 @@ void drawGLScene(AndroidContext *rc)
 
 
 #endif
-		//glTranslatef(0.0f, 0.0f, -3.3f);
-		//glTranslatef(0.0f, 0.0f, -2.3f);
 
 		/* Top Right Of The Quad    */
 		vertices[0][0]=rc->tex_width;
@@ -537,41 +485,36 @@ void drawGLScene(AndroidContext *rc)
 
 	/* Flush all drawings */
 	glFinish();
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("drawGLScene : end"));
-#endif /* DROID_EXTREME_LOGS */
 }
 
-int releaseTexture(AndroidContext *rc)
+static int releaseTexture(AndroidContext *rc)
 {
 	gl_check_error();
 	if (!rc)
 		return 0;
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Delete Texture"));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] Delete texture\n"));
 
-	if ( rc->texID >= 0)
-	{
+	if ( rc->texID >= 0) {
 		glDeleteTextures(1, &(rc->texID));
 		rc->texID = -1;
 	}
-	if (rc->texData)
-	{
+
+	if (rc->texData) {
 		gf_free(rc->texData);
 		rc->texData = NULL;
 	}
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Delete Texture DONE"))
 	gl_check_error();
 	return 0;
 }
 
-int createTexture(AndroidContext *rc)
+static int createTexture(AndroidContext *rc)
 {
 	if (!rc)
 		return 0;
-	if ( rc->texID >= 0 )
+	if (rc->texID >= 0)
 		releaseTexture(rc);
 
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("Android Create Texture Size: WxH: %dx%d", rc->tex_width, rc->tex_height));
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DroidVOUT] Create Texture %u x %u in RGBA\n", rc->tex_width, rc->tex_height));
 
 	glGenTextures( 1, &(rc->texID) );
 
@@ -592,12 +535,11 @@ int createTexture(AndroidContext *rc)
 	              GL_RGBA, GL_UNSIGNED_BYTE, NULL/*rc->texData*/ );
 
 	glBindTexture( GL_TEXTURE_2D, 0);
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Create Texture DONE"));
 	return 0;
 }
 
 
-u32 find_pow_2(u32 num)
+static u32 find_pow_2(u32 num)
 {
 	u32 res = 1;
 	while (res < num)
@@ -607,8 +549,8 @@ u32 find_pow_2(u32 num)
 
 static GF_Err droid_Resize(GF_VideoOutput *dr, u32 w, u32 h)
 {
-	RAWCTX;
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Resize: %dx%d", w, h));
+	DROIDVOUTCTX;
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] Resize: %u x %u", w, h));
 
 	rc->width = w;
 	rc->height = h;
@@ -622,13 +564,10 @@ static GF_Err droid_Resize(GF_VideoOutput *dr, u32 w, u32 h)
 	rc->tex_width = rc->width;
 	rc->tex_height = rc->height;
 #else
-	if ( rc->non_power_two )
-	{
+	if (rc->non_power_two) {
 		rc->tex_width = rc->width;
 		rc->tex_height = rc->height;
-	}
-	else
-	{
+	} else {
 		rc->tex_width = find_pow_2(rc->width);
 		rc->tex_height = find_pow_2(rc->height);
 	}
@@ -636,85 +575,62 @@ static GF_Err droid_Resize(GF_VideoOutput *dr, u32 w, u32 h)
 	gl_check_error();
 	resizeWindow(rc);
 
-	if ( rc->out_3d_type == 0 )
-	{
+	if (rc->out_3d_type == 0) {
 		createTexture(rc);
 	}
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Resize DONE", w, h));
 	gl_check_error();
 	return GF_OK;
 }
 
-GF_Err droid_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_flags)
+static GF_Err droid_Setup(GF_VideoOutput *dr, void *os_handle, void *os_display, u32 init_flags)
 {
-	RAWCTX;
+	DROIDVOUTCTX;
 
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Setup: %d", init_flags));
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] Setup with flags %08X\n", init_flags));
 
 
 #ifdef GPAC_USE_GLES2
-
-	if ( rc->out_3d_type == 0 ) {
+	if (rc->out_3d_type == 0) {
 		Bool res = GF_FALSE;
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("We are in OpenGL: disable mode"));
 		res = initGLES2(rc);
-		if(res==GF_FALSE) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_MMIO, ("ERROR Compiling ES2 Shaders"));
-		} else {	//set texture
+		if (res) {
 			glUseProgram(rc->base_program);
 			GLint loc = gf_glGetUniformLocation(rc->base_program, "img");
 			glUniform1i(loc,0);
 		}
 	}
-
 #else
-
-	if ( rc->out_3d_type == 0 )
-
+	if (rc->out_3d_type == 0)
 		initGL(rc);
 #endif //GPAC_USE_GLES2
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Setup DONE"));
 	return GF_OK;
 }
 
 
 static void droid_Shutdown(GF_VideoOutput *dr)
 {
-	RAWCTX;
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Shutdown\n"));
-
+	DROIDVOUTCTX;
+	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("[DroidVOUT] Shutdown\n"));
 	releaseTexture(rc);
-
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Shutdown DONE"));
 }
 
 
 static GF_Err droid_Flush(GF_VideoOutput *dr, GF_Window *dest)
 {
-	RAWCTX;
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Flush\n"));
-#endif /* DROID_EXTREME_LOGS */
-
+	DROIDVOUTCTX;
 	if ( rc->out_3d_type == 0 )
 		drawGLScene(rc);
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android Flush DONE"));
-#endif /* DROID_EXTREME_LOGS */
 	return GF_OK;
 }
 
 static GF_Err droid_LockBackBuffer(GF_VideoOutput *dr, GF_VideoSurface *vi, Bool do_lock)
 {
-	RAWCTX;
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android LockBackBuffer: %d", do_lock));
-#endif /* DROID_EXTREME_LOGS */
+	DROIDVOUTCTX;
+
 	if (do_lock) {
 		if (!vi) return GF_BAD_PARAM;
 
-		if ( rc->out_3d_type != 0 )
+		if (rc->out_3d_type != 0)
 			return GF_NOT_SUPPORTED;
 
 		memset(vi, 0, sizeof(GF_VideoSurface));
@@ -725,11 +641,8 @@ static GF_Err droid_LockBackBuffer(GF_VideoOutput *dr, GF_VideoSurface *vi, Bool
 		vi->pitch_x = NBPP;
 		vi->pitch_y = NBPP * rc->tex_width;
 		vi->pixel_format = RAW_OUT_PIXEL_FORMAT;
-	}
-	else
-	{
-		if (rc->locked_data)
-		{
+	} else {
+		if (rc->locked_data) {
 			//glBindTexture( GL_TEXTURE_2D, rc->texID);
 			//glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
 			//glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
@@ -738,21 +651,17 @@ static GF_Err droid_LockBackBuffer(GF_VideoOutput *dr, GF_VideoSurface *vi, Bool
 			//			GL_RGBA, GL_UNSIGNED_BYTE, rc->locked_data );
 		}
 	}
-#ifdef DROID_EXTREME_LOGS
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android LockBackBuffer DONE"));
-#endif /* DROID_EXTREME_LOGS */
 	return GF_OK;
 }
 
 static GF_Err droid_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 {
-	RAWCTX;
+	DROIDVOUTCTX;
 
 	if (!evt) return GF_OK;
 
 	switch (evt->type) {
 	case GF_EVENT_SIZE:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("GF_EVENT_SIZE( %d x %d)", evt->setup.width, evt->setup.height));
 		//in fullscreen mode: do not change viewport; just update perspective
 		if (rc->fullscreen) {
 #ifdef GPAC_USE_GLES2
@@ -781,10 +690,8 @@ static GF_Err droid_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 			return droid_Resize(dr, evt->setup.width, evt->setup.height);
 
 	case GF_EVENT_VIDEO_SETUP:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android OpenGL mode: %d", evt->setup.use_opengl));
 		if (!evt->setup.use_opengl) {
 			rc->out_3d_type = 0;
-//					initGL(rc);
 			droid_Resize(dr, evt->setup.width, evt->setup.height);
 		} else {
 			rc->out_3d_type = 1;
@@ -793,18 +700,13 @@ static GF_Err droid_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 		return GF_OK;
 
 	case GF_EVENT_SET_CURSOR:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("GF_EVENT_SET_CURSOR"));
 		return GF_OK;
 	case GF_EVENT_SET_CAPTION:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("GF_EVENT_SET_CAPTION"));
 		return GF_OK;
 	case GF_EVENT_SHOWHIDE:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("GF_EVENT_SHOWHIDE"));
 		return GF_OK;
 	default:
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Process Unknown Event: %d", evt->type));
 		return GF_OK;
-
 	case GF_EVENT_TEXT_EDITING_START:
 	case GF_EVENT_TEXT_EDITING_END:
 		return GF_NOT_SUPPORTED;
@@ -814,7 +716,7 @@ static GF_Err droid_ProcessEvent(GF_VideoOutput *dr, GF_Event *evt)
 
 static GF_Err droid_SetFullScreen(GF_VideoOutput *dr, Bool bOn, u32 *outWidth, u32 *outHeight)
 {
-	RAWCTX;
+	DROIDVOUTCTX;
 
 	*outWidth = dr->max_screen_width;
 	*outHeight = dr->max_screen_height;
@@ -826,7 +728,7 @@ GF_VideoOutput *NewAndroidVideoOutput()
 {
 	AndroidContext *pCtx;
 	GF_VideoOutput *driv = (GF_VideoOutput *) gf_malloc(sizeof(GF_VideoOutput));
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("Android Video Initialization in progress..."));
+	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("[DroidVOUT] Loading module"));
 	memset(driv, 0, sizeof(GF_VideoOutput));
 	GF_REGISTER_MODULE_INTERFACE(driv, GF_VIDEO_OUTPUT_INTERFACE, "Android Video Output", "gpac distribution")
 
@@ -847,8 +749,6 @@ GF_VideoOutput *NewAndroidVideoOutput()
 	driv->max_screen_height = 1024;
 
 	driv->hw_caps = GF_VIDEO_HW_OPENGL;// | GF_VIDEO_HW_OPENGL_OFFSCREEN_ALPHA;//GF_VIDEO_HW_DIRECT_ONLY;//
-
-	GF_LOG(GF_LOG_INFO, GF_LOG_MMIO, ("Android Video Init Done.\n"));
 	return (void *)driv;
 }
 
@@ -856,15 +756,12 @@ void DeleteAndroidVideoOutput(void *ifce)
 {
 	AndroidContext *rc;
 	GF_VideoOutput *driv = (GF_VideoOutput *) ifce;
-	if (!ifce)
-		return;
+	if (!ifce) return;
 	droid_Shutdown(driv);
 	rc = (AndroidContext *)driv->opaque;
-	if (rc)
-		gf_free(rc);
+	if (rc) gf_free(rc);
 	driv->opaque = NULL;
 	gf_free(driv);
-	GF_LOG(GF_LOG_DEBUG, GF_LOG_MMIO, ("Android vout deinit\n"));
 }
 
 /*interface query*/
