@@ -2,7 +2,7 @@
  *					GPAC Multimedia Framework
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2005-2020
+ *			Copyright (c) Telecom ParisTech 2005-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / X11 video output module
@@ -39,7 +39,7 @@ extern "C"
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 
-#if !defined(GPAC_DISABLE_3D) && !defined(GPAC_USE_GLES1X) && !defined(GPAC_USE_TINYGL)
+#if !defined(GPAC_DISABLE_3D) && !defined(GPAC_USE_GLES1X) && !defined(GPAC_USE_TINYGL) && defined(GPAC_HAS_X11_GLX)
 #define GPAC_HAS_OPENGL
 #endif
 
@@ -58,12 +58,6 @@ extern "C"
 #include <X11/extensions/Xvlib.h>
 #endif
 
-#if defined(ENABLE_JOYSTICK) || defined(ENABLE_JOYSTICK_NO_CURSOR)
-#include <linux/joystick.h>
-#include <unistd.h>
-#include <fcntl.h>
-#endif
-
 #define X11VID()	XWindow *xWindow = (XWindow *)vout->opaque;
 
 #define RGB555(r,g,b) (((r&248)<<7) + ((g&248)<<2)  + (b>>3))
@@ -121,10 +115,6 @@ typedef struct
 	Window gl_wnd;
 	u32 offscreen_type;
 #endif
-#if defined(ENABLE_JOYSTICK) || defined(ENABLE_JOYSTICK_NO_CURSOR)
-	/*joystick device file descriptor*/
-	s32 prev_x, prev_y, fd;
-#endif
 } XWindow;
 
 void StretchBits (void *dst, u32 dst_bpp, u32 dst_w, u32 dst_h, u32 dst_pitch,
