@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2020
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / Scene Compositor sub-project
@@ -28,7 +28,6 @@
 #include "visual_manager.h"
 #include "texturing.h"
 #include "nodes_stacks.h"
-#include <gpac/options.h>
 
 #ifndef GPAC_DISABLE_3D
 
@@ -216,6 +215,7 @@ void compositor_3d_draw_bitmap(Drawable *stack, DrawAspect2D *asp, GF_TraverseSt
 	} else {
 		visual_3d_set_state(tr_state->visual, V3D_STATE_BLEND, GF_FALSE);
 	}
+
 	/*ignore texture transform for bitmap*/
 	tr_state->mesh_num_textures = gf_sc_texture_enable(txh, NULL);
 	if (tr_state->mesh_num_textures) {
@@ -227,7 +227,11 @@ void compositor_3d_draw_bitmap(Drawable *stack, DrawAspect2D *asp, GF_TraverseSt
 				size.y = height;
 
 				stack->mesh = new_mesh();
-				mesh_new_rectangle(stack->mesh, size, NULL, GF_FALSE);
+				if (txh->stream) {
+					mesh_new_rectangle_ex(stack->mesh, size, NULL, txh->stream->flip, txh->stream->rotate);
+				} else {
+					mesh_new_rectangle(stack->mesh, size, NULL, GF_FALSE);
+				}
 			}
 		}
 		if (stack->mesh) {
