@@ -86,10 +86,10 @@ void gf_odm_del(GF_ObjectManager *odm)
 	gf_odm_reset_media_control(odm, 0);
 	gf_list_del(odm->ms_stack);
 	gf_list_del(odm->mc_stack);
-#endif
 
 	if (odm->type == GF_STREAM_INTERACT)
 		gf_input_sensor_delete(odm);
+#endif
 
 	if (odm->raw_frame_sema) gf_sema_del(odm->raw_frame_sema);
 
@@ -246,7 +246,9 @@ void gf_odm_disconnect(GF_ObjectManager *odm, u32 do_remove)
 		gf_filter_forward_gf_event(odm->parentscene->compositor->filter, &evt, GF_FALSE, GF_TRUE);
 
 		gf_scene_remove_object(odm->parentscene, odm, do_remove);
-		if (odm->subscene) gf_scene_del(odm->subscene);
+		if (odm->subscene) {
+			gf_scene_del(odm->subscene);
+		}
 		gf_odm_del(odm);
 		return;
 	}
@@ -273,7 +275,7 @@ static Bool gf_odm_should_auto_select(GF_ObjectManager *odm)
 	//if (odm->type == GF_STREAM_VISUAL) return GF_TRUE;
 
 	if (odm->parentscene && !odm->parentscene->is_dynamic_scene) {
-		return GF_TRUE;
+		return GF_FALSE;
 	}
 
 	if (odm->parentscene && odm->parentscene->root_od->addon) {
@@ -286,7 +288,7 @@ static Bool gf_odm_should_auto_select(GF_ObjectManager *odm)
 		GF_ObjectManager *an_odm = gf_list_get(odm->parentscene->resources, i);
 		if (an_odm==odm) continue;
 		if (an_odm->type != odm->type) continue;
-		//same type - if the first one has been autumatically activated, do not activate this one
+		//same type - if the first one has been automatically activated, do not activate this one
 		if (an_odm->state == GF_ODM_STATE_PLAY) return GF_FALSE;
 	}
 	return GF_TRUE;
@@ -359,7 +361,7 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_SceneNamespace *parent_ns, GF
 
 	/*restore OD ID */
 	if (odm->media_current_time) {
-		odm->ID = odm->media_current_time;
+		odm->ID = (u32) odm->media_current_time;
 		odm->media_current_time = 0;
 		odm->flags |= GF_ODM_REMOTE_OD;
 	}
@@ -373,7 +375,7 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_SceneNamespace *parent_ns, GF
 		assert(odm->subscene->root_od==odm);
 		odm->subscene->is_dynamic_scene = GF_TRUE;
 	} else if (odm->pid) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Terminal] Setting up object streams\n"));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("[Compositor] Setting up object streams\n"));
 
 		e = gf_odm_setup_pid(odm, for_pid);
 		if (e) {
@@ -412,6 +414,12 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_SceneNamespace *parent_ns, GF
 			odm->buffer_playout_ms = odm->buffer_max_ms = 1; //1 ms
 			odm->buffer_min_ms = 0;
 		}
+		prop = gf_filter_pid_get_property(for_pid ? for_pid : odm->pid, GF_PROP_PID_RAWGRAB);
+		if (prop && prop->value.uint) {
+			odm->buffer_playout_ms = 0;
+			odm->buffer_min_ms = 0;
+			odm->buffer_max_ms = 0;
+		}
 
 		GF_FEVT_INIT(evt, GF_FEVT_BUFFER_REQ, for_pid ? for_pid : odm->pid);
 		evt.buffer_req.max_buffer_us = odm->buffer_max_ms * 1000;
@@ -421,6 +429,10 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_SceneNamespace *parent_ns, GF
 
 		if (odm->buffer_min_ms * 1000 > evt.buffer_req.max_playout_us)
 			odm->buffer_min_ms = 0;
+
+		if (gf_filter_pid_is_sparse(for_pid ? for_pid : odm->pid))
+			odm->buffer_min_ms = odm->buffer_max_ms = odm->buffer_playout_ms = 0;
+
 	}
 
 	/*setup mediaobject info except for top-level OD*/
@@ -489,21 +501,29 @@ void gf_odm_setup_object(GF_ObjectManager *odm, GF_SceneNamespace *parent_ns, GF
 	have to wait for an entire image carousel period to start filling the buffers, which is sub-optimal
 	we also force a prefetch for object declared outside the OD stream to make sure we don't loose any data before object declaration and play
 	as can be the case with MPEG2 TS (first video packet right after the PMT) - this should be refined*/
-	else if ( ((odm->flags & GF_ODM_NO_TIME_CTRL) || (odm->flags & GF_ODM_NOT_IN_OD_STREAM)) && gf_odm_should_auto_select(odm) && (odm->parentscene->selected_service_id == odm->ServiceID)) {
-		Bool force_play = GF_FALSE;
+	else if ( ((odm->flags & GF_ODM_NO_TIME_CTRL) || (odm->flags & GF_ODM_NOT_IN_OD_STREAM)) && (odm->parentscene->selected_service_id == odm->ServiceID)) {
 
-		if (odm->state==GF_ODM_STATE_STOP) {
-			odm->flags |= GF_ODM_PREFETCH;
-			force_play = GF_TRUE;
-		}
+		if (gf_odm_should_auto_select(odm)) {
+			Bool force_play = GF_FALSE;
 
-		if (force_play) {
-			odm->flags |= GF_ODM_INITIAL_BROADCAST_PLAY;
-			odm->parentscene->selected_service_id = odm->ServiceID;
+			if (odm->state==GF_ODM_STATE_STOP) {
+				odm->flags |= GF_ODM_PREFETCH;
+				force_play = GF_TRUE;
+			}
 
-			GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] Inserted from input service %s - forcing play\n", odm->ID, odm->scene_ns->url));
-			odm->flags &= ~GF_ODM_NOT_SETUP;
+			if (force_play) {
+				odm->flags |= GF_ODM_INITIAL_BROADCAST_PLAY;
+				odm->parentscene->selected_service_id = odm->ServiceID;
+
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d] Inserted from input service %s - forcing play\n", odm->ID, odm->scene_ns->url));
+				odm->flags &= ~GF_ODM_NOT_SETUP;
+				gf_odm_start(odm);
+			}
+		} else {
+			//otherwise issue play/stop to stop the stream
+			//the play is needed because some filters (m2ts demux) start considering the pid is active (for reframers)
 			gf_odm_start(odm);
+			gf_odm_stop(odm, GF_FALSE);
 		}
 	}
 
@@ -572,6 +592,7 @@ GF_Err gf_odm_setup_pid(GF_ObjectManager *odm, GF_FilterPid *pid)
 	const GF_PropertyValue *prop;
 	u32 OD_OCR_ID=0;
 	u32 es_id=0;
+	u32 ckid_use_odid = 0;
 
 	/*find the clock for this new channel*/
 	ck = NULL;
@@ -586,6 +607,10 @@ GF_Err gf_odm_setup_pid(GF_ObjectManager *odm, GF_FilterPid *pid)
 	}
 #endif
 
+	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_ESID);
+	if (!prop) prop = gf_filter_pid_get_property(pid, GF_PROP_PID_ID);
+	if (prop) es_id = prop->value.uint;
+
 	if (odm->ck) {
 		ck = odm->ck;
 		goto clock_setup;
@@ -650,8 +675,10 @@ GF_Err gf_odm_setup_pid(GF_ObjectManager *odm, GF_FilterPid *pid)
 	if (!clockID) {
 		if (odm->ID == GF_MEDIA_EXTERNAL_ID) {
 			clockID = (u32) (intptr_t) odm->scene_ns;
+			ckid_use_odid = clockID;
 		} else {
 			clockID = odm->ID;
+			ckid_use_odid = clockID;
 		}
 	}
 
@@ -663,9 +690,6 @@ GF_Err gf_odm_setup_pid(GF_ObjectManager *odm, GF_FilterPid *pid)
 		assert(ck_namespace);
 	}
 
-	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_ESID);
-	if (!prop) prop = gf_filter_pid_get_property(pid, GF_PROP_PID_ID);
-	if (prop) es_id = prop->value.uint;
 
 	ck = gf_clock_attach(ck_namespace, scene, clockID, es_id, flag);
 	if (!ck) return GF_OUT_OF_MEM;
@@ -673,9 +697,6 @@ GF_Err gf_odm_setup_pid(GF_ObjectManager *odm, GF_FilterPid *pid)
 	ck->service_id = odm->ServiceID;
 	clock_inherited = GF_FALSE;
 
-	if (es_id==ck->clock_id)
-		odm->owns_clock = GF_TRUE;
-
 	if (scene->root_od->subscene && scene->root_od->subscene->is_dynamic_scene && !scene->root_od->ck)
 		scene->root_od->ck = ck;
 
@@ -684,6 +705,11 @@ clock_setup:
 	odm->ck = ck;
 	odm->clock_inherited = clock_inherited;
 
+	if (es_id==ck->clock_id)
+		odm->owns_clock = GF_TRUE;
+	else if (ckid_use_odid && (ckid_use_odid==ck->clock_id))
+		odm->owns_clock = GF_TRUE;
+
 	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_DELAY);
 	odm->timestamp_offset = prop ? prop->value.longsint : 0;
 	prop = gf_filter_pid_get_property(pid, GF_PROP_PID_TIMESCALE);
@@ -705,10 +731,10 @@ void gf_odm_update_duration(GF_ObjectManager *odm, GF_FilterPid *pid)
 	GF_PropertyEntry *pe = NULL;
 	const GF_PropertyValue *prop;
 	prop = gf_filter_pid_get_info(pid, GF_PROP_PID_DURATION, &pe);
-	if (prop) {
-		dur = prop->value.lfrac.num;
+	if (prop && prop->value.lfrac.den) {
+		dur = (prop->value.lfrac.num<0) ? -prop->value.lfrac.num : prop->value.lfrac.num;
 		dur *= 1000;
-		if (prop->value.lfrac.den) dur /= prop->value.lfrac.den;
+		dur /= prop->value.lfrac.den;
 	}
 	gf_filter_release_property(pe);
 	
@@ -772,6 +798,14 @@ void gf_odm_play(GF_ObjectManager *odm)
 		/*reset*/
 		gf_filter_pck_unref(odm->mo->pck);
 		odm->mo->pck = NULL;
+		//reset all textures pointing to this stream
+		u32 i, count = gf_list_count(scene->compositor->textures);
+		for (i=0; i<count; i++) {
+			GF_TextureHandler *txh = (GF_TextureHandler *)gf_list_get(scene->compositor->textures, i);
+			if (!txh || (txh->stream != odm->mo)) break;
+			txh->data = NULL;
+			txh->frame_ifce = NULL;
+		}
 	}
 
 	if (odm->parentscene) {
@@ -810,7 +844,7 @@ void gf_odm_play(GF_ObjectManager *odm)
 	}
 	/*play from current time*/
 	else {
-		ck_time = gf_clock_media_time(clock);
+		ck_time = (Double) gf_clock_media_time(clock);
 		ck_time /= 1000;
 		start_range_is_clock = 1;
 	}
@@ -830,7 +864,7 @@ void gf_odm_play(GF_ObjectManager *odm)
 				ck_time = (Double) odm->parentscene->root_od->addon->media_pts;
 				ck_time /= 90000;
 			} else {
-				ck_time = gf_clock_time(clock);
+				ck_time = (Double) gf_clock_time_absolute(clock);
 				ck_time /= 1000;
 			}
 		}
@@ -858,7 +892,7 @@ void gf_odm_play(GF_ObjectManager *odm)
 	}
 
 	com.play.speed = clock->speed;
-	if (ABS(com.play.speed)>scene->compositor->max_vspeed)
+	if (scene->compositor->max_vspeed && (ABS(com.play.speed) > ABS(scene->compositor->max_vspeed)))
 		com.play.drop_non_ref = GF_TRUE;
 
 #ifndef GPAC_DISABLE_VRML
@@ -875,7 +909,7 @@ void gf_odm_play(GF_ObjectManager *odm)
 		}
 
 		com.play.speed = FIX2FLT(ctrl->control->mediaSpeed);
-		if (ABS(com.play.speed)>scene->compositor->max_vspeed)
+		if (scene->compositor->max_vspeed && (ABS(com.play.speed) > ABS(scene->compositor->max_vspeed)))
 			com.play.drop_non_ref = GF_TRUE;
 		/*if the channel doesn't control the clock, jump to current time in the controled range, not just the beginning*/
 		if ((ctrl->stream != odm->mo) && (ck_time>com.play.start_range) && (com.play.end_range>com.play.start_range)
@@ -922,16 +956,18 @@ void gf_odm_play(GF_ObjectManager *odm)
 		}
 	}
 
-	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: At OTB %u requesting PLAY from %g to %g (clock init %d) - speed %g\n", odm->ID, odm->scene_ns->url, gf_filter_pid_get_name(odm->pid), gf_clock_time(clock), com.play.start_range, com.play.end_range, clock->clock_init, com.play.speed));
+	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: At OTB %u requesting PLAY from %g to %g (clock init %d) - speed %g\n", odm->ID, odm->scene_ns->url, odm->pid ? gf_filter_pid_get_name(odm->pid) : "ROOTSCENE", gf_clock_time(clock), com.play.start_range, com.play.end_range, clock->clock_init, com.play.speed));
 
 
 	if (odm->state != GF_ODM_STATE_PLAY) {
 		odm->state = GF_ODM_STATE_PLAY;
 
-		odm->nb_buffering ++;
-		scene->nb_buffering++;
-		//start buffering
-		gf_clock_buffer_on(odm->ck);
+		if (odm->buffer_max_ms) {
+			odm->nb_buffering++;
+			scene->nb_buffering++;
+			//start buffering
+			gf_clock_buffer_on(odm->ck);
+		}
 
 		odm->has_seen_eos = GF_FALSE;
 
@@ -951,7 +987,7 @@ void gf_odm_play(GF_ObjectManager *odm)
 				com.base.on_pid = xpid->pid;
 				odm->has_seen_eos = GF_FALSE;
 
-				odm->nb_buffering ++;
+				odm->nb_buffering++;
 				scene->nb_buffering++;
 				//start buffering
 				gf_clock_buffer_on(odm->ck);
@@ -1031,7 +1067,7 @@ void gf_odm_stop(GF_ObjectManager *odm, Bool force_close)
 	odm->has_seen_eos = GF_FALSE;
 	odm->state = GF_ODM_STATE_STOP;
 	GF_FEVT_INIT(com, GF_FEVT_STOP, odm->pid)
-	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s At OTB %u requesting STOP\n", odm->ID, odm->scene_ns->url, gf_filter_pid_get_name(odm->pid), odm->ck ? gf_clock_time(odm->ck) : 0 ));
+	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s At OTB %u requesting STOP\n", odm->ID, odm->scene_ns->url, odm->pid ? gf_filter_pid_get_name(odm->pid) : "ROOTSCENE", odm->ck ? gf_clock_time(odm->ck) : 0 ));
 
 	gf_filter_pid_send_event(odm->pid, &com);
 	gf_list_del_item(scene->compositor->systems_pids, odm->pid);
@@ -1053,7 +1089,7 @@ void gf_odm_stop(GF_ObjectManager *odm, Bool force_close)
 		scene->nb_buffering -= odm->nb_buffering;
 		while (odm->nb_buffering && odm->ck) {
 			gf_clock_buffer_off(odm->ck);
-			odm->nb_buffering --;
+			odm->nb_buffering--;
 		}
 		if (!scene->nb_buffering) {
 			gf_scene_buffering_info(scene, GF_TRUE);
@@ -1233,13 +1269,13 @@ void gf_odm_pause(GF_ObjectManager *odm)
 
 	//cleanup - we need to enter in stop state for broadcast modes
 	if (odm->flags & GF_ODM_NO_TIME_CTRL) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: no time control available in source filter, will not pause\n", odm->ID, odm->scene_ns->url, gf_filter_pid_get_name(odm->pid)));
+		GF_LOG(GF_LOG_WARNING, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: no time control available in source filter, will not pause\n", odm->ID, odm->scene_ns->url, odm->pid ? gf_filter_pid_get_name(odm->pid) : "ROOTSCENE"));
 		return;
 	}
 
 	scene = gf_scene_get_root_scene(scene);
 
-	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: At OTB %u requesting PAUSE (clock init %d)\n", odm->ID, odm->scene_ns->url, gf_filter_pid_get_name(odm->pid), gf_clock_time(odm->ck), odm->ck->clock_init ));
+	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: At OTB %u requesting PAUSE (clock init %d)\n", odm->ID, odm->scene_ns->url, odm->pid ? gf_filter_pid_get_name(odm->pid) : "ROOTSCENE", gf_clock_time(odm->ck), odm->ck->clock_init ));
 
 	GF_FEVT_INIT(com, GF_FEVT_PAUSE, odm->pid);
 	gf_clock_pause(odm->ck);
@@ -1305,12 +1341,14 @@ void gf_odm_resume(GF_ObjectManager *odm)
 	ctrl = gf_odm_get_mediacontrol(odm);
 #endif
 
-	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] CH%d: At OTB %u requesting RESUME (clock init %d)\n", odm->ID, odm->scene_ns->url, gf_filter_pid_get_name(odm->pid), gf_clock_time(odm->ck), odm->ck->clock_init ));
+	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[ODM%d %s] PID %s: At OTB %u requesting RESUME (clock init %d)\n", odm->ID, odm->scene_ns->url, odm->pid ? gf_filter_pid_get_name(odm->pid) : "ROOTSCENE", gf_clock_time(odm->ck), odm->ck->clock_init ));
 
 	GF_FEVT_INIT(com, GF_FEVT_RESUME, odm->pid);
 	com.play.speed = odm->ck->speed;
+#ifndef GPAC_DISABLE_VRML
 	if (ctrl) com.play.speed  = ctrl->control->mediaSpeed;
-	if (ABS(com.play.speed)>scene->compositor->max_vspeed)
+#endif
+	if (scene->compositor->max_vspeed && (ABS(com.play.speed) > ABS(scene->compositor->max_vspeed)))
 		com.play.drop_non_ref = GF_TRUE;
 
 	gf_clock_resume(odm->ck);
@@ -1359,8 +1397,9 @@ void gf_odm_set_speed(GF_ObjectManager *odm, Fixed speed, Bool adjust_clock_spee
 
 	GF_FEVT_INIT(com, GF_FEVT_SET_SPEED, odm->pid);
 
+	odm->too_slow = GF_FALSE;
 	com.play.speed = FIX2FLT(speed);
-	if (ABS(com.play.speed)>scene->compositor->max_vspeed)
+	if (scene->compositor->max_vspeed && (ABS(com.play.speed) > ABS(scene->compositor->max_vspeed)))
 		com.play.drop_non_ref = GF_TRUE;
 
 	gf_filter_pid_send_event(odm->pid, &com);
@@ -1453,18 +1492,18 @@ void gf_odm_init_segments(GF_ObjectManager *odm, GF_List *list, MFURL *url)
 	}
 }
 
-static Bool odm_update_buffer(GF_Scene *scene, GF_ObjectManager *odm, GF_FilterPid *pid, Bool *signal_eob)
+
+/*DO NOT use get_packet here, we don't want to trigger reconfig*/
+static Bool odm_update_buffer(GF_Scene *scene, GF_ObjectManager *odm, GF_FilterPid *pid, Bool check_full_buffer, Bool *signal_eob)
 {
 	u32 timescale;
-	u64 buffer_duration = gf_filter_pid_query_buffer_duration(pid, GF_FALSE)/1000;
+	u64 buffer_duration = gf_filter_pid_query_buffer_duration(pid, check_full_buffer)/1000;
 	if (odm->ck && ! odm->ck->clock_init) {
 		u64 time;
-		GF_FilterPacket *pck = gf_filter_pid_get_packet(pid);
-		if (!pck) return GF_TRUE;
-		timescale = gf_filter_pck_get_timescale(pck);
+		if (!gf_filter_pid_get_first_packet_cts(pid, &time))
+			return GF_TRUE;
+		timescale = gf_filter_pid_get_timescale(pid);
 
-		time = gf_filter_pck_get_cts(pck);
-		if (time==GF_FILTER_NO_TS) time = gf_filter_pck_get_dts(pck);
 		if (time==GF_FILTER_NO_TS) {
 			//this usually happens with BT/XMT playback
 			GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("No timestamp on first packet, using 0\n"));
@@ -1478,23 +1517,20 @@ static Bool odm_update_buffer(GF_Scene *scene, GF_ObjectManager *odm, GF_FilterP
 			time-= -odm->timestamp_offset;
 		}
 
-		time *= 1000;
-		time /= timescale;
-		gf_clock_set_time(odm->ck, (u32) time);
+		gf_clock_set_time(odm->ck, time, timescale);
 		odm->media_current_time = 0;
 		if (odm->parentscene) {
 			odm->parentscene->root_od->media_start_time = 0;
 			odm->parentscene->root_od->media_current_time = 0;
 		}
-		gf_odm_check_clock_mediatime(odm);
 
-		if (pck && gf_filter_pck_is_blocking_ref(pck))
+		if (gf_filter_pid_first_packet_is_blocking_ref(pid))
 			odm->blocking_media = GF_TRUE;
 	}
 
 	//TODO abort buffering when errors are found on the input chain !!
 	if (odm->blocking_media || (buffer_duration >= odm->buffer_playout_ms)) {
-		odm->nb_buffering --;
+		odm->nb_buffering--;
 		assert(scene->nb_buffering);
 		scene->nb_buffering--;
 		if (!scene->nb_buffering) {
@@ -1503,7 +1539,7 @@ static Bool odm_update_buffer(GF_Scene *scene, GF_ObjectManager *odm, GF_FilterP
 		if (odm->ck)
 			gf_clock_buffer_off(odm->ck);
 	} else if (gf_filter_pid_has_seen_eos(pid) ) {
-		odm->nb_buffering --;
+		odm->nb_buffering--;
 		assert(scene->nb_buffering);
 		scene->nb_buffering--;
 		//if eos while buffering, consider the last rebuffer an error
@@ -1522,6 +1558,7 @@ static Bool odm_update_buffer(GF_Scene *scene, GF_ObjectManager *odm, GF_FilterP
 	return GF_FALSE;
 }
 
+/*DO NOT use get_packet here, we don't want to trigger reconfig*/
 Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 {
 	u32 timescale;
@@ -1529,34 +1566,53 @@ Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 	GF_Scene *scene;
 	GF_FilterClockType ck_type;
 	u64 clock_reference;
-	GF_FilterPacket *pck;
-	Bool check_disc = GF_TRUE;
+	u64 pck_time = 0;
+	Bool has_pck = GF_FALSE;
+	Bool check_disc = GF_FALSE;
+	Bool check_full_buffer = GF_TRUE;
 
 	assert(odm);
 
-	if (!pid)
+	if (!pid) {
+		check_full_buffer = GF_FALSE;
 		pid = odm->pid;
+	} else {
+		//when a pid is given, this is a non-AV codec (bifs/od, svg, text), try to map media timeline
+		//otherwise we cannot since we don't want to use get_packet here
+		gf_odm_check_clock_mediatime(odm);
+	}
 
 	scene = odm->subscene ? odm->subscene : odm->parentscene;
 	if (!scene) return GF_FALSE;
-	pck = gf_filter_pid_get_packet(pid);
+
+	if (!gf_filter_pid_first_packet_is_empty(pid)) {
+		if (gf_filter_pid_get_first_packet_cts(pid, &pck_time))
+			has_pck = GF_TRUE;
+	}
 	ck_type = gf_filter_pid_get_clock_info(pid, &clock_reference, &timescale);
 
-	if (!odm->ck->clock_init && ck_type) {
-		clock_reference *= 1000;
-		clock_reference /= timescale;
-		gf_clock_set_time(odm->ck, (u32) clock_reference);
-		if (odm->parentscene)
-			odm->parentscene->root_od->media_start_time = 0;
-	}
+	if (!odm->ck->clock_init) {
+		if (ck_type) {
+			gf_clock_set_time(odm->ck, clock_reference, timescale);
+			if (odm->parentscene) odm->parentscene->root_od->media_start_time = 0;
+		}
+		else if (has_pck && odm->owns_clock && !odm->ck->clock_init) {
+			gf_clock_set_time(odm->ck, pck_time, gf_filter_pid_get_timescale(pid) );
 
+			if (gf_filter_pid_first_packet_is_blocking_ref(pid)) {
+				odm->blocking_media = GF_TRUE;
+				if (odm->nb_buffering)
+					odm_update_buffer(scene, odm, pid, check_full_buffer, &signal_eob);
+			}
+		}
+	}
 	if (odm->nb_buffering) {
 	 	GF_ODMExtraPid *xpid;
 	 	u32 i=0;
-	 	Bool ret = odm_update_buffer(scene, odm, pid, &signal_eob);
+		Bool ret = odm_update_buffer(scene, odm, pid, check_full_buffer, &signal_eob);
 	 	if (ret) return GF_TRUE;
 		while (odm->nb_buffering && (xpid = gf_list_enum(odm->extra_pids, &i))) {
-			ret = odm_update_buffer(scene, odm, xpid->pid, &signal_eob);
+			ret = odm_update_buffer(scene, odm, xpid->pid, GF_FALSE, &signal_eob);
 			if (ret) return GF_TRUE;
 		}
 	}
@@ -1569,7 +1625,7 @@ Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 			if (!an_odm->pid) continue;
 
 			if (an_odm->nb_buffering)
-	 			odm_update_buffer(scene, an_odm, an_odm->pid, &signal_eob);
+				odm_update_buffer(scene, an_odm, an_odm->pid, GF_FALSE, &signal_eob);
 		}
 	} else if (!odm->blocking_media && odm->buffer_min_ms && odm->pid && odm->ck && odm->ck->clock_init && !gf_filter_pid_has_seen_eos(odm->pid) ) {
 		u64 buffer_duration = gf_filter_pid_query_buffer_duration(odm->pid, GF_FALSE)/1000;
@@ -1587,31 +1643,31 @@ Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 
 	if (odm->parentscene && odm->parentscene->root_od->addon)
 		check_disc = GF_FALSE;
+	else if ((odm->type== GF_STREAM_VISUAL) || (odm->type== GF_STREAM_AUDIO))
+		check_disc = GF_TRUE;
+
 	//handle both PCR discontinuities or TS looping when no PCR disc is present/signaled
-	if (pck && check_disc) {
+	if (has_pck && check_disc) {
 		s32 diff=0;
-		u64 pck_time = 0;
 		u32 clock_time = gf_clock_time(odm->ck);
 		s32 diff_to = 0;
+		u64 ckref_orig = clock_reference;
 		if (ck_type) {
-			clock_reference *= 1000;
-			clock_reference /= timescale;
+			clock_reference = gf_timestamp_to_clocktime(clock_reference, timescale);
 			diff = (s32) clock_time + odm->buffer_playout_ms;
 			diff -= (s32) clock_reference;
 			GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("Clock %d (ODM %d) reference found "LLU" ms clock time %d ms - diff %d - type %d\n", odm->ck->clock_id, odm->ID, clock_reference, clock_time, diff, ck_type));
 
 			//if explicit clock discontinuity, mark clock
 			if (ck_type==GF_FILTER_CLOCK_PCR_DISC)
-				odm->ck->ocr_discontinuity_time = (u32) (1+clock_reference);
+				odm->ck->ocr_discontinuity_time = 1 + gf_timestamp_rescale(ckref_orig, timescale, 1000);
 		}
-		pck_time = gf_filter_pck_get_cts(pck);
-		timescale = gf_filter_pck_get_timescale(pck);
+		timescale = gf_filter_pid_get_timescale(pid);
 		if (pck_time != GF_FILTER_NO_TS) {
-			pck_time *= 1000;
-			pck_time /= timescale;
+			pck_time = gf_timestamp_to_clocktime(pck_time, timescale);
 			pck_time += 1;
 			diff = (u32) ((u64) clock_time - pck_time);
-			diff_to = odm->ck->ocr_discontinuity_time ? 500 : 8000;
+			diff_to = (s32) (odm->ck->ocr_discontinuity_time ? 500 : 8000) * ABS(FIX2INT(odm->ck->speed) );
 		}
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("Clock %d (ODM %d) pck time %d - clock ref "LLU" clock time %d - diff %d vs %d\n", odm->ck->clock_id, odm->ID, pck_time, clock_reference, clock_time, diff, diff_to));
 
@@ -1629,7 +1685,7 @@ Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 			if (diff_pck_old_clock > diff_pck_new_clock) {
 				u32 i, count;
 				GF_Scene *in_scene = odm->subscene ? odm->subscene : odm->parentscene;
-				GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("Clock %d (ODM %d) discontinuity detected "LLU" clock time %d - diff %d - type %d - pck time "LLU"\n", odm->ck->clock_id, odm->ID, clock_reference, clock_time, diff, ck_type, pck_time-1));
+				GF_LOG(GF_LOG_WARNING, GF_LOG_COMPTIME, ("Clock %d (ODM %d) discontinuity detected "LLU" clock time %d - diff %d - type %d - pck time "LLU"\n", odm->ck->clock_id, odm->ID, clock_reference, clock_time, diff, ck_type, pck_time-1));
 
 				count = gf_list_count(in_scene->resources);
 				for (i=0; i<count; i++) {
@@ -1638,15 +1694,25 @@ Bool gf_odm_check_buffering(GF_ObjectManager *odm, GF_FilterPid *pid)
 					an_odm->prev_clock_at_discontinuity_plus_one = 1 + clock_time;
 				}
 				odm->ck->clock_init = GF_FALSE;
-				gf_clock_set_time(odm->ck, odm->ck->ocr_discontinuity_time ? odm->ck->ocr_discontinuity_time - 1 : (u32) clock_reference);
+
+				if (odm->ck->ocr_discontinuity_time)
+					gf_clock_set_time(odm->ck, odm->ck->ocr_discontinuity_time - 1, 1000);
+				else
+					gf_clock_set_time(odm->ck, ckref_orig, timescale);
+
 				odm->ck->ocr_discontinuity_time = 0;
+				//if we had media time mapping, get media time in last clock value and reassign mapping
+				if (odm->ck->has_media_time_shift) {
+					u64 mtime = gf_clock_to_media_time(odm->ck, clock_time);
+					odm->ck->media_time_orig = mtime;
+					odm->ck->media_ts_orig = gf_timestamp_rescale(ckref_orig, timescale, 1000);
+				}
 			}
 		}
 	} else if (ck_type) {
-		clock_reference *= 1000;
-		clock_reference /= timescale;
+		clock_reference = gf_timestamp_rescale(clock_reference, timescale, 1000);
 		if (ck_type==GF_FILTER_CLOCK_PCR_DISC)
-			odm->ck->ocr_discontinuity_time = (u32) (1 + clock_reference);
+			odm->ck->ocr_discontinuity_time = 1 + clock_reference;
 
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_COMPTIME, ("Clock %d (ODM %d) received "LLU" type %d clock time %d no pending packets\n", odm->ck->clock_id, odm->ID, clock_reference, ck_type, gf_clock_time(odm->ck)));
 	}
@@ -1805,7 +1871,7 @@ static void get_codec_stats(GF_FilterPid *pid, GF_MediaInfo *info, Bool scalable
 	GF_FilterPidStatistics stats;
 	gf_filter_pid_get_statistics(pid, &stats, scalable_addon ? GF_STATS_LOCAL : GF_STATS_LOCAL_INPUTS);
 
-	info->avg_bitrate = stats.avgerage_bitrate;
+	info->avg_bitrate = stats.average_bitrate;
 	info->max_bitrate = stats.max_bitrate;
 	info->nb_dec_frames = stats.nb_processed;
 	info->max_dec_time = stats.max_process_time;
@@ -1824,6 +1890,7 @@ static void get_codec_stats(GF_FilterPid *pid, GF_MediaInfo *info, Bool scalable
 GF_EXPORT
 GF_Err gf_odm_get_object_info(GF_ObjectManager *odm, GF_MediaInfo *info)
 {
+	GF_FilterPid *gf_filter_pid_first_pid_for_source(GF_FilterPid *pid, GF_Filter *source);
 	const GF_PropertyValue *prop;
 	GF_ObjectManager *an_odm;
 	GF_FilterPid *pid;
@@ -1848,8 +1915,18 @@ GF_Err gf_odm_get_object_info(GF_ObjectManager *odm, GF_MediaInfo *info)
 		pid = an_odm->pid;
 	}
 
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode() && pid) {
+		GF_FilterPidStatistics stats;
+		gf_filter_pid_get_statistics(odm->pid, &stats, GF_STATS_SINK);
+		gf_filter_pid_set_rt_stats(pid, 0, 0, 0);
+		gf_filter_pid_share_origin(pid, pid);
+		if (odm->parentscene)
+			gf_filter_pid_first_pid_for_source(pid, odm->parentscene->root_od->scene_ns->source_filter);
+	}
+#endif
+
 	if (odm->subscene && !odm->pid && odm->addon && !gf_list_count(odm->subscene->resources)) {
-		GF_FilterPid *gf_filter_pid_first_pid_for_source(GF_FilterPid *pid, GF_Filter *source);
 		u32 i;
 		for (i=0; i<gf_list_count(odm->parentscene->resources); i++) {
 			GF_ObjectManager *par_odm = gf_list_get(odm->parentscene->resources, i);
@@ -1874,16 +1951,16 @@ GF_Err gf_odm_get_object_info(GF_ObjectManager *odm, GF_MediaInfo *info)
 	if (pid) {
 		/*since we don't remove ODs that failed setup, check for clock*/
 		if (odm->ck) {
-			info->current_time = odm->media_current_time;
+			//info->current_time = (Double) odm->media_current_time;
 			info->ntp_diff = odm->last_drawn_frame_ntp_diff;
-			info->current_time = gf_clock_media_time(odm->ck);
+			info->current_time = (Double) gf_clock_media_time(odm->ck);
 
 		}
 		info->current_time /= 1000;
 		info->nb_dropped = odm->nb_dropped;
 	} else if (odm->subscene) {
 		if (odm->subscene->root_od && odm->subscene->root_od->ck) {
-			info->current_time = gf_clock_media_time(odm->subscene->root_od->ck);
+			info->current_time = (Double) gf_clock_media_time(odm->subscene->root_od->ck);
 			info->current_time /= 1000;
 		}
 		info->duration = (Double) (s64)odm->subscene->duration;
@@ -2002,30 +2079,25 @@ void gf_odm_check_clock_mediatime(GF_ObjectManager *odm)
 	u64 timestamp;
 	u32 timescale;
 	u32 i;
-	Double media_time, shift;
+	Double media_time;
 	GF_Scene *scene;
 	const GF_PropertyValue *p;
-	GF_PropertyEntry *pe=NULL;
-	if (!odm->owns_clock) return;
-
-	if (odm->ck->has_media_time_shift) return;
+	if (!odm->owns_clock || odm->ck->has_media_time_shift) return;
 
 	timescale = gf_filter_pid_get_timescale(odm->pid);
 	if (!timescale) return;
 
-	p = gf_filter_pid_get_info_str(odm->pid, "time:timestamp", &pe);
+	GF_FilterPacket *pck = gf_filter_pid_get_packet(odm->pid);
+	if (!pck) return;
+
+	timestamp = gf_filter_pck_get_cts(pck);
+	if (timestamp == GF_FILTER_NO_TS) return;
+	p = gf_filter_pck_get_property(pck, GF_PROP_PCK_MEDIA_TIME);
 	if (!p) return;
-	timestamp = p->value.longuint;
-	p = gf_filter_pid_get_info_str(odm->pid, "time:media", &pe);
-	if (!p) return;
-	gf_filter_release_property(pe);
 	media_time = p->value.number;
 
-	shift = (Double) timestamp;
-	shift /= timescale;
-	shift -= ((Double)odm->ck->init_timestamp)/1000;
-	media_time += shift;
-	odm->ck->media_time_at_init = (u32) (media_time * 1000);
+	odm->ck->media_ts_orig = gf_timestamp_rescale(timestamp, timescale, 1000);
+	odm->ck->media_time_orig = (u64) (media_time * 1000);
 	odm->ck->has_media_time_shift = GF_TRUE;
 
 	scene = odm->subscene ? odm->subscene : odm->parentscene;
