@@ -24,7 +24,6 @@
  */
 
 #include "texturing.h"
-#include <gpac/options.h>
 
 #include "nodes_stacks.h"
 
@@ -167,7 +166,7 @@ void gf_sc_texture_restart(GF_TextureHandler *txh)
 {
 	if (!txh->is_open) return;
 	gf_sc_texture_release_stream(txh);
-	txh->stream_finished = 0;
+	txh->stream_finished = GF_FALSE;
 	gf_mo_restart(txh->stream);
 }
 
@@ -213,10 +212,12 @@ void gf_sc_texture_update_frame(GF_TextureHandler *txh, Bool disable_resync)
 
 	if (!txh->stream) {
 		txh->data = NULL;
+		txh->frame_ifce = NULL;
 		return;
 	}
 	if (txh->stream->config_changed) {
 		txh->data = NULL;
+		txh->frame_ifce = NULL;
 	}
 
 	/*already refreshed*/
@@ -308,8 +309,8 @@ void gf_sc_texture_update_frame(GF_TextureHandler *txh, Bool disable_resync)
 		}
 		return;
 	}
-	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[Texture %p] Updated new frame at clock time %d (%d ms) time %u ms\n", txh, gf_clock_time(txh->stream->odm->ck), gf_sys_clock(), ts));
-	txh->stream_finished = 0;
+	GF_LOG(GF_LOG_INFO, GF_LOG_COMPTIME, ("[Texture %p] Updated new frame at clock time %u (%u ms) time %u ms\n", txh, gf_clock_time(txh->stream->odm->ck), gf_sys_clock(), ts));
+	txh->stream_finished = GF_FALSE;
 	txh->needs_release = 1;
 	txh->last_frame_time = ts;
 	txh->size = size;
