@@ -480,6 +480,8 @@ static Bool tx_setup_format(GF_TextureHandler *txh)
 	case GF_PIXEL_VYUY:
 	case GF_PIXEL_UYVY:
 	case GF_PIXEL_GL_EXTERNAL:
+	case GF_PIXEL_YUV444_10_PACK:
+	case GF_PIXEL_V210:
 #if !defined(GPAC_USE_TINYGL) && !defined(GPAC_USE_GLES1X)
 		if (!compositor->visual->compositor->shader_mode_disabled) {
 			break;
@@ -1295,7 +1297,9 @@ void gf_sc_texture_set_stencil(GF_TextureHandler *txh, GF_EVGStencil * stencil)
 
 void gf_sc_texture_check_pause_on_first_load(GF_TextureHandler *txh, Bool do_freeze)
 {
-	if (!txh->stream || !txh->tx_io)
+	if (!txh->stream || !txh->tx_io || !txh->stream->odm)
+		return;
+	if (txh->stream->odm->flags & GF_ODM_IS_SPARSE)
 		return;
 
 	if (do_freeze) {
