@@ -169,7 +169,7 @@ u64 gf_evg_argb_to_ayuv_wide(GF_EVGSurface *surf, u64 col)
 #endif
 
 
-	return evg_make_col_wide(a, y, cb, cr);
+	return GF_COLW_ARGB(a, y, cb, cr);
 }
 u64 gf_evg_ayuv_to_argb_wide(GF_EVGSurface *surf, u64 col)
 {
@@ -203,9 +203,10 @@ u64 gf_evg_ayuv_to_argb_wide(GF_EVGSurface *surf, u64 col)
 
 #endif
 
-	return evg_make_col_wide(a, r, g, b);
+	return GF_COLW_ARGB(a, r, g, b);
 }
 
+#if 0 //unused
 void evg_make_ayuv_color_mx(GF_ColorMatrix *cmat, GF_ColorMatrix *yuv_cmat)
 {
 	GF_ColorMatrix cmx_y2r, cmx_r2y;
@@ -252,7 +253,7 @@ void evg_make_ayuv_color_mx(GF_ColorMatrix *cmat, GF_ColorMatrix *yuv_cmat)
 	gf_cmx_multiply(yuv_cmat, cmat);
 	gf_cmx_multiply(yuv_cmat, &cmx_y2r);
 }
-
+#endif
 
 /*
 			YUV420p part
@@ -277,20 +278,22 @@ static void overmask_yuv420p_const_run(u8 a, u8 val, u8 *ptr, u32 count, short x
 		count--;
 	}
 }
-void evg_yuv420p_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_yuv420p_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
-	char *pU = surf->pixels + surf->height *surf->pitch_y;
-	char *pV;
-	pU +=  y/2 * surf->pitch_y/2;
-	pV = pU + surf->height/2 * surf->pitch_y/2;
+	u8 *pU = rctx->surf->pixels + rctx->surf->height * rctx->surf->pitch_y;
+	u8 *pV;
+	pU +=  y/2 * rctx->surf->pitch_y/2;
+	pV = pU + rctx->surf->height/2 * rctx->surf->pitch_y/2;
+
+	//no need to swap u and V in const flush, they have been swaped when setting up the brush
 
 	//we are at an odd line, write uv
-	for (i=0; i<surf->width; i+=2) {
+	for (i=0; i<rctx->surf->width; i+=2) {
 		u8 dst;
 
 		//even line
-		a = surf->uv_alpha[i] + surf->uv_alpha[i+1];
+		a = rctx->uv_alpha[i] + rctx->uv_alpha[i+1];
 		//odd line
 		a += surf_uv_alpha[i] + surf_uv_alpha[i+1];
 
@@ -313,10 +316,10 @@ void evg_yuv420p_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu,
 			}
 		}
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, rctx->surf->uv_alpha_alloc);
 }
 
-void evg_yuv420p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	char *pY = surf->pixels;
 	u8 *surf_uv_alpha;
@@ -326,12 +329,12 @@ void evg_yuv420p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *su
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	}
 	else if (write_uv) {
-		surf_uv_alpha = surf->uv_alpha + surf->width;
+		surf_uv_alpha = rctx->uv_alpha + surf->width;
 	} else {
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	}
 
 	pY +=  y * surf->pitch_y;
@@ -359,13 +362,13 @@ void evg_yuv420p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *su
 			memset(surf_uv_alpha + spans[i].x, 0xFF, spans[i].len);
 		}
 	}
-	if (write_uv) {
-		surf->yuv_flush_uv(surf, surf_uv_alpha, cu, cv, y);
+	if (write_uv && !rctx->no_yuv_flush) {
+		surf->yuv_flush_uv(surf, rctx, surf_uv_alpha, cu, cv, y);
 	}
 
 }
 
-void evg_yuv420p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u32 a;
 	char *pY = surf->pixels;
@@ -376,11 +379,11 @@ void evg_yuv420p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	} else if (write_uv) {
-		surf_uv_alpha = surf->uv_alpha + surf->width;
+		surf_uv_alpha = rctx->uv_alpha + surf->width;
 	} else {
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	}
 
 	pY +=  y * surf->pitch_y;
@@ -418,26 +421,32 @@ void evg_yuv420p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *
 		}
 	}
 	//we are at an odd line, write uv
-	if (write_uv)
-		surf->yuv_flush_uv(surf, surf_uv_alpha, cu, cv, y);
+	if (write_uv && !rctx->no_yuv_flush)
+		surf->yuv_flush_uv(surf, rctx, surf_uv_alpha, cu, cv, y);
 }
 
 
-void evg_yuv420p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
+void evg_yuv420p_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
 {
 	u32 i;
-	char *pU, *pV;
+	u8 *pU, *pV;
 	pU = surf->pixels + surf->height *surf->pitch_y;
 	pU += y/2 * surf->pitch_y/2;
 	pV = pU + surf->height/2 * surf->pitch_y/2;
 
+	if (surf->swap_uv) {
+		u8 *tmp = pU;
+		pU = pV;
+		pV = tmp;
+	}
+
 	for (i=0; i<surf->width; i+=2) {
 		u32 a, a11, a12, a21, a22;
-		u32 idx1=3*i;
-		u32 idx2=3*i + 3;
+		u32 idx1 = 3*i;
+		u32 idx2 = idx1 + 3;
 		//get alpha
-		a11 = (u32)surf->uv_alpha[idx1];
-		a12 = (u32)surf->uv_alpha[idx2];
+		a11 = (u32)rctx->uv_alpha[idx1];
+		a12 = (u32)rctx->uv_alpha[idx2];
 		a21 = (u32)surf_uv_alpha[idx1];
 		a22 = (u32)surf_uv_alpha[idx2];
 
@@ -454,9 +463,9 @@ void evg_yuv420p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 			if (a!=0xFF) {
  				cdst = *pU;
 			}
-			c11 = (u32)surf->uv_alpha[idx1];
+			c11 = (u32)rctx->uv_alpha[idx1];
 			if (a11!=0xFF) c11 = mul255_zero(a11, c11 - cdst) + cdst;
-			c12 = (u32)surf->uv_alpha[idx2];
+			c12 = (u32)rctx->uv_alpha[idx2];
 			if (a12!=0xFF) c12 = mul255_zero(a12, c12 - cdst) + cdst;
 			c21 = (u32)surf_uv_alpha[idx1];
 			if (a21!=0xFF) c21 = mul255_zero(a21, c21 - cdst) + cdst;
@@ -473,9 +482,9 @@ void evg_yuv420p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 			if (a!=0xFF) {
  				cdst = *pV;
 			}
-			c11 = (u32)surf->uv_alpha[idx1];
+			c11 = (u32)rctx->uv_alpha[idx1];
 			if (a11!=0xFF) c11 = mul255_zero(a11, c11 - cdst) + cdst;
-			c12 = (u32)surf->uv_alpha[idx2];
+			c12 = (u32)rctx->uv_alpha[idx2];
 			if (a12!=0xFF) c12 = mul255_zero(a12, c12 - cdst) + cdst;
 			c21 = (u32)surf_uv_alpha[idx1];
 			if (a21!=0xFF) c21 = mul255_zero(a21, c21 - cdst) + cdst;
@@ -492,11 +501,11 @@ void evg_yuv420p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 		pV++;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 
 }
 
-void evg_yuv420p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	s32 i;
 	char *pY = surf->pixels;
@@ -505,13 +514,13 @@ void evg_yuv420p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	} else if (write_uv) {
 		//second line of storage (we store alpha, cr, cb for each pixel)
-		surf_uv_alpha = surf->uv_alpha + 3*surf->width;
+		surf_uv_alpha = rctx->uv_alpha + 3*surf->width;
 	} else {
 		//first line of storage
-		surf_uv_alpha = surf->uv_alpha;
+		surf_uv_alpha = rctx->uv_alpha;
 	}
 
 	pY +=  y * surf->pitch_y;
@@ -523,9 +532,8 @@ void evg_yuv420p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf
 		u32 *p_col;
 		u32 len;
 		len = spans[i].len;
+		p_col = surf->fill_run(surf->sten, rctx, &spans[i], y);
 		spanalpha = spans[i].coverage;
-		evg_fill_run(surf->sten, surf, spans[i].x, y, len);
-		p_col = surf->stencil_pix_run;
 
 		s_pY = pY + spans[i].x;
 		x = spans[i].x;
@@ -559,8 +567,8 @@ void evg_yuv420p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf
 		}
 	}
 	//compute final u,v for both lines
-	if (write_uv) {
-		surf->yuv_flush_uv(surf, surf_uv_alpha, 0, 0, y);
+	if (write_uv && !rctx->no_yuv_flush) {
+		surf->yuv_flush_uv(surf, rctx, surf_uv_alpha, 0, 0, y);
 	}
 }
 
@@ -569,14 +577,26 @@ GF_Err evg_surface_clear_yuv420p(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 	s32 i;
 	u8 cy, cb, cr;
 	GF_EVGSurface *surf = (GF_EVGSurface *)_surf;
-	char *pY, *pU, *pV;
+	u8 *pY, *pU, *pV;
 
 	pY = surf->pixels + rc.y * surf->pitch_y + rc.x;
 	pU = surf->pixels + surf->height * surf->pitch_y + rc.y/2 * surf->pitch_y/2 + rc.x/2;
 	pV = surf->pixels + 5*surf->height * surf->pitch_y/4 + rc.y/2 * surf->pitch_y/2 + rc.x/2;
+	if (surf->swap_uv) {
+		u8 *tmp = pU;
+		pU = pV;
+		pV = tmp;
+	}
 
 	gf_evg_rgb_to_yuv(surf, col, &cy, &cb, &cr);
 
+	if (!rc.x && !rc.y && (rc.width==_surf->width) && (rc.height==_surf->height)) {
+		memset(pY, cy, _surf->pitch_y * _surf->height);
+		memset(pU, cb, _surf->pitch_y/2 * _surf->height/2);
+		memset(pV, cr, _surf->pitch_y/2 * _surf->height/2);
+		return GF_OK;
+	}
+
 	for (i = 0; i < rc.height; i++) {
 		memset(pY, cy, rc.width);
 		pY += surf->pitch_y;
@@ -596,7 +616,7 @@ GF_Err evg_surface_clear_yuv420p(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 */
 
 
-void evg_nv12_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_nv12_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
 	char *pU = surf->pixels + surf->height *surf->pitch_y;
@@ -606,7 +626,7 @@ void evg_nv12_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32
 		u8 dst;
 
 		//even line
-		a = surf->uv_alpha[i] + surf->uv_alpha[i+1];
+		a = rctx->uv_alpha[i] + rctx->uv_alpha[i+1];
 		//odd line
 		a += surf_uv_alpha[i] + surf_uv_alpha[i+1];
 
@@ -630,10 +650,10 @@ void evg_nv12_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32
 			}
 		}
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
-void evg_nv12_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_nv12_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i;
 	char *pU;
@@ -647,8 +667,8 @@ void evg_nv12_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 c
 		u32 idx2=3*i + 3;
 
 		//get alpha
-		a11 = (u32)surf->uv_alpha[idx1];
-		a12 = (u32)surf->uv_alpha[idx2];
+		a11 = (u32)rctx->uv_alpha[idx1];
+		a12 = (u32)rctx->uv_alpha[idx2];
 		a21 = (u32)surf_uv_alpha[idx1];
 		a22 = (u32)surf_uv_alpha[idx2];
 		a = a11+a12+a21+a22;
@@ -665,9 +685,9 @@ void evg_nv12_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 c
 			if (a!=0xFF)
 				cdst = pU[surf->idx_u];
 
-			c11 = (u32)surf->uv_alpha[idx1];
+			c11 = (u32)rctx->uv_alpha[idx1];
 			if (a11!=0xFF) c11 = mul255_zero(a11, c11 - cdst) + cdst;
-			c12 = (u32)surf->uv_alpha[idx2];
+			c12 = (u32)rctx->uv_alpha[idx2];
 			if (a12!=0xFF) c12 = mul255_zero(a12, c12 - cdst) + cdst;
 			c21 = (u32)surf_uv_alpha[idx1];
 			if (a21!=0xFF) c21 = mul255_zero(a21, c21 - cdst) + cdst;
@@ -684,9 +704,9 @@ void evg_nv12_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 c
 			if (a!=0xFF)
 				cdst = pU[surf->idx_v];
 
-			c11 = (u32)surf->uv_alpha[idx1];
+			c11 = (u32)rctx->uv_alpha[idx1];
 			if (a11!=0xFF) c11 = mul255_zero(a11, c11 - cdst) + cdst;
-			c12 = (u32)surf->uv_alpha[idx2];
+			c12 = (u32)rctx->uv_alpha[idx2];
 			if (a12!=0xFF) c12 = mul255_zero(a12, c12 - cdst) + cdst;
 			c21 = (u32)surf_uv_alpha[idx1];
 			if (a21!=0xFF) c21 = mul255_zero(a21, c21 - cdst) + cdst;
@@ -702,7 +722,7 @@ void evg_nv12_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 c
 		pU+=2;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
 
@@ -758,7 +778,7 @@ GF_Err evg_surface_clear_nv12(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col, B
 			YUV422 part
 */
 
-void evg_yuv422p_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_yuv422p_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
 	char *pU = surf->pixels + surf->height *surf->pitch_y;
@@ -769,7 +789,7 @@ void evg_yuv422p_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu,
 	for (i=0; i<surf->width; i+=2) {
 		u8 dst;
 
-		a = surf->uv_alpha[i] + surf->uv_alpha[i+1];
+		a = rctx->uv_alpha[i] + rctx->uv_alpha[i+1];
 
 		if (a) {
 			char *s_ptr_u, *s_ptr_v;
@@ -789,10 +809,10 @@ void evg_yuv422p_flush_uv_const(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 cu,
 			}
 		}
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
-void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
+void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
 {
 	u32 i;
 	char *pU, *pV;
@@ -805,8 +825,8 @@ void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 		u32 idx1=3*i;
 		u32 idx2=3*i + 3;
 		//get alpha
-		a1 = (u32)surf->uv_alpha[idx1];
-		a2 = (u32)surf->uv_alpha[idx2];
+		a1 = (u32)rctx->uv_alpha[idx1];
+		a2 = (u32)rctx->uv_alpha[idx2];
 		a = a1+a2;
 		if (a) {
 			u8 cdst=0;
@@ -818,9 +838,9 @@ void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 			if (a != 0xFF)
 				cdst = *pU;
 
-			c1 = (u32)surf->uv_alpha[idx1+1];
+			c1 = (u32)rctx->uv_alpha[idx1+1];
 			if (a1!=0xFF) c1 = mul255_zero(a1, c1 - cdst) + cdst;
-			c2 = (u32)surf->uv_alpha[idx2+1];
+			c2 = (u32)rctx->uv_alpha[idx2+1];
 			if (a2!=0xFF) c2 = mul255_zero(a2, c1 - cdst) + cdst;
 			chroma_u = c1 + c2;
 			chroma_u /= 2;
@@ -829,9 +849,9 @@ void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 			if (a != 0xFF)
 				cdst = *pV;
 
-			c1 = (u32)surf->uv_alpha[idx1+2];
+			c1 = (u32)rctx->uv_alpha[idx1+2];
 			if (a1!=0xFF) c1 = mul255_zero(a1, c1 - cdst) + cdst;
-			c2 = (u32)surf->uv_alpha[idx2+2];
+			c2 = (u32)rctx->uv_alpha[idx2+2];
 			if (a2!=0xFF) c2 = mul255_zero(a2, c1 - cdst) + cdst;
 			chroma_v = c1 + c2;
 			chroma_v /= 2;
@@ -844,7 +864,7 @@ void evg_yuv422p_flush_uv_var(GF_EVGSurface *surf, u8 *surf_uv_alpha, s32 _cu, s
 		pV++;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
 
@@ -867,6 +887,14 @@ GF_Err evg_surface_clear_yuv422p(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 
 	gf_evg_rgb_to_yuv(surf, col, &cy, &cb, &cr);
 
+	if (!rc.x && !rc.y && (rc.width==_surf->width) && (rc.height==_surf->height)) {
+		memset(pY, cy, _surf->pitch_y * _surf->height);
+		memset(pU, cb, _surf->pitch_y/2 * _surf->height);
+		memset(pV, cr, _surf->pitch_y/2 * _surf->height);
+		return GF_OK;
+	}
+
+
 	for (i = 0; i < rc.height; i++) {
 		memset(pY, cy, rc.width);
 		pY += surf->pitch_y;
@@ -882,7 +910,7 @@ GF_Err evg_surface_clear_yuv422p(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 			YUV444 part
 */
 
-void evg_yuv444p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	char *pY, *pU, *pV;
 	s32 i;
@@ -923,7 +951,7 @@ void evg_yuv444p_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *su
 	}
 }
 
-void evg_yuv444p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u32 a;
 	char *pY, *pU, *pV;
@@ -974,7 +1002,7 @@ void evg_yuv444p_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *
 	}
 }
 
-void evg_yuv444p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	s32 i;
 	char *pY, *pU, *pV;
@@ -989,9 +1017,8 @@ void evg_yuv444p_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf
 		u32 *p_col;
 		char *s_pY, *s_pU, *s_pV;
 		len = spans[i].len;
+		p_col = surf->fill_run(surf->sten, rctx, &spans[i], y);
 		spanalpha = spans[i].coverage;
-		evg_fill_run(surf->sten, surf, spans[i].x, y, len);
-		p_col = surf->stencil_pix_run;
 
 		s_pY = pY + spans[i].x;
 		s_pU = pU + spans[i].x;
@@ -1043,6 +1070,13 @@ GF_Err evg_surface_clear_yuv444p(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 
 	gf_evg_rgb_to_yuv(surf, col, &cy, &cb, &cr);
 
+	if (!rc.x && !rc.y && (rc.width==_surf->width) && (rc.height==_surf->height)) {
+		memset(pY, cy, _surf->pitch_y * _surf->height);
+		memset(pU, cb, _surf->pitch_y * _surf->height);
+		memset(pV, cr, _surf->pitch_y * _surf->height);
+		return GF_OK;
+	}
+
 	for (i = 0; i < rc.height; i++) {
 		memset(pY, cy, rc.width);
 		pY += surf->pitch_y;
@@ -1066,7 +1100,7 @@ static void overmask_yuvy(char *dst, u8 c, u32 alpha)
 	s32 dstc = (*dst) & 0xFF;
 	*dst = mul255(srca, srcc - dstc) + dstc;
 }
-void evg_yuyv_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuyv_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	char *pY;
 	s32 i;
@@ -1091,13 +1125,13 @@ void evg_yuyv_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 		if (spans[i].x%2) s_pY += 2;
 
 		if (spans[i].coverage != 0xFF) {
-			memset(surf->uv_alpha + spans[i].x, spans[i].coverage, len);
+			memset(rctx->uv_alpha + spans[i].x, spans[i].coverage, len);
 			while (len--) {
 				overmask_yuvy(s_pY, cy, spans[i].coverage);
 				s_pY += 2;
 			}
 		} else  {
-			memset(surf->uv_alpha + spans[i].x, 0xFF, len);
+			memset(rctx->uv_alpha + spans[i].x, 0xFF, len);
 
 			while (len--) {
 				*s_pY = cy;
@@ -1107,8 +1141,8 @@ void evg_yuyv_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 	}
 
 	for (i=0; i<(s32) surf->width; i+=2) {
-		u32 a = (u32)surf->uv_alpha[i];
-		a += (u32)surf->uv_alpha[i + 1];
+		u32 a = (u32)rctx->uv_alpha[i];
+		a += (u32)rctx->uv_alpha[i + 1];
 		if (a) {
 			a /=2;
 			if (a==0xFF) {
@@ -1121,10 +1155,10 @@ void evg_yuyv_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 		}
 		pY+=4;
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
-void evg_yuyv_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuyv_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	char *pY;
 	s32 i;
@@ -1145,7 +1179,7 @@ void evg_yuyv_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *sur
 
 		fin = mul255(a, spans[i].coverage);
 
-		memset(surf->uv_alpha + spans[i].x, (u8)fin, len);
+		memset(rctx->uv_alpha + spans[i].x, (u8)fin, len);
 		while (len--) {
 			overmask_yuvy(s_pY + surf->idx_y1, cy, fin);
 			s_pY += 2;
@@ -1153,8 +1187,8 @@ void evg_yuyv_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *sur
 	}
 	pY = surf->pixels + y * surf->pitch_y;
 	for (i=0; i<(s32) surf->width; i+=2) {
-		u32 p_a = surf->uv_alpha[i];
-		p_a += surf->uv_alpha[i + 1];
+		u32 p_a = rctx->uv_alpha[i];
+		p_a += rctx->uv_alpha[i + 1];
 		p_a /=2;
 		if (p_a==0xFF) {
 			pY[surf->idx_u] = cu;
@@ -1165,11 +1199,11 @@ void evg_yuyv_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *sur
 		}
 		pY+=4;
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 
 }
 
-void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	s32 i;
 	char *pY;
@@ -1185,9 +1219,8 @@ void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 		s_pY = pY + (spans[i].x/2) * 4;
 		if (spans[i].x%2) s_pY += 2;
 
+		p_col = surf->fill_run(surf->sten, rctx, &spans[i], y);
 		spanalpha = spans[i].coverage;
-		evg_fill_run(surf->sten, surf, spans[i].x, y, len);
-		p_col = surf->stencil_pix_run;
 
 		x = spans[i].x;
 
@@ -1205,13 +1238,13 @@ void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 					overmask_yuv420p(col_a, cy, s_pY + surf->idx_y1, spanalpha);
 
 					u8 a = mul255(col_a, spanalpha);
-					surf->uv_alpha[idx] = a;
+					rctx->uv_alpha[idx] = a;
 				} else {
 					s_pY[surf->idx_y1] = cy;
-					surf->uv_alpha[idx] = 0xFF;
+					rctx->uv_alpha[idx] = 0xFF;
 				}
-				surf->uv_alpha[idx+1] = cb;
-				surf->uv_alpha[idx+2] = cr;
+				rctx->uv_alpha[idx+1] = cb;
+				rctx->uv_alpha[idx+2] = cr;
 			}
 			s_pY+=2;
 			p_col++;
@@ -1224,14 +1257,14 @@ void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 		u32 idx1=3*i;
 		u32 idx2=3*i + 3;
 		//get alpha
-		a = (u32)surf->uv_alpha[idx1] + (u32)surf->uv_alpha[idx2];
+		a = (u32)rctx->uv_alpha[idx1] + (u32)rctx->uv_alpha[idx2];
 		if (a) {
 			u32 chroma;
 
 			a /= 2;
 
 			//get cb
-			chroma = (u32)surf->uv_alpha[idx1+1] + (u32)surf->uv_alpha[idx2+1];
+			chroma = (u32)rctx->uv_alpha[idx1+1] + (u32)rctx->uv_alpha[idx2+1];
 			chroma /= 2;
 			if (a==0xFF) {
 				pY[surf->idx_u] = chroma;
@@ -1239,7 +1272,7 @@ void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 				overmask_yuvy(pY + surf->idx_u, chroma, a);
 			}
 			//get cr
-			chroma = (u32)surf->uv_alpha[idx1+2] + (u32)surf->uv_alpha[idx2+2];
+			chroma = (u32)rctx->uv_alpha[idx1+2] + (u32)rctx->uv_alpha[idx2+2];
 			chroma /= 2;
 			if (a==0xFF) {
 				pY[surf->idx_v] = chroma;
@@ -1250,7 +1283,7 @@ void evg_yuyv_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
 		pY+=4;
 	}
 
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 
 }
 
@@ -1343,10 +1376,10 @@ static void overmask_yuv420p_10_const_run(u16 a, u16 val, u16 *ptr, u32 count, s
 	}
 }
 
-void evg_yuv420p_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_yuv420p_10_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
-	u16 *surf_uv_alpha_even = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha_even = (u16 *) rctx->uv_alpha;
 	u16 *surf_uv_alpha_odd = (u16 *) _surf_uv_alpha;
 	char *pU = surf->pixels + surf->height *surf->pitch_y;
 	char *pV;
@@ -1380,10 +1413,10 @@ void evg_yuv420p_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32
 			}
 		}
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
-void evg_yuv420p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u16 *pY;
 	u16 *surf_uv_alpha;
@@ -1393,12 +1426,12 @@ void evg_yuv420p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = (u16 *)surf->uv_alpha;
+		surf_uv_alpha = (u16 *)rctx->uv_alpha;
 	}
 	else if (write_uv) {
-		surf_uv_alpha = ((u16 *)surf->uv_alpha) + surf->width;
+		surf_uv_alpha = ((u16 *)rctx->uv_alpha) + surf->width;
 	} else {
-		surf_uv_alpha = (u16 *)surf->uv_alpha;
+		surf_uv_alpha = (u16 *)rctx->uv_alpha;
 	}
 
 	pY =  (u16 *) (surf->pixels + y * surf->pitch_y);
@@ -1431,13 +1464,13 @@ void evg_yuv420p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface
 			}
 		}
 	}
-	if (write_uv) {
-		surf->yuv_flush_uv(surf, (u8 *)surf_uv_alpha, cu, cv, y);
+	if (write_uv && !rctx->no_yuv_flush) {
+		surf->yuv_flush_uv(surf, rctx, (u8 *)surf_uv_alpha, cu, cv, y);
 	}
 
 }
 
-void evg_yuv420p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u32 a;
 	u16 *pY;
@@ -1448,11 +1481,11 @@ void evg_yuv420p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurfac
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = (u16 *)surf->uv_alpha;
+		surf_uv_alpha = (u16 *)rctx->uv_alpha;
 	} else if (write_uv) {
-		surf_uv_alpha = (u16 *)surf->uv_alpha + surf->width;
+		surf_uv_alpha = (u16 *)rctx->uv_alpha + surf->width;
 	} else {
-		surf_uv_alpha = (u16 *)surf->uv_alpha;
+		surf_uv_alpha = (u16 *)rctx->uv_alpha;
 	}
 
 	pY = (u16 *) (surf->pixels + y * surf->pitch_y);
@@ -1498,14 +1531,14 @@ void evg_yuv420p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurfac
 		}
 	}
 	//we are at an odd line, write uv
-	if (write_uv)
-		surf->yuv_flush_uv(surf, (u8*)surf_uv_alpha, cu, cv, y);
+	if (write_uv && !rctx->no_yuv_flush)
+		surf->yuv_flush_uv(surf, rctx, (u8*)surf_uv_alpha, cu, cv, y);
 }
 
-void evg_yuv420p_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
+void evg_yuv420p_10_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
 {
 	u32 i;
-	u16 *surf_uv_alpha_even = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha_even = (u16 *) rctx->uv_alpha;
 	u16 *surf_uv_alpha_odd = (u16 *) _surf_uv_alpha;
 	u16 *pU, *pV;
 	pU = (u16 *) (surf->pixels + surf->height *surf->pitch_y + y/2 * surf->pitch_y/2);
@@ -1573,11 +1606,11 @@ void evg_yuv420p_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 _c
 		pV++;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 
 }
 
-void evg_yuv420p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv420p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	s32 i;
 	u16 *pY;
@@ -1586,13 +1619,13 @@ void evg_yuv420p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *s
 
 	if (surf->is_422) {
 		write_uv = GF_TRUE;
-		surf_uv_alpha = (u16 *) surf->uv_alpha;
+		surf_uv_alpha = (u16 *) rctx->uv_alpha;
 	} else if (write_uv) {
 		//second line of storage (we store alpha, cr, cb for each pixel)
-		surf_uv_alpha =  ((u16 *) surf->uv_alpha) + 3*surf->width;
+		surf_uv_alpha =  ((u16 *) rctx->uv_alpha) + 3*surf->width;
 	} else {
 		//first line of storage
-		surf_uv_alpha =  (u16 *) surf->uv_alpha;
+		surf_uv_alpha =  (u16 *) rctx->uv_alpha;
 	}
 
 	pY = (u16 *) (surf->pixels + y * surf->pitch_y);
@@ -1604,9 +1637,8 @@ void evg_yuv420p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *s
 		u16 *s_pY;
 		short x;
 		len = spans[i].len;
+		p_col = surf->fill_run(surf->sten, rctx, &spans[i], y);
 		spanalpha = spans[i].coverage;
-		evg_fill_run(surf->sten, surf, spans[i].x, y, len);
-		p_col = surf->stencil_pix_run;
 
 		s_pY = pY + spans[i].x;
 		x = spans[i].x;
@@ -1640,8 +1672,8 @@ void evg_yuv420p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *s
 		}
 	}
 	//compute final u,v for both lines
-	if (write_uv) {
-		surf->yuv_flush_uv(surf, (u8 *)surf_uv_alpha, 0, 0, y);
+	if (write_uv && !rctx->no_yuv_flush) {
+		surf->yuv_flush_uv(surf, rctx, (u8 *)surf_uv_alpha, 0, 0, y);
 	}
 }
 
@@ -1701,10 +1733,10 @@ GF_Err evg_surface_clear_yuv420p_10(GF_EVGSurface *_surf, GF_IRect rc, GF_Color
 */
 
 
-void evg_nv12_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_nv12_10_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
-	u16 *surf_uv_alpha_even = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha_even = (u16 *) rctx->uv_alpha;
 	u16 *surf_uv_alpha_odd = (u16 *)_surf_uv_alpha;
 	u8 *pUV = surf->pixels + surf->height * surf->pitch_y + y/2 * surf->pitch_y;
 	u8 *pU = pUV + 2*surf->idx_u;
@@ -1733,14 +1765,14 @@ void evg_nv12_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu,
 		pU += 4;
 		pV += 4;
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
 
-void evg_nv12_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_nv12_10_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i;
-	u16 *surf_uv_alpha_even = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha_even = (u16 *) rctx->uv_alpha;
 	u16 *surf_uv_alpha_odd = (u16 *) _surf_uv_alpha;
 	u8 *pUV = surf->pixels + surf->height *surf->pitch_y + y/2 * surf->pitch_y;
 	u8 *pU = pUV + 2*surf->idx_u;
@@ -1806,7 +1838,7 @@ void evg_nv12_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu, s
 		pV += 4;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
 
@@ -1879,12 +1911,12 @@ GF_Err evg_surface_clear_nv12_10(GF_EVGSurface *_surf, GF_IRect rc, GF_Color col
 			YUV422 10 part
 */
 
-void evg_yuv422p_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
+void evg_yuv422p_10_flush_uv_const(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 cu, s32 cv, s32 y)
 {
 	u32 i, a;
 	char *pU = surf->pixels + surf->height *surf->pitch_y;
 	char *pV;
-	u16 *surf_uv_alpha = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha = (u16 *) rctx->uv_alpha;
 	pU +=  y * surf->pitch_y/2;
 	pV = pU + surf->height * surf->pitch_y/2;
 
@@ -1912,13 +1944,13 @@ void evg_yuv422p_10_flush_uv_const(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32
 			}
 		}
 	}
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
-void evg_yuv422p_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
+void evg_yuv422p_10_flush_uv_var(GF_EVGSurface *surf, EVGRasterCtx *rctx, u8 *_surf_uv_alpha, s32 _cu, s32 _cv,  s32 y)
 {
 	u32 i;
-	u16 *surf_uv_alpha = (u16 *) surf->uv_alpha;
+	u16 *surf_uv_alpha = (u16 *) rctx->uv_alpha;
 	u16 *pU = (u16 *) (surf->pixels + surf->height *surf->pitch_y + y * surf->pitch_y/2);
 	u16 *pV = (u16 *) (surf->pixels + surf->height *surf->pitch_y + surf->height * surf->pitch_y/2 + y * surf->pitch_y/2);
 
@@ -1966,7 +1998,7 @@ void evg_yuv422p_10_flush_uv_var(GF_EVGSurface *surf, u8 *_surf_uv_alpha, s32 _c
 		pV++;
 	}
 	//reset for next pass
-	memset(surf->uv_alpha, 0, surf->uv_alpha_alloc);
+	memset(rctx->uv_alpha, 0, surf->uv_alpha_alloc);
 }
 
 
@@ -2024,7 +2056,7 @@ GF_Err evg_surface_clear_yuv422p_10(GF_EVGSurface *_surf, GF_IRect rc, GF_Color
 			YUV444 10bits part
 */
 
-void evg_yuv444p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u16 *pY, *pU, *pV;
 	s32 i;
@@ -2069,7 +2101,7 @@ void evg_yuv444p_10_fill_const(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface
 	}
 }
 
-void evg_yuv444p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	u32 a;
 	u16 *pY, *pU, *pV;
@@ -2126,7 +2158,7 @@ void evg_yuv444p_10_fill_const_a(s32 y, s32 count, EVG_Span *spans, GF_EVGSurfac
 	}
 }
 
-void evg_yuv444p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf)
+void evg_yuv444p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *surf, EVGRasterCtx *rctx)
 {
 	s32 i;
 	u16 *pY, *pU, *pV;
@@ -2141,9 +2173,8 @@ void evg_yuv444p_10_fill_var(s32 y, s32 count, EVG_Span *spans, GF_EVGSurface *s
 		u64 *p_col;
 		u16 *s_pY, *s_pU, *s_pV;
 		len = spans[i].len;
+		p_col = surf->fill_run(surf->sten, rctx, &spans[i], y);
 		spanalpha = spans[i].coverage;
-		evg_fill_run(surf->sten, surf, spans[i].x, y, len);
-		p_col = surf->stencil_pix_run;
 
 		s_pY = pY + spans[i].x;
 		s_pU = pU + spans[i].x;
