@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2021
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / software 2D rasterizer module
@@ -1722,52 +1722,62 @@ GF_EVGStencil *evg_texture_brush()
 
 u32 get_pix_argb(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(*(pix) & 0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF, *(pix+3) & 0xFF);
 }
 u32 get_pix_rgba(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(*(pix+3) & 0xFF, *(pix) & 0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF);
 }
+u32 get_pix_grba(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	return GF_COL_ARGB(*(pix+3) & 0xFF, *(pix+1) & 0xFF, *(pix) & 0xFF, *(pix+2) & 0xFF);
+}
 u32 get_pix_abgr(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(*(pix) & 0xFF, *(pix+3) & 0xFF, *(pix+2) & 0xFF, *(pix+1) & 0xFF);
 }
 u32 get_pix_bgra(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(*(pix+3) & 0xFF, *(pix+2) & 0xFF, *(pix+1) & 0xFF, *(pix) & 0xFF);
 }
 u32 get_pix_rgbx(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *(pix) & 0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF);
 }
 u32 get_pix_xrgb(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF, *(pix+3) & 0xFF);
 }
 u32 get_pix_xbgr(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *(pix+3) & 0xFF, *(pix+2) & 0xFF, *(pix+1) & 0xFF);
 }
 u32 get_pix_bgrx(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *(pix+2) & 0xFF, *(pix+1) & 0xFF, *(pix) & 0xFF);
 }
 u32 get_pix_rgb_24(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *pix & 0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF);
 }
+u32 get_pix_gbr_24(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	return GF_COL_ARGB(0xFF, *(pix+1) & 0xFF, *(pix+2) & 0xFF, *pix & 0xFF);
+}
 u32 get_pix_bgr_24(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	return GF_COL_ARGB(0xFF, *(pix+2) & 0xFF, * (pix+1) & 0xFF, *pix & 0xFF);
 }
 u32 get_pix_444(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
@@ -1790,7 +1800,7 @@ u32 get_pix_555(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 }
 u32 get_pix_565(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	u32 r = (pix[0]>>3) & 0x1f;
 	u32 g = (pix[0])&0x7;
 	g<<=3;
@@ -1798,16 +1808,32 @@ u32 get_pix_565(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 	u32 b = pix[1] & 0x1f;
 	return GF_COL_ARGB(0xFF, (r << 3), (g << 2), (b << 3));
 }
+u32 get_pix_palette_alpha(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u32 val = *pix;
+	if (val >= _this->palette_colors) return GF_COL_ARGB(0xFF, 0xFF, 0xFF, 0xFF);
+	pix = (u8*)_this->palette + val*4;
+	return GF_COL_ARGB(pix[_this->pidx_a], pix[_this->pidx_r], pix[_this->pidx_g], pix[_this->pidx_b]);
+}
+u32 get_pix_palette(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u32 val = *pix;
+	if (val >= _this->palette_colors) return GF_COL_ARGB(0xFF, 0xFF, 0xFF, 0xFF);
+	pix = (u8*)_this->palette + val*3;
+	return GF_COL_ARGB(0xFF, pix[_this->pidx_r], pix[_this->pidx_g], pix[_this->pidx_b]);
+}
 u32 get_pix_grey(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	u8 val = *pix;
 	return GF_COL_ARGB(0xFF, val, val, val);
 }
 u32 get_pix_alphagrey(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
 	u8 a, g;
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	a = *pix;
 	g = *(pix+1);
 	return GF_COL_ARGB(a, g, g, g);
@@ -1815,7 +1841,7 @@ u32 get_pix_alphagrey(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 u32 get_pix_greyalpha(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 {
 	u8 a, g;
-	char *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
+	u8 *pix = _this->pixels + y * _this->stride + _this->Bpp*x;
 	g = *pix;
 	a = *(pix+1);
 	return GF_COL_ARGB(a, g, g, g);
@@ -1833,6 +1859,10 @@ u32 get_pix_yuv420p(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 #define GET_LE_10BIT_AS_8(_ptr) ( (((u16)(_ptr)[1])<<8 | (u16)(_ptr)[0] ) >> 2 )
 #define GET_LE_10BIT_AS_16(_ptr) ( (((u16)(_ptr)[1])<<8 | (u16)(_ptr)[0] ) << 6 )
 
+//#define GET_LE_10BIT_LEFT_AS_8(_ptr) ( (((u16)(_ptr)[1])<<8 | (u16)(_ptr)[0] ) >> 8 )
+#define GET_LE_10BIT_LEFT_AS_8(_ptr) (_ptr[1] )
+#define GET_LE_10BIT_LEFT_AS_16(_ptr) ( (((u16)(_ptr)[1])<<8 | (u16)(_ptr)[0] ) & 0xFFC0 )
+
 #define GET_BE_10BIT_AS_8(_ptr) ( (*(u16 *)(_ptr)) >> 2 )
 #define GET_BE_10BIT_AS_16(_ptr) ( (*(u16 *)(_ptr)) << 6 )
 
@@ -1841,6 +1871,10 @@ u32 get_pix_yuv420p(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 #define GET_LE_10BIT_AS_8(_ptr) ( (*(u16 *)(_ptr)) >> 2 )
 #define GET_LE_10BIT_AS_16(_ptr) ( (*(u16 *)(_ptr)) << 6 )
 
+#define GET_LE_10BIT_LEFT_AS_8(_ptr) ( (*(u16 *)(_ptr)) >> 8 )
+#define GET_LE_10BIT_LEFT_AS_16(_ptr) ( (*(u16 *)(_ptr)) & 0xFFC0 )
+
+
 #define GET_BE_10BIT_AS_8(_ptr) ( (((u16)(_ptr)[0])<<8 | (u16)(_ptr)[1] ) >> 2 )
 #define GET_BE_10BIT_AS_16(_ptr) ( (((u16)(_ptr)[0])<<8 | (u16)(_ptr)[1] ) << 6 )
 
@@ -2046,8 +2080,165 @@ u32 get_pix_vyuy(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
 	u8 luma = (x%2) ? pY[3] : pY[1];
 	return GF_COL_ARGB(0xFF, luma, u, v);
 }
+u32 get_pix_yuv444_10(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 vy, vu, vv;
+	u32 val;
+	u8 *p_src = _this->pixels + y * _this->stride + x*4;
+	val = p_src[3]; val<<=8;
+	val |= p_src[2]; val<<=8;
+	val |= p_src[1]; val<<=8;
+	val |= p_src[0];
 
+	vu = (u8) ( ((val>>2) & 0x3FF) >> 2);
+	vy = (u8) ( ((val>>12) & 0x3FF) >> 2);
+	vv = (u8) ( ((val>>22)) >> 2);
 
+	return GF_COL_ARGB(0xFF, vy, vu, vv);
+}
+u64 get_pix_yuv444_10_wide(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u16 vy, vu, vv;
+	u32 val;
+	u8 *p_src = _this->pixels + y * _this->stride + x*4;
+	val = p_src[3]; val<<=8;
+	val |= p_src[2]; val<<=8;
+	val |= p_src[1]; val<<=8;
+	val |= p_src[0];
+
+	vu = (u16) ( ((val>>2) & 0x3FF) << 6);
+	vy = (u16) ( ((val>>12) & 0x3FF) << 6);
+	vv = (u16) ( ((val>>22)) << 6);
+	return GF_COLW_ARGB(0xFFFF, vy, vu, vv);
+}
+
+u32 get_pix_v210(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 vy, vu, vv;
+	u32 val1, val2;
+	u32 x_offset = x%6;
+	x-=x_offset;
+	u8 *p_src = _this->pixels + y * _this->stride + (x * 16) / 6;
+	if (x_offset>=4) p_src += 8;
+	else if (x_offset>=2) p_src += 4;
+
+	val1 = p_src[3]; val1<<=8;
+	val1 |= p_src[2]; val1<<=8;
+	val1 |= p_src[1]; val1<<=8;
+	val1 |= p_src[0];
+
+	if (x_offset==0) {
+		vu = (u8) ( ((val1) & 0x3FF) >> 2);
+		vy = (u8) ( ((val1>>10) & 0x3FF) >> 2);
+		vv = (u8) ( ((val1>>20) & 0x3FF) >> 2);
+	} else {
+		val2 = p_src[7]; val2<<=8;
+		val2 |= p_src[6]; val2<<=8;
+		val2 |= p_src[5]; val2<<=8;
+		val2 |= p_src[4];
+
+		if (x_offset==1) {
+			vu = (u8) ( ((val1) & 0x3FF) >> 2);
+			vv = (u8) ( ((val1>>20) & 0x3FF) >> 2);
+			vy = (u8) ( ((val2) & 0x3FF) >> 2);
+		} else if (x_offset==2) {
+			vu = (u8) ( ((val1>>10) & 0x3FF) >> 2);
+			vy = (u8) ( ((val1>>20) & 0x3FF) >> 2);
+			vv = (u8) ( ((val2) & 0x3FF) >> 2);
+		} else if (x_offset==3) {
+			vu = (u8) ( ((val1>>10) & 0x3FF) >> 2);
+			vv = (u8) ( ((val2) & 0x3FF) >> 2);
+			vy = (u8) ( ((val2>>10) & 0x3FF) >> 2);
+		} else if (x_offset==4) {
+			vu = (u8) ( ((val1>>20) & 0x3FF) >> 2);
+			vy = (u8) ( ((val2) & 0x3FF) >> 2);
+			vv = (u8) ( ((val2>>10) & 0x3FF) >> 2);
+		} else {
+			vu = (u8) ( ((val1>>20) & 0x3FF) >> 2);
+			vv = (u8) ( ((val2>>10) & 0x3FF) >> 2);
+			vy = (u8) ( ((val2>>20) & 0x3FF) >> 2);
+		}
+	}
+	return GF_COL_ARGB(0xFF, vy, vu, vv);
+}
+u64 get_pix_v210_wide(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u16 vy, vu, vv;
+	u32 val1, val2;
+	u32 x_offset = x%6;
+	x-=x_offset;
+	u8 *p_src = _this->pixels + y * _this->stride + (x * 16) / 6;
+	if (x_offset>=4) p_src += 8;
+	else if (x_offset>=2) p_src += 4;
+
+	val1 = p_src[3]; val1<<=8;
+	val1 |= p_src[2]; val1<<=8;
+	val1 |= p_src[1]; val1<<=8;
+	val1 |= p_src[0];
+
+	if (x_offset==0) {
+		vu = (u16) ( ((val1) & 0x3FF) << 6);
+		vy = (u16) ( ((val1>>10) & 0x3FF) << 6);
+		vv = (u16) ( ((val1>>20) & 0x3FF) << 6);
+	} else {
+		val2 = p_src[7]; val2<<=8;
+		val2 |= p_src[6]; val2<<=8;
+		val2 |= p_src[5]; val2<<=8;
+		val2 |= p_src[4];
+
+		if (x_offset==1) {
+			vu = (u16) ( ((val1) & 0x3FF) << 6);
+			vv = (u16) ( ((val1>>20) & 0x3FF) << 6);
+			vy = (u16) ( ((val2) & 0x3FF) << 6);
+		} else if (x_offset==2) {
+			vu = (u16) ( ((val1>>10) & 0x3FF) << 6);
+			vy = (u16) ( ((val1>>20) & 0x3FF) << 6);
+			vv = (u16) ( ((val2) & 0x3FF) << 6);
+		} else if (x_offset==3) {
+			vu = (u16) ( ((val1>>10) & 0x3FF) << 6);
+			vv = (u16) ( ((val2) & 0x3FF) << 6);
+			vy = (u16) ( ((val2>>10) & 0x3FF) << 6);
+		} else if (x_offset==4) {
+			vu = (u16) ( ((val1>>20) & 0x3FF) << 6);
+			vy = (u16) ( ((val2) & 0x3FF) << 6);
+			vv = (u16) ( ((val2>>10) & 0x3FF) << 6);
+		} else {
+			vu = (u16) ( ((val1>>20) & 0x3FF) << 6);
+			vv = (u16) ( ((val2>>10) & 0x3FF) << 6);
+			vy = (u16) ( ((val2>>20) & 0x3FF) << 6);
+		}
+	}
+	return GF_COLW_ARGB(0xFFFF, vy, vu, vv);
+}
+
+u32 get_pix_yuyv_10(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u8 vy, vu, vv;
+	Bool odd = x%2;
+	u8 *p_src = _this->pixels + y * _this->stride + (x/2)*8;
+
+	vu = GET_LE_10BIT_LEFT_AS_8(p_src + _this->off_u);
+	if (odd)
+		vy = GET_LE_10BIT_LEFT_AS_8(p_src + _this->off_y+4);
+	else
+		vy = GET_LE_10BIT_LEFT_AS_8(p_src + _this->off_y);
+	vv = GET_LE_10BIT_LEFT_AS_8(p_src + _this->off_v);
+	return GF_COL_ARGB(0xFF, vy, vu, vv);
+}
+u64 get_pix_yuyv_10_wide(EVG_Texture *_this, u32 x, u32 y, EVGRasterCtx *ctx)
+{
+	u16 vy, vu, vv;
+	u32 odd = x%2;
+	u8 *p_src = _this->pixels + y * _this->stride + (x/2)*8;
+
+	vu = GET_LE_10BIT_LEFT_AS_16(p_src + _this->off_u);
+	if (odd)
+		vy = GET_LE_10BIT_LEFT_AS_16(p_src + _this->off_y+4);
+	else
+		vy = GET_LE_10BIT_LEFT_AS_16(p_src + _this->off_y);
+	vv = GET_LE_10BIT_LEFT_AS_16(p_src + _this->off_v);
+	return GF_COLW_ARGB(0xFFFF, vy, vu, vv);
+}
 u64 default_get_pixel_wide(struct __evg_texture *_this, u32 x, u32 y, EVGRasterCtx *rctx)
 {
 	return evg_col_to_wide( _this->tx_get_pixel(_this, x, y, rctx) );
@@ -2109,7 +2300,13 @@ static void texture_set_callbacks(EVG_Texture *_this)
 		_this->tx_get_pixel = get_pix_565;
 		return;
 	case GF_PIXEL_GREYSCALE:
-		_this->tx_get_pixel = get_pix_grey;
+		if (_this->palette && _this->palette_colors) {
+			if (_this->palette_comp==4)
+				_this->tx_get_pixel = get_pix_palette_alpha;
+			else
+				_this->tx_get_pixel = get_pix_palette;
+		} else
+			_this->tx_get_pixel = get_pix_grey;
 		return;
 	case GF_PIXEL_ALPHAGREY:
 		_this->tx_get_pixel = get_pix_alphagrey;
@@ -2185,9 +2382,58 @@ static void texture_set_callbacks(EVG_Texture *_this)
 		_this->tx_get_pixel = get_pix_rgba;
 		_this->is_transparent = 1;
 		break;
+	case GF_PIXEL_UYVA444_PACK:
+		_this->tx_get_pixel = get_pix_grba;
+		_this->is_transparent = 1;
+		break;
 	case GF_PIXEL_YUV444_PACK:
 		_this->tx_get_pixel = get_pix_rgb_24;
 		break;
+	case GF_PIXEL_VYU444_PACK:
+		_this->tx_get_pixel = get_pix_gbr_24;
+		break;
+	case GF_PIXEL_YUV444_10_PACK:
+		_this->tx_get_pixel = get_pix_yuv444_10;
+		_this->tx_get_pixel_wide = get_pix_yuv444_10_wide;
+		_this->is_wide = 1;
+		break;
+	case GF_PIXEL_V210:
+		_this->tx_get_pixel = get_pix_v210;
+		_this->tx_get_pixel_wide = get_pix_v210_wide;
+		_this->is_wide = 1;
+		break;
+	case GF_PIXEL_YUYV_10:
+		_this->off_y=0;
+		_this->off_u=2;
+		_this->off_v=4;
+		_this->tx_get_pixel = get_pix_yuyv_10;
+		_this->tx_get_pixel_wide = get_pix_yuyv_10_wide;
+		_this->is_wide = 1;
+		break;
+	case GF_PIXEL_YVYU_10:
+		_this->off_y=0;
+		_this->off_u=4;
+		_this->off_v=2;
+		_this->tx_get_pixel = get_pix_yuyv_10;
+		_this->tx_get_pixel_wide = get_pix_yuyv_10_wide;
+		_this->is_wide = 1;
+		break;
+	case GF_PIXEL_UYVY_10:
+		_this->off_y=2;
+		_this->off_u=0;
+		_this->off_v=4;
+		_this->tx_get_pixel = get_pix_yuyv_10;
+		_this->tx_get_pixel_wide = get_pix_yuyv_10_wide;
+		_this->is_wide = 1;
+		break;
+	case GF_PIXEL_VYUY_10:
+		_this->off_y=2;
+		_this->off_u=4;
+		_this->off_v=2;
+		_this->tx_get_pixel = get_pix_yuyv_10;
+		_this->tx_get_pixel_wide = get_pix_yuyv_10_wide;
+		_this->is_wide = 1;
+		break;
 	default:
 		return;
 	}
@@ -2250,6 +2496,8 @@ static GF_Err gf_evg_stencil_set_texture_internal(GF_EVGStencil * st, u32 width,
 
 	_this->pixels = NULL;
 	_this->is_yuv = GF_FALSE;
+	_this->palette = NULL;
+	_this->palette_colors = 0;
 
 	switch (pixelFormat) {
 	case GF_PIXEL_ARGB:
@@ -2304,14 +2552,39 @@ static GF_Err gf_evg_stencil_set_texture_internal(GF_EVGStencil * st, u32 width,
 		if (!stride)
 			stride = 4 * width;
 		break;
+
+	case GF_PIXEL_YUYV_10:
+	case GF_PIXEL_YVYU_10:
+	case GF_PIXEL_UYVY_10:
+	case GF_PIXEL_VYUY_10:
+		_this->is_yuv = GF_TRUE;
+		_this->Bpp = 2;
+		if (!stride)
+			stride = 4 * width;
+		break;
 	case GF_PIXEL_YUVA444_PACK:
+	case GF_PIXEL_UYVA444_PACK:
+		_this->Bpp = 4;
+		_this->is_yuv = GF_TRUE;
+		break;
+	case GF_PIXEL_YUV444_10_PACK:
 		_this->Bpp = 4;
 		_this->is_yuv = GF_TRUE;
 		break;
 	case GF_PIXEL_YUV444_PACK:
+	case GF_PIXEL_VYU444_PACK:
 		_this->Bpp = 3;
 		_this->is_yuv = GF_TRUE;
 		break;
+	case GF_PIXEL_V210:
+		_this->Bpp = 2;
+		_this->is_yuv = GF_TRUE;
+		if (!stride) {
+			stride = width;
+			while (stride % 48) stride++;
+			stride = stride * 16 / 6; //4 x 32 bits to represent 6 pixels
+		}
+		break;
 	default:
 		return GF_NOT_SUPPORTED;
 	}
@@ -2342,6 +2615,64 @@ GF_Err gf_evg_stencil_set_texture(GF_EVGStencil *stencil, u8 *pixels, u32 width,
 	return gf_evg_stencil_set_texture_internal(stencil, width, height, pixelFormat, pixels, stride, NULL, NULL, 0, NULL, 0);
 }
 
+GF_EXPORT
+GF_Err gf_evg_stencil_set_palette(GF_EVGStencil *stencil, const u8 *palette, u32 pix_fmt, u32 nb_cols)
+{
+	EVG_Texture *_this = (EVG_Texture *) stencil;
+	if (!_this || (_this->type != GF_STENCIL_TEXTURE) || !palette || !nb_cols)
+		return GF_BAD_PARAM;
+	if (_this->pixel_format != GF_PIXEL_GREYSCALE) return GF_BAD_PARAM;
+	_this->palette = palette;
+	_this->palette_colors = nb_cols;
+	_this->palette_pfmt = pix_fmt;
+	switch (pix_fmt) {
+	case GF_PIXEL_RGB:
+		_this->palette_comp = 3;
+		_this->pidx_r=0;
+		_this->pidx_g=1;
+		_this->pidx_b=2;
+		break;
+	case GF_PIXEL_BGR:
+		_this->palette_comp = 3;
+		_this->pidx_r=2;
+		_this->pidx_g=1;
+		_this->pidx_b=0;
+		break;
+	case GF_PIXEL_RGBA:
+		_this->palette_comp = 4;
+		_this->pidx_r=0;
+		_this->pidx_g=1;
+		_this->pidx_b=2;
+		_this->pidx_a=3;
+		break;
+	case GF_PIXEL_ARGB:
+		_this->palette_comp = 4;
+		_this->pidx_r=1;
+		_this->pidx_g=2;
+		_this->pidx_b=3;
+		_this->pidx_a=0;
+		break;
+	case GF_PIXEL_ABGR:
+		_this->palette_comp = 4;
+		_this->pidx_r=3;
+		_this->pidx_g=2;
+		_this->pidx_b=1;
+		_this->pidx_a=0;
+		break;
+	case GF_PIXEL_BGRA:
+		_this->palette_comp = 4;
+		_this->pidx_r=2;
+		_this->pidx_g=1;
+		_this->pidx_b=0;
+		_this->pidx_a=3;
+		break;
+	default:
+		_this->palette = NULL;
+		_this->palette_colors = 0;
+		return GF_NOT_SUPPORTED;
+	}
+	return GF_OK;
+}
 GF_EXPORT
 GF_Err gf_evg_stencil_get_texture_planes(GF_EVGStencil *stencil, u8 **pY_or_RGB, u8 **pU, u8 **pV, u8 **pA, u32 *stride, u32 *stride_uv)
 {
@@ -2536,6 +2867,15 @@ u32 gf_evg_stencil_get_pixel_yuv(GF_EVGStencil *st, s32 x, s32 y)
 	return gf_evg_stencil_get_pixel_intern((EVG_Texture *)st, x, y, GF_TRUE);
 }
 
+u32 gf_evg_stencil_get_pixel_fast(GF_EVGStencil *st, s32 x, s32 y)
+{
+	return ((EVG_Texture *)st)->tx_get_pixel((EVG_Texture *)st, x, y, NULL);
+}
+u64 gf_evg_stencil_get_pixel_wide_fast(GF_EVGStencil *st, s32 x, s32 y)
+{
+	return ((EVG_Texture *)st)->tx_get_pixel_wide((EVG_Texture *)st, x, y, NULL);
+}
+
 
 
 static u64 gf_evg_stencil_get_pixel_wide_intern(EVG_Texture *_this, s32 x, s32 y, Bool want_yuv)
