@@ -249,7 +249,7 @@ static GF_FilterPacket *gf_filter_pck_clone_frame_interface(GF_FilterPid *pid, G
 	p = gf_filter_pid_get_property(ref->pid, GF_PROP_PID_PIXFMT);
 	pf = p ? p->value.uint : 0;
 	//not supported
-	if (!w || !h || !pf) {
+	if (!w || !h || !pf || !ref->frame_ifce->get_plane) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_FILTER, ("Missing width/height/pf in frame interface cloning, not supported\n"));
 		return NULL;
 	}
@@ -412,7 +412,7 @@ GF_FilterPacket *gf_filter_pck_new_alloc_destructor(GF_FilterPid *pid, u32 data_
 	GF_FilterPacket *pck = gf_filter_pck_new_alloc_internal(pid, data_size, data);
 	if (pck && destruct) {
 		pck->destructor = destruct;
-		if (pid->filter->freg->flags & GF_FS_REG_MAIN_THREAD)
+		if (pid->filter->nb_main_thread_forced)
 			pck->info.flags |= GF_PCKF_FORCE_MAIN;
 	}
 	return pck;
@@ -446,7 +446,7 @@ GF_FilterPacket *gf_filter_pck_new_shared_internal(GF_FilterPid *pid, const u8 *
 	}
 	gf_filter_pck_reset_props(pck, pid);
 
-	if (destruct && (pid->filter->freg->flags & GF_FS_REG_MAIN_THREAD))
+	if (destruct && pid->filter->nb_main_thread_forced)
 		pck->info.flags |= GF_PCKF_FORCE_MAIN;
 
 	assert(pck->pid);
@@ -520,6 +520,7 @@ GF_FilterPacket *gf_filter_pck_new_frame_interface(GF_FilterPid *pid, GF_FilterF
 	pck->destructor = destruct;
 	pck->frame_ifce = frame_ifce;
 	pck->filter_owns_mem = 2;
+	//GL frame interface must be processed by main thread
 	if (frame_ifce && frame_ifce->get_gl_texture)
 		pck->info.flags |= GF_PCKF_FORCE_MAIN;
 	return pck;
@@ -861,16 +862,6 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		return GF_BAD_PARAM;
 	}
 
-	if (pid->discard_input_packets) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s reset pending, discarding input packet\n", pid->filter->name, pid->name));
-		safe_int_inc(&pck->reference_count);
-		assert(pck->reference_count);
-		if (safe_int_dec(&pck->reference_count) == 0) {
-			gf_filter_packet_destroy(pck);
-		}
-		return GF_OK;
-	}
-
 	is_cmd_pck = (pck->info.flags & GF_PCK_CMD_MASK);
 
 	//special case for source filters (no input pids), mark as playing once we have a packet sent
@@ -947,7 +938,11 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 		}
 	}
 
-	if (pid->filter->out_pid_connection_pending || pid->filter->has_pending_pids || pid->init_task_pending) {
+	if (pid->filter->out_pid_connection_pending
+		|| pid->filter->has_pending_pids
+		|| pid->init_task_pending
+		|| (from_filter && pid->filter->postponed_packets)
+	) {
 		GF_LOG(GF_LOG_INFO, GF_LOG_FILTER, ("Filter %s PID %s connection pending, queuing packet\n", pck->pid->filter->name, pck->pid->name));
 		if (!pid->filter->postponed_packets) pid->filter->postponed_packets = gf_list_new();
 		gf_list_add(pid->filter->postponed_packets, pck);
@@ -1066,7 +1061,8 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			else if ((u32) duration < pid->min_pck_duration) pid->min_pck_duration = (u32) duration;
 		}
 
-		if (!pck->info.duration && pid->min_pck_duration)
+		//set duration if pid is not sparse. If sparse (text & co) don't set it because we will likely get it wrong
+		if (!pid->is_sparse && !pck->info.duration && pid->min_pck_duration)
 			pck->info.duration = (u32) duration;
 
 		//may happen if we don't have DTS, only CTS signaled and B-frames
@@ -1083,15 +1079,15 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			u8 bend = (pck->info.flags & GF_PCKF_BLOCK_END) ? 1 : 0;
 
 			if ((pck->info.dts != GF_FILTER_NO_TS) && (pck->info.cts != GF_FILTER_NO_TS) ) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet DTS "LLU" CTS "LLU" SAP %d seek %d duration %d S/E %d/%d\n", pck->pid->filter->name, pck->pid->name, pck->info.dts, pck->info.cts, sap_type, seek, pck->info.duration, bstart, bend));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet DTS "LLU" CTS "LLU" SAP %d seek %d duration %d S/E %d/%d size %u\n", pck->pid->filter->name, pck->pid->name, pck->info.dts, pck->info.cts, sap_type, seek, pck->info.duration, bstart, bend, pck->data_length));
 			}
 			else if ((pck->info.cts != GF_FILTER_NO_TS) ) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet CTS "LLU" SAP %d seek %d duration %d S/E %d/%d\n", pck->pid->filter->name, pck->pid->name, pck->info.cts, sap_type, seek, pck->info.duration, bstart, bend));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet CTS "LLU" SAP %d seek %d duration %d S/E %d/%d size %u\n", pck->pid->filter->name, pck->pid->name, pck->info.cts, sap_type, seek, pck->info.duration, bstart, bend, pck->data_length));
 			}
 			else if ((pck->info.dts != GF_FILTER_NO_TS) ) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet DTS "LLU" SAP %d seek %d duration %d S/E %d/%d\n", pck->pid->filter->name, pck->pid->name, pck->info.dts, sap_type, seek, pck->info.duration, bstart, bend));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet DTS "LLU" SAP %d seek %d duration %d S/E %d/%d size %u\n", pck->pid->filter->name, pck->pid->name, pck->info.dts, sap_type, seek, pck->info.duration, bstart, bend, pck->data_length));
 			} else {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet no DTS/PTS SAP %d seek %d duration %d S/E %d/%d\n", pck->pid->filter->name, pck->pid->name, sap_type, seek, pck->info.duration, bstart, bend));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s PID %s sent packet no DTS/PTS SAP %d seek %d duration %d S/E %d/%d size %u\n", pck->pid->filter->name, pck->pid->name, sap_type, seek, pck->info.duration, bstart, bend, pck->data_length));
 			}
 		}
 #endif
@@ -1110,7 +1106,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 	assert(pck->pid);
 	count = pck->pid->num_destinations;
-	//check if processing this packet must be done on main thread (OpenGL interface or source filter asked fo this) 
+	//check if processing this packet must be done on main thread (OpenGL interface or source filter asked for this)
 	Bool force_main_thread = (pck->info.flags & GF_PCKF_FORCE_MAIN) ? GF_TRUE : GF_FALSE;
 
 	for (i=0; i<count; i++) {
@@ -1179,6 +1175,8 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Dispatching packet from filter %s to filter %s - %d packet in PID %s buffer ("LLU" us buffer)\n", pid->filter->name, dst->filter->name, gf_fq_count(dst->packets), pid->name, dst->buffer_duration ));
 
+		u64 us_duration = 0;
+
 		if (cktype) {
 			safe_int_inc(&dst->filter->pending_packets);
 			gf_fq_add(dst->packets, inst);
@@ -1187,7 +1185,7 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			if (pck->info.flags & GF_PCKF_BLOCK_START) {
 				//missed end of previous, aggregate all before excluding this packet
 				if (!dst->last_block_ended) {
-					GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s: Missed end of block signaling but got start of block - queuing for potential reaggregation\n", pid->filter->name));
+					GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("Filter %s: Missed end of block signaling but got start of block - performing reaggregation\n", pid->filter->name));
 
 					//post process task if we have been reaggregating a packet
 					post_task = gf_filter_aggregate_packets(dst);
@@ -1212,8 +1210,8 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 						dst->nb_reagg_pck++;
 
 					if (pck->info.duration && timescale) {
-						duration = gf_timestamp_rescale(pck->info.duration, timescale, 1000000);
-						safe_int64_add(&dst->buffer_duration, duration);
+						us_duration = gf_timestamp_rescale(pck->info.duration, timescale, 1000000);
+						safe_int64_add(&dst->buffer_duration, us_duration);
 					}
 					inst->pck->info.flags |= GF_PCKF_BLOCK_START;
 					safe_int_inc(&dst->filter->pending_packets);
@@ -1278,23 +1276,23 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			}
 
 		} else {
-			duration=0;
+			u32 pck_dur=0;
 			//store start of block info
 			if (pck->info.flags & GF_PCKF_BLOCK_START) {
 				dst->first_block_started = GF_TRUE;
-				duration = pck->info.duration;
+				pck_dur = pck->info.duration;
 			}
 			if (pck->info.flags & GF_PCKF_BLOCK_END) {
 				//we didn't get a start for this end of block use the packet duration
 				if (!dst->first_block_started) {
-					duration=pck->info.duration;
+					pck_dur=pck->info.duration;
 				}
 				dst->first_block_started = GF_FALSE;
 			}
 
-			if (duration && timescale) {
-				duration = gf_timestamp_rescale(duration, timescale, 1000000);
-				safe_int64_add(&dst->buffer_duration, duration);
+			if (pck_dur && timescale) {
+				us_duration = gf_timestamp_rescale(pck_dur, timescale, 1000000);
+				safe_int64_add(&dst->buffer_duration, us_duration);
 			}
 			safe_int_inc(&dst->filter->pending_packets);
 //
@@ -1311,6 +1309,12 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 			//will be updated during packet drop of target
 			if (pid->nb_buffer_unit < nb_pck) pid->nb_buffer_unit = nb_pck;
 			if ((s64) pid->buffer_duration < dst->buffer_duration) pid->buffer_duration = dst->buffer_duration;
+			//if computed duration of packet is larger than pid max_buffer_time, update
+			//this is to make sure playback at speed > 1 won't trigger blocking state
+			//otherwise we would have max_buffer_time=1ms (default) and a single AU dispatched would block unless speed is AU_DUR_ms/1ms ...
+			if (us_duration && pid->max_buffer_time && (pid->max_buffer_time<us_duration))
+				pid->max_buffer_time = us_duration;
+				
 			gf_mx_v(pid->filter->tasks_mx);
 
 			//post process task
@@ -1332,10 +1336,10 @@ GF_Err gf_filter_pck_send_internal(GF_FilterPacket *pck, Bool from_filter)
 	assert(pck->reference_count);
 	if (safe_int_dec(&pck->reference_count) == 0) {
 		if (!nb_dispatch) {
-			if (nb_discard) {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("All PID destinations on filter %s are in discard mode - discarding\n", pid->filter->name));
+			if (count) {
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("All destinations of PID %s:%s are in discard mode - discarding\n", pid->filter->name, pid->name));
 			} else {
-				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("No PID destination on filter %s for packet - discarding\n", pid->filter->name));
+				GF_LOG(GF_LOG_DEBUG, GF_LOG_FILTER, ("PID %s:%s has no destination for packet - discarding\n", pid->filter->name, pid->name));
 			}
 		}
 		gf_filter_packet_destroy(pck);
@@ -1442,6 +1446,7 @@ GF_EXPORT
 const u8 *gf_filter_pck_get_data(GF_FilterPacket *pck, u32 *size)
 {
 	assert(pck);
+	assert(pck->pck);
 	assert(size);
 	//get true packet pointer
 	pck=pck->pck;
