@@ -30,6 +30,7 @@
 
 #ifdef GPAC_HAS_QJS
 
+static GF_Err gf_fs_load_script_ex(GF_FilterSession *fs, const char *jsfile, JSContext *in_ctx, GF_Filter *for_filter);
 
 
 static JSClassID fs_class_id;
@@ -37,6 +38,11 @@ typedef struct __jsfs_task
 {
 	JSValue fun;
 	JSValue _obj;
+
+	//for event callback, we allow 2 user-defined functions
+	JSValue fun2;
+	JSValue _obj2;
+
 	u32 type;
 	JSContext *ctx;
 } JSFS_Task;
@@ -50,6 +56,10 @@ static void jsfs_mark(JSRuntime *rt, JSValueConst val, JS_MarkFunc *mark_func)
 			JSFS_Task *task = gf_list_get(fs->jstasks, i);
             JS_MarkValue(rt, task->fun, mark_func);
             JS_MarkValue(rt, task->_obj, mark_func);
+            if (!JS_IsUndefined(task->_obj2)) {
+				JS_MarkValue(rt, task->fun2, mark_func);
+				JS_MarkValue(rt, task->_obj2, mark_func);
+			}
 		}
 		gf_fs_lock_filters(fs, GF_TRUE);
 		count = gf_list_count(fs->filters);
@@ -281,6 +291,15 @@ static JSValue jsfs_lock_filters(JSContext *ctx, JSValueConst this_val, int argc
 	return JS_UNDEFINED;
 }
 
+Bool gf_sys_enable_remotery(Bool start, Bool is_shutdown);
+
+static JSValue jsfs_enable_rmt(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	GF_FilterSession *fs = JS_GetOpaque(this_val, fs_class_id);
+    if (!fs) return GF_JS_EXCEPTION(ctx);
+	gf_sys_enable_remotery(GF_TRUE, GF_FALSE);
+	return JS_UNDEFINED;
+}
 
 static JSValue jsfs_rmt_send(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
@@ -297,7 +316,6 @@ static JSValue jsfs_rmt_send(JSContext *ctx, JSValueConst this_val, int argc, JS
 
 void jsfs_on_filter_created(GF_Filter *new_filter)
 {
-	new_filter->jsval = JS_UNDEFINED;
 	if (!new_filter->session->new_f_task) return;
 	jsfs_exec_task_custom(new_filter->session->new_f_task, NULL, new_filter, NULL);
 }
@@ -325,6 +343,8 @@ void jsfs_on_filter_destroyed(GF_Filter *del_filter)
 Bool jsfs_on_event(GF_FilterSession *fs, GF_Event *evt)
 {
 	GF_FilterEvent fevt;
+	Bool pass2=GF_FALSE;
+	JSValue fun, obj;
 	JSValue js_init_evt_obj(JSContext *ctx, const GF_FilterEvent *evt);
 
 	JSValue arg, ret;
@@ -336,9 +356,13 @@ Bool jsfs_on_event(GF_FilterSession *fs, GF_Event *evt)
 	fevt.user_event.event = *evt;
 	fevt.base.type = GF_FEVT_USER;
 
-	arg = js_init_evt_obj(fs->on_evt_task->ctx, &fevt);
+	fun = fs->on_evt_task->fun;
+	obj = fs->on_evt_task->_obj;
 
-	ret = JS_Call(fs->on_evt_task->ctx, fs->on_evt_task->fun, fs->on_evt_task->_obj, 1, &arg);
+
+retry:
+	arg = js_init_evt_obj(fs->on_evt_task->ctx, &fevt);
+	ret = JS_Call(fs->on_evt_task->ctx, fun, obj, 1, &arg);
 	JS_SetOpaque(arg, NULL);
 	JS_FreeValue(fs->on_evt_task->ctx, arg);
 
@@ -353,11 +377,113 @@ Bool jsfs_on_event(GF_FilterSession *fs, GF_Event *evt)
 		evt->clipboard.text = NULL;
 	}
 	JS_FreeValue(fs->on_evt_task->ctx, ret);
+	if (!pass2 && !res && !JS_IsUndefined(fs->on_evt_task->_obj2)) {
+		pass2 = GF_TRUE;
+		fun = fs->on_evt_task->fun2;
+		obj = fs->on_evt_task->_obj2;
+		goto retry;
+	}
 	js_std_loop(fs->on_evt_task->ctx);
 	gf_js_lock(fs->on_evt_task->ctx, GF_FALSE);
 	return res;
 }
 
+typedef struct
+{
+	void (*on_usr_pass)(void *usr_cbk, const char *usr_name, const char *password, Bool store_info);
+	void *async_usr_data;
+} JSFAuthContext;
+
+static JSClassID jsf_auth_class_id;
+static void jsf_auth_finalizer(JSRuntime *rt, JSValue val)
+{
+	JSFAuthContext *actx = JS_GetOpaque(val, jsf_auth_class_id);
+    if (!actx) return;
+
+	if (actx->on_usr_pass) {
+		actx->on_usr_pass(actx->async_usr_data, NULL, NULL, GF_FALSE);
+	}
+    gf_free(actx);
+}
+static JSClassDef jsf_auth_class = {
+    "AsyncAuth",
+    .finalizer = jsf_auth_finalizer
+};
+
+static JSValue js_auth_done(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	JSFAuthContext *actx = JS_GetOpaque(this_val, jsf_auth_class_id);
+	Bool store = GF_FALSE;
+	if (!actx) return GF_JS_EXCEPTION(ctx);
+	const char *user, *pass;
+
+	user = argc ? JS_ToCString(ctx, argv[0]) : NULL;
+	pass = (argc>1) ? JS_ToCString(ctx, argv[1]) : NULL;
+	if (argc==3)
+		store = JS_ToBool(ctx, argv[2]);
+
+	if (!user || !user[0] || !pass || !pass[0])
+		actx->on_usr_pass(actx->async_usr_data, NULL, NULL, GF_FALSE);
+	else
+		actx->on_usr_pass(actx->async_usr_data, user, pass, store);
+
+	//callback done, set to null. If object gets destroyed without the done() being called, we will callback with failure
+	actx->on_usr_pass = NULL;
+	if (user) JS_FreeCString(ctx, user);
+	if (pass) JS_FreeCString(ctx, pass);
+	return JS_UNDEFINED;
+}
+
+static const JSCFunctionListEntry jsf_auth_funcs[] = {
+	JS_CFUNC_DEF("done", 0, js_auth_done),
+};
+
+
+Bool jsfs_on_auth(GF_FilterSession *fs, GF_Event *evt)
+{
+	JSValue args[5], ret, obj;
+	Bool res = GF_TRUE;
+	JSContext *ctx;
+	assert(fs->on_auth_task);
+	ctx = fs->on_auth_task->ctx;
+	gf_js_lock(ctx, GF_TRUE);
+
+	JSFAuthContext *actx;
+	GF_SAFEALLOC(actx, JSFAuthContext);
+	if (!actx) {
+		gf_js_lock(ctx, GF_FALSE);
+		return GF_FALSE;
+	}
+	actx->on_usr_pass = evt->auth.on_usr_pass;
+	actx->async_usr_data = evt->auth.async_usr_data;
+	obj = JS_NewObjectClass(ctx, jsf_auth_class_id);
+	JS_SetOpaque(obj, actx);
+
+	args[0] = JS_NewString(ctx, evt->auth.site_url);
+	args[1] = JS_NewString(ctx, evt->auth.user);
+	args[2] = JS_NewString(ctx, evt->auth.password);
+	args[3] = JS_NewBool(ctx, evt->auth.secure);
+	args[4] = obj;
+
+	ret = JS_Call(ctx, fs->on_auth_task->fun, fs->on_auth_task->_obj, 5, args);
+
+	JS_FreeValue(ctx, args[0]);
+	JS_FreeValue(ctx, args[1]);
+	JS_FreeValue(ctx, args[2]);
+	JS_FreeValue(ctx, args[3]);
+	JS_FreeValue(ctx, args[4]);
+
+	if (JS_IsException(ret)) {
+		js_dump_error(ctx);
+		res = GF_FALSE;
+	}
+	JS_FreeValue(ctx, ret);
+
+	js_std_loop(ctx);
+	gf_js_lock(ctx, GF_FALSE);
+	return res;
+}
+
 
 static JSValue jsfs_set_fun_callback(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv, u32 cbk_type)
 {
@@ -378,6 +504,8 @@ static JSValue jsfs_set_fun_callback(JSContext *ctx, JSValueConst this_val, int
 		task = fs->del_f_task;
 	} else if (cbk_type==4) {
 		task = fs->on_evt_task;
+	} else if (cbk_type==5) {
+		task = fs->on_auth_task;
 	} else {
 		count = gf_list_count(fs->jstasks);
 		for (i=0; i<count; i++) {
@@ -390,6 +518,10 @@ static JSValue jsfs_set_fun_callback(JSContext *ctx, JSValueConst this_val, int
 		if (task) {
 			JS_FreeValue(ctx, task->fun);
 			JS_FreeValue(ctx, task->_obj);
+			if (!JS_IsUndefined(task->_obj2)) {
+				JS_FreeValue(ctx, task->fun2);
+				JS_FreeValue(ctx, task->_obj2);
+			}
 			gf_list_del_item(fs->jstasks, task);
 			gf_free(task);
 		}
@@ -401,19 +533,30 @@ static JSValue jsfs_set_fun_callback(JSContext *ctx, JSValueConst this_val, int
 			fs->del_f_task = NULL;
 		else if (cbk_type == 4)
 			fs->on_evt_task = NULL;
+		else if (cbk_type == 5)
+			fs->on_auth_task = NULL;
 
 		return JS_UNDEFINED;
 	}
 
 	if (task) {
-		JS_FreeValue(ctx, task->fun);
-		JS_FreeValue(ctx, task->_obj);
+		if ((cbk_type == 4) && JS_IsUndefined(task->fun2)) {
+			task->fun2 = task->fun;
+			task->_obj2 = task->_obj;
+			task->fun = JS_UNDEFINED;
+			task->_obj = JS_UNDEFINED;
+		} else {
+			JS_FreeValue(ctx, task->fun);
+			JS_FreeValue(ctx, task->_obj);
+		}
 	} else {
 		GF_SAFEALLOC(task, JSFS_Task);
 		if (!task) return GF_JS_EXCEPTION(ctx);
 		gf_list_add(fs->jstasks, task);
 		task->type = cbk_type;
 		task->ctx = ctx;
+		task->fun2 = JS_UNDEFINED;
+		task->_obj2 = JS_UNDEFINED;
 	}
 	task->fun = JS_DupValue(ctx, argv[0]);
 	task->_obj = JS_DupValue(ctx, this_val);
@@ -432,6 +575,8 @@ static JSValue jsfs_set_fun_callback(JSContext *ctx, JSValueConst this_val, int
 		fs->del_f_task = task;
 	else if (cbk_type == 4)
 		fs->on_evt_task = task;
+	else if (cbk_type == 5)
+		fs->on_auth_task = task;
     return JS_UNDEFINED;
 }
 
@@ -451,6 +596,10 @@ static JSValue jsfs_set_event_fun(JSContext *ctx, JSValueConst this_val, int arg
 {
 	return jsfs_set_fun_callback(ctx, this_val, argc, argv, 4);
 }
+static JSValue jsfs_set_auth_fun(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	return jsfs_set_fun_callback(ctx, this_val, argc, argv, 5);
+}
 
 enum
 {
@@ -760,64 +909,59 @@ static JSValue jsff_get_pid_sinks(JSContext *ctx, JSValueConst this_val, int arg
 	return ret;
 }
 
-static JSValue jsff_all_args(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+static void get_filter_args(JSContext *ctx, GF_FilterSession *fs,  const GF_FilterRegister *freg, GF_Filter *f_inst, Bool val_only, JSValue array)
 {
 	u32 idx, a_idx;
-	JSValue res;
-	Bool val_only = GF_TRUE;
 	const GF_FilterArgs *args;
-	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
-	if (!f)
-		return GF_JS_EXCEPTION(ctx);
 
-	if (argc && JS_IsBool(argv[0]) && JS_ToBool(ctx, argv[0]))
-		val_only = GF_FALSE;
-
-	res = JS_NewArray(ctx);
 	idx=0;
 	a_idx=0;
-	args = f->instance_args ? f->instance_args : f->freg->args;
+	args = (f_inst && f_inst->instance_args) ? f_inst->instance_args : freg->args;
+
 	while (args && args[idx].arg_name) {
 		const GF_FilterArgs *arg = &args[idx];
 		GF_PropertyValue p;
 		JSValue aval;
-		if (arg->flags & GF_FS_ARG_HINT_HIDE) {
+		if ((arg->flags & GF_FS_ARG_HINT_HIDE)
+			|| !strcmp(arg->arg_name, "*")
+		) {
 			idx++;
 			continue;
 		}
 
 		aval = JS_NewObject(ctx);
 		JS_SetPropertyStr(ctx, aval, "name", JS_NewString(ctx, arg->arg_name) );
-		if (f->instance_args) {
+		JS_SetPropertyStr(ctx, aval, "type", JS_NewString(ctx, gf_props_get_type_name(arg->arg_type)) );
+		if (f_inst && f_inst->instance_args) {
 			char szArgName[100];
 			char *arg_start;
-			snprintf(szArgName, 100, "%c%s%c", f->session->sep_args, arg->arg_name, f->session->sep_name);
-			arg_start = f->orig_args ? strstr(f->orig_args, szArgName) : NULL;
+			snprintf(szArgName, 100, "%c%s%c", f_inst->session->sep_args, arg->arg_name, f_inst->session->sep_name);
+			arg_start = f_inst->orig_args ? strstr(f_inst->orig_args, szArgName) : NULL;
 			if (arg_start) {
 				char *arg_end;
 				arg_start += strlen(szArgName);
 				arg_start = gf_strdup(arg_start);
-				arg_end = strchr(arg_start, f->session->sep_args);
+				arg_end = strchr(arg_start, fs->sep_args);
 				if (arg_end) arg_end[0] = 0;
 				JS_SetPropertyStr(ctx, aval, "value", JS_NewString(ctx, arg_start) );
 				gf_free(arg_start);
 			} else {
-				snprintf(szArgName, 100, "%c%s", f->session->sep_args, arg->arg_name);
-				arg_start = f->orig_args ? strstr(f->orig_args, szArgName) : NULL;
+				snprintf(szArgName, 100, "%c%s", fs->sep_args, arg->arg_name);
+				arg_start = f_inst->orig_args ? strstr(f_inst->orig_args, szArgName) : NULL;
 				if (arg_start && (arg->arg_type==GF_PROP_BOOL)) {
 					p.type = GF_PROP_BOOL;
 					p.value.boolean = GF_TRUE;
 					JS_SetPropertyStr(ctx, aval, "value", jsf_NewProp(ctx, &p) );
 				} else {
-					snprintf(szArgName, 100, "%c%c%s", f->session->sep_args, f->session->sep_neg, arg->arg_name);
-					arg_start = f->orig_args ? strstr(f->orig_args, szArgName) : NULL;
+					snprintf(szArgName, 100, "%c%c%s", fs->sep_args, fs->sep_neg, arg->arg_name);
+					arg_start = f_inst->orig_args ? strstr(f_inst->orig_args, szArgName) : NULL;
 					if (arg_start && (arg->arg_type==GF_PROP_BOOL)) {
 						p.type = GF_PROP_BOOL;
 						p.value.boolean = GF_FALSE;
 						JS_SetPropertyStr(ctx, aval, "value", jsf_NewProp(ctx, &p) );
 					} else {
 						if (arg->arg_default_val) {
-							gf_props_parse_value(arg->arg_type, arg->arg_name, arg->arg_default_val, arg->min_max_enum, f->session->sep_list);
+							gf_props_parse_value(arg->arg_type, arg->arg_name, arg->arg_default_val, arg->min_max_enum, f_inst->session->sep_list);
 							JS_SetPropertyStr(ctx, aval, "value", jsf_NewProp(ctx, &p) );
 						} else {
 							JS_SetPropertyStr(ctx, aval, "value", JS_NULL);
@@ -825,10 +969,17 @@ static JSValue jsff_all_args(JSContext *ctx, JSValueConst this_val, int argc, JS
 					}
 				}
 			}
-		} else {
-			gf_filter_get_arg(f, arg->arg_name, &p);
+		} else if (f_inst) {
+			gf_filter_get_arg(f_inst, arg->arg_name, &p);
 			JS_SetPropertyStr(ctx, aval, "value", jsf_NewProp(ctx, &p) );
+		} else if (arg->arg_default_val) {
+			p = gf_filter_parse_prop_solve_env_var(fs, NULL, arg->arg_type, arg->arg_name, arg->arg_default_val, arg->min_max_enum);
+			JS_SetPropertyStr(ctx, aval, "value", jsf_NewProp(ctx, &p) );
+			gf_props_reset_single(&p);
+		} else {
+			JS_SetPropertyStr(ctx, aval, "value", JS_NULL);
 		}
+
 		if (!val_only) {
 			if (arg->arg_desc)
 				JS_SetPropertyStr(ctx, aval, "desc", JS_NewString(ctx, arg->arg_desc) );
@@ -845,13 +996,182 @@ static JSValue jsff_all_args(JSContext *ctx, JSValueConst this_val, int argc, JS
 				JS_SetPropertyStr(ctx, aval, "hint", JS_NewString(ctx, "expert") );
 			}
 		}
-		JS_SetPropertyUint32(ctx, res, idx, aval);
+		JS_SetPropertyUint32(ctx, array, idx, aval);
 		idx++;
 		a_idx++;
 	}
+}
+
+static JSValue jsff_all_args(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	Bool val_only = GF_TRUE;
+	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
+	if (!f)
+		return GF_JS_EXCEPTION(ctx);
+
+	if (argc && JS_IsBool(argv[0]) && JS_ToBool(ctx, argv[0]))
+		val_only = GF_FALSE;
+
+	JSValue res = JS_NewArray(ctx);
+	get_filter_args(ctx, f->session, f->freg, f, val_only, res);
 	return res;
 }
 
+static const GF_FilterRegister *find_reg(GF_FilterSession *fs, char *name)
+{
+	u32 i, count = gf_list_count(fs->registry);
+	for (i=0; i<count; i++) {
+		const GF_FilterRegister *freg = gf_list_get(fs->registry, i);
+		if (!strcmp(freg->name, name)) return freg;
+	}
+	return NULL;
+}
+
+#include <gpac/module.h>
+
+static Bool jsfs_get_filter_args(JSContext *ctx, GF_FilterSession *fs, GF_FilterSession **meta_fs, char *regname, GF_Filter *finst, JSValue args)
+{
+	GF_Filter *new_f = NULL;
+	const GF_FilterRegister *freg = NULL;
+	char *sep = strchr(regname, fs->sep_args);
+
+	if (sep) sep[0] = 0;
+	freg = finst ? finst->freg : find_reg(fs, regname);
+	if (sep) sep[0] = fs->sep_args;
+
+	if (!freg && !finst) {
+		GF_Err e;
+		Bool probe;
+		if (!strncmp(regname, "src=", 4)) {
+			gf_fs_load_source_dest_internal(fs, regname+4, NULL, NULL, &e, NULL, NULL, GF_TRUE, GF_FALSE, &probe, &freg);
+		} else if (!strncmp(regname, "dst=", 4)) {
+			gf_fs_load_source_dest_internal(fs, regname+4, NULL, NULL, &e, NULL, NULL, GF_FALSE, GF_FALSE, &probe, &freg);
+		}
+	}
+
+	if (!freg) return GF_FALSE;
+
+	if (!(freg->flags & GF_FS_REG_META)) {
+		get_filter_args(ctx, fs, freg, NULL, GF_FALSE, args);
+		return GF_TRUE;
+	}
+
+	//don't load a full session, only allocate object and create registry list
+	if (! *meta_fs) {
+		GF_SAFEALLOC(*meta_fs, GF_FilterSession)
+		(*meta_fs)->registry = gf_list_new();
+		(*meta_fs)->blacklist = fs->blacklist;
+
+		//load internal filters
+		gf_fs_reg_all(*meta_fs, *meta_fs);
+
+		//load external modules
+		u32 i, count = gf_modules_count();
+		for (i=0; i<count; i++) {
+			GF_FilterRegister *a_freg = (GF_FilterRegister *) gf_modules_load_filter(i, *meta_fs);
+			if (a_freg) {
+				gf_fs_add_filter_register(*meta_fs, a_freg);
+			}
+		}
+		(*meta_fs)->blacklist = NULL;
+	}
+
+	gf_fs_lock_filters(fs, GF_TRUE);
+	if (!finst) {
+		GF_Err e;
+		char *fdesc, szFmt[30];
+		fdesc = gf_strdup(regname);
+		sprintf(szFmt, "%cgpac", fs->sep_args);
+		if (strstr(fdesc, szFmt)) {
+			sprintf(szFmt, "%c_GFTMP", fs->sep_args);
+		} else {
+			sprintf(szFmt, "%c%cgpac%c%c_GFTMP", fs->sep_args, fs->sep_args, fs->sep_args, fs->sep_args);
+		}
+		gf_dynstrcat(&fdesc, szFmt, NULL);
+
+		if (!strncmp(fdesc, "src=", 4)) {
+			new_f = gf_fs_load_source(fs, fdesc+4, NULL, NULL, &e);
+		} else if (!strncmp(fdesc, "dst=", 4)) {
+			new_f = gf_fs_load_destination(fs, fdesc+4, NULL, NULL, &e);
+		} else {
+			new_f = gf_fs_load_filter(fs, fdesc, &e);
+		}
+		if (fdesc) gf_free(fdesc);
+
+		if (!new_f) {
+			gf_fs_lock_filters(fs, GF_FALSE);
+			return GF_FALSE;
+		}
+		finst = new_f;
+	}
+
+	//get all args
+	get_filter_args(ctx, fs, finst->freg, finst, GF_FALSE, args);
+	const char *inst_names = gf_filter_meta_get_instances(finst);
+	while (inst_names && inst_names[0]) {
+		char szFName[100];
+		sep = strchr(inst_names, ' ');
+		if (sep) sep[0] = 0;
+		sprintf(szFName, "%s:%s", finst->freg->name, inst_names);
+
+		freg = find_reg(*meta_fs, szFName);
+		if (freg) {
+			get_filter_args(ctx, fs, freg, NULL, GF_FALSE, args);
+		}
+
+		if (!sep) break;
+		sep[0] = ' ';
+		inst_names = sep+1;
+	}
+
+	//delete filter
+	if (new_f) {
+		gf_list_del_item(fs->filters, new_f);
+		if (!new_f->finalized && new_f->freg->finalize) {
+			new_f->freg->finalize(new_f);
+		}
+		gf_filter_del(new_f);
+	}
+
+	gf_fs_lock_filters(fs, GF_FALSE);
+	return GF_TRUE;
+}
+
+static void del_meta_fs(GF_FilterSession *metafs)
+{
+	while (gf_list_count(metafs->registry)) {
+		GF_FilterRegister *freg = gf_list_pop_back(metafs->registry);
+		if (freg->register_free)
+			freg->register_free(metafs, freg);
+	}
+	gf_list_del(metafs->registry);
+	gf_free(metafs);
+}
+
+static JSValue jsfs_filter_args(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	char *name = NULL;
+	GF_FilterSession *fs = JS_GetOpaque(this_val, fs_class_id);
+	if (!fs || !argc) return GF_JS_EXCEPTION(ctx);
+
+	name = (char *)JS_ToCString(ctx, argv[0]);
+	if (!name) return GF_JS_EXCEPTION(ctx);
+
+	GF_FilterSession *meta_fs = NULL;
+	JSValue args = JS_NewArray(ctx);
+	Bool res = jsfs_get_filter_args(ctx, fs, &meta_fs, name, NULL, args);
+	JS_FreeCString(ctx, name);
+	if (meta_fs)
+		del_meta_fs(meta_fs);
+
+	if (!res) {
+		JS_FreeValue(ctx, args);
+		return GF_JS_EXCEPTION(ctx);
+	}
+	return args;
+}
+
+
 static JSValue jsff_get_arg(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
 	u32 idx;
@@ -889,19 +1209,23 @@ GF_Err jsf_ToProp_ex(GF_Filter *filter, JSContext *ctx, JSValue value, u32 p4cc,
 static JSValue jsff_update(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
 	const char *aname;
+	u32 prop_mask=0;
 	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
-	if (!f || (argc!=2) )
+	if (!f || (argc<2) )
 		return GF_JS_EXCEPTION(ctx);
 
 	aname = JS_ToCString(ctx, argv[0]);
 	if (!aname) return GF_JS_EXCEPTION(ctx);
+	if (argc>2) {
+		JS_ToInt32(ctx, &prop_mask, argv[2]);
+	}
 	if (JS_IsString(argv[1])) {
 		const char *aval = JS_ToCString(ctx, argv[1]);
 		if (!aval) {
 			JS_FreeCString(ctx, aname);
 			return GF_JS_EXCEPTION(ctx);
 		}
-		gf_fs_send_update(f->session, NULL, f, aname, aval, 0);
+		gf_fs_send_update(f->session, NULL, f, aname, aval, prop_mask);
 		JS_FreeCString(ctx, aval);
 		JS_FreeCString(ctx, aname);
 		return JS_UNDEFINED;
@@ -921,7 +1245,7 @@ static JSValue jsff_update(JSContext *ctx, JSValueConst this_val, int argc, JSVa
 			return err;
 		}
 		gf_props_dump_val(&p, szDump, GF_PROP_DUMP_DATA_PTR, NULL);
-		gf_fs_send_update(f->session, NULL, f, aname, szDump, 0);
+		gf_fs_send_update(f->session, NULL, f, aname, szDump, prop_mask);
 		gf_props_reset_single(&p);
 	}
 
@@ -959,6 +1283,7 @@ static JSValue jsff_insert_filter(JSContext *ctx, JSValueConst this_val, int arg
 	GF_Filter *new_f;
 	GF_Err e;
 	Bool is_source = GF_FALSE;
+	GF_FilterPid *opid=NULL;
 	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
 	if (!f || !argc)
 		return GF_JS_EXCEPTION(ctx);
@@ -967,10 +1292,21 @@ static JSValue jsff_insert_filter(JSContext *ctx, JSValueConst this_val, int arg
 	if (!fname) return GF_JS_EXCEPTION(ctx);
 	link_args = NULL;
 	if (argc>1) {
-		link_args = JS_ToCString(ctx, argv[1]);
-		if (!link_args) {
-			JS_FreeCString(ctx, fname);
-			return GF_JS_EXCEPTION(ctx);
+		u32 offset=1;
+		if (!JS_IsString(argv[1])) {
+			s32 opid_idx=-1;
+			if (JS_ToInt32(ctx, &opid_idx, argv[1])) return GF_JS_EXCEPTION(ctx);
+			if (opid_idx<0) return GF_JS_EXCEPTION(ctx);
+			opid = gf_filter_get_opid(f, opid_idx);
+			if (!opid) return GF_JS_EXCEPTION(ctx);
+			offset=2;
+		}
+		if ((u32) argc>offset) {
+			link_args = JS_ToCString(ctx, argv[offset]);
+			if (!link_args) {
+				JS_FreeCString(ctx, fname);
+				return GF_JS_EXCEPTION(ctx);
+			}
 		}
 	}
 	gf_fs_lock_filters(f->session, GF_TRUE);
@@ -996,9 +1332,11 @@ static JSValue jsff_insert_filter(JSContext *ctx, JSValueConst this_val, int arg
 		gf_filter_set_source(new_f, (GF_Filter *) f, link_args);
 
 	gf_fs_lock_filters(f->session, GF_FALSE);
+
+	f->dst_filter = new_f;
 	
 	//reconnect outputs of source
-	gf_filter_reconnect_output((GF_Filter *) f);
+	gf_filter_reconnect_output((GF_Filter *) f, opid);
 
 	JS_FreeCString(ctx, fname);
 	if (link_args) JS_FreeCString(ctx, link_args);
@@ -1006,6 +1344,105 @@ static JSValue jsff_insert_filter(JSContext *ctx, JSValueConst this_val, int arg
 	return jsfs_new_filter_obj(ctx, new_f);
 }
 
+static Bool jsfs_get_filter_args(JSContext *ctx, GF_FilterSession *fs, GF_FilterSession **meta_fs, char *regname, GF_Filter *finst, JSValue args);
+static void del_meta_fs(GF_FilterSession *metafs);
+
+static JSValue jsff_compute_link(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	const char *fname;
+	char *fdesc=NULL;
+	char szFmt[20];
+	GF_Filter *new_f;
+	GF_Err e;
+	Bool load_args=GF_FALSE;
+	s32 opid_idx=-1;
+	GF_FilterPid *opid=NULL;
+	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
+	if (!f || (argc<2))
+		return GF_JS_EXCEPTION(ctx);
+
+	if (JS_ToInt32(ctx, &opid_idx, argv[0])) return GF_JS_EXCEPTION(ctx);
+	if (opid_idx<0) return GF_JS_EXCEPTION(ctx);
+	opid = gf_filter_get_opid(f, opid_idx);
+	if (!opid) return GF_JS_EXCEPTION(ctx);
+	fname = JS_ToCString(ctx, argv[1]);
+	if (!fname) return GF_JS_EXCEPTION(ctx);
+	if (!strncmp(fname, "src=", 4) || !strlen(fname)) {
+		JS_FreeCString(ctx, fname);
+		return GF_JS_EXCEPTION(ctx);
+	}
+	if ((argc==3) && JS_ToBool(ctx, argv[2]))
+		load_args = GF_TRUE;
+
+	fdesc = gf_strdup(fname);
+	JS_FreeCString(ctx, fname);
+	sprintf(szFmt, "%cgpac", f->session->sep_args);
+	if (strstr(fdesc, szFmt)) {
+		sprintf(szFmt, "%c_GFTMP", f->session->sep_args);
+	} else {
+		sprintf(szFmt, "%c%cgpac%c%c_GFTMP", f->session->sep_args, f->session->sep_args, f->session->sep_args, f->session->sep_args);
+	}
+	gf_dynstrcat(&fdesc, szFmt, NULL);
+
+	gf_fs_lock_filters(f->session, GF_TRUE);
+	if (!strncmp(fname, "dst=", 4)) {
+		new_f = gf_fs_load_destination(f->session, fdesc+4, NULL, NULL, &e);
+	} else {
+		new_f = gf_fs_load_filter(f->session, fdesc, &e);
+	}
+	gf_free(fdesc);
+
+	if (!new_f) {
+		JSValue ret = js_throw_err_msg(ctx, e, "Cannot load filter %s: %s\n", fname, gf_error_to_string(e));
+		gf_fs_lock_filters(f->session, GF_FALSE);
+		return ret;
+	}
+
+	JSValue res;
+	GF_List *fchain = gf_filter_pid_compute_link(opid, new_f);
+	if (fchain) {
+		res = JS_NewArray(ctx);
+		u32 i, count = gf_list_count(fchain);
+		GF_FilterSession *meta_fs=NULL;
+		for (i=0; i<count; i+=2) {
+			const GF_FilterRegister *freg = gf_list_get(fchain, i);
+
+			if (load_args) {
+				GF_Filter *finst = NULL;
+				JSValue obj = JS_NewObject(ctx);
+				JS_SetPropertyUint32(ctx, res, i/2, obj);
+				JS_SetPropertyStr(ctx, obj, "fname", JS_NewString(ctx, freg->name));
+				JSValue args = JS_NewArray(ctx);
+				JS_SetPropertyStr(ctx, obj, "args", args);
+				if (i+2>=count) finst = new_f;
+
+				jsfs_get_filter_args(ctx, f->session, &meta_fs, (char *) freg->name, finst, args);
+			} else {
+				JS_SetPropertyUint32(ctx, res, i/2, JS_NewString(ctx, freg->name));
+			}
+		}
+		gf_list_del(fchain);
+		if (meta_fs)
+			del_meta_fs(meta_fs);
+	} else {
+		res = JS_NULL;
+	}
+	gf_list_del_item(f->session->filters, new_f);
+	if (!new_f->finalized && new_f->freg->finalize) {
+		new_f->freg->finalize(new_f);
+	}
+	gf_filter_del(new_f);
+	gf_fs_lock_filters(f->session, GF_FALSE);
+	return res;
+}
+
+static JSValue jsff_require_source_id(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
+	if (!f) return GF_JS_EXCEPTION(ctx);
+	gf_filter_require_source_id(f);
+	return JS_UNDEFINED;
+}
 static JSValue jsff_bind(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
 {
 	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
@@ -1024,6 +1461,74 @@ static JSValue jsff_bind(JSContext *ctx, JSValueConst this_val, int argc, JSValu
 	return js_throw_err_msg(ctx, GF_BAD_PARAM, "filter class %s has no JS bind capabilities", f->freg->name);
 }
 
+static JSValue jsff_get_stats(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv, Bool is_input)
+{
+	JSValue res;
+	u32 mode=GF_STATS_LOCAL;
+	GF_Filter *f = JS_GetOpaque(this_val, fs_f_class_id);
+	u32 idx;
+	if (!f || (argc<1) )
+		return GF_JS_EXCEPTION(ctx);
+
+	if (JS_ToInt32(ctx, &idx, argv[0]))
+		return GF_JS_EXCEPTION(ctx);
+
+	GF_FilterPid *pid;
+	if (is_input) pid = gf_filter_get_ipid(f, idx);
+	else pid = gf_filter_get_opid(f, idx);
+	if (!pid)
+		return GF_JS_EXCEPTION(ctx);
+
+	if (argc>1)
+		JS_ToInt32(ctx, &mode, argv[1]);
+
+	GF_FilterPidStatistics stats;
+	gf_filter_pid_get_statistics(pid, &stats, (GF_FilterPidStatsLocation) mode);
+	res = JS_NewObject(ctx);
+	if (JS_IsException(res)) return res;
+
+	JS_SetPropertyStr(ctx, res, "disconnected", stats.disconnected ? JS_TRUE : JS_FALSE);
+#define SET_U32(_name) \
+	JS_SetPropertyStr(ctx, res, #_name, JS_NewInt32(ctx, stats._name));
+
+#define SET_U64(_name) \
+	JS_SetPropertyStr(ctx, res, #_name, JS_NewInt64(ctx, stats._name));
+
+	SET_U32(average_process_rate)
+	SET_U32(max_process_rate)
+	SET_U32(average_bitrate)
+	SET_U32(max_bitrate)
+	SET_U32(nb_processed)
+	SET_U32(max_process_time)
+	SET_U64(total_process_time)
+	SET_U64(first_process_time)
+	SET_U64(last_process_time)
+	SET_U32(min_frame_dur)
+	SET_U32(nb_saps)
+	SET_U32(max_sap_process_time)
+	SET_U64(total_sap_process_time)
+	SET_U64(max_buffer_time)
+	SET_U64(max_playout_time)
+	SET_U64(min_playout_time)
+	SET_U64(buffer_time)
+	SET_U32(nb_buffer_units)
+	SET_U64(last_rt_report)
+	SET_U32(rtt)
+	SET_U32(jitter)
+	SET_U32(loss_rate)
+	return res;
+}
+
+static JSValue jsff_ipid_stats(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	return jsff_get_stats(ctx, this_val, argc, argv, GF_TRUE);
+}
+
+static JSValue jsff_opid_stats(JSContext *ctx, JSValueConst this_val, int argc, JSValueConst *argv)
+{
+	return jsff_get_stats(ctx, this_val, argc, argv, GF_FALSE);
+}
+
 #define JS_CGETSET_MAGIC_DEF_ENUM(name, fgetter, fsetter, magic) { name, JS_PROP_CONFIGURABLE|JS_PROP_ENUMERABLE, JS_DEF_CGETSET_MAGIC, magic, .u = { .getset = { .get = { .getter_magic = fgetter }, .set = { .setter_magic = fsetter } } } }
 
 static const JSCFunctionListEntry fs_f_funcs[] = {
@@ -1066,6 +1571,10 @@ static const JSCFunctionListEntry fs_f_funcs[] = {
 	JS_CFUNC_DEF("insert", 0, jsff_insert_filter),
 	JS_CFUNC_DEF("bind", 0, jsff_bind),
 	JS_CFUNC_DEF("lock", 0, jsff_lock),
+	JS_CFUNC_DEF("ipid_stats", 0, jsff_ipid_stats),
+	JS_CFUNC_DEF("opid_stats", 0, jsff_opid_stats),
+	JS_CFUNC_DEF("compute_link", 0, jsff_compute_link),
+	JS_CFUNC_DEF("require_source_id", 0, jsff_require_source_id),
 };
 
 static JSValue jsfs_new_filter_obj(JSContext *ctx, GF_Filter *f)
@@ -1245,6 +1754,7 @@ static const JSCFunctionListEntry fs_funcs[] = {
     JS_CFUNC_DEF("abort", 0, jsfs_abort),
     JS_CFUNC_DEF("get_filter", 0, jsfs_get_filter),
     JS_CFUNC_DEF("lock_filters", 0, jsfs_lock_filters),
+    JS_CFUNC_DEF("enable_rmt", 0, jsfs_enable_rmt),
     JS_CFUNC_DEF("rmt_send", 0, jsfs_rmt_send),
     JS_CFUNC_DEF("set_rmt_fun", 0, jsfs_set_rmt_fun),
     JS_CFUNC_DEF("set_new_filter_fun", 0, jsfs_set_new_filter_fun),
@@ -1254,6 +1764,8 @@ static const JSCFunctionListEntry fs_funcs[] = {
     JS_CFUNC_DEF("fire_event", 0, jsfs_fire_event),
     JS_CFUNC_DEF("reporting", 0, jsfs_reporting),
     JS_CFUNC_DEF("new_filter", 0, jsfs_new_filter),
+    JS_CFUNC_DEF("set_auth_fun", 0, jsfs_set_auth_fun),
+	JS_CFUNC_DEF("filter_args", 0, jsfs_filter_args),
 
 };
 
@@ -1311,6 +1823,12 @@ GF_Err gf_fs_load_js_api(JSContext *c, GF_FilterSession *fs)
 	JS_NewClassID(&fs_f_class_id);
 	JS_NewClass(rt, fs_f_class_id, &fs_f_class);
 
+	JS_NewClassID(&jsf_auth_class_id);
+	JS_NewClass(rt, jsf_auth_class_id, &jsf_auth_class);
+	JSValue proto = JS_NewObjectClass(c, jsf_auth_class_id);
+	JS_SetPropertyFunctionList(c, proto, jsf_auth_funcs, countof(jsf_auth_funcs));
+	JS_SetClassProto(c, jsf_auth_class_id, proto);
+
 
 	fs_obj = JS_NewObjectClass(c, fs_class_id);
     JS_SetPropertyFunctionList(c, fs_obj, fs_funcs, countof(fs_funcs));
@@ -1321,8 +1839,7 @@ GF_Err gf_fs_load_js_api(JSContext *c, GF_FilterSession *fs)
     return GF_OK;
 }
 
-GF_EXPORT
-GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
+static GF_Err gf_fs_load_script_ex(GF_FilterSession *fs, const char *jsfile, JSContext *in_ctx, GF_Filter *for_filter)
 {
 	GF_Err e;
     JSValue global_obj;
@@ -1334,24 +1851,27 @@ GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
 
 
 	if (!fs) return GF_BAD_PARAM;
-	if (fs->js_ctx) return GF_NOT_SUPPORTED;
+	if (!in_ctx) {
+		if (fs->js_ctx) return GF_NOT_SUPPORTED;
 
 #ifdef GPAC_HAS_QJS
-	ctx = gf_js_create_context();
-	if (!ctx) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_SCRIPT, ("[JSF] Failed to load QuickJS context\n"));
-		return GF_IO_ERR;
+		ctx = gf_js_create_context();
+		if (!ctx) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_SCRIPT, ("[JSF] Failed to load QuickJS context\n"));
+			return GF_IO_ERR;
+		}
+		JS_SetContextOpaque(ctx, fs);
+
+		global_obj = JS_GetGlobalObject(ctx);
+		gf_fs_load_js_api(ctx, fs);
+		fs->js_ctx = ctx;
+
+		JS_SetPropertyStr(fs->js_ctx, global_obj, "_gpac_log_name", JS_NewString(fs->js_ctx, gf_file_basename(jsfile) ) );
+		JS_SetPropertyStr(fs->js_ctx, global_obj, "_gpac_script_src", JS_NewString(fs->js_ctx, jsfile ) );
+		JS_FreeValue(fs->js_ctx, global_obj);
+	} else {
+		ctx = in_ctx;
 	}
-	JS_SetContextOpaque(ctx, fs);
-
-    global_obj = JS_GetGlobalObject(ctx);
-	gf_fs_load_js_api(ctx, fs);
-	fs->js_ctx = ctx;
-
-	JS_SetPropertyStr(fs->js_ctx, global_obj, "_gpac_log_name", JS_NewString(fs->js_ctx, gf_file_basename(jsfile) ) );
-	JS_SetPropertyStr(fs->js_ctx, global_obj, "_gpac_script_src", JS_NewString(fs->js_ctx, jsfile ) );
-    JS_FreeValue(fs->js_ctx, global_obj);
-
 
 	//load script
 	if (!strncmp(jsfile, "$GSHARE/", 8)) {
@@ -1370,22 +1890,26 @@ GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
 		return e;
 	}
 
- 	if (!gf_opts_get_bool("core", "no-js-mods") && JS_DetectModule((char *)buf, buf_len)) {
-		qjs_init_all_modules(fs->js_ctx, GF_FALSE, GF_FALSE);
+	if (in_ctx || (!gf_opts_get_bool("core", "no-js-mods") && JS_DetectModule((char *)buf, buf_len))) {
+		qjs_init_all_modules(ctx, GF_FALSE, GF_FALSE);
 		flags = JS_EVAL_TYPE_MODULE;
 	}
 
-	ret = JS_Eval(fs->js_ctx, (char *)buf, buf_len, jsfile, flags);
+	JSValue global = JS_GetGlobalObject(ctx);
+	JS_SetPropertyStr(ctx, global, "parent_filter", for_filter ? jsfs_new_filter_obj(ctx, for_filter) : JS_NULL);
+	JS_FreeValue(ctx, global);
+
+	ret = JS_Eval(ctx, (char *)buf, buf_len, jsfile, flags);
 	gf_free(buf);
 
 	if (JS_IsException(ret)) {
 		GF_LOG(GF_LOG_ERROR, GF_LOG_SCRIPT, ("[JSF] Error loading script %s\n", jsfile));
-        js_dump_error(fs->js_ctx);
-		JS_FreeValue(fs->js_ctx, ret);
+        js_dump_error(ctx);
+		JS_FreeValue(ctx, ret);
 		return GF_BAD_PARAM;
 	}
-	JS_FreeValue(fs->js_ctx, ret);
-	js_std_loop(fs->js_ctx);
+	JS_FreeValue(ctx, ret);
+	js_std_loop(ctx);
 	return GF_OK;
 #else
 	return GF_NOT_SUPPORTED;
@@ -1393,10 +1917,23 @@ GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
 
 }
 
+GF_EXPORT
+GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
+{
+	return gf_fs_load_script_ex(fs, jsfile, NULL, NULL);
+}
+
 void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx)
 {
 	u32 i, count=gf_list_count(fs->jstasks);
 
+	gf_js_lock(js_ctx, GF_TRUE);
+	fs->new_f_task = NULL;
+	fs->del_f_task = NULL;
+	fs->on_evt_task = NULL;
+	fs->on_auth_task = NULL;
+	gf_js_lock(js_ctx, GF_FALSE);
+
 	for (i=0; i<count; i++) {
 		JSFS_Task *task = gf_list_get(fs->jstasks, i);
 		if (js_ctx && (task->ctx != js_ctx))
@@ -1405,6 +1942,10 @@ void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx)
 		gf_js_lock(js_ctx, GF_TRUE);
 		JS_FreeValue(task->ctx, task->fun);
 		JS_FreeValue(task->ctx, task->_obj);
+		if (!JS_IsUndefined(task->_obj2)) {
+			JS_FreeValue(task->ctx, task->fun2);
+			JS_FreeValue(task->ctx, task->_obj2);
+		}
 		gf_js_lock(js_ctx, GF_FALSE);
 
 		gf_free(task);
@@ -1412,7 +1953,6 @@ void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx)
 		i--;
 		count--;
 	}
-
 	if (fs->js_ctx) {
 		gf_js_delete_context(fs->js_ctx);
 		fs->js_ctx = NULL;
@@ -1422,6 +1962,31 @@ void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx)
 		fs->jstasks = NULL;
 	}
 }
+void gf_filter_load_script(GF_Filter *filter, const char *js_file, const char *filters_blacklist)
+{
+	if (filter->finalized || filter->removed)
+		return;
+
+	gf_mx_p(filter->session->filters_mx);
+	if (filters_blacklist) {
+		u32 i, count = gf_list_count(filter->session->filters);
+		for (i=0; i<count; i++) {
+			GF_Filter *af = gf_list_get(filter->session->filters, i);
+			if (!strstr(filters_blacklist, af->freg->name)) continue;
+
+			if (!strcmp(af->freg->name, "compositor")) {
+				GF_PropertyValue p;
+				gf_filter_get_arg(af, "player", &p);
+				if (p.value.uint==0) continue;
+			}
+
+			gf_mx_v(filter->session->filters_mx);
+			return;
+		}
+	}
+	gf_fs_load_script_ex(filter->session, js_file, filter->session->js_ctx, filter);
+	gf_mx_v(filter->session->filters_mx);
+}
 
 #else
 GF_EXPORT
@@ -1432,6 +1997,10 @@ GF_Err gf_fs_load_script(GF_FilterSession *fs, const char *jsfile)
 void gf_fs_unload_script(GF_FilterSession *fs, void *js_ctx)
 {
 
+}
+void gf_filter_load_script(GF_Filter *filter, const char *js_file, const char *filters_blacklist)
+{
+
 }
 
 #endif
