@@ -140,7 +140,6 @@ GF_Err bifs_dec_process(GF_Filter *filter)
 	GF_Err e;
 	Double ts_offset;
 	u64 now, cts;
-	u32 obj_time;
 	u32 i, count;
 	const char *data;
 	u32 size, ESID=0;
@@ -189,24 +188,18 @@ GF_Err bifs_dec_process(GF_Filter *filter)
 		if (prop) ESID = prop->value.uint;
 
 		cts = gf_filter_pck_get_cts( pck );
-		ts_offset = (Double) cts;
-		ts_offset /= gf_filter_pck_get_timescale(pck);
+		cts = gf_timestamp_to_clocktime(cts, gf_filter_pck_get_timescale(pck) );
 
-		gf_odm_check_buffering(odm, pid);
-
-
-		//we still process any frame before our clock time even when buffering
-		obj_time = gf_clock_time(odm->ck);
-		if (ts_offset * 1000 > obj_time) {
-			gf_sc_sys_frame_pending(scene->compositor, ts_offset, obj_time, filter);
+		if (!gf_sc_check_sys_frame(scene, odm, pid, filter, cts, 0))
 			continue;
-		}
 
+		ts_offset = (Double) cts;
+		ts_offset /= 1000.0;
 		now = gf_sys_clock_high_res();
 		e = gf_bifs_decode_au(ctx->bifs_dec, ESID, data, size, ts_offset);
 		now = gf_sys_clock_high_res() - now;
 
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[BIFS] ODM%d #CH%d at %d decoded AU TS %u in "LLU" us\n", odm->ID, ESID, obj_time, cts, now));
+		GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[BIFS] ODM%d #CH%d decoded AU TS %u in "LLU" us\n", odm->ID, ESID, cts, now));
 
 		gf_filter_pid_drop_packet(pid);
 
