@@ -48,16 +48,17 @@ typedef struct
 {
 	//opts
 	GF_Fraction fps;
-	Bool importer;
+	Bool importer, noreframe;
 
 	GF_FilterPid *ipid;
 
 	GF_FilterPid *v_opid;
-	Bool v_in_use;
+	Bool v_in_use, video_done;
 	u32 nb_frames, cur_frame, nb_frame_sent;
 	u32 dummy, nvops;
 	const char *src_url;
 	avi_t *avi;
+	u64 file_size;
 
 	Bool use_file_fps;
 	GF_Fraction64 duration;
@@ -114,7 +115,9 @@ static void avidmx_setup(GF_Filter *filter, GF_AVIDmxCtx *ctx)
 		codecid = GF_CODECID_AVC;
 		unframed = GF_FALSE;
 	} else if (!stricmp(comp, "DIV3") || !stricmp(comp, "DIV4")) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[AVIDmx] Video format %s not compliant with MPEG-4 Visual - please recompress the file first\n", comp));
+//		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[AVIDmx] Video format %s not compliant with MPEG-4 Visual - please recompress the file first\n", comp));
+		codecid = GF_CODECID_MSPEG4_V3;
+		unframed = GF_FALSE;
 	} else if (!comp[0]) {
 		codecid = GF_CODECID_RAW;
 		pfmt = GF_PIXEL_BGR;
@@ -160,8 +163,9 @@ static void avidmx_setup(GF_Filter *filter, GF_AVIDmxCtx *ctx)
 			gf_filter_pid_set_property_str(ctx->v_opid, "nocts", &PROP_BOOL( GF_TRUE ) );
 		} else if (ctx->avi->extradata_size && ctx->avi->extradata) {
 			gf_filter_pid_set_property(ctx->v_opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA(ctx->avi->extradata, ctx->avi->extradata_size) );
-
 		}
+		if (ctx->noreframe)
+			gf_filter_pid_set_property(ctx->v_opid, GF_PROP_PID_UNFRAMED, NULL);
 	}
 
 	unframed = GF_FALSE;
@@ -329,6 +333,8 @@ static void avidmx_setup(GF_Filter *filter, GF_AVIDmxCtx *ctx)
 				}
 			}
 
+			if (ctx->noreframe)
+				gf_filter_pid_set_property(st->opid, GF_PROP_PID_UNFRAMED, NULL);
 		}
 	}
 }
@@ -360,7 +366,7 @@ GF_Err avidmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 		ctx->ipid = pid;
 
 		//we work with full file only, send a play event on source to indicate that
-		GF_FEVT_INIT(fevt, GF_FEVT_PLAY, pid);
+		GF_FEVT_INIT(fevt, GF_FEVT_PLAY_HINT, pid);
 		fevt.play.start_range = 0;
 		fevt.base.on_pid = ctx->ipid;
 		fevt.play.full_file_only = GF_TRUE;
@@ -398,6 +404,7 @@ static Bool avidmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		if (evt->base.on_pid==ctx->v_opid) {
 			u32 frame_idx = 0;
 			ctx->v_playing = GF_TRUE;
+			ctx->video_done = GF_FALSE;
 			if ((evt->play.start_range==0) && !ctx->v_init_play_done) {
 				ctx->v_init_play_done = GF_TRUE;
 				return GF_TRUE;
@@ -405,6 +412,11 @@ static Bool avidmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 			ctx->v_init_play_done = GF_TRUE;
 			frame_idx = (u32) (ctx->avi->fps * evt->play.start_range);
 			if (frame_idx) {
+#ifdef GPAC_ENABLE_COVERAGE
+				if (gf_sys_is_cov_mode()) {
+					AVI_seek_start(ctx->avi);
+				}
+#endif
 				AVI_set_video_position(ctx->avi, frame_idx);
 			} else {
 				AVI_seek_start(ctx->avi);
@@ -417,6 +429,7 @@ static Bool avidmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 				AVIAstream *st = gf_list_get(ctx->audios, i);
 				if (st->opid != evt->base.on_pid) continue;
 				st->playing = GF_TRUE;
+				st->audio_done = GF_FALSE;
 				if ((evt->play.start_range==0) && !st->init_play_done) {
 					st->init_play_done = GF_TRUE;
 					return GF_TRUE;
@@ -438,7 +451,7 @@ static Bool avidmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 			for (i=0; i<gf_list_count(ctx->audios); i++) {
 				AVIAstream *st = gf_list_get(ctx->audios, i);
 				if (st->opid != evt->base.on_pid) continue;
-				st->playing = GF_TRUE;
+				st->playing = GF_FALSE;
 			}
 		}
 		//don't cancel event
@@ -458,7 +471,8 @@ GF_Err avidmx_process(GF_Filter *filter)
 {
 	GF_AVIDmxCtx *ctx = gf_filter_get_udta(filter);
 	u32 i, count, nb_done, nb_active=0;
-	Bool start, end, video_done;
+	Bool start, end;
+	u32 a_pc=0;
 
 	if (!ctx->avi) {
 		GF_FilterPacket *pck = gf_filter_pid_get_packet(ctx->ipid);
@@ -471,7 +485,6 @@ GF_Err avidmx_process(GF_Filter *filter)
 		if (!end) {
 			return GF_OK;
 		}
-
 		ctx->avi = AVI_open_input_file((char *)ctx->src_url, 1);
 		if (!ctx->avi) {
 			GF_Err e = GF_NON_COMPLIANT_BITSTREAM;
@@ -479,10 +492,16 @@ GF_Err avidmx_process(GF_Filter *filter)
 			gf_filter_setup_failure(filter, e);
 			return GF_NOT_SUPPORTED;
 		}
+		FILE *fp = gf_fopen(ctx->src_url, "r");
+		if (fp) {
+			ctx->file_size = gf_fsize(fp);
+			gf_fclose(fp);
+		}
+
 		avidmx_setup(filter, ctx);
 		return GF_OK;
 	}
-	video_done = GF_FALSE;
+
 	if (ctx->v_in_use && ctx->v_playing && (ctx->cur_frame < ctx->nb_frames) && !gf_filter_pid_would_block(ctx->v_opid) ) {
 		u32 key;
 		u64 file_offset, cts;
@@ -518,35 +537,52 @@ GF_Err avidmx_process(GF_Filter *filter)
 		ctx->cur_frame++;
 		if (ctx->cur_frame < ctx->nb_frames)
 			gf_filter_post_process_task(filter);
-		else
-			video_done = GF_TRUE;
-	} else if (!ctx->v_in_use) {
-		video_done = GF_TRUE;
-	} else if (ctx->cur_frame >= ctx->nb_frames) {
-		video_done = GF_TRUE;
+		else if (!ctx->video_done) {
+			ctx->video_done = GF_TRUE;
+			gf_filter_pid_set_eos(ctx->v_opid);
+		}
+	} else if (!ctx->v_in_use || (ctx->cur_frame >= ctx->nb_frames)) {
+		if (!ctx->video_done) {
+			if (ctx->v_in_use) {
+				gf_filter_pid_set_eos(ctx->v_opid);
+			}
+			ctx->video_done = GF_TRUE;
+		}
 	}
 	
 	nb_done = 0;
 	count = gf_list_count(ctx->audios);
 	for (i=0; i<count; i++) {
 		s32 size;
+		u32 pc=0;
 		AVIAstream *st = gf_list_get(ctx->audios, i);
 		if (st->audio_done || !st->in_use) {
 			nb_done++;
 			continue;
 		}
-		if (!ctx->v_playing) video_done = GF_TRUE;
+		if (!ctx->v_playing)
+			ctx->video_done = GF_TRUE;
+
 		if (!st->playing || gf_filter_pid_would_block(st->opid) )
 			continue;
 		AVI_set_audio_track(ctx->avi, st->stream_num);
 		nb_active++;
 
+restart:
 		size = AVI_audio_size(ctx->avi, st->aud_frame);
 
 		if (st->seek_to_ts && size) {
 			if (st->seek_to_ts > st->audio_ts) {
 				st->aud_frame ++;
-				continue;
+				if (st->audio_bps) {
+					u32 nb_samples = (8*size) / (st->audio_bps * st->nb_channels);
+					st->audio_ts += nb_samples;
+				} else if (st->is_aac) {
+					st->audio_ts += 1024;
+				} else {
+					st->audio_ts ++;
+				}
+				goto restart;
 			}
 			else
 				st->seek_to_ts = 0;
@@ -562,6 +598,10 @@ GF_Err avidmx_process(GF_Filter *filter)
 			file_offset = gf_ftell(ctx->avi->fdes);
 			AVI_read_audio(ctx->avi, pck_data, size, (int*)&continuous);
 
+			if (ctx->file_size) {
+				pc = (u32) ( file_offset * 100  / ctx->file_size);
+			}
+
 			if (st->audio_bps) {
 				u32 nb_samples = (8*size) / (st->audio_bps * st->nb_channels);
 				gf_filter_pck_set_cts(dst_pck, st->audio_ts);
@@ -581,20 +621,40 @@ GF_Err avidmx_process(GF_Filter *filter)
 			st->aud_frame ++;
 		} else {
 			st->audio_done = GF_TRUE;
+			gf_filter_pid_set_eos(st->opid);
 			nb_done++;
+			pc = 100;
 		}
+		if (!a_pc || (pc<a_pc)) a_pc = pc;
+	}
+	if (gf_filter_reporting_enabled(filter)) {
+		char szStatus[1024];
+		u32 v_pc=0;
+		if (ctx->v_in_use) {
+			v_pc = ctx->cur_frame * 100;
+			v_pc /= ctx->nb_frames;
+		}
+
+		if (ctx->video_done && (nb_done==count) ) {
+			sprintf(szStatus, "100 %%");
+			v_pc=100;
+		}
+		else if (count && ctx->v_in_use) {
+			sprintf(szStatus, "V %d %% A %d %%", v_pc, a_pc);
+			if (v_pc>a_pc) v_pc=a_pc;
+		} else if (count) {
+			sprintf(szStatus, "A %d %%", a_pc);
+			v_pc = a_pc;
+		} else {
+			sprintf(szStatus, "V %d %%", v_pc);
+		}
+		gf_filter_update_status(filter, v_pc, szStatus);
+	}
+	if (ctx->video_done && (nb_done==count) ) {
+		return GF_EOS;
 	}
 	if (!nb_active)
 		return GF_OK;
-	if (video_done && (nb_done==count) ) {
-		if (ctx->v_opid && ctx->v_in_use) gf_filter_pid_set_eos(ctx->v_opid);
-
-		for (i=0; i<count;i++) {
-			AVIAstream *st = gf_list_get(ctx->audios, i);
-			gf_filter_pid_set_eos(st->opid);
-		}
-		return GF_EOS;
-	}
 	gf_filter_post_process_task(filter);
 	return GF_OK;
 }
@@ -626,6 +686,7 @@ void avidmx_finalize(GF_Filter *filter)
 static const char * avidmx_probe_data(const u8 *data, u32 size, GF_FilterProbeScore *score)
 {
 	if (size<12) return NULL;
+	*score = GF_FPROBE_NOT_SUPPORTED;
 	if (strncmp(data, "RIFF", 4)) return NULL;
 	if (strncmp(data+8, "AVI ", 4)) return NULL;
 	*score = GF_FPROBE_SUPPORTED;
@@ -647,6 +708,7 @@ static const GF_FilterArgs AVIDmxArgs[] =
 {
 	{ OFFS(fps), "import frame rate, default is AVI one", GF_PROP_FRACTION, "1/0", NULL, 0},
 	{ OFFS(importer), "compatibility with old importer, displays import results", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(noreframe), "skip media reframer", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{0}
 };
 
