@@ -31,6 +31,7 @@
 
 #include <gpac/mpegts.h>
 #include <gpac/thread.h>
+#include <gpac/internal/media_dev.h>
 
 typedef struct {
 	char *fragment;
@@ -59,7 +60,7 @@ typedef struct
 {
 	//opts
 	const char *temi_url;
-	Bool dsmcc, seeksrc, sigfrag;
+	Bool dsmcc, seeksrc, sigfrag, dvbtxt;
 
 	GF_Filter *filter;
 	GF_FilterPid *ipid;
@@ -72,7 +73,7 @@ typedef struct
 	u64 file_size;
 	Bool in_seek;
 	Bool initial_play_done;
-	u32 nb_playing;
+	u32 nb_playing, nb_stop_pending;
 
 	//duration estimation
 	GF_Fraction64 duration;
@@ -203,13 +204,13 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 	if (stream->flags & GF_M2TS_GPAC_CODEC_ID) {
 		codecid = stream->stream_type;
 		stype = gf_codecid_type(codecid);
-		unframed = GF_TRUE;
+		if (stream->gpac_meta_dsi)
+			stype = stream->gpac_meta_dsi[4];
 		if (!stype) {
 			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSDmx] Unrecognized gpac codec %s - ignoring pid\n", gf_4cc_to_str(codecid) ));
 			return;
 		}
 	} else {
-
 		switch (stream->stream_type) {
 		case GF_M2TS_VIDEO_MPEG1:
 			stype = GF_STREAM_VISUAL;
@@ -272,6 +273,12 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 		case GF_M2TS_VIDEO_VC1:
 			stype = GF_STREAM_VISUAL;
 			codecid = GF_CODECID_SMPTE_VC1;
+			stream->flags |= GF_M2TS_CHECK_VC1;
+			break;
+		case GF_M2TS_VIDEO_AV1:
+			stype = GF_STREAM_VISUAL;
+			codecid = GF_CODECID_AV1;
+			unframed = GF_TRUE;
 			break;
 		case GF_M2TS_AUDIO_MPEG1:
 			stype = GF_STREAM_AUDIO;
@@ -312,6 +319,12 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 		case GF_M2TS_AUDIO_EC3:
 			stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_EAC3;
+			unframed = GF_TRUE;
+			break;
+		case GF_M2TS_AUDIO_TRUEHD:
+			stype = GF_STREAM_AUDIO;
+			codecid = GF_CODECID_TRUEHD;
+			unframed = GF_TRUE;
 			break;
 		case GF_M2TS_AUDIO_DTS:
 			stype = GF_STREAM_AUDIO;
@@ -327,7 +340,7 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 		case GF_M2TS_SYSTEMS_MPEG4_PES:
 			if (!esd) {
 				m4sys_iod_stream = GF_TRUE;
-				count = gf_list_count(stream->program->pmt_iod->ESDescriptors);
+				count = stream->program->pmt_iod ? gf_list_count(stream->program->pmt_iod->ESDescriptors) : 0;
 				for (i=0; i<count; i++) {
 					esd = gf_list_get(stream->program->pmt_iod->ESDescriptors, i);
 					if (esd->ESID == stream->mpeg4_es_id) break;
@@ -365,6 +378,20 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 			orig_stype = GF_STREAM_AUDIO;
 			codecid = GF_CODECID_EAC3;
 			break;
+		case GF_M2TS_DVB_SUBTITLE:
+			stype = GF_STREAM_TEXT;
+			codecid = GF_CODECID_DVB_SUBS;
+			stream->flags |= GF_M2TS_ES_FULL_AU;
+			break;
+		case GF_M2TS_DVB_TELETEXT:
+			if (!ctx->dvbtxt) {
+				GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSDmx] DVB teletext pid skipped, use --dvbtxt to enable\n", stream->stream_type));
+				return;
+			}
+			stype = GF_STREAM_TEXT;
+			codecid = GF_CODECID_DVB_TELETEXT;
+			stream->flags |= GF_M2TS_ES_FULL_AU;
+			break;
 		default:
 			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("[M2TSDmx] Stream type 0x%02X not supported - ignoring pid\n", stream->stream_type));
 			return;
@@ -386,6 +413,17 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 	stream->user = opid;
 	stream->flags |= GF_M2TS_ES_ALREADY_DECLARED;
 
+	u32 d_type = orig_stype ? orig_stype : stype;
+	switch (d_type) {
+	case GF_STREAM_AUDIO:
+	case GF_STREAM_VISUAL:
+		stream->flags |= GF_M2TS_CHECK_DISC;
+		break;
+	default:
+		stream->flags &= ~GF_M2TS_CHECK_DISC;
+		break;
+	}
+
 	stname = gf_stream_type_name(stype);
 	sprintf(szName, "P%d%c%d", stream->program->number, stname[0], 1+gf_list_find(stream->program->streams, stream));
 	gf_filter_pid_set_name(opid, szName);
@@ -428,12 +466,47 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 
 		gf_filter_pid_set_property(opid, GF_PROP_PID_TIMESCALE, &PROP_UINT(90000) );
 		gf_filter_pid_set_property(opid, GF_PROP_PID_CLOCK_ID, &PROP_UINT(stream->program->pcr_pid) );
+
+		if ((stream->flags&GF_M2TS_ES_IS_PES) && stream->gpac_meta_dsi) {
+			char *cname;
+			GF_BitStream *bs = gf_bs_new(stream->gpac_meta_dsi, stream->gpac_meta_dsi_size, GF_BITSTREAM_READ);
+			u32 val = gf_bs_read_u32(bs); //codec ID (meta codec identifier)
+			gf_filter_pid_set_property(opid, GF_PROP_PID_CODECID, &PROP_UINT(val) );
+			gf_bs_read_u8(bs); //stream type
+			gf_bs_read_u8(bs); //version
+			val = gf_bs_read_u32(bs); //codecID for meta codec, e.g. an AVCodecID or other
+			gf_filter_pid_set_property(opid, GF_PROP_PID_META_DEMUX_CODEC_ID, &PROP_UINT(val) );
+			cname = gf_bs_read_utf8(bs); //meta codec name
+			gf_filter_pid_set_property(opid, GF_PROP_PID_META_DEMUX_CODEC_NAME, cname ? &PROP_STRING_NO_COPY(cname) : NULL );
+			val = gf_bs_read_u32(bs); //meta opaque
+			gf_filter_pid_set_property(opid, GF_PROP_PID_META_DEMUX_OPAQUE, &PROP_UINT(val) );
+			u32 dsi_len = gf_bs_read_u32(bs);
+			if (dsi_len) {
+				u32 pos = (u32) gf_bs_get_position(bs);
+				gf_filter_pid_set_property(opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA(stream->gpac_meta_dsi+pos, dsi_len) );
+				gf_bs_skip_bytes(bs, dsi_len);
+			} else {
+				gf_filter_pid_set_property(opid, GF_PROP_PID_DECODER_CONFIG, NULL);
+			}
+			if (stype==GF_STREAM_VISUAL) {
+				val = gf_bs_read_u32(bs);
+				gf_filter_pid_set_property(opid, GF_PROP_PID_WIDTH, &PROP_UINT(val) );
+				val = gf_bs_read_u32(bs);
+				gf_filter_pid_set_property(opid, GF_PROP_PID_HEIGHT, &PROP_UINT(val) );
+			} else if (stype==GF_STREAM_AUDIO) {
+				val = gf_bs_read_u32(bs);
+				gf_filter_pid_set_property(opid, GF_PROP_PID_SAMPLE_RATE, &PROP_UINT(val) );
+				val = gf_bs_read_u32(bs);
+				gf_filter_pid_set_property(opid, GF_PROP_PID_NUM_CHANNELS, &PROP_UINT(val) );
+			}
+			gf_bs_del(bs);
+		}
 	}
 	gf_filter_pid_set_property(opid, GF_PROP_PID_SCALABLE, has_scal_layer ? &PROP_BOOL(GF_TRUE) : NULL);
 
 	gf_filter_pid_set_property(opid, GF_PROP_PID_SERVICE_ID, &PROP_UINT(stream->program->number) );
 
-	if (stream->lang) {
+	if ((stream->flags&GF_M2TS_ES_IS_PES) && stream->lang) {
 		char szLang[4];
 		szLang[0] = (stream->lang>>16) & 0xFF;
 		szLang[1] = (stream->lang>>8) & 0xFF;
@@ -442,7 +515,20 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 		if (szLang[2]==' ') szLang[2] = 0;
 		gf_filter_pid_set_property(opid, GF_PROP_PID_LANGUAGE, &PROP_STRING(szLang) );
 	}
+	if (codecid == GF_CODECID_DVB_SUBS) {
+		char szLang[4];
+		memcpy(szLang, stream->sub.language, 3);
+		szLang[3]=0;
+		gf_filter_pid_set_property(opid, GF_PROP_PID_LANGUAGE, &PROP_STRING(szLang) );
 
+		u8 dsi[5];
+		dsi[0] = stream->sub.composition_page_id>>8;
+		dsi[1] = stream->sub.composition_page_id & 0xFF;
+		dsi[2] = stream->sub.ancillary_page_id>>8;
+		dsi[3] = stream->sub.ancillary_page_id & 0xFF;
+		dsi[4] = stream->sub.type;
+		gf_filter_pid_set_property(opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA(dsi, 5));
+	}
 
 	if (ctx->duration.num>1) {
 		gf_filter_pid_set_property(opid, GF_PROP_PID_DURATION, &PROP_FRAC64(ctx->duration) );
@@ -450,7 +536,7 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 	}
 	/*indicate our coding dependencies if any*/
 	if (!m4sys_stream) {
-		if (stream->depends_on_pid) {
+		if ((stream->flags&GF_M2TS_ES_IS_PES) && stream->depends_on_pid) {
 			gf_filter_pid_set_property(opid, GF_PROP_PID_DEPENDENCY_ID, &PROP_UINT(stream->depends_on_pid) );
 			if ((stream->stream_type == GF_M2TS_VIDEO_HEVC_TEMPORAL) || (stream->stream_type == GF_M2TS_VIDEO_HEVC_MCTS)) {
 				gf_filter_pid_set_property(opid, GF_PROP_PID_SUBLAYER, &PROP_BOOL(GF_TRUE) );
@@ -461,7 +547,7 @@ static void m2tsdmx_declare_pid(GF_M2TSDmxCtx *ctx, GF_M2TS_PES *stream, GF_ESD
 		}
 	}
 
-	if (stream->dv_info[0]) {
+	if ((stream->flags & GF_M2TS_ES_IS_PES) && (stream->dv_info[0])) {
 		gf_filter_pid_set_property(opid, GF_PROP_PID_DOLBY_VISION, &PROP_DATA(stream->dv_info, 24) );
 		u32 dvtype=0;
 		if (stream->dv_info[24]) {
@@ -540,25 +626,93 @@ static void m2tsdmx_send_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_PES_PCK *pck)
 	GF_FilterPid *opid;
 	GF_FilterPacket *dst_pck;
 	u8 * data;
+	//we don't have end of frame signaling by default
+	Bool au_end = GF_FALSE;
+	GF_FilterSAPType sap_type = GF_FILTER_SAP_NONE;
 
 	/*pcr not initialized, don't send any data*/
 //	if (! pck->stream->program->first_dts) return;
 	if (!pck->stream->user) return;
 	opid = pck->stream->user;
 
-	dst_pck = gf_filter_pck_new_alloc(opid, pck->data_len, &data);
-	if (!dst_pck) return;
+	u8 *ptr = pck->data;
+	u32 len = pck->data_len;
 
-	memcpy(data, pck->data, pck->data_len);
-	//we don't have end of frame signaling
-	gf_filter_pck_set_framing(dst_pck, (pck->flags & GF_M2TS_PES_PCK_AU_START) ? GF_TRUE : GF_FALSE, GF_FALSE);
+	//skip dataID and stream ID
+	if (pck->stream->stream_type==GF_M2TS_DVB_SUBTITLE) {
+		ptr+=2;
+		len-=2;
+	}
+	//for now GF_M2TS_ES_FULL_AU is only used for text, all rap
+	if (pck->stream->flags & GF_M2TS_ES_FULL_AU) {
+		au_end = GF_TRUE;
+		sap_type = GF_FILTER_SAP_1;
+	}
+
+	if (pck->stream->flags & GF_M2TS_CHECK_VC1) {
+		//extract seq header
+		u32 start, next, sc_size, sc_size2, sc_size3, hdr_len=0;
+
+		start = next = gf_media_nalu_next_start_code(ptr, len, &sc_size);
+		if ((next<len) && (ptr[next+sc_size]==0x0F)) {
+			u32 ephdr = gf_media_nalu_next_start_code (ptr+next+sc_size, len-next-sc_size, &sc_size2);
+			if ((ephdr + next + sc_size < len) && (ptr[next+sc_size+ephdr+sc_size2]==0x0E)) {
+				u32 end = gf_media_nalu_next_start_code (ptr+next+sc_size+ephdr+sc_size2, len-next-sc_size-ephdr-sc_size2, &sc_size3);
+				if (end + ephdr + next + sc_size + sc_size2 < len)
+					hdr_len = end + ephdr + sc_size2 + next + sc_size;
+			} else if ((ephdr + next + sc_size < len) && (ptr[next+sc_size+ephdr+sc_size2]==0x0D)) {
+				hdr_len = ephdr + next + sc_size;
+			}
+		}
+		if (hdr_len) {
+			u8 *dsi=NULL;
+			u32 dsi_len;
+			ptr += start;
+			len -= start;
+			gf_media_vc1_seq_header_to_dsi(ptr, len, &dsi, &dsi_len);
+			if (dsi)
+				gf_filter_pid_set_property(opid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_len));
+
+			ptr += hdr_len;
+			len -= hdr_len;
+			pck->stream->flags &= ~GF_M2TS_CHECK_VC1;
+		}
+	}
+
+
+	dst_pck = gf_filter_pck_new_alloc(opid, len, &data);
+	if (!dst_pck) return;
+	memcpy(data, ptr, len);
+
+	gf_filter_pck_set_framing(dst_pck, (pck->flags & GF_M2TS_PES_PCK_AU_START) ? GF_TRUE : GF_FALSE, au_end);
 
 	if (pck->flags & GF_M2TS_PES_PCK_AU_START) {
+		if (pck->flags & GF_M2TS_PES_PCK_RAP)
+			sap_type = GF_FILTER_SAP_1;
+
 		gf_filter_pck_set_cts(dst_pck, pck->PTS);
 		if (pck->DTS != pck->PTS) {
 			gf_filter_pck_set_dts(dst_pck, pck->DTS);
 		}
-		gf_filter_pck_set_sap(dst_pck, (pck->flags & GF_M2TS_PES_PCK_RAP) ? GF_FILTER_SAP_1 : GF_FILTER_SAP_NONE);
+		gf_filter_pck_set_sap(dst_pck, sap_type);
+
+		if (pck->stream->flags & GF_M2TS_ES_IS_PES) {
+			GF_M2TS_PES *pes = (GF_M2TS_PES *)pck->stream;
+			if (pes->map_utc) {
+				s64 diff = pck->PTS;
+				diff -= pes->map_utc_pcr;
+				diff = gf_timestamp_rescale_signed(diff, 90000, 1000);
+				gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_UTC_TIME, &PROP_LONGUINT(pes->map_utc+diff) );
+				pes->map_utc=0;
+			}
+			if (pes->map_pcr) {
+				Double diff = (Double) pck->PTS;
+				diff -= pes->map_pcr;
+				diff /= 90000;
+				gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_MEDIA_TIME, &PROP_DOUBLE(ctx->media_start_range+diff) );
+				pes->map_pcr=0;
+			}
+		}
 	}
 	m2tdmx_merge_temi(opid, (GF_M2TS_ES *)pck->stream, dst_pck);
 
@@ -566,7 +720,9 @@ static void m2tsdmx_send_packet(GF_M2TSDmxCtx *ctx, GF_M2TS_PES_PCK *pck)
 		pck->stream->is_seg_start = GF_FALSE;
 		gf_filter_pck_set_property(dst_pck, GF_PROP_PCK_CUE_START, &PROP_BOOL(GF_TRUE));
 	}
+
 	gf_filter_pck_send(dst_pck);
+	ctx->nb_stop_pending = 0;
 }
 
 static GF_M2TS_ES *m2tsdmx_get_m4sys_stream(GF_M2TSDmxCtx *ctx, u32 m4sys_es_id)
@@ -791,9 +947,8 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 			}
 			gf_filter_pck_send(dst_pck);
 
-			if (map_time) {
-				gf_filter_pid_set_info_str(stream->user, "time:timestamp", &PROP_LONGUINT(pcr) );
-				gf_filter_pid_set_info_str(stream->user, "time:media", &PROP_DOUBLE(ctx->media_start_range) );
+			if (map_time && (stream->flags & GF_M2TS_ES_IS_PES) ) {
+				((GF_M2TS_PES*)stream)->map_pcr = pcr;
 			}
 		}
 
@@ -814,13 +969,14 @@ static void m2tsdmx_on_event(GF_M2TS_Demuxer *ts, u32 evt_type, void *param)
 			u32 j, count2 = gf_list_count(prog->streams);
 			for (j=0; j<count2; j++) {
 				GF_M2TS_ES * stream = gf_list_get(prog->streams, j);
-				if (stream->user) {
-					gf_filter_pid_set_info(stream->user, GF_PROP_PID_UTC_TIME, & PROP_LONGUINT(utc_ts) );
-					gf_filter_pid_set_info(stream->user, GF_PROP_PID_UTC_TIMESTAMP, & PROP_LONGUINT(prog->last_pcr_value / 300) );
+				if (stream->user && (stream->flags & GF_M2TS_ES_IS_PES)) {
+					GF_M2TS_PES*pes = (GF_M2TS_PES*)stream;
+					pes->map_utc = utc_ts;
+					pes->map_utc_pcr = prog->last_pcr_value/300;
 				}
 			}
-			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[M2TS In] Mapping TDT Time %04d/%02d/%02d %02d:%02d:%02d and PCR time "LLD" on program %d\n",
-				                                       tdt->year, tdt->month, tdt->day, tdt->hour, tdt->minute, tdt->second, prog->last_pcr_value/300, prog->number));
+			GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[M2TS In] Mapping TDT Time %04d-%02d-%02dT%02d:%02d:%02d and PCR time "LLD" on program %d\n",
+				                                       tdt->year, tdt->month+1, tdt->day, tdt->hour, tdt->minute, tdt->second, prog->last_pcr_value/300, prog->number));
 		}
 	}
 		break;
@@ -952,17 +1108,10 @@ static GF_Err m2tsdmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool i
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-		u32 i, count = gf_list_count(ctx->ts->programs);
+		u32 i, count = gf_filter_get_opid_count(filter);
 		for (i=0; i<count; i++) {
-			GF_M2TS_Program *prog = gf_list_get(ctx->ts->programs, i);
-			u32 j, count2 = gf_list_count(prog->streams);
-			for (j=0; j<count2; j++) {
-				GF_M2TS_ES *es = gf_list_get(prog->streams, j);
-				if (es->user) {
-					GF_FilterPid *opid = es->user;
-					gf_filter_pid_remove(opid);
-				}
-			}
+			GF_FilterPid *opid = gf_filter_get_opid(filter, i);
+			gf_filter_pid_remove( opid);
 		}
 		return GF_OK;
 	}
@@ -1089,6 +1238,8 @@ static Bool m2tsdmx_process_event(GF_Filter *filter, const GF_FilterEvent *com)
 	if (!com->base.on_pid) return GF_FALSE;
 	switch (com->base.type) {
 	case GF_FEVT_PLAY:
+		if (com->play.initial_broadcast_play==2)
+			return GF_TRUE;
 		pes = m2tsdmx_get_stream(ctx, com->base.on_pid);
 		if (!pes) {
 			if (com->base.on_pid == ctx->eit_pid) {
@@ -1105,8 +1256,20 @@ static Bool m2tsdmx_process_event(GF_Filter *filter, const GF_FilterEvent *com)
 
 		/*this is a multiplex, only trigger the play command for the first activated stream*/
 		ctx->nb_playing++;
-		if (ctx->nb_playing>1) return GF_TRUE;
 
+		if (ctx->nb_playing>1) {
+			Bool skip_com = GF_TRUE;
+			//PLAY/STOP may arrive at different times depending on the length of filter chains on each PID
+			//we stack number of STOP received and trigger seek when we have the same amount of play
+			if (ctx->nb_stop_pending==ctx->nb_playing) {
+				skip_com = GF_FALSE;
+			}
+			if (skip_com) {
+				return GF_TRUE;
+			}
+		}
+
+		ctx->nb_stop_pending = 0;
 		//not file, don't cancel the event
 		if (!ctx->is_file) {
 			ctx->initial_play_done = GF_TRUE;
@@ -1127,15 +1290,13 @@ static Bool m2tsdmx_process_event(GF_Filter *filter, const GF_FilterEvent *com)
 		if (!ctx->initial_play_done) {
 			ctx->initial_play_done = GF_TRUE;
 			//seek will not change the current source state, don't send a seek
-			if (!file_pos)
-				return GF_TRUE;
+			if (!file_pos) return GF_TRUE;
 		}
 
 		//file and seek, cancel the event and post a seek event to source
 		ctx->in_seek = GF_TRUE;
 		//we seek so consider the mux tuned in
 		ctx->mux_tune_state = DMX_TUNE_DONE;
-
 		//post a seek
 		GF_FEVT_INIT(fevt, GF_FEVT_SOURCE_SEEK, ctx->ipid);
 		fevt.seek.start_offset = file_pos;
@@ -1150,11 +1311,14 @@ static Bool m2tsdmx_process_event(GF_Filter *filter, const GF_FilterEvent *com)
 			}
 			return GF_FALSE;
 		}
-		/* In case of EOS, we may receive a stop command after no one is playing */
-		if (ctx->nb_playing)
-			ctx->nb_playing--;
-
 		gf_m2ts_set_pes_framing(pes, GF_M2TS_PES_FRAMING_SKIP);
+
+		if (com->play.initial_broadcast_play==2)
+			return GF_TRUE;
+
+		ctx->nb_stop_pending++;
+		if (ctx->nb_playing) ctx->nb_playing--;
+
 		//don't cancel event if still playing
 		return ctx->nb_playing ? GF_TRUE : GF_FALSE;
 
@@ -1229,17 +1393,24 @@ restart:
 	if (ctx->in_seek) {
 		gf_m2ts_reset_parsers(ctx->ts);
 		ctx->in_seek = GF_FALSE;
-	} else if (check_block) {
+	} else if (check_block && !ctx->wait_for_progs) {
 		u32 i, nb_streams, would_block = 0;
 		nb_streams = gf_filter_get_opid_count(filter);
 		for (i=0; i<nb_streams; i++) {
 			GF_FilterPid *opid = gf_filter_get_opid(filter, i);
-			if ( gf_filter_pid_would_block(opid) ) {
+			if (!gf_filter_pid_is_playing(opid)) {
+				would_block++;
+			} else if ( gf_filter_pid_would_block(opid) ) {
 				would_block++;
 			}
 		}
-		if (would_block && (would_block==nb_streams))
+		if (would_block && (would_block==nb_streams)) {
+			//keep filter alive
+			if (ctx->nb_playing) {
+				gf_filter_ask_rt_reschedule(filter, 0);
+			}
 			return GF_OK;
+		}
 
 		check_block = GF_FALSE;
 	}
@@ -1279,6 +1450,8 @@ static const GF_FilterCapability M2TSDmxCaps[] =
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_SCENE),
+	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_TEXT),
+	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_METADATA),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_OD),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_PRIVATE_SCENE),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_ENCRYPTED),
@@ -1292,6 +1465,7 @@ static const GF_FilterArgs M2TSDmxArgs[] =
 	{ OFFS(dsmcc), "enable DSMCC receiver", GF_PROP_BOOL, "no", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(seeksrc), "seek local source file back to origin once all programs are setup", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(sigfrag), "signal segment boundaries on output packets for DASH or HLS sources", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(dvbtxt), "export DVB teletext streams", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{0}
 };
 
