@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2021
+ *			Copyright (c) Telecom ParisTech 2017-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / common ffmpeg filters
@@ -56,9 +56,11 @@
 #define FF_IFMT_CAST
 #endif
 
-
-//rendering/translating to internal supported formats for text streams is not yet implemented
-//#define FF_SUB_SUPPORT
+//consider stable bsf api after major version 59
+#if (LIBAVCODEC_VERSION_MAJOR > 58)
+#define FFMPEG_HAS_BSF
+#include <libavcodec/bsf.h>
+#endif
 
 
 GF_FilterArgs ffmpeg_arg_translate(const struct AVOption *opt);
@@ -71,12 +73,15 @@ enum{
 	FF_REG_TYPE_ENCODE,
 	FF_REG_TYPE_MUX,
 	FF_REG_TYPE_AVF,
+#ifdef FFMPEG_HAS_BSF
+	FF_REG_TYPE_BSF,
+#endif
 };
 
-void ffmpeg_build_register(GF_FilterSession *session, GF_FilterRegister *orig_reg, const GF_FilterArgs *default_args, u32 nb_def_args, u32 reg_type);
+GF_FilterRegister *ffmpeg_build_register(GF_FilterSession *session, GF_FilterRegister *orig_reg, const GF_FilterArgs *default_args, u32 nb_def_args, u32 reg_type);
 
-u32 ffmpeg_pixfmt_from_gpac(u32 pfmt);
-u32 ffmpeg_pixfmt_to_gpac(u32 pfmt, Bool no_warn);
+enum AVPixelFormat ffmpeg_pixfmt_from_gpac(u32 pfmt, Bool no_warn);
+u32 ffmpeg_pixfmt_to_gpac(enum AVPixelFormat pfmt, Bool no_warn);
 //return GPAC pix format from codec tag
 u32 ffmpeg_pixfmt_from_codec_tag(u32 codec_tag, Bool *is_full_range);
 //check if FF pixfmt is an old format fullrange
@@ -98,3 +103,29 @@ u64 ffmpeg_channel_layout_to_gpac(u64 ff_ch_layout);
 
 //this will destroy unknown_options if set
 void ffmpeg_report_options(GF_Filter *filter, AVDictionary *unknown_options, AVDictionary *all_options);
+
+void ffmpeg_register_set_dyn_help(GF_FilterRegister *reg);
+
+//output allocated with av_malloc
+GF_Err ffmpeg_extradata_from_gpac(u32 gpac_codec_id, const u8 *dsi_in, u32 dsi_in_size, u8 **dsi_out, u32 *dsi_out_size);
+//output allocated with gf_malloc
+GF_Err ffmpeg_extradata_to_gpac(u32 gpac_codec_id, const u8 *data, u32 size, u8 **dsi_out, u32 *dsi_out_size);
+
+void ffmpeg_tags_from_gpac(GF_FilterPid *pid, AVDictionary **metadata);
+void ffmpeg_tags_to_gpac(AVDictionary *metadata, GF_FilterPid *pid);
+
+void ffmpeg_generate_gpac_dsi(GF_FilterPid *out_pid, u32 gpac_codec_id, u32 color_primaries, u32 transfer_characteristics, u32 colorspace, const u8 *data, u32 size);
+
+#if (LIBAVCODEC_VERSION_MAJOR > 56)
+/*fills codecpar from pid properties. This assumes the codecpar was properly initialized.
+codec_id, codec_type, and extradata are NOT set by this function
+*/
+GF_Err ffmpeg_codec_par_from_gpac(GF_FilterPid *pid, AVCodecParameters *codecpar, u32 ffmpeg_timescale);
+
+/*sets pid properties from codecpar:
+codec_id, codec_type, and extradata are NOT set by this function
+*/
+GF_Err ffmpeg_codec_par_to_gpac(AVCodecParameters *codecpar, GF_FilterPid *opid, u32 ffmpeg_timescale);
+#endif
+
+GF_Err ffmpeg_update_arg(const char *log_name, void *ctx, AVDictionary **options, const char *arg_name, const GF_PropertyValue *arg_val);
