@@ -26,6 +26,7 @@
 #include <gpac/setup.h>
 #include <gpac/bitstream.h>
 #include <gpac/avparse.h>
+#include <gpac/internal/media_dev.h>
 
 #ifdef GPAC_HAS_FFMPEG
 
@@ -38,13 +39,13 @@ typedef struct _gf_ffenc_ctx
 	//opts
 	Bool all_intra;
 	char *c;
-	Bool ls;
+	Bool ls, rld;
 	u32 pfmt;
 	GF_Fraction fintra;
 	Bool rc;
 
 	//internal data
-	Bool initialized;
+	Bool gen_dsi;
 
 	u32 gop_size;
 	u32 target_rate;
@@ -85,7 +86,8 @@ typedef struct _gf_ffenc_ctx
 	u64 cts_first_frame_plus_one;
 
 	//audio state
-	u32 channels, sample_rate, channel_layout, bytes_per_sample;
+	u32 channels, sample_rate, bytes_per_sample;
+	u64 channel_layout;
 	//ffmpeg one
 	u32 sample_fmt;
 	//we store input audio frame in this buffer until we have enough data for one encoder frame
@@ -109,6 +111,7 @@ typedef struct _gf_ffenc_ctx
 	Bool infmt_negociate;
 	Bool remap_ts;
 	Bool force_reconfig;
+	u32 setup_failed;
 
 	u32 dsi_crc;
 
@@ -133,7 +136,9 @@ typedef struct _gf_ffenc_ctx
 	AVPacket *pkt;
 #endif
 
-	u32 premul_timescale;
+	Bool args_updated;
+
+	FILE *logfile_pass1;
 } GF_FFEncodeCtx;
 
 static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove, Bool is_force_reconf);
@@ -168,7 +173,6 @@ static GF_Err ffenc_initialize(GF_Filter *filter)
 {
 	u32 codec_id;
 	GF_FFEncodeCtx *ctx = (GF_FFEncodeCtx *) gf_filter_get_udta(filter);
-	ctx->initialized = GF_TRUE;
 	ctx->src_packets = gf_list_new();
 	ctx->sdbs = gf_bs_new((u8*)ctx, 1, GF_BITSTREAM_READ);
 
@@ -193,6 +197,13 @@ static GF_Err ffenc_initialize(GF_Filter *filter)
 	}
 	if (!ctx->force_codec) return GF_NOT_SUPPORTED;
 	ffenc_override_caps(filter, ctx->force_codec->type);
+
+
+	if (gf_filter_is_temporary(filter)) {
+		gf_filter_meta_set_instances(filter, ctx->force_codec->name);
+		return GF_OK;
+	}
+
 	return GF_OK;
 }
 
@@ -214,7 +225,14 @@ static void ffenc_finalize(GF_Filter *filter)
 	av_packet_free(&ctx->pkt);
 #endif
 
+	if (ctx->logfile_pass1) {
+		if (ctx->encoder && ctx->encoder->stats_out) fprintf(ctx->logfile_pass1, "%s", ctx->encoder->stats_out);
+		gf_fclose(ctx->logfile_pass1);
+	}
+
 	if (ctx->encoder) {
+		if (ctx->encoder->stats_in)
+			gf_free(ctx->encoder->stats_in);
 		avcodec_free_context(&ctx->encoder);
 	}
 	if (ctx->sdbs) gf_bs_del(ctx->sdbs);
@@ -229,11 +247,11 @@ static void ffenc_copy_pid_props(GF_FFEncodeCtx *ctx)
 	if (!ctx->codecid) {
 		gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_CODECID, &PROP_UINT(GF_CODECID_FFMPEG) );
 		if (ctx->encoder) {
-			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_FFMPEG_CODEC_ID, &PROP_UINT(ctx->encoder->codec->id) );
+			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_META_DEMUX_CODEC_ID, &PROP_UINT(ctx->encoder->codec->id) );
 
 			const char *cname = avcodec_get_name(ctx->encoder->codec->id);
 			if (cname)
-				gf_filter_pid_set_property_str(ctx->out_pid, "ffmpeg:codec", &PROP_STRING(cname ) );
+				gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_META_DEMUX_CODEC_NAME, &PROP_STRING(cname ) );
 		}
 	} else {
 		gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_CODECID, &PROP_UINT(ctx->codecid) );
@@ -242,16 +260,45 @@ static void ffenc_copy_pid_props(GF_FFEncodeCtx *ctx)
 	gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_ISOM_SUBTYPE, NULL);
 	gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_PROFILE_LEVEL, NULL);
 
+	ctx->gen_dsi = GF_FALSE;
 	switch (ctx->codecid) {
+	//reframe all these codecs for proper ISOBMFF+DSI formating
 	case GF_CODECID_AVC:
 	case GF_CODECID_HEVC:
-	case GF_CODECID_MPEG4_PART2:
+	case GF_CODECID_VVC:
+	case GF_CODECID_AV1:
 		gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_UNFRAMED, &PROP_BOOL(GF_TRUE) );
 		gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_UNFRAMED_FULL_AU, &PROP_BOOL(GF_TRUE) );
 		break;
+
+	//for these, we will need to generate dsi from first frame - this avoids using reframers only for DSI extraction
+	case GF_CODECID_MPEG1:
+	case GF_CODECID_MPEG2_422:
+	case GF_CODECID_MPEG2_SNR:
+	case GF_CODECID_MPEG2_HIGH:
+	case GF_CODECID_MPEG2_MAIN:
+	case GF_CODECID_MPEG2_SIMPLE:
+	case GF_CODECID_MPEG2_SPATIAL:
+	case GF_CODECID_MPEG4_PART2:
+	case GF_CODECID_VP8:
+	case GF_CODECID_VP9:
+	case GF_CODECID_VP10:
+	case GF_CODECID_AC3:
+	case GF_CODECID_EAC3:
+	case GF_CODECID_TRUEHD:
+		ctx->gen_dsi = GF_TRUE;
+		break;
+
+	case GF_CODECID_FLAC:
+	case GF_CODECID_OPUS:
+	case GF_CODECID_VORBIS:
 	default:
-		if (ctx->encoder && ctx->encoder->extradata_size && ctx->encoder->extradata) {
-			gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA(ctx->encoder->extradata, ctx->encoder->extradata_size) );
+		if (ctx->encoder && ctx->encoder->extradata) {
+			u8 *dsi;
+			u32 dsi_size;
+			GF_Err e = ffmpeg_extradata_to_gpac(ctx->codecid, ctx->encoder->extradata, ctx->encoder->extradata_size, &dsi, &dsi_size);
+			if (!e)
+				gf_filter_pid_set_property(ctx->out_pid, GF_PROP_PID_DECODER_CONFIG, &PROP_DATA_NO_COPY(dsi, dsi_size) );
 		}
 		break;
 	}
@@ -271,6 +318,8 @@ static void ffenc_copy_pid_props(GF_FFEncodeCtx *ctx)
 	}
 }
 
+
+
 static u64 ffenc_get_cts(GF_FFEncodeCtx *ctx, GF_FilterPacket *pck)
 {
 	u64 cts = gf_filter_pck_get_cts(pck);
@@ -372,11 +421,21 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	pck = gf_filter_pid_get_packet(ctx->in_pid);
 
 	if (!ctx->encoder) {
-		if (ctx->infmt_negociate) return GF_OK;
+		//no encoder: if negociating input format or input pid props not known yet, wait
+		if (ctx->infmt_negociate || !ctx->setup_failed) return GF_OK;
 
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] encoder reconfiguration failed, aborting stream\n"));
-		gf_filter_pid_set_eos(ctx->out_pid);
-		return GF_EOS;
+		if (ctx->setup_failed==1) {
+			GF_FilterEvent fevt;
+
+			ctx->setup_failed = 2;
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] encoder reconfiguration failed, aborting stream\n"));
+			gf_filter_pid_set_eos(ctx->out_pid);
+
+			gf_filter_pid_set_discard(ctx->in_pid, GF_TRUE);
+			GF_FEVT_INIT(fevt, GF_FEVT_STOP, ctx->in_pid);
+			gf_filter_pid_send_event(ctx->in_pid, &fevt);
+		}
+		return GF_SERVICE_ERROR;
 	}
 
 	if (!pck) {
@@ -439,6 +498,10 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	}
 
 	if (force_intra) {
+		if (ctx->args_updated) {
+			force_intra = 2;
+			ctx->args_updated = GF_FALSE;
+		}
 		//file switch we force a full reset to force injecting xPS in the stream
 		//we could also inject them manually but we don't have them !!
 		if ((ctx->rc && ctx->nb_frames_in) || (force_intra==2)) {
@@ -503,9 +566,9 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		ctx->frame->pkt_duration = gf_filter_pck_get_duration(pck);
 
 //use signed version of timestamp rescale since we may have negative dts
-#define SCALE_TS(_ts) if (_ts != GF_FILTER_NO_TS) { _ts = gf_timestamp_rescale_signed(_ts, ctx->premul_timescale, ctx->encoder->time_base.den); }
-#define UNSCALE_TS(_ts) if (_ts != AV_NOPTS_VALUE)  { _ts = gf_timestamp_rescale_signed(_ts, ctx->encoder->time_base.den, ctx->premul_timescale); }
-#define UNSCALE_DUR(_ts) { _ts = gf_timestamp_rescale(_ts, ctx->encoder->time_base.den, ctx->premul_timescale); }
+#define SCALE_TS(_ts) if (_ts != GF_FILTER_NO_TS) { _ts = gf_timestamp_rescale_signed(_ts, ctx->timescale, ctx->encoder->time_base.den); }
+#define UNSCALE_TS(_ts) if (_ts != AV_NOPTS_VALUE)  { _ts = gf_timestamp_rescale_signed(_ts, ctx->encoder->time_base.den, ctx->timescale); }
+#define UNSCALE_DUR(_ts) { _ts = gf_timestamp_rescale(_ts, ctx->encoder->time_base.den, ctx->timescale); }
 
 		if (ctx->remap_ts) {
 			SCALE_TS(ctx->frame->pts);
@@ -600,6 +663,7 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 				GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[FFEnc] codec flush done, triggering reconfiguration\n"));
 				avcodec_close(ctx->encoder);
 				ctx->encoder = NULL;
+				ctx->setup_failed = 0;
 				e = ffenc_configure_pid_ex(filter, ctx->in_pid, GF_FALSE, GF_TRUE);
 				ctx->init_cts_setup = bck_init_cts;
 				return e;
@@ -751,6 +815,10 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 			gf_filter_pck_set_duration(dst_pck, (u32) ctx->frame->pkt_duration);
 		}
 	}
+	if (ctx->gen_dsi) {
+		ffmpeg_generate_gpac_dsi(ctx->out_pid, ctx->codecid, ctx->encoder->color_primaries, ctx->encoder->color_trc, ctx->encoder->colorspace, output, size);
+		ctx->gen_dsi = GF_FALSE;
+	}
 
 	ffenc_log_video(filter, ctx, pkt, gf_filter_reporting_enabled(filter));
 
@@ -784,12 +852,10 @@ static GF_Err ffenc_process_video(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	return GF_OK;
 }
 
-
-
 static void ffenc_audio_append_samples(struct _gf_ffenc_ctx *ctx, const u8 *data, u32 size, u32 sample_offset, u32 nb_samples)
 {
 	u8 *dst;
-	u32 f_idx, s_idx;
+	u32 f_idx, s_idx, offset, bytes;
 	u32 i, bytes_per_chan, src_frame_size;
 
 	if (!ctx->audio_buffer || !nb_samples)
@@ -818,7 +884,14 @@ static void ffenc_audio_append_samples(struct _gf_ffenc_ctx *ctx, const u8 *data
 	if (s_idx) {
 		assert(s_idx + nb_samples <= (u32) ctx->encoder->frame_size);
 	}
-	dst = ctx->audio_buffer + (f_idx * ctx->channels * ctx->encoder->frame_size + s_idx) * bytes_per_chan;
+
+	offset = (f_idx * ctx->channels * ctx->encoder->frame_size + s_idx) * bytes_per_chan;
+	bytes = nb_samples * ctx->channels * bytes_per_chan;
+	if (bytes + offset > ctx->audio_buffer_size) {
+		ctx->audio_buffer_size = bytes+offset;
+		ctx->audio_buffer = gf_realloc(ctx->audio_buffer, sizeof(u8)*ctx->audio_buffer_size);
+	}
+	dst = ctx->audio_buffer + offset;
 	while (nb_samples) {
 		const u8 *src = NULL;
 		u32 nb_samples_to_copy = nb_samples;
@@ -864,11 +937,21 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	pck = gf_filter_pid_get_packet(ctx->in_pid);
 
 	if (!ctx->encoder) {
-		if (ctx->infmt_negociate) return GF_OK;
-		
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] encoder reconfiguration failed, aborting stream\n"));
-		gf_filter_pid_set_eos(ctx->out_pid);
-		return GF_EOS;
+		//no encoder: if negociating input format or input pid props not known yet, wait
+		if (ctx->infmt_negociate || !ctx->setup_failed) return GF_OK;
+
+		if (ctx->setup_failed==1) {
+			GF_FilterEvent fevt;
+
+			ctx->setup_failed = 2;
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] encoder reconfiguration failed, aborting stream\n"));
+			gf_filter_pid_set_eos(ctx->out_pid);
+
+			gf_filter_pid_set_discard(ctx->in_pid, GF_TRUE);
+			GF_FEVT_INIT(fevt, GF_FEVT_STOP, ctx->in_pid);
+			gf_filter_pid_send_event(ctx->in_pid, &fevt);
+		}
+		return GF_SERVICE_ERROR;
 	}
 
 	if (!pck) {
@@ -879,7 +962,14 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 	if (ctx->reconfig_pending) pck = NULL;
 
 	if (ctx->encoder->frame_size && (ctx->encoder->frame_size <= (s32) ctx->samples_in_audio_buffer)) {
-		avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt, ctx->audio_buffer, ctx->bytes_per_sample * ctx->encoder->frame_size, 0);
+		ctx->frame->nb_samples = ctx->encoder->frame_size;
+		res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt,
+			ctx->audio_buffer, ctx->audio_buffer_size /*ctx->bytes_per_sample * ctx->encoder->frame_size*/, 0);
+		if (res<0) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] Error filling raw audio frame: %s\n", av_err2str(res) ));
+			ctx->samples_in_audio_buffer = 0;
+			return GF_SERVICE_ERROR;
+		}
 
 		from_internal_buffer_only = GF_TRUE;
 
@@ -915,8 +1005,8 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 			ffenc_audio_append_samples(ctx, data, size, 0, nb_copy);
 
 			ctx->frame->nb_samples = ctx->encoder->frame_size;
-			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt, ctx->audio_buffer, ctx->encoder->frame_size*ctx->bytes_per_sample, 0);
-
+			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt,
+				ctx->audio_buffer, ctx->audio_buffer_size/*ctx->encoder->frame_size*ctx->bytes_per_sample*/, 0);
 		} else {
 			ctx->frame->nb_samples = size / ctx->bytes_per_sample;
 			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt, data, size, 0);
@@ -940,7 +1030,8 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		if (ctx->encoder->frame_size) {
 			nb_samples = ctx->encoder->frame_size - ctx->samples_in_audio_buffer;
 			ffenc_audio_append_samples(ctx, NULL, 0, 0, nb_samples);
-			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt, ctx->audio_buffer, ctx->encoder->frame_size * ctx->bytes_per_sample, 0);
+			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt,
+				ctx->audio_buffer, ctx->audio_buffer_size/*ctx->encoder->frame_size * ctx->bytes_per_sample*/, 0);
 		} else {
 			res = avcodec_fill_audio_frame(ctx->frame, ctx->channels, ctx->sample_fmt, ctx->audio_buffer, ctx->samples_in_audio_buffer * ctx->bytes_per_sample, 0);
 		}
@@ -1035,6 +1126,7 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 				ctx->reconfig_pending = GF_FALSE;
 				avcodec_free_context(&ctx->encoder);
 				ctx->encoder = NULL;
+				ctx->setup_failed = 0;
 				GF_LOG(GF_LOG_DEBUG, GF_LOG_CODEC, ("[FFEnc] codec flush done, triggering reconfiguration\n"));
 				e = ffenc_configure_pid_ex(filter, ctx->in_pid, GF_FALSE, GF_TRUE);
 				ctx->init_cts_setup = bck_init_cts;
@@ -1055,13 +1147,23 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 
 		if (ctx->samples_in_audio_buffer > nb_samples_to_drop) {
 			offset = nb_samples_to_drop * ctx->bytes_per_sample;
-			len = (ctx->samples_in_audio_buffer - nb_samples_to_drop) * ctx->bytes_per_sample;
+			len = (ctx->samples_in_audio_buffer - nb_samples_to_drop);
+			//if planar we must move entire frames
+			if (ctx->planar_audio) {
+				u32 nb_p = len / ctx->encoder->frame_size;
+				if (nb_p * ctx->encoder->frame_size < len) nb_p++;
+				len = nb_p * ctx->encoder->frame_size;
+			}
+			len *= ctx->bytes_per_sample;
+			if (len + offset > ctx->audio_buffer_size) {
+				ctx->audio_buffer_size = len+offset;
+				ctx->audio_buffer = gf_realloc(ctx->audio_buffer, sizeof(u8) * ctx->audio_buffer_size);
+			}
 			memmove(ctx->audio_buffer, ctx->audio_buffer + offset, sizeof(u8)*len);
 			ctx->samples_in_audio_buffer -= nb_samples_to_drop;
 		} else {
 			ctx->samples_in_audio_buffer = 0;
 		}
-
 	}
 
 	//increase timestamp
@@ -1146,6 +1248,10 @@ static GF_Err ffenc_process_audio(GF_Filter *filter, struct _gf_ffenc_ctx *ctx)
 		gf_list_del_item(ctx->src_packets, src_pck);
 		gf_filter_pck_unref(src_pck);
 	}
+	if (ctx->gen_dsi) {
+		ffmpeg_generate_gpac_dsi(ctx->out_pid, ctx->codecid, ctx->encoder->color_primaries, ctx->encoder->color_trc, ctx->encoder->colorspace, output, size);
+		ctx->gen_dsi = GF_FALSE;
+	}
 
 	gf_filter_pck_set_cts(dst_pck, pkt->pts + ctx->ts_shift);
 	gf_filter_pck_set_dts(dst_pck, pkt->dts + ctx->ts_shift);
@@ -1280,13 +1386,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	//declare our output pid to make sure we connect the chain
 	ctx->in_pid = pid;
 	if (!ctx->out_pid) {
-		char szCodecName[1000];
 		ctx->out_pid = gf_filter_pid_new(filter);
-
-		//to change once we implement on-the-fly codec change
-		sprintf(szCodecName, "ffenc:%s", codec->name ? codec->name : "unknown");
-		gf_filter_set_name(filter, szCodecName);
-		gf_filter_pid_set_framing_mode(ctx->in_pid, GF_TRUE);
 	}
 	if (type==GF_STREAM_AUDIO) {
 		ctx->process = ffenc_process_audio;
@@ -1311,10 +1411,10 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			ctx->discontunity = GF_TRUE;
 		}
 	}
-
+	//macro to check if prop exists and has non-0 value
 #define GET_PROP(_a, _code, _name) \
 	prop = gf_filter_pid_get_property(pid, _code); \
-	if (!prop) {\
+	if (!prop || !prop->value.uint) {\
 		GF_LOG(GF_LOG_INFO, GF_LOG_CODEC, ("[FFEnc] Input %s unknown, waiting for reconfigure\n", _name)); \
 		return GF_OK; \
 	}\
@@ -1353,6 +1453,9 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 		GET_PROP(ctx->sample_rate, GF_PROP_PID_SAMPLE_RATE, "sample rate")
 		GET_PROP(ctx->channels, GF_PROP_PID_NUM_CHANNELS, "nb channels")
 		GET_PROP(afmt, GF_PROP_PID_AUDIO_FORMAT, "audio format")
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_CHANNEL_LAYOUT);
+		ctx->channel_layout = prop ? prop->value.longuint : 0;
 	}
 
 
@@ -1386,7 +1489,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	if (type==GF_STREAM_VISUAL) {
 		u32 force_pfmt = AV_PIX_FMT_NONE;
 		if (ctx->pfmt) {
-			u32 ff_pfmt = ffmpeg_pixfmt_from_gpac(ctx->pfmt);
+			u32 ff_pfmt = ffmpeg_pixfmt_from_gpac(ctx->pfmt, GF_FALSE);
 			i=0;
 			while (codec->pix_fmts) {
 				if (codec->pix_fmts[i] == AV_PIX_FMT_NONE) break;
@@ -1407,7 +1510,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 				change_input_fmt = force_pfmt;
 			}
 		}
-		ctx->pixel_fmt = ffmpeg_pixfmt_from_gpac(pfmt);
+		ctx->pixel_fmt = ffmpeg_pixfmt_from_gpac(pfmt, GF_FALSE);
 		//check pixel format
 		if (force_pfmt == AV_PIX_FMT_NONE) {
 			change_input_fmt = AV_PIX_FMT_NONE;
@@ -1476,7 +1579,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 					return GF_NOT_SUPPORTED;
 				}
 			} else if (ctx->pfmt) {
-				ff_pmft = ffmpeg_pixfmt_from_gpac(ctx->pfmt);
+				ff_pmft = ffmpeg_pixfmt_from_gpac(ctx->pfmt, GF_FALSE);
 			}
 			pfmt = ffmpeg_pixfmt_to_gpac(ff_pmft, GF_FALSE);
 			gf_filter_pid_negociate_property(ctx->in_pid, GF_PROP_PID_PIXFMT, &PROP_UINT(pfmt) );
@@ -1486,9 +1589,10 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 		}
 	} else {
 		u32 change_input_sr = 0;
+		u64 change_chan_layout = 0;
 		//check audio format
 		ctx->sample_fmt = ffmpeg_audio_fmt_from_gpac(afmt);
-		change_input_fmt = 0;
+		change_input_fmt = AV_SAMPLE_FMT_NONE;
 		while (codec->sample_fmts) {
 			if (codec->sample_fmts[i] == AV_SAMPLE_FMT_NONE) break;
 			if (codec->sample_fmts[i] == ctx->sample_fmt) {
@@ -1509,7 +1613,33 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			}
 			i++;
 		}
-		if ((ctx->sample_fmt != change_input_fmt) || (ctx->sample_rate != change_input_sr)) {
+
+		i=0;
+		if (!ctx->channel_layout) {
+			ctx->channel_layout = gf_audio_fmt_get_layout_from_cicp(gf_audio_fmt_get_cicp_layout(ctx->channels, 0, 0));
+		}
+		u64 ff_ch_layout = ffmpeg_channel_layout_from_gpac(ctx->channel_layout);
+		if (!codec->channel_layouts)
+			change_chan_layout = ctx->channel_layout;
+
+		while (codec->channel_layouts) {
+			if (!codec->channel_layouts[i]) break;
+			if (codec->channel_layouts[i] == ff_ch_layout) {
+				change_chan_layout = ctx->channel_layout;
+				break;
+			}
+			i++;
+		}
+		//vorbis in ffmpeg currently requires stereo but channel_layouts is not set
+		if (ctx->codecid==GF_CODECID_VORBIS) {
+			change_chan_layout = gf_audio_fmt_get_layout_from_cicp(gf_audio_fmt_get_cicp_layout(2, 0, 0));
+		}
+
+
+		if ((ctx->sample_fmt != change_input_fmt)
+			|| (ctx->sample_rate != change_input_sr)
+			|| (ctx->channel_layout != change_chan_layout)
+		) {
 			if (ctx->sample_fmt != change_input_fmt) {
 				ctx->sample_fmt = codec->sample_fmts ? codec->sample_fmts[0] : AV_SAMPLE_FMT_S16;
 				afmt = ffmpeg_audio_fmt_to_gpac(ctx->sample_fmt);
@@ -1518,6 +1648,13 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			if (ctx->sample_rate != change_input_sr) {
 				gf_filter_pid_negociate_property(ctx->in_pid, GF_PROP_PID_SAMPLE_RATE, &PROP_UINT(codec->supported_samplerates[0]) );
 			}
+			if (ctx->channel_layout != change_chan_layout) {
+				if (!change_chan_layout)
+					change_chan_layout = ffmpeg_channel_layout_to_gpac(codec->channel_layouts[0]);
+				u32 nb_chans = gf_audio_fmt_get_num_channels_from_layout(change_chan_layout);
+				gf_filter_pid_negociate_property(ctx->in_pid, GF_PROP_PID_NUM_CHANNELS, &PROP_UINT(nb_chans) );
+				gf_filter_pid_negociate_property(ctx->in_pid, GF_PROP_PID_CHANNEL_LAYOUT, &PROP_LONGUINT(change_chan_layout) );
+			}
 			ctx->infmt_negociate = GF_TRUE;
 		} else {
 			ctx->infmt_negociate = GF_FALSE;
@@ -1535,7 +1672,10 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	ctx->in_tk_delay = prop ? prop->value.longsint : 0;
 
 	ctx->encoder = avcodec_alloc_context3(codec);
-	if (! ctx->encoder) return GF_OUT_OF_MEM;
+	if (! ctx->encoder) {
+		ctx->setup_failed = 1;
+		return GF_OUT_OF_MEM;
+	}
 
 	ctx->encoder->codec_tag = ff_codectag;
 	if (type==GF_STREAM_VISUAL) {
@@ -1568,18 +1708,31 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			ctx->encoder->framerate.den = prop->value.frac.den;
 			gf_media_get_reduced_frame_rate(&ctx->encoder->framerate.num, &ctx->encoder->framerate.den);
 
-			//if framerate exact or drop frame, use fps.num as timebase
-			//this is needed because some codecs in libavcodec check this value to derive the profile/levels
-			//so providing a too high time_base will increase the profile/levels ...
-			if ((ctx->encoder->framerate.den == 1) || (ctx->encoder->framerate.den == 1001)) {
+			//by default use input timescale as timebase for encoder, but:
+			//
+			// - mpeg12enc (maybe other codecs?) uses timebase to compute framerate so 1/25000 will fail, we must pass the fps
+			if ((codec->id==AV_CODEC_ID_MPEG1VIDEO)
+				|| (codec->id==AV_CODEC_ID_MPEG2VIDEO)
+				|| (codec->id==AV_CODEC_ID_H264)
+				|| (codec->id==AV_CODEC_ID_HEVC)
+#ifdef FFMPEG_ENABLE_VVC
+				|| (codec->id==AV_CODEC_ID_VVC)
+#endif
+			) {
+				ctx->encoder->time_base.num = ctx->encoder->framerate.den;
 				ctx->encoder->time_base.den = ctx->encoder->framerate.num;
 			}
-			//some codecs in libavcodec will complain if timebase is too high
-			//if fps is set and its num is quite small compared to our input timescale, use the num
-			//otherwise we try to keep the same timescale as input
-			else if ((u32) ctx->encoder->framerate.num * 100 < ctx->timescale) {
+			//- if framerate indicates drop frame, use fps.num as timebase
+			else if ((ctx->encoder->framerate.den % 1001)==0) {
 				ctx->encoder->time_base.den = ctx->encoder->framerate.num;
 			}
+			//- fps num less than input timescale
+			else if ((u32) ctx->encoder->framerate.num < ctx->timescale) {
+				//if timescale is a multiple of fps num, rescaling will not introduce rounding error so use fps num
+				if (!(ctx->timescale % ctx->encoder->framerate.num))
+					ctx->encoder->time_base.den = ctx->encoder->framerate.num;
+				//otherwise we would feed consecutive frames with same timestamp when rescaling and loose timing, so use ctx->timescale (default)
+			}
 		} else {
 			GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[FFEnc] Unknown frame rate for PID %s, will use 25 fps - use `:#FPS=VAL` on input to force frame rate\n", gf_filter_pid_get_name(pid) ));
 			ctx->encoder->framerate.num = 25;
@@ -1604,6 +1757,24 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			av_dict_set(&ctx->options, "forced-idr", "1", 0);
 		}
 
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_PRIMARIES);
+		ctx->encoder->color_primaries = prop ? prop->value.uint : AVCOL_PRI_UNSPECIFIED;
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_RANGE);
+		if (prop) ctx->encoder->color_range = (prop->value.uint==1) ? AVCOL_RANGE_JPEG : AVCOL_RANGE_MPEG;
+		else ctx->encoder->color_range = AVCOL_RANGE_UNSPECIFIED;
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_TRANSFER);
+		ctx->encoder->color_trc = prop ? prop->value.uint : AVCOL_TRC_UNSPECIFIED;
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_MX);
+		ctx->encoder->colorspace = prop ? prop->value.uint : AVCOL_SPC_UNSPECIFIED;
+
+		prop = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_CHROMALOC);
+		ctx->encoder->chroma_sample_location = prop ? prop->value.uint : AVCHROMA_LOC_UNSPECIFIED;
+
+
 		//we don't use out of band headers, since x264 in ffmpeg (and likely other) do not output in MP4 format but
 		//in annexB (extradata only contains SPS/PPS/etc in annexB)
 		//so we indicate unframed for these codecs and use our own filter for annexB->MP4
@@ -1646,12 +1817,15 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 			ctx->timescale = ctx->sample_rate;
 		}
 
-		//for aac
+		//enable expermimental encoders
 		switch (ctx->codecid) {
 		case GF_CODECID_AAC_MPEG4:
 		case GF_CODECID_AAC_MPEG2_MP:
 		case GF_CODECID_AAC_MPEG2_LCP:
 		case GF_CODECID_AAC_MPEG2_SSRP:
+		case GF_CODECID_OPUS:
+		case GF_CODECID_TRUEHD:
+		case GF_CODECID_VORBIS:
 			av_dict_set(&ctx->options, "strict", "experimental", 0);
 			break;
 		}
@@ -1667,7 +1841,7 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 		ctx->frame->format = ctx->encoder->sample_fmt;
 
 		ctx->audio_buffer_size = ctx->sample_rate;
-		ctx->audio_buffer = gf_realloc(ctx->audio_buffer, sizeof(char) * ctx->enc_buffer_size);
+		ctx->audio_buffer = gf_realloc(ctx->audio_buffer, sizeof(char) * ctx->audio_buffer_size);
 		ctx->bytes_per_sample = ctx->channels * gf_audio_fmt_bit_depth(afmt) / 8;
 		ctx->init_cts_setup = GF_TRUE;
 
@@ -1702,25 +1876,76 @@ static GF_Err ffenc_configure_pid_ex(GF_Filter *filter, GF_FilterPid *pid, Bool
 	else if (ctx->gop_size) ctx->encoder->gop_size = ctx->gop_size;
 
 	//by default let libavcodec decide - if single thread is required, let the user define -threads option
-	if (codec->capabilities & AV_CODEC_CAP_AUTO_THREADS)
-		ctx->encoder->thread_count = 0;
+	ctx->encoder->thread_count = 0;
 
+	//setup 2 pass encoding
+	if (ctx->encoder->flags & (AV_CODEC_FLAG_PASS1|AV_CODEC_FLAG_PASS2)) {
+		char szLogFile[GF_MAX_PATH];
+		const GF_PropertyValue *p = gf_filter_pid_get_property_str(pid, "logpass");
+		if (p && (p->type==GF_PROP_STRING) && p->value.string) {
+			sprintf(szLogFile, "%s", p->value.string);
+		} else {
+			u32 id=0;
+			p = gf_filter_pid_get_property(pid, GF_PROP_PID_ID);
+			if (p) id = p->value.uint;
+			sprintf(szLogFile, "ffenc2pass-%d.log", id);
+		}
+		if (ctx->rc) {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODEC, ("[FFEnc] Multi-pass encoding not compatible with `rc`, disabling reset coder flag\n", szLogFile));
+			ctx->rc = 0;
+		}
+		if (!strcmp(codec->name, "libx264")) {
+			av_dict_set(&options, "stats", szLogFile, AV_DICT_DONT_OVERWRITE);
+		} else {
+			if (ctx->encoder->flags & AV_CODEC_FLAG_PASS2) {
+				u32 len=0;
+				GF_Err e = gf_file_load_data(szLogFile, (u8**) &ctx->encoder->stats_in, &len);
+				if (!e && !gf_utf8_is_legal(ctx->encoder->stats_in, len)) {
+					e = GF_NON_COMPLIANT_BITSTREAM;
+				}
+				if (e) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] Error reading log file %s for pass-2 encoding: %s\n", szLogFile, gf_error_to_string(e) ));
+					if (ctx->encoder->stats_in) gf_free(ctx->encoder->stats_in);
+					return e;
+				}
+			}
+			if (ctx->encoder->flags & AV_CODEC_FLAG_PASS1) {
+				FILE *f = gf_fopen(szLogFile, "w");
+				if (!f) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] Error opening log file %s for pass-1 encoding\n", szLogFile));
+					return GF_IO_ERR;
+				}
+				ctx->logfile_pass1 = f;
+			}
+		}
+	}
 
 	av_dict_copy(&options, ctx->options, 0);
-	res = avcodec_open2(ctx->encoder, codec, &options );
+	res = avcodec_open2(ctx->encoder, codec, &options);
 	if (res < 0) {
 		if (options) av_dict_free(&options);
 		GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] PID %s failed to open codec context: %s\n", gf_filter_pid_get_name(pid), av_err2str(res) ));
 		avcodec_free_context(&ctx->encoder);
 		ctx->encoder = NULL;
+		ctx->setup_failed = 1;
 		return GF_BAD_PARAM;
 	}
-	//precompute gpac_timescale * encoder->time_base.num for rescale operations
-	//do that AFTER opening the codec, since some codecs may touch this field ...
-	ctx->premul_timescale = ctx->timescale;
-	ctx->premul_timescale *= ctx->encoder->time_base.num;
 
-	ctx->remap_ts = (ctx->encoder->time_base.den && (ctx->encoder->time_base.den != ctx->premul_timescale)) ? GF_TRUE : GF_FALSE;
+	if (ctx->c) gf_free(ctx->c);
+	ctx->c = gf_strdup(codec->name);
+
+	{
+		char szCodecName[1000];
+		if (ctx->encoder->thread_count>1)
+			sprintf(szCodecName, "ffenc:%s (%d frames)", codec->name ? codec->name : "unknown", ctx->encoder->thread_count);
+		else
+			sprintf(szCodecName, "ffenc:%s", codec->name ? codec->name : "unknown");
+		gf_filter_set_name(filter, szCodecName);
+		gf_filter_pid_set_framing_mode(ctx->in_pid, GF_TRUE);
+	}
+
+
+	ctx->remap_ts = (ctx->encoder->time_base.den != ctx->timescale) ? GF_TRUE : GF_FALSE;
 	if (!ctx->target_rate)
 		ctx->target_rate = (u32)ctx->encoder->bit_rate;
 
@@ -1740,11 +1965,12 @@ static GF_Err ffenc_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 
 static GF_Err ffenc_update_arg(GF_Filter *filter, const char *arg_name, const GF_PropertyValue *arg_val)
 {
-	s32 res;
 	char szOverrideOpt[1000];
-	Bool do_override = GF_FALSE;
+	GF_PropertyValue arg_val_override;
 	GF_FFEncodeCtx *ctx = gf_filter_get_udta(filter);
 
+	if (!strcmp(arg_name, "rld")) return GF_OK;
+
 	if (!strcmp(arg_name, "global_header"))	return GF_OK;
 	else if (!strcmp(arg_name, "local_header"))	return GF_OK;
 	else if (!strcmp(arg_name, "low_delay"))	ctx->low_delay = GF_TRUE;
@@ -1768,33 +1994,54 @@ static GF_Err ffenc_update_arg(GF_Filter *filter, const char *arg_name, const GF
 			ctx->target_rate *= 1000;
 
 		sprintf(szOverrideOpt, "%d", ctx->target_rate);
-		do_override = GF_TRUE;
+		arg_val_override.type = GF_PROP_STRING;
+		arg_val_override.value.string = szOverrideOpt;
+		arg_val = &arg_val_override;
 	}
 
 	//initial parsing of arguments
-	if (!ctx->initialized) {
-		const char *arg_val_str;
-		switch (arg_val->type) {
-		case GF_PROP_STRING:
-			if (do_override)
-				arg_val_str = szOverrideOpt;
-			else {
-				arg_val_str = arg_val->value.string;
-				if (!arg_val_str) arg_val_str = "1";
+	if (!ctx->encoder) {
+		return ffmpeg_update_arg("FFEnc", NULL, &ctx->options, arg_name, arg_val);
+	}
+	//updates of arguments
+	u64 value=0;
+	switch (arg_val->type) {
+	case GF_PROP_STRING:
+	{
+		GF_PropertyValue res = gf_props_parse_value(GF_PROP_LUINT, arg_name, arg_val->value.string, NULL, 0);
+		value = res.value.longuint;
+	}
+		break;
+	case GF_PROP_UINT: value = arg_val->value.uint; break;
+	case GF_PROP_LUINT: value = arg_val->value.longuint; break;
+	default:
+		return GF_NOT_SUPPORTED;
+	}
+
+	if (ctx->rld) {
+		Bool skip=GF_FALSE;
+		char szVal[100];
+		sprintf(szVal, LLU, value);
+		AVDictionaryEntry *key = av_dict_get(ctx->options, arg_name, NULL, 0);
+		if (key) {
+			if ((arg_val->type==GF_PROP_STRING) && !strcmp(key->value, arg_val->value.string))
+				skip=GF_TRUE;
+			else if (!strcmp(key->value, szVal))
+				skip=GF_TRUE;
+		}
+		if (!skip) {
+			av_dict_set(&ctx->options, arg_name, szVal, 0);
+			//if video and fintra is set, reload args at next intra
+			if (ctx->width && (ctx->fintra.num>=0)) {
+				ctx->args_updated = GF_TRUE;
+			} else {
+				ctx->reconfig_pending = GF_TRUE;
+				ctx->force_reconfig = GF_TRUE;
 			}
-			res = av_dict_set(&ctx->options, arg_name, arg_val_str, 0);
-			if (res<0) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] Failed to set option %s:%s\n", arg_name, arg_val_str ));
-			}
-			break;
-		default:
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CODEC, ("[FFEnc] Failed to set option %s, unrecognized type %d\n", arg_name, arg_val->type ));
-			return GF_NOT_SUPPORTED;
 		}
 		return GF_OK;
 	}
-	//updates of arguments, not supported for ffmpeg decoders
-	return GF_NOT_SUPPORTED;
+	return ffmpeg_update_arg("FFEnc", ctx->encoder, NULL, arg_name, arg_val);
 }
 
 static Bool ffenc_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
@@ -1849,6 +2096,12 @@ GF_FilterRegister FFEncodeRegister = {
 		"\n"
 		"When forcing a closed GOP boundary, the filter will flush, destroy and recreate the encoder to make sure a clean context is used, as currently many encoders in libavcodec do not support clean reset when forcing picture types.\n"
 		"If [-fintra]() is not set and the output of the encoder is a DASH session in live profile without segment timeline, [-fintra]() will be set to the target segment duration and [-rc]() will be set.\n"
+		"\n"
+		"The filter will look for property `logpass` on input PID to set 2-pass log filename, otherwise defaults to `ffenc2pass-PID.log`.\n"
+		"\n"
+		"Arguments may be updated at runtime. If [-rld]() is set, the encoder will be flushed then reloaded with new options.\n"
+		"If codec is video and [-fintra]() is set, reload will happen at next forced intra; otherwise, reload happens at next encode.\n"
+		"The [-rld]() option is usually needed for dynamic updates of rate control parameters, since most encoders in ffmpeg do not support it.\n"
 	)
 	.private_size = sizeof(GF_FFEncodeCtx),
 	SETCAPS(FFEncodeCaps),
@@ -1858,19 +2111,20 @@ GF_FilterRegister FFEncodeRegister = {
 	.process = ffenc_process,
 	.process_event = ffenc_process_event,
 	.update_arg = ffenc_update_arg,
-	.flags = GF_FS_REG_META,
+	.flags = GF_FS_REG_META | GF_FS_REG_TEMP_INIT,
 };
 
 #define OFFS(_n)	#_n, offsetof(GF_FFEncodeCtx, _n)
 static const GF_FilterArgs FFEncodeArgs[] =
 {
-	{ OFFS(c), "codec identifier. Can be any supported GPAC codec name or ffmpeg codec name", GF_PROP_STRING, NULL, NULL, 0},
+	{ OFFS(c), "codec identifier. Can be any supported GPAC codec name or ffmpeg codec name - updated to ffmpeg codec name after initialization", GF_PROP_STRING, NULL, NULL, 0},
 	{ OFFS(pfmt), "pixel format for input video. When not set, input format is used", GF_PROP_PIXFMT, "none", NULL, 0},
 	{ OFFS(fintra), "force intra / IDR frames at the given period in sec, e.g. `fintra=2` will force an intra every 2 seconds and `fintra=1001/1000` will force an intra every 30 frames on 30000/1001=29.97 fps video; ignored for audio", GF_PROP_FRACTION, "-1/1", NULL, 0},
 
 	{ OFFS(all_intra), "only produce intra frames", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_UPDATE|GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(ls), "log stats", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(rc), "reset encoder when forcing intra frame (some encoders might not support intra frame forcing)", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
+	{ OFFS(rld), "force reloading of encoder when arguments are updated", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT|GF_FS_ARG_UPDATE},
 
 	{ "*", -1, "any possible options defined for AVCodecContext and sub-classes. see `gpac -hx ffenc` and `gpac -hx ffenc:*`", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_META},
 	{0}
@@ -1880,8 +2134,7 @@ const int FFENC_STATIC_ARGS = (sizeof (FFEncodeArgs) / sizeof (GF_FilterArgs)) -
 
 const GF_FilterRegister *ffenc_register(GF_FilterSession *session)
 {
-	ffmpeg_build_register(session, &FFEncodeRegister, FFEncodeArgs, FFENC_STATIC_ARGS, FF_REG_TYPE_ENCODE);
-	return &FFEncodeRegister;
+	return ffmpeg_build_register(session, &FFEncodeRegister, FFEncodeArgs, FFENC_STATIC_ARGS, FF_REG_TYPE_ENCODE);
 }
 
 
