@@ -39,16 +39,24 @@
 
 #ifdef GPAC_HAS_FFMPEG
 
+#if (LIBAVCODEC_VERSION_MAJOR>58)
+#include <libavutil/mastering_display_metadata.h>
+#endif
+
+
 typedef struct
 {
 	AVStream *stream;
 	Bool ts_rescale;
 	AVRational in_scale;
+	u32 in_timescale;
 	Bool in_seg_flush;
 	u32 cts_shift;
-
+	s64 ts_shift;
+	Bool ready;
 	Bool suspended;
 	Bool reconfig_stream;
+	Bool webvtt;
 } GF_FFMuxStream;
 
 enum{
@@ -70,10 +78,11 @@ enum
 typedef struct
 {
 	//options
-	char *dst, *mime, *ffmt;
+	char *dst, *mime, *ffmt, *ext;
 	Double start, speed;
 	u32 block_size;
-	Bool interleave, nodisc, ffiles, noinit;
+	Bool nodisc, ffiles, noinit, keepts;
+	GF_Fraction ileave;
 
 	AVFormatContext *muxer;
 	//decode options
@@ -94,6 +103,9 @@ typedef struct
 	FILE *gfio;
 
 	u32 cur_file_idx_plus_one;
+	u32 probe_init;
+
+	u64 ts_regulate;
 
 #if (LIBAVCODEC_VERSION_MAJOR < 59)
 	AVPacket pkt;
@@ -107,6 +119,10 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 static GF_Err ffmx_init_mux(GF_Filter *filter, GF_FFMuxCtx *ctx)
 {
 	u32 i, nb_pids;
+	u64 min_ts=0;
+	u64 min_delay_skip=0;
+	u32 min_delay_ts=0;
+	u32 min_timescale=1000;
 	int res;
 
 	assert(ctx->status==FFMX_STATE_AVIO_OPEN);
@@ -136,6 +152,45 @@ static GF_Err ffmx_init_mux(GF_Filter *filter, GF_FFMuxCtx *ctx)
 		} else {
 			st->ts_rescale = GF_TRUE;
 		}
+		if (!ctx->keepts) {
+			GF_FilterPacket *pck = gf_filter_pid_get_packet(ipid);
+			u64 ts = gf_filter_pck_get_dts(pck);
+			if (ts == GF_FILTER_NO_TS) ts = gf_filter_pck_get_cts(pck);
+			if (ts == GF_FILTER_NO_TS) continue;
+
+			if (!min_ts || (min_ts * st->in_scale.den > ts * min_timescale)) {
+				min_ts = ts;
+				min_timescale = st->in_scale.den;
+			}
+		}
+
+		const GF_PropertyValue *p = gf_filter_pid_get_property(ipid, GF_PROP_PID_DELAY);
+		if (p) {
+			if (p->value.longsint < 0) {
+				if (!min_delay_skip || (gf_timestamp_greater(-p->value.longsint, st->in_scale.den, min_delay_skip, min_delay_ts))) {
+					min_delay_skip = -p->value.longsint;
+					min_delay_ts = st->in_scale.den;
+				}
+			}
+		}
+	}
+
+	for (i=0; i<nb_pids; i++) {
+		GF_FilterPid *ipid = gf_filter_get_ipid(filter, i);
+		GF_FFMuxStream *st = gf_filter_pid_get_udta(ipid);
+
+		//get pid delay
+		const GF_PropertyValue *p = gf_filter_pid_get_property(ipid, GF_PROP_PID_DELAY);
+		s64 delay = p ? p->value.longsint : 0;
+		//add min skip, delay is now >=0
+		delay += gf_timestamp_rescale(min_delay_skip, min_delay_ts, st->in_scale.den);
+		//may happen due to rounding
+		if (delay<0) delay=0;
+		st->ts_shift = delay;
+
+		if (!ctx->keepts) {
+			st->ts_shift = st->ts_shift - gf_timestamp_rescale(min_ts, min_timescale, st->in_scale.den);
+		}
 	}
 
 	res = avformat_write_header(ctx->muxer, &options);
@@ -209,6 +264,15 @@ static GF_Err ffmx_open_url(GF_FFMuxCtx *ctx, char *final_name)
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Fail to open AVIO context for %s - error %s\n", dst, av_err2str(res) ));
 			return GF_FILTER_NOT_SUPPORTED;
 		}
+
+#if (LIBAVFORMAT_VERSION_MAJOR < 59)
+		strncpy(ctx->muxer->filename, dst, 1023);
+		ctx->muxer->filename[1023]=0;
+#else
+		av_freep(&ctx->muxer->url);
+		ctx->muxer->url = av_strdup(dst);
+#endif
+
 	}
 	ctx->status = FFMX_STATE_AVIO_OPEN;
 	return GF_OK;
@@ -243,7 +307,7 @@ static GF_Err ffmx_initialize_ex(GF_Filter *filter, Bool use_templates)
 	if (sep && strchr(sep+1, '$'))
 		use_templates = GF_TRUE;
 
-	ofmt = av_guess_format(ctx->ffmt, url, ctx->mime);
+	ofmt = av_guess_format(ctx->ext ? ctx->ext : ctx->ffmt, url, ctx->mime);
 	//if protocol is present, we may fail at guessing the format
 	if (!ofmt && !ctx->ffmt) {
 		u32 len;
@@ -268,6 +332,12 @@ static GF_Err ffmx_initialize_ex(GF_Filter *filter, Bool use_templates)
 	ctx->muxer->oformat = FF_OFMT_CAST ofmt;
 
 	ctx->status = FFMX_STATE_ALLOC;
+
+	if (gf_filter_is_temporary(filter)) {
+		gf_filter_meta_set_instances(filter, ofmt->name);
+		return GF_OK;
+	}
+
 	//templates are used, we need to postpone opening the url until we have a PID and a first packet
 	if (use_templates)
 		return GF_OK;
@@ -277,6 +347,12 @@ static GF_Err ffmx_initialize_ex(GF_Filter *filter, Bool use_templates)
 
 static GF_Err ffmx_initialize(GF_Filter *filter)
 {
+	GF_FFMuxCtx *ctx = (GF_FFMuxCtx *) gf_filter_get_udta(filter);
+	if (ctx->ileave.num<=0) {
+		ctx->ileave.num = 0;
+	} else {
+		ctx->ts_regulate = ctx->ileave.num;
+	}
 	return ffmx_initialize_ex(filter, GF_FALSE);
 }
 static GF_Err ffmx_start_seg(GF_Filter *filter, GF_FFMuxCtx *ctx, const char *seg_name)
@@ -427,7 +503,7 @@ static GF_Err ffmx_close_seg(GF_Filter *filter, GF_FFMuxCtx *ctx, Bool send_evt_
 		evt.seg_size.is_init = 0;
 	}
 	evt.seg_size.media_range_start = ctx->offset_at_seg_start;
-	evt.seg_size.media_range_end = ctx->muxer->pb ? ctx->muxer->pb->written-1 : 0;
+	evt.seg_size.media_range_end = ctx->muxer->pb ? (ctx->muxer->pb->written-1) : 0;
 	ctx->offset_at_seg_start = evt.seg_size.media_range_end;
 
 	gf_filter_pid_send_event(pid, &evt);
@@ -441,9 +517,27 @@ void ffmx_inject_config(GF_FilterPid *pid, GF_FFMuxStream *st, AVPacket *pkt)
 	if (st->reconfig_stream & FFMX_INJECT_DSI) {
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_DECODER_CONFIG);
 		if (p && (p->type==GF_PROP_DATA) && p->value.data.ptr) {
-			data = av_packet_new_side_data(pkt, AV_PKT_DATA_NEW_EXTRADATA, p->value.data.size);
-			if (data)
-				memcpy(data, p->value.data.ptr, p->value.data.size);
+			u32 dsi_size = p->value.data.size;
+			u32 codec_id = 0;
+			const GF_PropertyValue *cid = gf_filter_pid_get_property(pid, GF_PROP_PID_CODECID);
+			if (cid) codec_id = cid->value.uint;
+
+			if (codec_id==GF_CODECID_FLAC) {
+				dsi_size-=4;
+			} else if (codec_id==GF_CODECID_OPUS) {
+				dsi_size+=8;
+			}
+			data = av_packet_new_side_data(pkt, AV_PKT_DATA_NEW_EXTRADATA, dsi_size);
+			if (data) {
+				if (codec_id==GF_CODECID_FLAC) {
+					memcpy(data, p->value.data.ptr+4, p->value.data.size-4);
+				} else if (codec_id==GF_CODECID_OPUS) {
+					memcpy(data, "OpusHead", 8);
+					memcpy(data+8, p->value.data.ptr, p->value.data.size);
+				} else {
+					memcpy(data, p->value.data.ptr, p->value.data.size);
+				}
+			}
 		}
 	}
 	if (st->reconfig_stream & FFMX_INJECT_VID_INFO) {
@@ -493,6 +587,41 @@ void ffmx_inject_config(GF_FilterPid *pid, GF_FFMuxStream *st, AVPacket *pkt)
 		}
 	}
 }
+static void ffmx_inject_webvtt(GF_FilterPacket *ipck, AVPacket *pkt)
+{
+	u32 len;
+	u8 *data;
+	const GF_PropertyValue *p;
+	const char *vtt_pre=NULL, *vtt_cueid=NULL, *vtt_settings=NULL;
+	p = gf_filter_pck_get_property_str(ipck, "vtt_pre");
+	if (p) vtt_pre = p->value.string;
+	p = gf_filter_pck_get_property_str(ipck, "vtt_cueid");
+	if (p) vtt_cueid = p->value.string;
+	p = gf_filter_pck_get_property_str(ipck, "vtt_settings");
+	if (p) vtt_settings = p->value.string;
+
+	if (vtt_cueid) {
+		len = (u32) strlen(vtt_cueid);
+		data = av_packet_new_side_data(pkt, AV_PKT_DATA_WEBVTT_IDENTIFIER, len);
+		if (data) memcpy(data, vtt_cueid, len);
+	}
+	if (vtt_settings) {
+		len = (u32) strlen(vtt_settings);
+		data = av_packet_new_side_data(pkt, AV_PKT_DATA_WEBVTT_SETTINGS, len);
+		if (data) memcpy(data, vtt_settings, len);
+	}
+	if (vtt_pre) {
+		len = (u32) strlen(vtt_pre);
+		data = av_packet_new_side_data(pkt, AV_PKT_DATA_MATROSKA_BLOCKADDITIONAL, len+8);
+		if (data) {
+			//set additional blockID = 1 (64 BE int)
+			memset(data, 0, 7);
+			data[7] = 1;
+			memcpy(data+8, vtt_pre, len);
+		}
+	}
+}
+
 
 static GF_Err ffmx_process(GF_Filter *filter)
 {
@@ -504,17 +633,42 @@ static GF_Err ffmx_process(GF_Filter *filter)
 		Bool all_ready = GF_TRUE;
 		Bool needs_reinit = (ctx->status==FFMX_STATE_ALLOC) ? GF_TRUE : GF_FALSE;
 
+		//potpone until no pending connections so that we don't write header before all streams are declared
+		if (gf_filter_connections_pending(filter))
+			return GF_OK;
+
+		if (!ctx->probe_init) ctx->probe_init = gf_sys_clock();
+		else if (gf_sys_clock() - ctx->probe_init > 10000) {
+			for (i=0; i<nb_pids; i++) {
+				GF_FilterPid *ipid = gf_filter_get_ipid(filter, i);
+				gf_filter_pid_set_discard(ipid, GF_TRUE);
+			}
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Could not initialize stream list in 10s, aborting\n"));
+			return GF_SERVICE_ERROR;
+		}
+
 		for (i=0; i<nb_pids; i++) {
 			GF_FilterPid *ipid = gf_filter_get_ipid(filter, i);
-			GF_FFMuxStream *st = gf_filter_pid_get_udta(ipid);
 			GF_FilterPacket *ipck = gf_filter_pid_get_packet(ipid);
+			GF_FFMuxStream *st = gf_filter_pid_get_udta(ipid);
+			if (!st) continue;
 
-			if (needs_reinit) {
+			if (!st->ready) {
+				const GF_PropertyValue *p = gf_filter_pid_get_property(ipid, GF_PROP_PID_DECODER_CONFIG);
+				if (!p) return GF_OK;
+			}
+
+			if (needs_reinit || !st->ready) {
 				e = ffmx_configure_pid(filter, ipid, GF_FALSE);
 				if (e) return e;
 			}
 			if (!ipck) {
-				all_ready = GF_FALSE;
+				//don't wait for subtitles & other streams
+				if ((st->stream->codecpar->codec_type==AVMEDIA_TYPE_VIDEO)
+					|| (st->stream->codecpar->codec_type==AVMEDIA_TYPE_VIDEO)
+				) {
+					all_ready = GF_FALSE;
+				}
 				continue;
 			}
 
@@ -536,6 +690,7 @@ static GF_Err ffmx_process(GF_Filter *filter)
 	if (ctx->status==FFMX_STATE_EOS) return GF_EOS;
 	else if (ctx->status==FFMX_STATE_ERROR) return GF_SERVICE_ERROR;
 
+	u32 nb_skip = 0;
 	nb_segs_done = 0;
 	nb_done = 0;
 	nb_suspended = 0;
@@ -581,6 +736,12 @@ static GF_Err ffmx_process(GF_Filter *filter)
 				continue;
 			}
 
+			//ahead of our interleaving window, don't write yet - av_interleaved_write_frame is not enough for performing regulation
+			if (ctx->ileave.num && gf_timestamp_greater(cts+st->ts_shift, st->in_timescale, ctx->ts_regulate, ctx->ileave.den)) {
+				nb_skip++;
+				break;
+			}
+
 			p = gf_filter_pck_get_property(ipck, GF_PROP_PCK_FILENUM);
 			if (ctx->dash_mode) {
 				if (p) {
@@ -627,15 +788,21 @@ static GF_Err ffmx_process(GF_Filter *filter)
 			if (sap==GF_FILTER_SAP_1) pkt->flags = AV_PKT_FLAG_KEY;
 			pkt->data = (u8 *) gf_filter_pck_get_data(ipck, &pkt->size);
 
+			if (st->ts_shift) {
+				pkt->dts = pkt->dts + st->ts_shift;
+				pkt->pts = pkt->pts + st->ts_shift;
+			}
 			if (st->ts_rescale) {
 				av_packet_rescale_ts(pkt, st->in_scale, st->stream->time_base);
 			}
 
 			if (st->reconfig_stream) {
 				ffmx_inject_config(ipid, st, pkt);
+			} else if (st->webvtt) {
+				ffmx_inject_webvtt(ipck, pkt);
 			}
 
-			if (ctx->interleave) {
+			if (ctx->ileave.num) {
 				res = av_interleaved_write_frame(ctx->muxer, pkt);
 			} else {
 				res = av_write_frame(ctx->muxer, pkt);
@@ -644,10 +811,9 @@ static GF_Err ffmx_process(GF_Filter *filter)
 				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Fail to write packet %sto %s - error %s\n", st->reconfig_stream ? "with reconfig side data " : "", AVFMT_URL(ctx->muxer), av_err2str(res) ));
 				e = GF_IO_ERR;
 			}
-			if (st->reconfig_stream) {
-				av_packet_free_side_data(pkt);
-				st->reconfig_stream = 0;
-			}
+
+			if (pkt->side_data) av_packet_free_side_data(pkt);
+			st->reconfig_stream = 0;
 
 			gf_filter_pid_drop_packet(ipid);
 			ctx->nb_pck_in_seg++;
@@ -655,9 +821,18 @@ static GF_Err ffmx_process(GF_Filter *filter)
 		}
 	}
 
+	if (nb_skip + nb_done >= nb_pids) {
+		ctx->ts_regulate += ctx->ileave.num;
+	}
+
 	//done writing file
 	if (nb_suspended && (nb_suspended==nb_pids)) {
-		av_write_trailer(ctx->muxer);
+		int res = av_write_trailer(ctx->muxer);
+		if (res) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Failed to write trailer for %s: %s\n", ctx->dst, av_err2str(res) ));
+			e = GF_SERVICE_ERROR;
+		}
+
 		if (ctx->muxer)	avformat_free_context(ctx->muxer);
 		ctx->muxer = NULL;
 		ctx->status = FFMX_STATE_ALLOC;
@@ -691,7 +866,11 @@ static GF_Err ffmx_process(GF_Filter *filter)
 			if (ctx->dash_mode) {
 				ffmx_close_seg(filter, ctx, GF_FALSE);
 			} else {
-				av_write_trailer(ctx->muxer);
+				int res = av_write_trailer(ctx->muxer);
+				if (res) {
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Failed to write trailer for %s: %s\n", ctx->dst, av_err2str(res) ));
+					return GF_SERVICE_ERROR;
+				}
 			}
 			ctx->status = FFMX_STATE_EOS;
 		}
@@ -700,6 +879,19 @@ static GF_Err ffmx_process(GF_Filter *filter)
 	return GF_OK;
 }
 
+//dovi_meta.h not exported in old releases, just redefine
+typedef struct {
+    u8 dv_version_major;
+    u8 dv_version_minor;
+    u8 dv_profile;
+    u8 dv_level;
+    u8 rpu_present_flag;
+    u8 el_present_flag;
+    u8 bl_present_flag;
+    u8 dv_bl_signal_compatibility_id;
+} Ref_FFAVDoviRecord;
+
+
 static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove)
 {
 	int res;
@@ -707,6 +899,7 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	Bool check_disc = GF_FALSE;
 	u32 streamtype, codec_id;
 	u32 ff_codec_id, ff_st, ff_codec_tag;
+	GF_Err e;
 	const GF_PropertyValue *p, *dsi;
 	GF_FFMuxStream *st;
 	GF_FFMuxCtx *ctx = (GF_FFMuxCtx *) gf_filter_get_udta(filter);
@@ -740,6 +933,38 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	if (!p) return GF_NOT_SUPPORTED;
 	codec_id = p->value.uint;
 
+	Bool stream_ready = GF_TRUE;
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DECODER_CONFIG);
+	if (!p) {
+		//we must wait for decoder config to be present for these codecs
+		switch (codec_id) {
+		case GF_CODECID_MPEG1:
+		case GF_CODECID_MPEG2_422:
+		case GF_CODECID_MPEG2_SNR:
+		case GF_CODECID_MPEG2_HIGH:
+		case GF_CODECID_MPEG2_MAIN:
+		case GF_CODECID_MPEG2_SIMPLE:
+		case GF_CODECID_MPEG2_SPATIAL:
+		case GF_CODECID_MPEG4_PART2:
+		case GF_CODECID_AVC:
+		case GF_CODECID_HEVC:
+		case GF_CODECID_VVC:
+		case GF_CODECID_AV1:
+		case GF_CODECID_VP8:
+		case GF_CODECID_VP9:
+		case GF_CODECID_VP10:
+		case GF_CODECID_AC3:
+		case GF_CODECID_EAC3:
+		case GF_CODECID_FLAC:
+		case GF_CODECID_OPUS:
+		case GF_CODECID_VORBIS:
+			stream_ready = GF_FALSE;
+			break;
+		default:
+			break;
+		}
+	}
+
 	ff_st = ffmpeg_stream_type_from_gpac(streamtype);
 	if (codec_id==GF_CODECID_RAW) {
 		ff_codec_tag = 0;
@@ -801,6 +1026,11 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	if (!st) {
 		GF_FilterEvent evt;
 
+		if (ctx->status > FFMX_STATE_HDR_DONE) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Cannot add new stream to container, header already written\n"));
+			return GF_NOT_SUPPORTED;
+		}
+
 		GF_SAFEALLOC(st, GF_FFMuxStream);
 		if (!st) return GF_OUT_OF_MEM;
 		gf_filter_pid_set_udta(pid, st);
@@ -809,6 +1039,7 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		gf_filter_pid_init_play_event(pid, &evt, ctx->start, ctx->speed, "FFMux");
 		gf_filter_pid_send_event(pid, &evt);
 	}
+	st->ready =  stream_ready;
 
 	if (ctx->status==FFMX_STATE_ALLOC) {
 		char szPath[GF_MAX_PATH];
@@ -836,6 +1067,9 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		if (!st->stream) return GF_NOT_SUPPORTED;
 	}
 	avst = st->stream;
+	st->webvtt = GF_FALSE;
+	if ((codec_id==GF_CODECID_WEBVTT) && !strcmp( ctx->muxer->oformat->name, "matroska"))
+		st->webvtt = GF_TRUE;
 
 	dsi = gf_filter_pid_get_property(pid, GF_PROP_PID_DECODER_CONFIG);
 
@@ -889,13 +1123,8 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	avst->codecpar->codec_tag = ff_codec_tag;
 
 	if (dsi && dsi->value.data.ptr) {
-		if (avst->codecpar->extradata) av_free(avst->codecpar->extradata);
-		avst->codecpar->extradata_size = dsi->value.data.size;
-		avst->codecpar->extradata = av_malloc(avst->codecpar->extradata_size+ AV_INPUT_BUFFER_PADDING_SIZE);
-		if (avst->codecpar->extradata) {
-			memcpy(avst->codecpar->extradata, dsi->value.data.ptr, avst->codecpar->extradata_size);
-			memset(avst->codecpar->extradata + dsi->value.data.size, 0, AV_INPUT_BUFFER_PADDING_SIZE);
-		}
+		e = ffmpeg_extradata_from_gpac(codec_id, dsi->value.data.ptr, dsi->value.data.size, &avst->codecpar->extradata, &avst->codecpar->extradata_size);
+		if (e) return e;
 	}
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_ID);
 	if (p) avst->id = p->value.uint;
@@ -906,12 +1135,13 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		avst->time_base.num = 1;
 	}
 	st->in_scale = avst->time_base;
+	st->in_timescale = avst->time_base.den;
 
 	avst->start_time = 0;
 	avst->duration = 0;
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DURATION);
 	if (p && p->value.lfrac.den) {
-		avst->duration = p->value.lfrac.num;
+		avst->duration = (p->value.lfrac.num<0) ? -p->value.lfrac.num : p->value.lfrac.num;
 		avst->duration *= avst->time_base.den;
 		avst->duration /= p->value.lfrac.den;
 	}
@@ -920,56 +1150,19 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 	if (p) avst->codecpar->bit_rate = p->value.uint;
 
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_CTS_SHIFT);
-	if (p) st->cts_shift = p->value.uint;
+	st->cts_shift = p ? p->value.uint : 0;
+
+	e = ffmpeg_codec_par_from_gpac(pid, avst->codecpar, st->in_scale.den);
+	if (e) return e;
 
 	if (streamtype==GF_STREAM_VISUAL) {
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_WIDTH);
-		if (p) avst->codecpar->width = p->value.uint;
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_HEIGHT);
-		if (p) avst->codecpar->height = p->value.uint;
-
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_FPS);
 		if (p) {
 			avst->r_frame_rate.num = p->value.frac.num;
 			avst->r_frame_rate.den = p->value.frac.den;
 			avst->avg_frame_rate = avst->r_frame_rate;
 		}
-		if (codec_id==GF_CODECID_RAW) {
-			p = gf_filter_pid_get_property(pid, GF_PROP_PID_PIXFMT);
-			if (p) {
-				avst->codecpar->format = ffmpeg_pixfmt_from_gpac(p->value.uint);
-				avst->codecpar->codec_tag = avcodec_pix_fmt_to_codec_tag(avst->codecpar->format);
-			}
-		}
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_SAR);
-		if (p) {
-			avst->codecpar->sample_aspect_ratio.num = p->value.frac.num;
-			avst->codecpar->sample_aspect_ratio.den = p->value.frac.den;
-		}
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_PRIMARIES);
-		if (p) avst->codecpar->color_primaries = p->value.uint;
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_RANGE);
-		if (p) avst->codecpar->color_range = (p->value.uint==1) ? AVCOL_RANGE_JPEG : AVCOL_RANGE_MPEG;
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_TRANSFER);
-		if (p) avst->codecpar->color_trc = p->value.uint;
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_MX);
-		if (p) avst->codecpar->color_space = p->value.uint;
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_COLR_CHROMALOC);
-		if (p) avst->codecpar->chroma_location = p->value.uint;
-
-		p = gf_filter_pid_get_property(pid, GF_PROP_PID_DELAY);
-		if (p && avst->r_frame_rate.num && avst->r_frame_rate.den) {
-			s64 delay = p->value.longsint;
-			delay *= avst->r_frame_rate.num;
-			delay /= avst->r_frame_rate.den;
-			avst->codecpar->video_delay = (s32) delay;
-		}
-
+		avst->sample_aspect_ratio = avst->codecpar->sample_aspect_ratio;
 	}
 	else if (streamtype==GF_STREAM_AUDIO) {
 		u64 ch_layout;
@@ -988,17 +1181,20 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 			if (p) avst->codecpar->format =  ffmpeg_audio_fmt_from_gpac(p->value.uint);
 		}
 
-		ch_layout = AV_CH_LAYOUT_MONO;
+		ch_layout = 0;
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_CHANNEL_LAYOUT);
 		if (p)
 			ch_layout = p->value.longuint;
 		else if (avst->codecpar->channels==2)
-			ch_layout = AV_CH_LAYOUT_STEREO;
-		avst->codecpar->channel_layout = ffmpeg_channel_layout_from_gpac(ch_layout);
+			ch_layout = GF_AUDIO_CH_FRONT_LEFT|GF_AUDIO_CH_FRONT_RIGHT;
+		else if (avst->codecpar->channels==1)
+			ch_layout = GF_AUDIO_CH_FRONT_CENTER;
+
+		avst->codecpar->channel_layout = ch_layout ? ffmpeg_channel_layout_from_gpac(ch_layout) : 0;
 
 		p = gf_filter_pid_get_property(pid, GF_PROP_PID_DELAY);
 		if (p && (p->value.sint<0) && samplerate) {
-			s64 pad = p->value.longsint;
+			s64 pad = -p->value.longsint;
 			if (st->in_scale.den != samplerate) {
 				pad *= samplerate;
 				pad /= st->in_scale.den;
@@ -1011,6 +1207,93 @@ static GF_Err ffmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_r
 		*/
 	}
 
+	ffmpeg_tags_from_gpac(pid, &avst->metadata);
+
+	//remap our props to side data
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_ISOM_TRACK_MATRIX);
+	if (p && (p->type==GF_PROP_UINT_LIST) && (p->value.uint_list.nb_items==9)) {
+		u8 *data = av_malloc(sizeof(u32) * 9);
+		if (data) {
+			memcpy(data, p->value.uint_list.vals, sizeof(u32)*9);
+			av_stream_add_side_data(st->stream, AV_PKT_DATA_DISPLAYMATRIX, data, 32*9);
+		}
+	}
+#if (LIBAVCODEC_VERSION_MAJOR>58)
+	//icc profile if any
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_ICC_PROFILE);
+	if (p && ((p->type==GF_PROP_DATA) || (p->type==GF_PROP_CONST_DATA))) {
+		u8 *data = av_malloc(p->value.data.size);
+		if (data) {
+			memcpy(data, p->value.data.ptr, p->value.data.size);
+			av_stream_add_side_data(st->stream, AV_PKT_DATA_ICC_PROFILE, data, p->value.data.size);
+		}
+	}
+	//clli
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_CONTENT_LIGHT_LEVEL);
+	if (p && ((p->type==GF_PROP_DATA) || (p->type==GF_PROP_CONST_DATA)) && (p->value.data.size==4)) {
+		GF_BitStream *bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+		AVContentLightMetadata *data = av_malloc(sizeof(AVContentLightMetadata));
+		if (data) {
+			data->MaxCLL = gf_bs_read_u16(bs);
+			data->MaxFALL = gf_bs_read_u16(bs);;
+			av_stream_add_side_data(st->stream, AV_PKT_DATA_CONTENT_LIGHT_LEVEL, (u8*) data, sizeof(AVContentLightMetadata));
+		}
+		gf_bs_del(bs);
+	}
+	//mdcv
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_MASTER_DISPLAY_COLOUR);
+	if (p && ((p->type==GF_PROP_DATA) || (p->type==GF_PROP_CONST_DATA)) && (p->value.data.size==24)) {
+		u8 mdcv[24];
+		const int chroma_den = 50000;
+		const int luma_den = 10000;
+		memset(mdcv, 0, sizeof(u8)*24);
+		AVMasteringDisplayMetadata *data = av_malloc(sizeof(AVMasteringDisplayMetadata));
+		GF_BitStream *bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+		if (data) {
+			data->display_primaries[1][0].num = gf_bs_read_u16(bs);
+			data->display_primaries[1][0].den = chroma_den;
+			data->display_primaries[1][1].num = gf_bs_read_u16(bs);
+			data->display_primaries[1][1].den = chroma_den;
+			data->display_primaries[2][0].num = gf_bs_read_u16(bs);
+			data->display_primaries[2][0].den = chroma_den;
+			data->display_primaries[2][1].num = gf_bs_read_u16(bs);
+			data->display_primaries[2][1].den = chroma_den;
+			data->display_primaries[0][0].num = gf_bs_read_u16(bs);
+			data->display_primaries[0][0].den = chroma_den;
+			data->display_primaries[0][1].num = gf_bs_read_u16(bs);
+			data->display_primaries[0][1].den = chroma_den;
+			data->white_point[0].num = gf_bs_read_u16(bs);
+			data->white_point[0].den = chroma_den;
+			data->white_point[1].num = gf_bs_read_u16(bs);
+			data->white_point[1].den = chroma_den;
+			data->max_luminance.num = gf_bs_read_u32(bs);
+			data->max_luminance.den = luma_den;
+			data->min_luminance.num = gf_bs_read_u32(bs);
+			data->min_luminance.den = luma_den;
+			av_stream_add_side_data(st->stream, AV_PKT_DATA_MASTERING_DISPLAY_METADATA, (u8*) data, sizeof(AVMasteringDisplayMetadata));
+    	}
+    	gf_bs_del(bs);
+	}
+	//dolby vision
+	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DOLBY_VISION);
+	if (p && ((p->type==GF_PROP_DATA) || (p->type==GF_PROP_CONST_DATA)) && (p->value.data.size==24)) {
+		GF_BitStream *bs = gf_bs_new(p->value.data.ptr, p->value.data.size, GF_BITSTREAM_READ);
+		Ref_FFAVDoviRecord *data = av_malloc(sizeof(Ref_FFAVDoviRecord));
+		if (data) {
+			data->dv_version_major = gf_bs_read_u8(bs);
+			data->dv_version_minor = gf_bs_read_u8(bs);
+			data->dv_profile = gf_bs_read_int(bs, 7);
+			data->dv_level = gf_bs_read_int(bs, 6);
+			data->rpu_present_flag = gf_bs_read_int(bs, 1);
+			data->el_present_flag = gf_bs_read_int(bs, 1);
+			data->bl_present_flag = gf_bs_read_int(bs, 1);
+			data->dv_bl_signal_compatibility_id = gf_bs_read_int(bs, 4);
+			av_stream_add_side_data(st->stream, AV_PKT_DATA_DOVI_CONF, (u8*) data, sizeof(Ref_FFAVDoviRecord));
+		}
+		gf_bs_del(bs);
+	}
+#endif
+
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DASH_MODE);
 	if (p && (p->value.uint==1)) {
 		ctx->dash_mode = GF_TRUE;
@@ -1029,7 +1312,10 @@ static void ffmx_finalize(GF_Filter *filter)
 		if (ctx->dash_mode) {
 			ffmx_close_seg(filter, ctx, GF_FALSE);
 		} else {
-			av_write_trailer(ctx->muxer);
+			int res = av_write_trailer(ctx->muxer);
+			if (res) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Failed to write trailer for %s: %s\n", ctx->dst, av_err2str(res) ));
+			}
 		}
 		ctx->status = FFMX_STATE_TRAILER_DONE;
 	} 
@@ -1058,26 +1344,8 @@ static void ffmx_finalize(GF_Filter *filter)
 
 static GF_Err ffmx_update_arg(GF_Filter *filter, const char *arg_name, const GF_PropertyValue *arg_val)
 {
-	s32 res;
 	GF_FFMuxCtx *ctx = gf_filter_get_udta(filter);
-
-	//initial parsing of arguments
-	if (!ctx->muxer) {
-		switch (arg_val->type) {
-		case GF_PROP_STRING:
-			res = av_dict_set(&ctx->options, arg_name, arg_val->value.string, 0);
-			if (res<0) {
-				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Failed to set option %s:%s\n", arg_name, arg_val ));
-			}
-			break;
-		default:
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[FFMux] Failed to set option %s:%s, unrecognized type %d\n", arg_name, arg_val, arg_val->type ));
-			return GF_NOT_SUPPORTED;
-		}
-		return GF_OK;
-	}
-	//updates of arguments, not supported for ffmpeg decoders
-	return GF_NOT_SUPPORTED;
+	return ffmpeg_update_arg("FFMux", ctx->muxer, &ctx->options, arg_name, arg_val);
 }
 
 static GF_FilterProbeScore ffmx_probe_url(const char *url, const char *mime)
@@ -1110,14 +1378,20 @@ static const GF_FilterCapability FFMuxCaps[] =
 {
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_VISUAL),
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
-	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
-
 	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
 	CAP_UINT(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_CODECID, GF_CODECID_NONE),
-
-#ifdef FF_SUB_SUPPORT
+	{0},
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_TEXT),
-#endif
+	CAP_BOOL(GF_CAPS_INPUT, GF_PROP_PID_UNFRAMED, GF_TRUE),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_WEBVTT),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_SIMPLE_TEXT),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_SUBS_TEXT),
+	{0},
+	//these  ones are framed (DVB subs, simple text)
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_TEXT),
+	CAP_BOOL(GF_CAPS_INPUT_EXCLUDED, GF_PROP_PID_UNFRAMED, GF_TRUE),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_SIMPLE_TEXT),
+	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_CODECID, GF_CODECID_DVB_SUBS),
 };
 
 GF_FilterRegister FFMuxRegister = {
@@ -1162,12 +1436,14 @@ static const GF_FilterArgs FFMuxArgs[] =
 	{ OFFS(dst), "location of destination file or remote URL", GF_PROP_NAME, NULL, NULL, 0},
 	{ OFFS(start), "set playback start offset. A negative value means percent of media duration with -1 equal to duration", GF_PROP_DOUBLE, "0.0", NULL, 0},
 	{ OFFS(speed), "set playback speed. If negative and start is 0, start is set to -1", GF_PROP_DOUBLE, "1.0", NULL, 0},
-	{ OFFS(interleave), "write frame in interleave mode", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(ileave), "interleave window duration in second, a value of 0 disable interleaving", GF_PROP_FRACTION, "1", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(nodisc), "ignore stream configuration changes while multiplexing, may result in broken streams", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(mime), "set mime type for graph resolution", GF_PROP_NAME, NULL, NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(ffiles), "force complete files to be created for each segment in DASH modes", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_EXPERT},
 	{ OFFS(ffmt), "force ffmpeg output format for the given URL", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(block_size), "block size used to read file when using avio context", GF_PROP_UINT, "4096", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(keepts), "do not shift input timeline back to 0", GF_PROP_BOOL, "true", NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(ext), "force ffmpeg output format for the given URL", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_HINT_HIDE},
 	{ "*", -1, "any possible options defined for AVFormatContext and sub-classes (see `gpac -hx ffmx` and `gpac -hx ffmx:*`)", GF_PROP_STRING, NULL, NULL, GF_FS_ARG_META},
 	{0}
 };
@@ -1176,8 +1452,7 @@ const int FFMX_STATIC_ARGS = (sizeof (FFMuxArgs) / sizeof (GF_FilterArgs)) - 1;
 
 const GF_FilterRegister *ffmx_register(GF_FilterSession *session)
 {
-	ffmpeg_build_register(session, &FFMuxRegister, FFMuxArgs, FFMX_STATIC_ARGS, FF_REG_TYPE_MUX);
-	return &FFMuxRegister;
+	return ffmpeg_build_register(session, &FFMuxRegister, FFMuxArgs, FFMX_STATIC_ARGS, FF_REG_TYPE_MUX);
 }
 
 #else
