@@ -67,13 +67,31 @@ typedef struct
 	Bool fullrange;
 
 	u32 o_bpp, offset_w, offset_h;
+
+	Bool unpack_v410, repack_v410, unpack_v210, repack_v210;
+	u32 orig_in_stride, repack_stride;
+	u8 *unpack_buf, *repack_buf;
+
+	u32 unpack_generic, unpack_depth;
+	Bool unpack_yuv, unpack_alpha;
+
+	u32 repack_generic, repack_depth;
+	Bool repack_yuv, repack_alpha;
+
 	GF_EVGSurface *surf;
+	GF_EVGStencil *tx;
+	GF_Path *path;
 } GF_FFSWScaleCtx;
 
+#include <libavutil/intreadwrite.h>
+
+u32 gf_evg_stencil_get_pixel_fast(GF_EVGStencil *st, s32 x, s32 y);
+u64 gf_evg_stencil_get_pixel_wide_fast(GF_EVGStencil *st, s32 x, s32 y);
+
 static GF_Err ffsws_process(GF_Filter *filter)
 {
 	const char *data;
-	u8 *output;
+	u8 *output, *pck_data;
 	u32 osize;
 	s32 res;
 	u8 *src_planes[5];
@@ -111,7 +129,7 @@ static GF_Err ffsws_process(GF_Filter *filter)
 	frame_ifce = gf_filter_pck_get_frame_interface(pck);
 	//we may have buffer input (padding) but shall not have smaller
 	if (osize && (ctx->out_src_size > osize) ) {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[FFSWS] Mismatched in source osize, expected %d got %d - stride issue ?\n", ctx->out_src_size, osize));
+		GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[FFSWS] Mismatch in source osize, expected %d got %d - stride issue ?\n", ctx->out_src_size, osize));
 		gf_filter_pid_drop_packet(ctx->ipid);
 		return GF_NOT_SUPPORTED;
 	}
@@ -126,17 +144,194 @@ static GF_Err ffsws_process(GF_Filter *filter)
 	if (data) {
 		src_planes[0] = (u8 *) data;
 
-		if (ctx->nb_src_planes==1) {
-		} else if (ctx->nb_src_planes==2) {
+		if (ctx->unpack_v410) {
+			u32 i, j;
+			for (j=0; j<ctx->h; j++) {
+				const u8 *p_src = data + j * ctx->orig_in_stride;
+				u8 *p_y = ctx->unpack_buf + j * ctx->w * 2;
+				u8 *p_u = p_y + ctx->h * ctx->w * 2;
+				u8 *p_v = p_u + ctx->h * ctx->w * 2;
+
+				for (i=0; i<ctx->w; i++) {
+					u32 val = AV_RL32(p_src);
+					*(u16*) p_u = (val>>2) & 0x3FF;
+					*(u16*) p_y = (val>>12) & 0x3FF;
+					*(u16*) p_v = (val>>22);
+
+					p_u+=2;
+					p_v+=2;
+					p_y+=2;
+					p_src+=4;
+				}
+			}
+			src_planes[0] = ctx->unpack_buf;
 			src_planes[1] = src_planes[0] + ctx->src_stride[0]*ctx->h;
-		} else if (ctx->nb_src_planes==3) {
-			src_planes[1] = src_planes[0] + ctx->src_stride[0] * ctx->h;
-			src_planes[2] = src_planes[1] + ctx->src_stride[1] * ctx->src_uv_height;
-		} else if (ctx->nb_src_planes==4) {
-			src_planes[1] = src_planes[0] + ctx->src_stride[0] * ctx->h;
-			src_planes[2] = src_planes[1] + ctx->src_stride[1] * ctx->src_uv_height;
-			src_planes[3] = src_planes[2] + ctx->src_stride[2] * ctx->src_uv_height;
+			src_planes[2] = src_planes[1] + ctx->src_stride[1]*ctx->h;
+		} else if (ctx->unpack_v210) {
+			u32 i, j;
+			for (j=0; j<ctx->h; j++) {
+				u32 cur_w = 0;
+				const u8 *p_src = data + j * ctx->orig_in_stride;
+				u8 *p_y = ctx->unpack_buf + j * ctx->w * 2;
+				u8 *p_u = ctx->unpack_buf + j * ctx->w + ctx->h * ctx->w * 2;
+				u8 *p_v = p_u + ctx->h * ctx->w;
+
+				for (i=0; i<ctx->w; i+=6) {
+					u32 val;
+					u16 u, v, y;
+					//first word is xx Cr0 Y0 Cb0
+					val = AV_RL32(p_src);
+					u = (val) & 0x3FF;
+					*(u16*) p_y = (val>>10) & 0x3FF;
+					v = (val>>20) & 0x3FF;
+					*(u16*) p_u = u;
+					*(u16*) p_v = v;
+					p_u+=2;
+					p_v+=2;
+					p_y+=2;
+					p_src+=4;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+
+					//second word is xx Y2 Cb1 Y1
+					val = AV_RL32(p_src);
+					y = (val) & 0x3FF;
+
+					*(u16*) p_y = y;
+					p_y+=2;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+
+					u = (val>>10) & 0x3FF;
+					y = (val>>20) & 0x3FF;
+					p_src+=4;
+
+					//third word is xx Cb2 Y3 Cr1
+					val = AV_RL32(p_src);
+					v = (val) & 0x3FF;
+					*(u16*) p_y = y;
+					*(u16*) p_u = u;
+					*(u16*) p_v = v;
+					p_u+=2;
+					p_v+=2;
+					p_y+=2;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+
+					y = (val>>10) & 0x3FF;
+					*(u16*) p_y = y;
+					p_y+=2;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+
+					u = (val>>20) & 0x3FF;
+					p_src+=4;
+
+					//fourth word is xx Y5 Cr2 Y4
+					val = AV_RL32(p_src);
+					y = (val) & 0x3FF;
+					v = (val>>10) & 0x3FF;
+
+					*(u16*) p_y = y;
+					*(u16*) p_u = u;
+					*(u16*) p_v = v;
+					p_u+=2;
+					p_v+=2;
+					p_y+=2;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+
+					y = (val>>20) & 0x3FF;
+					*(u16*) p_y = y;
+					p_y+=2;
+					p_src+=4;
+					cur_w++;
+					if (cur_w>=ctx->w) break;
+				}
+			}
+			src_planes[0] = ctx->unpack_buf;
+			src_planes[1] = src_planes[0] + ctx->src_stride[0]*ctx->h;
+			src_planes[2] = src_planes[1] + ctx->src_stride[1]*ctx->h;
+		} else if (ctx->unpack_generic) {
+			GF_Err e = gf_evg_stencil_set_texture_planes(ctx->tx, ctx->w, ctx->h, ctx->s_pfmt, data, ctx->orig_in_stride, NULL, NULL, 0, NULL, 0);
+			if (e) return e;
+
+			u32 i, j;
+			Bool wide = GF_FALSE;
+			Bool alpha = ctx->unpack_alpha;
+			u32 Bpp = ctx->unpack_yuv ? 1 : (alpha ? 4 : 3);
+			u32 bshift = 0;
+			if (ctx->unpack_yuv && (ctx->unpack_depth>8)) {
+				wide = GF_TRUE;
+				Bpp = 2;
+				bshift = 16 - ctx->unpack_depth;
+			}
+			for (j=0; j<ctx->h; j++) {
+				u8 *p_u=NULL, *p_v=NULL, *p_a=NULL;
+				u8 *p_y = ctx->unpack_buf + j * ctx->w * Bpp;
+				if (ctx->unpack_yuv) {
+					p_u = p_y + ctx->h * ctx->w * Bpp;
+					p_v = p_u + ctx->h * ctx->w * Bpp;
+					if (alpha)
+						p_a = p_v + ctx->h * ctx->w * Bpp;
+				}
+
+				for (i=0; i<ctx->w; i++) {
+					if (ctx->unpack_yuv) {
+						if (wide) {
+							u64 col = gf_evg_stencil_get_pixel_wide_fast(ctx->tx, i ,j);
+							*(u16*)p_y = GF_COLW_R(col) >> bshift;
+							*(u16*)p_u = GF_COLW_G(col) >> bshift;
+							*(u16*)p_v = GF_COLW_B(col) >> bshift;
+							if (alpha)
+								*(u16*)p_a = GF_COLW_A(col) >> bshift;
+						} else {
+							u32 col = gf_evg_stencil_get_pixel_fast(ctx->tx, i ,j);
+							*p_y = GF_COL_R(col);
+							*p_u = GF_COL_G(col);
+							*p_v = GF_COL_B(col);
+							if (alpha)
+								*p_a = GF_COL_A(col);
+						}
+						p_y+=Bpp;
+						p_u+=Bpp;
+						p_v+=Bpp;
+						if (alpha)
+							p_a+=Bpp;
+					} else {
+						u32 col = gf_evg_stencil_get_pixel_fast(ctx->tx, i ,j);
+						p_y[0] = GF_COL_R(col);
+						p_y[1] = GF_COL_G(col);
+						p_y[2] = GF_COL_B(col);
+						if (alpha) {
+							p_y[3] = GF_COL_A(col);
+							p_y+=4;
+						} else {
+							p_y+=3;
+						}
+					}
+				}
+			}
+
+			src_planes[0] = ctx->unpack_buf;
+			if (ctx->unpack_yuv) {
+				src_planes[1] = src_planes[0] + ctx->src_stride[0]*ctx->h;
+				src_planes[2] = src_planes[1] + ctx->src_stride[1]*ctx->h;
+			}
+		} else {
+			if (ctx->nb_src_planes==1) {
+			} else if (ctx->nb_src_planes==2) {
+				src_planes[1] = src_planes[0] + ctx->src_stride[0]*ctx->h;
+			} else if (ctx->nb_src_planes==3) {
+				src_planes[1] = src_planes[0] + ctx->src_stride[0] * ctx->h;
+				src_planes[2] = src_planes[1] + ctx->src_stride[1] * ctx->src_uv_height;
+			} else if (ctx->nb_src_planes==4) {
+				src_planes[1] = src_planes[0] + ctx->src_stride[0] * ctx->h;
+				src_planes[2] = src_planes[1] + ctx->src_stride[1] * ctx->src_uv_height;
+				src_planes[3] = src_planes[2] + ctx->src_stride[2] * ctx->src_uv_height;
+			}
 		}
+
 	} else if (frame_ifce && frame_ifce->get_plane) {
 		u32 i=0;
 		for (i=0; i<ctx->nb_src_planes; i++) {
@@ -148,6 +343,12 @@ static GF_Err ffsws_process(GF_Filter *filter)
 		gf_filter_pid_drop_packet(ctx->ipid);
 		return GF_NOT_SUPPORTED;
 	}
+
+	pck_data = output;
+	if (ctx->repack_v410 || ctx->repack_v210 || ctx->repack_generic) {
+		output = ctx->repack_buf;
+	}
+
 	dst_planes[0] = output;
 	dst_planes[0] += ctx->dst_stride[0] * ctx->offset_h + ctx->offset_w*ctx->o_bpp;
 
@@ -194,7 +395,113 @@ static GF_Err ffsws_process(GF_Filter *filter)
 		return GF_NOT_SUPPORTED;
 	}
 
-	if (ctx->swap_idx_1 || ctx->swap_idx_2) {
+
+	if (ctx->repack_v410) {
+		u32 i, j;
+		output = pck_data;
+		for (j=0; j<ctx->oh; j++) {
+			u8 *p_dst = output + j * ctx->repack_stride;
+			u8 *p_y = ctx->repack_buf + j * ctx->ow * 2;
+			u8 *p_u = p_y + ctx->oh * ctx->ow * 2;
+			u8 *p_v = p_u + ctx->oh * ctx->ow * 2;
+
+			for (i=0; i<ctx->ow; i++) {
+				u32 val = 0;
+				val = *(u16 *) p_v; val<<=10;
+				val |= *(u16 *) p_y; val<<=10;
+				val |= *(u16 *) p_u; val<<=2;
+
+				p_dst[0] = val & 0xFF;
+				p_dst[1] = (val>>8) & 0xFF;
+				p_dst[2] = (val>>16) & 0xFF;
+				p_dst[3] = (val>>24) & 0xFF;
+
+				p_u+=2;
+				p_v+=2;
+				p_y+=2;
+				p_dst+=4;
+			}
+		}
+	}
+	else if (ctx->repack_v210) {
+		u32 i, j;
+		output = pck_data;
+		for (j=0; j<ctx->oh; j++) {
+			u8 *p_dst = output + j * ctx->repack_stride;
+			u8 *p_y = ctx->repack_buf + j * ctx->ow * 2;
+			u8 *p_u = ctx->repack_buf + ctx->oh * ctx->ow * 2 + j * ctx->ow;
+			u8 *p_v = p_u + ctx->oh * ctx->ow;
+			u32 cur_uv_w=0;
+
+			for (i=0; i<ctx->ow; i+=6) {
+				u32 val = 0;
+				val = *(u16 *) p_u;
+				p_u+=2;
+				val |= ((u32) *(u16 *) p_y) << 10;
+				p_y+=2;
+				val |= ((u32) *(u16 *) p_v) << 20;
+				p_v+=2;
+
+#define PUT_WORD_LE \
+				p_dst[0] = val & 0xFF;\
+				p_dst[1] = (val>>8) & 0xFF;\
+				p_dst[2] = (val>>16) & 0xFF;\
+				p_dst[3] = (val>>24) & 0xFF;\
+				p_dst+=4;\
+
+				PUT_WORD_LE
+
+				val = *(u16 *) p_y;
+				p_y+=2;
+
+				cur_uv_w+=2;
+				if (cur_uv_w>=ctx->ow) {
+					PUT_WORD_LE
+					break;
+				}
+				val |= ((u32) *(u16 *) p_u) << 10;
+				p_u+=2;
+				val |= ((u32) *(u16 *) p_y) << 20;
+				p_y+=2;
+
+				PUT_WORD_LE
+
+				val = *(u16 *) p_v;
+				p_v+=2;
+				val |= ((u32) *(u16 *) p_y) << 10;
+				p_y+=2;
+
+				cur_uv_w+=2;
+				if (cur_uv_w>=ctx->ow) {
+					PUT_WORD_LE
+					break;
+				}
+
+				val |= ((u32) *(u16 *) p_u) << 20;
+				p_u+=2;
+
+				PUT_WORD_LE
+
+				val = *(u16 *) p_y;
+				p_y+=2;
+				val |= ((u32) *(u16 *) p_v) << 10;
+				p_v+=2;
+				val |= ((u32) *(u16 *) p_y) << 20;
+				p_y+=2;
+
+				PUT_WORD_LE
+
+				cur_uv_w+=2;
+				if (cur_uv_w>=ctx->ow)
+					break;
+			}
+		}
+	} else if (ctx->repack_generic) {
+		gf_evg_surface_attach_to_buffer(ctx->surf, pck_data, ctx->ow, ctx->oh, 0, ctx->repack_stride, ctx->ofmt);
+		gf_evg_stencil_set_texture(ctx->tx, ctx->repack_buf, ctx->ow, ctx->oh, ctx->dst_stride[0], ctx->repack_generic);
+		gf_evg_surface_set_path(ctx->surf, ctx->path);
+		gf_evg_surface_fill(ctx->surf, ctx->tx);
+	} else if (ctx->swap_idx_1 || ctx->swap_idx_2) {
 		u32 i, j;
 		for (i=0; i<ctx->h; i++) {
 			u8 *dst = output + ctx->dst_stride[0]*i;
@@ -400,8 +707,62 @@ static GF_Err ffsws_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		nb_par = 0;
 		Bool res;
 		u32 mode = get_sws_mode(ctx->scale, &nb_par);
-		u32 ff_src_pfmt = ffmpeg_pixfmt_from_gpac(ofmt);
-		u32 ff_dst_pfmt = ffmpeg_pixfmt_from_gpac(ctx->ofmt);
+
+		u32 ff_src_pfmt, ff_dst_pfmt;
+		ctx->unpack_v410 = ctx->repack_v410 = GF_FALSE;
+		ctx->unpack_v210 = ctx->repack_v210 = GF_FALSE;
+		ctx->unpack_generic = ctx->repack_generic = 0;
+
+		//v410 and v210 as treated as codecs in ffmpeg, we deal with them as pixel formats
+		if (ofmt == GF_PIXEL_YUV444_10_PACK) {
+			ctx->unpack_v410 = GF_TRUE;
+			ff_src_pfmt = ffmpeg_pixfmt_from_gpac(GF_PIXEL_YUV444_10, GF_FALSE);
+		} else if (ofmt == GF_PIXEL_V210) {
+			ctx->unpack_v210 = GF_TRUE;
+			ff_src_pfmt = ffmpeg_pixfmt_from_gpac(GF_PIXEL_YUV422_10, GF_FALSE);
+		} else {
+			ff_src_pfmt = ffmpeg_pixfmt_from_gpac(ofmt, GF_TRUE);
+			if (ff_src_pfmt==AV_PIX_FMT_NONE) {
+				if (gf_evg_stencil_set_texture(ctx->tx, (u8*)ctx, 4, 2, 0, ofmt)==GF_OK) {
+					ctx->unpack_yuv = gf_pixel_fmt_is_yuv(ofmt);
+					ctx->unpack_depth = gf_pixel_is_wide_depth(ofmt);
+					ctx->unpack_alpha = gf_pixel_fmt_is_transparent(ofmt);
+					if (ctx->unpack_yuv) {
+						if (ctx->unpack_alpha) ctx->unpack_generic = GF_PIXEL_YUVA444;
+						else ctx->unpack_generic = (ctx->unpack_depth>8) ? GF_PIXEL_YUV444_10 : GF_PIXEL_YUV444;
+					} else {
+						ctx->unpack_generic = ctx->unpack_alpha ? GF_PIXEL_RGBA : GF_PIXEL_RGB;
+					}
+					ff_src_pfmt = ffmpeg_pixfmt_from_gpac(ctx->unpack_generic, GF_FALSE);
+				}
+			}
+		}
+		if (ctx->ofmt == GF_PIXEL_YUV444_10_PACK) {
+			ctx->repack_v410 = GF_TRUE;
+			ff_dst_pfmt = ffmpeg_pixfmt_from_gpac(GF_PIXEL_YUV444_10, GF_FALSE);
+		} else if (ctx->ofmt == GF_PIXEL_V210) {
+			ctx->repack_v210 = GF_TRUE;
+			ff_dst_pfmt = ffmpeg_pixfmt_from_gpac(GF_PIXEL_YUV422_10, GF_FALSE);
+		} else {
+			ff_dst_pfmt = ffmpeg_pixfmt_from_gpac(ctx->ofmt, GF_TRUE);
+			if (ff_dst_pfmt == AV_PIX_FMT_NONE) {
+				if (gf_evg_surface_attach_to_buffer(ctx->surf, (u8 *) ctx, 4, 2, 0, 0, ctx->ofmt)==GF_OK) {
+					ctx->repack_yuv = gf_pixel_fmt_is_yuv(ctx->ofmt);
+					ctx->repack_depth = gf_pixel_is_wide_depth(ctx->ofmt);
+					ctx->repack_alpha = gf_pixel_fmt_is_transparent(ctx->ofmt);
+					if (ctx->repack_yuv) {
+						if (ctx->repack_alpha) ctx->repack_generic = GF_PIXEL_YUVA444;
+						else ctx->repack_generic = (ctx->repack_depth>8) ? GF_PIXEL_YUV444_10 : GF_PIXEL_YUV444;
+					} else {
+						ctx->repack_generic = ctx->repack_alpha ? GF_PIXEL_RGBA : GF_PIXEL_RGB;
+					}
+					ff_dst_pfmt = ffmpeg_pixfmt_from_gpac(ctx->repack_generic, GF_FALSE);
+				}
+			}
+		}
+
+		if ((ff_src_pfmt==AV_PIX_FMT_NONE) || (ff_dst_pfmt==AV_PIX_FMT_NONE))
+			return GF_NOT_SUPPORTED;
 
 		//get layout info for source
 		memset(ctx->src_stride, 0, sizeof(ctx->src_stride));
@@ -413,8 +774,31 @@ static GF_Err ffsws_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[FFSWS] Failed to query source pixel format characteristics\n"));
 			return GF_NOT_SUPPORTED;
 		}
-		if (ctx->nb_src_planes==3) ctx->src_stride[2] = ctx->src_stride[1];
-		if (ctx->nb_src_planes==4) ctx->src_stride[3] = ctx->src_stride[0];
+		if (ctx->unpack_v410) {
+			ctx->unpack_buf = gf_realloc(ctx->unpack_buf, w*h*3*2);
+			ctx->nb_src_planes = 3;
+			ctx->orig_in_stride = ctx->src_stride[0];
+			ctx->src_stride[2] = ctx->src_stride[1] = ctx->src_stride[0] = 2*w;
+			ctx->src_uv_height = h;
+		} else if (ctx->unpack_v210) {
+			ctx->unpack_buf = gf_realloc(ctx->unpack_buf, w*h*2*2);
+			ctx->nb_src_planes = 3;
+			ctx->orig_in_stride = ctx->src_stride[0];
+			ctx->src_stride[0] = 2*w;
+			ctx->src_stride[2] = ctx->src_stride[1] = w;
+			ctx->src_uv_height = h;
+		} else if (ctx->unpack_generic) {
+			u32 planes = ctx->unpack_alpha ? 4 : 3;
+			u32 Bpp = (ctx->unpack_depth>8) ? 2 : 1;
+			ctx->unpack_buf = gf_realloc(ctx->unpack_buf, w*h*planes*Bpp);
+			ctx->nb_src_planes = ctx->unpack_yuv ? planes : 1;
+			ctx->orig_in_stride = ctx->src_stride[0];
+			ctx->src_stride[2] = ctx->src_stride[1] = ctx->src_stride[0] = Bpp*w;
+			ctx->src_uv_height = h;
+		} else {
+			if (ctx->nb_src_planes==3) ctx->src_stride[2] = ctx->src_stride[1];
+			if (ctx->nb_src_planes==4) ctx->src_stride[3] = ctx->src_stride[0];
+		}
 
 		ctx->o_bpp = gf_pixel_get_bytes_per_pixel(ofmt);
 		//get layout info for dest
@@ -427,7 +811,36 @@ static GF_Err ffsws_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 		if (ctx->nb_planes==3) ctx->dst_stride[2] = ctx->dst_stride[1];
 		if (ctx->nb_planes==4) ctx->dst_stride[3] = ctx->dst_stride[0];
 
+		if (ctx->repack_v410) {
+			ctx->nb_planes = 3;
+			ctx->repack_stride = ctx->dst_stride[0];
+			ctx->repack_buf = gf_realloc(ctx->repack_buf, ctx->ow * ctx->oh * 3 * 2);
+			ctx->dst_stride[2] = ctx->dst_stride[1] = ctx->dst_stride[0] = 2 * ctx->ow;
+			ctx->dst_uv_height = ctx->oh;
+			ctx->o_bpp = 2;
+		}
+		else if (ctx->repack_v210) {
+			ctx->nb_planes = 3;
+			ctx->repack_stride = ctx->dst_stride[0];
+			ctx->repack_buf = gf_realloc(ctx->repack_buf, ctx->ow * ctx->oh * 2 * 2);
+			ctx->dst_stride[0] = 2 * ctx->ow;
+			ctx->dst_stride[2] = ctx->dst_stride[1] = ctx->ow;
+			ctx->dst_uv_height = ctx->oh;
+			ctx->o_bpp = 2;
+		}
+		else if (ctx->repack_generic) {
+			u32 osize;
+			ctx->repack_stride = ctx->dst_stride[0];
+			ctx->dst_stride[0] = ctx->dst_stride[1] = ctx->dst_stride[2] = ctx->dst_stride[3] = 0;
+			gf_pixel_get_size_info(ctx->repack_generic, ctx->ow, ctx->oh, &osize, &ctx->dst_stride[0] , &ctx->dst_stride[1], &ctx->nb_planes, &ctx->dst_uv_height);
+			ctx->repack_buf = gf_realloc(ctx->repack_buf, osize);
+			ctx->dst_stride[2] = ctx->dst_stride[1];
+			if (ctx->repack_alpha)
+				ctx->dst_stride[3] = ctx->dst_stride[0];
 
+			gf_path_reset(ctx->path);
+			gf_path_add_rect_center(ctx->path, 0, 0, INT2FIX(ctx->ow), INT2FIX(ctx->oh));
+		}
 		if (nb_par) {
 			if ((nb_par==1) && (ctx->p1!=GF_MAX_DOUBLE) ) {
 				par[0] = ctx->p1;
@@ -490,9 +903,17 @@ static GF_Err ffsws_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_WIDTH, &PROP_UINT(ctx->ow));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_HEIGHT, &PROP_UINT(ctx->oh));
-	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE, &PROP_UINT(ctx->dst_stride[0]));
-	if (ctx->nb_planes>1)
-		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE_UV, &PROP_UINT(ctx->dst_stride[1]));
+
+	if (ctx->repack_v410 || ctx->repack_v210 || ctx->repack_generic) {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE, &PROP_UINT(ctx->repack_stride));
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE_UV, NULL );
+	} else {
+		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE, &PROP_UINT(ctx->dst_stride[0]));
+		if (ctx->nb_planes>1)
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE_UV, &PROP_UINT(ctx->dst_stride[1]));
+		else
+			gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_STRIDE_UV, NULL );
+	}
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CODECID, &PROP_UINT(GF_CODECID_RAW));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_PIXFMT, &PROP_UINT(ctx->ofmt));
@@ -504,6 +925,12 @@ static GF_Err ffsws_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_
 	} else {
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SAR, NULL );
 	}
+
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CLAP_X, NULL);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CLAP_Y, NULL);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CLAP_W, NULL);
+	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CLAP_H, NULL);
+
 	return GF_OK;
 }
 
@@ -511,7 +938,9 @@ static GF_Err ffsws_initialize(GF_Filter *filter)
 {
 	GF_FFSWScaleCtx *ctx = gf_filter_get_udta(filter);
 	ffmpeg_setup_logs(GF_LOG_MEDIA);
-	ctx->surf = gf_evg_surface_new(GF_FALSE);
+	ctx->surf = gf_evg_surface_new(GF_TRUE);
+	ctx->tx = gf_evg_stencil_new(GF_STENCIL_TEXTURE);
+	ctx->path = gf_path_new();
 
 	if (ctx->osar.num &&
 	 ((ctx->osar.num<0) || (ctx->osar.num < (s32) ctx->osar.den))
@@ -525,7 +954,11 @@ static void ffsws_finalize(GF_Filter *filter)
 {
 	GF_FFSWScaleCtx *ctx = gf_filter_get_udta(filter);
 	if (ctx->swscaler) sws_freeContext(ctx->swscaler);
+	if (ctx->unpack_buf) gf_free(ctx->unpack_buf);
+	if (ctx->repack_buf) gf_free(ctx->repack_buf);
 	gf_evg_surface_delete(ctx->surf);
+	gf_evg_stencil_delete(ctx->tx);
+	gf_path_del(ctx->path);
 	return;
 }
 
@@ -618,6 +1051,7 @@ GF_FilterRegister FFSWSRegister = {
 	.args = FFSWSArgs,
 	.configure_pid = ffsws_configure_pid,
 	SETCAPS(FFSWSCaps),
+	.flags = GF_FS_REG_ALLOW_CYCLIC,
 	.initialize = ffsws_initialize,
 	.finalize = ffsws_finalize,
 	.process = ffsws_process,
