@@ -572,5 +572,8 @@ GF_FilterRegister PipeInRegister = {
 
 const GF_FilterRegister *pipein_register(GF_FilterSession *session)
 {
+	if (gf_opts_get_bool("temp", "get_proto_schemes")) {
+		gf_opts_set_key("temp_in_proto", PipeInRegister.name, "pipe");
+	}
 	return &PipeInRegister;
 }
