@@ -45,9 +45,9 @@ typedef struct
 	char address[GF_MAX_IP_NAME_LEN];
 
 	u64 start_time, last_stats_time;
+	u32 init_time;
 	u64 nb_bytes;
 	Bool done;
-
 } GF_SockInClient;
 
 typedef struct
@@ -65,16 +65,19 @@ typedef struct
 	u32 reorder_pck;
 	u32 reorder_delay;
 #endif
+	GF_PropStringList ssm, ssmx;
 
 	GF_SockInClient sock_c;
 	GF_List *clients;
 	Bool had_clients;
 	Bool is_udp;
+	Bool is_stop;
 
 	char *buffer;
 
 	GF_SockGroup *active_sockets;
-	u64 last_rcv_time;
+	u32 last_rcv_time;
+	u32 last_timeout_sec;
 } GF_SockInCtx;
 
 
@@ -117,7 +120,6 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 		GF_LOG(GF_LOG_ERROR, GF_LOG_NETWORK, ("[SockIn] Failed to open socket for %s\n", ctx->src));
 		return GF_IO_ERR;
 	}
-	gf_sk_group_register(ctx->active_sockets, ctx->sock_c.socket);
 
 	/*setup port and src*/
 	port = ctx->port;
@@ -129,9 +131,10 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 		str[0] = 0;
 	}
 
+
 	/*do we have a source ?*/
 	if (gf_sk_is_multicast_address(url)) {
-		e = gf_sk_setup_multicast(ctx->sock_c.socket, url, port, 0, 0, ctx->ifce);
+		e = gf_sk_setup_multicast_ex(ctx->sock_c.socket, url, port, 0, 0, ctx->ifce, (const char **)ctx->ssm.vals, ctx->ssm.nb_items, (const char **)ctx->ssmx.vals, ctx->ssmx.nb_items);
 		ctx->listen = GF_FALSE;
 	} else if ((sock_type==GF_SOCK_TYPE_UDP) 
 #ifdef GPAC_HAS_SOCK_UN 
@@ -155,6 +158,9 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 		e = gf_sk_connect(ctx->sock_c.socket, url, port, NULL);
 	}
 
+	strcpy(ctx->sock_c.address, "unknown");
+	gf_sk_get_remote_address(ctx->sock_c.socket, ctx->sock_c.address);
+
 	if (str) str[0] = ':';
 
 	if (e) {
@@ -163,6 +169,8 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 		return e;
 	}
 
+	gf_sk_group_register(ctx->active_sockets, ctx->sock_c.socket);
+
 	GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[SockIn] opening %s%s\n", ctx->src, ctx->listen ? " in server mode" : ""));
 
 	if (ctx->block_size<2000)
@@ -176,6 +184,9 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 	gf_sk_set_buffer_size(ctx->sock_c.socket, 0, ctx->block_size);
 	gf_sk_set_block_mode(ctx->sock_c.socket, (!ctx->is_udp && ctx->block) ? GF_FALSE : GF_TRUE);
 
+	if (!ctx->is_udp)
+		gf_filter_set_blocking(filter, GF_TRUE);
+
 	ctx->buffer = gf_malloc(ctx->block_size + 1);
 	if (!ctx->buffer) return GF_OUT_OF_MEM;
 	//ext/mime given and not mpeg2, disable probe
@@ -186,6 +197,9 @@ static GF_Err sockin_initialize(GF_Filter *filter)
 		ctx->clients = gf_list_new();
 		if (!ctx->clients) return GF_OUT_OF_MEM;
 	}
+
+	ctx->sock_c.init_time = gf_sys_clock();
+
 	return GF_OK;
 }
 
@@ -243,11 +257,14 @@ static void sockin_rtp_destructor(GF_Filter *filter, GF_FilterPid *pid, GF_Filte
 static Bool sockin_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 {
 	if (!evt->base.on_pid) return GF_FALSE;
+	GF_SockInCtx *ctx = (GF_SockInCtx *) gf_filter_get_udta(filter);
 
 	switch (evt->base.type) {
 	case GF_FEVT_PLAY:
+		ctx->is_stop = GF_FALSE;
 		return GF_TRUE;
 	case GF_FEVT_STOP:
+		ctx->is_stop = GF_TRUE;
 		return GF_TRUE;
 	default:
 		break;
@@ -273,7 +290,6 @@ static GF_Err sockin_read_client(GF_Filter *filter, GF_SockInCtx *ctx, GF_SockIn
 	}
 
 	if (!sock_c->start_time) sock_c->start_time = gf_sys_clock_high_res();
-
 	pos = 0;
 	nb_read=0;
 	while (pos < ctx->block_size) {
@@ -308,6 +324,11 @@ static GF_Err sockin_read_client(GF_Filter *filter, GF_SockInCtx *ctx, GF_SockIn
 		pos += read;
 	}
 	if (!nb_read) return GF_OK;
+
+	if (!sock_c->nb_bytes) {
+		GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[SockIn] Reception started after %u ms\n", gf_sys_clock() - sock_c->init_time));
+	}
+
 	sock_c->nb_bytes += nb_read;
 	sock_c->done = GF_FALSE;
 
@@ -399,24 +420,37 @@ static GF_Err sockin_read_client(GF_Filter *filter, GF_SockInCtx *ctx, GF_SockIn
 	return GF_OK;
 }
 
-static Bool sockin_check_eos(GF_SockInCtx *ctx)
+static GF_Err sockin_check_eos(GF_Filter *filter, GF_SockInCtx *ctx)
 {
-	u64 now;
-	if (!ctx->timeout) return GF_FALSE;
+	u32 now;
+	if (!ctx->timeout) return GF_OK;
 
-	now = gf_sys_clock_high_res();
+	now = gf_sys_clock();
 	if (!ctx->last_rcv_time) {
 		ctx->last_rcv_time = now;
-		return GF_FALSE;
+		return GF_OK;
 	}
-	if (now - ctx->last_rcv_time < ctx->timeout*1000) {
-		return GF_FALSE;
+	if (now - ctx->last_rcv_time < ctx->timeout) {
+		u32 tout = (ctx->timeout - (now - ctx->last_rcv_time)) / 1000;
+		if (tout != ctx->last_timeout_sec) {
+			ctx->last_timeout_sec = tout;
+			GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[SockIn] Waiting for %u seconds\r", tout));
+		}
+		return GF_OK;
 	}
-	if (ctx->sock_c.pid && !ctx->sock_c.done) {
-		gf_filter_pid_set_eos(ctx->sock_c.pid);
+	if (!ctx->sock_c.done) {
+		if (ctx->sock_c.pid)
+			gf_filter_pid_set_eos(ctx->sock_c.pid);
 		ctx->sock_c.done = GF_TRUE;
+		if (ctx->sock_c.nb_bytes) {
+			GF_LOG(GF_LOG_INFO, GF_LOG_NETWORK, ("[SockIn] No data received for %d ms, assuming end of stream\n", ctx->timeout));
+		} else {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_NETWORK, ("[SockIn] No data received after %d ms, aborting\n", ctx->timeout));
+			gf_filter_setup_failure(filter, GF_IP_NETWORK_FAILURE);
+			return GF_IP_NETWORK_FAILURE;
+		}
 	}
-	return GF_TRUE;
+	return GF_EOS;
 }
 
 static GF_Err sockin_process(GF_Filter *filter)
@@ -426,11 +460,17 @@ static GF_Err sockin_process(GF_Filter *filter)
 	u32 i, count;
 	GF_SockInCtx *ctx = (GF_SockInCtx *) gf_filter_get_udta(filter);
 
+	if (ctx->is_stop) return GF_EOS;
+
 	e = gf_sk_group_select(ctx->active_sockets, 1, GF_SK_SELECT_READ);
 	if (e==GF_IP_NETWORK_EMPTY) {
 		if (ctx->is_udp) {
-			if (sockin_check_eos(ctx) )
-				return GF_EOS;
+			e = sockin_check_eos(filter, ctx);
+			if (e) return e;
+			if (!ctx->sock_c.nb_bytes) {
+				gf_filter_ask_rt_reschedule(filter, 10000);
+				return GF_OK;
+			}
 		} else if (!gf_list_count(ctx->clients)) {
 			gf_filter_ask_rt_reschedule(filter, 1000);
 			return GF_OK;
@@ -467,6 +507,7 @@ static GF_Err sockin_process(GF_Filter *filter)
 				gf_list_add(ctx->clients, sc);
 				ctx->had_clients = GF_TRUE;
 				gf_sk_group_register(ctx->active_sockets, sc->socket);
+				sc->init_time = gf_sys_clock();
 			}
 		}
 	}
@@ -537,6 +578,9 @@ static const GF_FilterArgs SockInArgs[] =
 	{ OFFS(reorder_pck), "number of packets delay for RTP reordering (M2TS over RTP) ", GF_PROP_UINT, "100", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(reorder_delay), "number of ms delay for RTP reordering (M2TS over RTP)", GF_PROP_UINT, "10", NULL, GF_FS_ARG_HINT_ADVANCED},
 #endif
+
+	{ OFFS(ssm), "list of IP to include for source-specific multicast", GF_PROP_STRING_LIST, NULL, NULL, GF_FS_ARG_HINT_EXPERT},
+	{ OFFS(ssmx), "list of IP to exclude for source-specific multicast", GF_PROP_STRING_LIST, NULL, NULL, GF_FS_ARG_HINT_EXPERT},
 	{0}
 };
 
@@ -583,6 +627,13 @@ GF_FilterRegister SockInRegister = {
 
 const GF_FilterRegister *sockin_register(GF_FilterSession *session)
 {
+	if (gf_opts_get_bool("temp", "get_proto_schemes")) {
+#ifdef GPAC_HAS_SOCK_UN
+		gf_opts_set_key("temp_in_proto", SockInRegister.name, "tcp,udp,tcpu,udpu");
+#else
+		gf_opts_set_key("temp_in_proto", SockInRegister.name, "tcp,udp");
+#endif
+	}
 	return &SockInRegister;
 }
 
