@@ -107,7 +107,7 @@ GF_Err ac3dmx_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 		if (p && p->value.string && strstr(p->value.string, "eac3")) ctx->is_eac3 = GF_TRUE;
 		else {
 			p = gf_filter_pid_get_property(pid, GF_PROP_PID_FILE_EXT);
-			if (p && p->value.string && strstr(p->value.string, "eac3")) ctx->is_eac3 = GF_TRUE;
+			if (p && p->value.string && (strstr(p->value.string, "eac3")||strstr(p->value.string, "ec3"))) ctx->is_eac3 = GF_TRUE;
 		}
 	}
 	if (ctx->is_eac3) {
@@ -211,7 +211,7 @@ static void ac3dmx_check_pid(GF_Filter *filter, GF_AC3DmxCtx *ctx)
 		ctx->opid = gf_filter_pid_new(filter);
 		ac3dmx_check_dur(filter, ctx);
 	}
-	if ((ctx->sample_rate == ctx->hdr.sample_rate) && (ctx->nb_ch == ctx->hdr.channels) && !ctx->copy_props) return;
+	if ((ctx->sample_rate == ctx->hdr.sample_rate) && (ctx->nb_ch == ctx->hdr.streams[0].channels) && !ctx->copy_props) return;
 
 	ctx->copy_props = GF_FALSE;
 	//copy properties at init or reconfig
@@ -228,7 +228,7 @@ static void ac3dmx_check_pid(GF_Filter *filter, GF_AC3DmxCtx *ctx)
 		gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_CAN_DATAREF, & PROP_BOOL(GF_TRUE ) );
 
 
-	ctx->nb_ch = ctx->hdr.channels;
+	ctx->nb_ch = ctx->hdr.streams[0].channels;
 	if (!ctx->timescale) {
 		//we change sample rate, change cts
 		if (ctx->cts && (ctx->sample_rate != ctx->hdr.sample_rate)) {
@@ -267,8 +267,6 @@ static Bool ac3dmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		if (!ctx->is_playing) {
 			ctx->is_playing = GF_TRUE;
 			ctx->cts = 0;
-			ctx->ac3_buffer_size = 0;
-			ctx->resume_from = 0;
 		}
 		if (! ctx->is_file) {
 			return GF_FALSE;
@@ -304,6 +302,7 @@ static Bool ac3dmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 	case GF_FEVT_STOP:
 		//don't cancel event
 		ctx->is_playing = GF_FALSE;
+		ctx->cts = 0;
 		return GF_FALSE;
 
 	case GF_FEVT_SET_SPEED:
@@ -367,6 +366,12 @@ GF_Err ac3dmx_process(GF_Filter *filter)
 			return GF_OK;
 		}
 
+		//max EAC3 frame is 4096, AC3 is 3840 - if we store more than 2 frames consider we have garbage
+		if (ctx->ac3_buffer_size>10000) {
+			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[AC3Dmx] Trashing %d garbage bytes\n", ctx->ac3_buffer_size));
+			ctx->ac3_buffer_size = 0;
+		}
+
 		if (ctx->byte_offset != GF_FILTER_NO_BO) {
 			u64 byte_offset = gf_filter_pck_get_byte_offset(pck);
 			if (!ctx->ac3_buffer_size) {
@@ -390,6 +395,9 @@ GF_Err ac3dmx_process(GF_Filter *filter)
 	//input pid sets some timescale - we flushed pending data , update cts
 	if (ctx->timescale && pck) {
 		cts = gf_filter_pck_get_cts(pck);
+		//init cts at first packet
+		if (!ctx->cts && (cts != GF_FILTER_NO_TS))
+			ctx->cts = cts;
 	}
 
 	if (cts == GF_FILTER_NO_TS) {
@@ -416,14 +424,18 @@ GF_Err ac3dmx_process(GF_Filter *filter)
 		Bool res;
 		u32 sync_pos, bytes_to_drop=0;
 
-
 		res = ctx->ac3_parser_bs(ctx->bs, &ctx->hdr, GF_TRUE);
 
 		sync_pos = (u32) gf_bs_get_position(ctx->bs);
 
 		//startcode not found or not enough bytes, gather more
-		if (!res || (remain < sync_pos + ctx->hdr.framesize))
+		if (!res || (remain < sync_pos + ctx->hdr.framesize)) {
+			if (sync_pos && ctx->hdr.framesize) {
+				start += sync_pos;
+				remain -= sync_pos;
+			}
 			break;
+		}
 
 		ac3dmx_check_pid(filter, ctx);
 
@@ -505,7 +517,7 @@ GF_Err ac3dmx_process(GF_Filter *filter)
 		ctx->ac3_buffer_size = 0;
 		return ac3dmx_process(filter);
 	} else {
-		if (remain) {
+		if (remain && (remain < ctx->ac3_buffer_size)) {
 			memmove(ctx->ac3_buffer, start, remain);
 		}
 		ctx->ac3_buffer_size = remain;
@@ -522,41 +534,88 @@ static void ac3dmx_finalize(GF_Filter *filter)
 	if (ctx->indexes) gf_free(ctx->indexes);
 }
 
-static const char *ac3dmx_probe_data(const u8 *data, u32 size, GF_FilterProbeScore *score)
+static const char *ac3dmx_probe_data(const u8 *_data, u32 _size, GF_FilterProbeScore *score)
 {
-	u32 nb_frames=0;
+	GF_AC3Config ahdr;
+	u32 i, nb_frames=0;
 	Bool has_broken_frames = GF_FALSE;
 	u32 pos=0;
+	const u8 *data = _data;
+	u32 size = _size;
+	u32 nb_ac3_frames=0;
+
+	//check AC3
 	while (1) {
-		GF_AC3Config ahdr;
-		if (! gf_ac3_parser((u8 *) data, size, &pos, &ahdr, GF_FALSE) )
+		ahdr.sample_rate = 0;
+		if (! gf_ac3_parser((u8 *) data, size, &pos, &ahdr, GF_FALSE) ) {
+			if (ahdr.sample_rate) nb_frames++;
 		 	break;
+		}
 		u32 fsize = ahdr.framesize;
 		if (pos) {
 			nb_frames=0;
 			has_broken_frames = GF_TRUE;
-			//what is before is bigger than max ac3 frame size (1920), this is packaged ac3 (mkv) at best
-			if (pos > 2000)
+			//what is before is bigger than max ac3 frame size (1920 words), this is packaged ac3 (mkv) at best
+			if (pos > 4000)
 				break;
 		}
 		nb_frames++;
-		if (fsize > size+pos) break;
+		if (fsize > size+pos) {
+			if (!pos && (nb_frames==1)) nb_frames++;
+			break;
+		}
 		if (nb_frames>4) break;
 		if (size < fsize+pos) break;
 		size -= fsize+pos;
 		data += fsize+pos;
 	}
-	if (nb_frames>2) {
+	nb_ac3_frames = nb_frames;
+
+	//check EAC3
+	pos=0;
+	data = _data;
+	size = _size;
+	nb_frames = 0;
+	GF_BitStream *bs = gf_bs_new(data, size, GF_BITSTREAM_READ);
+	while (gf_bs_available(bs)) {
+		ahdr.sample_rate = 0;
+		if (!gf_eac3_parser_bs(bs, &ahdr, GF_FALSE)) {
+			if (ahdr.sample_rate) nb_frames++;
+			break;
+		}
+
+		if (pos != (u32) gf_bs_get_position(bs))
+			has_broken_frames = GF_TRUE;
+		nb_frames += ahdr.nb_streams;
+		for (i=0; i<ahdr.nb_streams; i++)
+			nb_frames += ahdr.streams[i].nb_dep_sub;
+		gf_bs_skip_bytes(bs, ahdr.framesize);
+		if (!pos && (nb_frames==1) && !gf_bs_available(bs)) nb_frames++;
+		pos+=ahdr.framesize;
+		if (nb_frames>4) break;
+	}
+	gf_bs_del(bs);
+
+	if (nb_frames<nb_ac3_frames) {
+		nb_frames = 0;
+	}
+	if (nb_ac3_frames>=2) {
 		*score = has_broken_frames ? GF_FPROBE_MAYBE_NOT_SUPPORTED : GF_FPROBE_SUPPORTED;
 		return "audio/ac3";
 	}
+
+	if (nb_frames>=2) {
+		*score = has_broken_frames ? GF_FPROBE_MAYBE_NOT_SUPPORTED : GF_FPROBE_SUPPORTED;
+		return "audio/eac3";
+	}
+
 	return NULL;
 }
 
 static const GF_FilterCapability AC3DmxCaps[] =
 {
 	CAP_UINT(GF_CAPS_INPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_FILE),
-	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_FILE_EXT, "ac3|eac3"),
+	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_FILE_EXT, "ac3|eac3|ec3"),
 	CAP_STRING(GF_CAPS_INPUT, GF_PROP_PID_MIME, "audio/x-ac3|audio/ac3|audio/x-eac3|audio/eac3"),
 	CAP_UINT(GF_CAPS_OUTPUT, GF_PROP_PID_STREAM_TYPE, GF_STREAM_AUDIO),
 	CAP_UINT(GF_CAPS_OUTPUT_STATIC, GF_PROP_PID_CODECID, GF_CODECID_AC3),
