@@ -27,6 +27,8 @@
 #include <gpac/constants.h>
 #include <gpac/filters.h>
 
+#if !defined(GPAC_DISABLE_AV_PARSERS)
+
 
 static u32 USACSampleRates[] = {96000, 88200, 64000, 48000, 44100, 32000, 24000, 22050, 16000, 12000, 11025, 8000, 7350, 0, 0,
 	57600, 51200, 40000, 38400, 34150, 28800, 25600, 20000, 19200, 17075, 14400, 12800, 9600};
@@ -84,6 +86,7 @@ typedef struct
 	u32 nb_unknown_pck;
 	u32 bitrate;
 	Bool copy_props;
+	Bool is_sync;
 } GF_MHASDmxCtx;
 
 
@@ -195,12 +198,11 @@ static void mhas_dmx_check_dur(GF_Filter *filter, GF_MHASDmxCtx *ctx)
 			if (!mhas_last_cfg) mhas_sap = 0;
 		//config
 		} else if (mhas_type==1) {
-			u32 sr = 0;
 			/*u32 pl = */gf_bs_read_u8(bs);
 			u32 idx = gf_bs_read_int(bs, 5);
 			if (idx==0x1f)
 				duration.den = gf_bs_read_int(bs, 24);
-			else if (sr < nb_usac_sr) {
+			else if (idx < nb_usac_sr) {
 				duration.den = USACSampleRates[idx];
 			}
 			idx = gf_bs_read_int(bs, 3);
@@ -390,6 +392,7 @@ static Bool mhas_dmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		ctx->is_playing = GF_FALSE;
 		if (ctx->src_pck) gf_filter_pck_unref(ctx->src_pck);
 		ctx->src_pck = NULL;
+		ctx->cts = 0;
 		//don't cancel event
 		return GF_FALSE;
 
@@ -513,6 +516,9 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 	//input pid sets some timescale - we flushed pending data , update cts
 	if (ctx->timescale && in_pck) {
 		cts = gf_filter_pck_get_cts(in_pck);
+		//init cts at first packet
+		if (!ctx->cts && (cts != GF_FILTER_NO_TS))
+			ctx->cts = cts;
 	}
 
 	if (cts == GF_FILTER_NO_TS) {
@@ -573,14 +579,16 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 		if (mhas_type>18) {
 			ctx->nb_unknown_pck++;
 			if (ctx->nb_unknown_pck > ctx->pcksync) {
-				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MHASDmx] %d packets of unknwon type, considering sync was lost\n"));
+				GF_LOG(ctx->is_sync ? GF_LOG_WARNING : GF_LOG_DEBUG, GF_LOG_MEDIA, ("[MHASDmx] %d packets of unknown type, considering sync was lost\n"));
+				ctx->is_sync = GF_FALSE;
 				consumed = 0;
 				ctx->nosync = GF_TRUE;
 				ctx->nb_unknown_pck = 0;
 				break;
 			}
 		} else if (!mhas_size) {
-			GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MHASDmx] MHAS packet with 0 payload size, considering sync was lost\n"));
+			GF_LOG(ctx->is_sync ? GF_LOG_WARNING : GF_LOG_DEBUG, GF_LOG_MEDIA, ("[MHASDmx] MHAS packet with 0 payload size, considering sync was lost\n"));
+			ctx->is_sync = GF_FALSE;
 			consumed = 0;
 			ctx->nosync = GF_TRUE;
 			ctx->nb_unknown_pck = 0;
@@ -595,6 +603,8 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_MEDIA, ("[MHASDmx] incomplete packet type %d %s label "LLU" size "LLU" - keeping in buffer\n", mhas_type, mhas_pck_name(mhas_type), mhas_label, mhas_size));
 			break;
 		}
+		ctx->is_sync = GF_TRUE;
+
 		//frame
 		if (mhas_type==2) {
 			mhas_sap = gf_bs_peek_bits(ctx->bs, 1, 0);
@@ -610,7 +620,7 @@ GF_Err mhas_dmx_process(GF_Filter *filter)
 			u32 idx = gf_bs_read_int(ctx->bs, 5);
 			if (idx==0x1f)
 				sr = gf_bs_read_int(ctx->bs, 24);
-			else if (sr < nb_usac_sr) {
+			else if (idx < nb_usac_sr) {
 				sr = USACSampleRates[idx];
 			}
 			ctx->nb_unknown_pck = 0;
@@ -908,3 +918,11 @@ const GF_FilterRegister *mhas_dmx_register(GF_FilterSession *session)
 {
 	return &MHASDmxRegister;
 }
+
+#else
+
+const GF_FilterRegister *mhas_dmx_register(GF_FilterSession *session)
+{
+	return NULL;
+}
+#endif //#if !defined(GPAC_DISABLE_AV_PARSERS)
