@@ -75,6 +75,8 @@ typedef struct
 
 	GF_FilterPid *vpid;
 	Bool copy_props;
+
+	Bool is_sync;
 } GF_MP3DmxCtx;
 
 
@@ -341,15 +343,27 @@ static void mp3_dmx_flush_id3(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 
 static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 {
-	u32 sr;
+	u32 sr, codec_id;
 
 	if (!ctx->opid) {
 		ctx->opid = gf_filter_pid_new(filter);
 		mp3_dmx_check_dur(filter, ctx);
 	}
 
+	codec_id = gf_mp3_object_type_indication(ctx->hdr);
+	switch (gf_mp3_layer(ctx->hdr)) {
+	case 3:
+		if (ctx->forcemp3) {
+			codec_id = GF_CODECID_MPEG_AUDIO;
+		}
+		break;
+	case 1:
+		codec_id = GF_CODECID_MPEG_AUDIO_L1;
+		break;
+	}
+
 	if ((ctx->sr == gf_mp3_sampling_rate(ctx->hdr)) && (ctx->nb_ch == gf_mp3_num_channels(ctx->hdr) )
-		&& (ctx->codecid == gf_mp3_object_type_indication(ctx->hdr) )
+		&& (ctx->codecid == codec_id )
 		&& !ctx->copy_props
 	)
 		return;
@@ -370,17 +384,7 @@ static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 	if (!ctx->timescale) gf_filter_pid_set_name(ctx->opid, "audio");
 
 	ctx->nb_ch = gf_mp3_num_channels(ctx->hdr);
-	ctx->codecid = gf_mp3_object_type_indication(ctx->hdr);
-	switch (gf_mp3_layer(ctx->hdr)) {
-	case 3:
-		if (ctx->forcemp3) {
-			ctx->codecid = GF_CODECID_MPEG_AUDIO;
-		}
-		break;
-	case 1:
-		ctx->codecid = GF_CODECID_MPEG_AUDIO_L1;
-		break;
-	}
+
 	sr = gf_mp3_sampling_rate(ctx->hdr);
 
 	if (!ctx->timescale) {
@@ -390,6 +394,7 @@ static void mp3_dmx_check_pid(GF_Filter *filter, GF_MP3DmxCtx *ctx)
 		}
 	}
 	ctx->sr = sr;
+	ctx->codecid = codec_id;
 
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_TIMESCALE, & PROP_UINT(ctx->timescale ? ctx->timescale : sr));
 	gf_filter_pid_set_property(ctx->opid, GF_PROP_PID_SAMPLE_RATE, & PROP_UINT(sr));
@@ -461,6 +466,7 @@ static Bool mp3_dmx_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 		ctx->is_playing = GF_FALSE;
 		if (ctx->src_pck) gf_filter_pck_unref(ctx->src_pck);
 		ctx->src_pck = NULL;
+		ctx->cts = 0;
 		//don't cancel event
 		return GF_FALSE;
 
@@ -547,6 +553,9 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 	//input pid sets some timescale - we flushed pending data , update cts
 	if (ctx->timescale && pck) {
 		cts = gf_filter_pck_get_cts(pck);
+		//init cts at first packet
+		if (!ctx->cts && (cts != GF_FILTER_NO_TS))
+			ctx->cts = cts;
 	}
 
 	if (cts == GF_FILTER_NO_TS) {
@@ -623,7 +632,8 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 				if ((sync[size]=='T') && (sync[size+1]=='A') && (sync[size+2]=='G')) {
 					skip_id3v1=GF_TRUE;
 				} else {
-					GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MP3Dmx] invalid frame, resyncing\n"));
+					GF_LOG(ctx->is_sync ? GF_LOG_WARNING : GF_LOG_DEBUG, GF_LOG_MEDIA, ("[MP3Dmx] invalid frame, resyncing\n"));
+					ctx->is_sync = GF_FALSE;
 					goto drop_byte;
 				}
 			}
@@ -639,6 +649,7 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 			ctx->resume_from = (u32) (sync - ctx->mp3_buffer + 1);
 			return GF_OK;
 		}
+		ctx->is_sync = GF_TRUE;
 
 		nb_samp = gf_mp3_window_size(ctx->hdr);
 
@@ -657,6 +668,10 @@ GF_Err mp3_dmx_process(GF_Filter *filter)
 		}
 
 		if (!ctx->in_seek) {
+			if (bytes_skipped + size > remain) {
+				GF_LOG(GF_LOG_WARNING, GF_LOG_MEDIA, ("[MP3Dmx] truncated frame of size %u (remains %d)\n", size, remain-bytes_skipped));
+				break;
+			}
 			dst_pck = gf_filter_pck_new_alloc(ctx->opid, size, &output);
 			if (!dst_pck) break;
 			memcpy(output, sync, size);
@@ -896,4 +911,3 @@ const GF_FilterRegister *mp3_dmx_register(GF_FilterSession *session)
 	return NULL;
 }
 #endif // GPAC_DISABLE_AV_PARSERS
-
