@@ -51,11 +51,15 @@ typedef struct
 {
 	//filter args
 	Double index;
+	Bool auxac3;
 
-	//only one input pid declared
+	//only one input pid
 	GF_FilterPid *ipid;
-	//only one output pid declared
+	//truehd output pid
 	GF_FilterPid *opid;
+	//aux AC3 output pid
+	GF_FilterPid *aux_opid;
+	GF_Filter *filter;
 
 	GF_BitStream *bs;
 	u64 file_pos, cts;
@@ -73,11 +77,17 @@ typedef struct
 	Bool is_file, file_loaded;
 	Bool initial_play_done;
 
+	//ref src pck
 	GF_FilterPacket *src_pck;
+	//current src, not ref
+	GF_FilterPacket *src_current;
 
 	TrueHDIdx *indexes;
 	u32 index_alloc_size, index_size;
 	Bool copy_props;
+
+	GF_AC3Header ac3_hdr;
+	Bool is_sync;
 } GF_TrueHDDmxCtx;
 
 
@@ -99,6 +109,7 @@ GF_Err truehd_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 	if (! gf_filter_pid_check_caps(pid))
 		return GF_NOT_SUPPORTED;
 
+	ctx->filter = filter;
 	ctx->ipid = pid;
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_TIMESCALE);
 	if (p) ctx->timescale = p->value.uint;
@@ -112,17 +123,86 @@ GF_Err truehd_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remove
 	return GF_OK;
 }
 
-static GF_Err truehd_parse_frame(GF_BitStream *bs, TrueHDHdr *hdr)
+static void truehd_aux_ac3(GF_TrueHDDmxCtx *ctx, GF_BitStream *bs, GF_AC3Header *hdr)
+{
+	u8 *data;
+	u32 size;
+
+	if (!ctx->aux_opid) {
+		ctx->aux_opid = gf_filter_pid_new(ctx->filter);
+		if (!ctx->aux_opid) return;
+		gf_filter_pid_set_name(ctx->aux_opid, "aux_ac3");
+	}
+
+	if (memcmp(&ctx->ac3_hdr, hdr, sizeof(GF_AC3Header))) {
+		memcpy(&ctx->ac3_hdr, hdr, sizeof(GF_AC3Header));
+
+		//copy properties from output pid
+		if (ctx->opid) {
+			gf_filter_pid_copy_properties(ctx->aux_opid, ctx->opid);
+		} else {
+			gf_filter_pid_copy_properties(ctx->aux_opid, ctx->ipid);
+		}
+		//don't change codec type if reframing an ES (for HLS SAES)
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_STREAM_TYPE, & PROP_UINT( GF_STREAM_AUDIO));
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_CODECID, & PROP_UINT( GF_CODECID_AC3));
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_UNFRAMED, & PROP_BOOL(GF_FALSE) );
+
+		if (ctx->duration.num)
+			gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_DURATION, & PROP_FRAC64(ctx->duration));
+
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_TIMESCALE, & PROP_UINT(ctx->timescale ? ctx->timescale : hdr->sample_rate));
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_SAMPLE_RATE, & PROP_UINT(hdr->sample_rate));
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_NUM_CHANNELS, & PROP_UINT(hdr->streams[0].channels) );
+
+		gf_odf_ac3_cfg_write(hdr, &data, &size);
+		gf_filter_pid_set_property(ctx->aux_opid, GF_PROP_PID_DECODER_CONFIG, & PROP_DATA_NO_COPY(data, size) );
+	}
+
+	GF_FilterPacket *dst_pck = gf_filter_pck_new_alloc(ctx->aux_opid, hdr->framesize, &data);
+	gf_bs_read_data(bs, data, hdr->framesize);
+	gf_filter_pck_merge_properties(ctx->src_pck ? ctx->src_pck : ctx->src_current, dst_pck);
+	gf_filter_pck_set_byte_offset(dst_pck, GF_FILTER_NO_BO);
+
+	gf_filter_pck_set_dts(dst_pck, ctx->cts);
+	gf_filter_pck_set_cts(dst_pck, ctx->cts);
+	gf_filter_pck_set_sap(dst_pck, GF_FILTER_SAP_1);
+	gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
+	gf_filter_pck_send(dst_pck);
+}
+
+static GF_Err truehd_parse_frame(GF_TrueHDDmxCtx *ctx, GF_BitStream *bs, TrueHDHdr *hdr, u64 *frame_start)
 {
 	memset(hdr, 0, sizeof(TrueHDHdr));
 
 	u32 avail = (u32) gf_bs_available(bs);
+	*frame_start = gf_bs_get_position(bs);
+	hdr->frame_size = 0;
 	//we need 8 bytes for base header (up to sync marker)
-	if (avail<8) {
-		hdr->frame_size = 0;
-		return GF_OK;
-	}
+	if (avail<8)
+		return GF_BUFFER_TOO_SMALL;
 
+	while (gf_bs_available(bs)) {
+		u32 sync = gf_bs_peek_bits(bs, 16, 0);
+		if (sync==0x0B77) {
+			GF_AC3Header ac3hdr;
+			if (!gf_ac3_parser_bs(bs, &ac3hdr, GF_TRUE))
+				return GF_OK;
+			//make sure we have the complete frame + 8 byte of TrueHD header
+			if (gf_bs_available(bs) < ac3hdr.framesize + 8)
+				return GF_BUFFER_TOO_SMALL;
+
+			if (ctx && ctx->auxac3 && ctx->is_playing) {
+				truehd_aux_ac3(ctx, bs, &ac3hdr);
+			} else {
+				gf_bs_skip_bytes(bs, ac3hdr.framesize);
+			}
+			avail = (u32) gf_bs_available(bs);
+			*frame_start = gf_bs_get_position(bs);
+			continue;
+		}
+		break;
+	}
 	/*u8 nibble = */gf_bs_read_int(bs, 4);
 	hdr->frame_size = 2 * gf_bs_read_int(bs, 12);
 	hdr->time = gf_bs_read_u16(bs);
@@ -135,7 +215,7 @@ static GF_Err truehd_parse_frame(GF_BitStream *bs, TrueHDHdr *hdr)
 	//we need 12 bytes until peak rate - to update if we decide to parse more
 	if (avail < 12) {
 		hdr->frame_size = 0;
-		return GF_OK;
+		return GF_BUFFER_TOO_SMALL;
 	}
 
 	hdr->format = gf_bs_peek_bits(bs, 32, 0);
@@ -225,8 +305,8 @@ static void truehd_check_dur(GF_Filter *filter, GF_TrueHDDmxCtx *ctx)
 	cur_dur = 0;
 	while (	gf_bs_available(bs) > 8 ) {
 		TrueHDHdr hdr;
-		u64 pos = gf_bs_get_position(bs);
-		GF_Err e = truehd_parse_frame(bs, &hdr);
+		u64 pos;
+		GF_Err e = truehd_parse_frame(NULL, bs, &hdr, &pos);
 		if (e) break;
 
 		if (hdr.sync) {
@@ -410,6 +490,7 @@ static Bool truehd_process_event(GF_Filter *filter, const GF_FilterEvent *evt)
 	case GF_FEVT_STOP:
 		//don't cancel event
 		ctx->is_playing = GF_FALSE;
+		ctx->cts = 0;
 		return GF_FALSE;
 
 	case GF_FEVT_SET_SPEED:
@@ -457,6 +538,8 @@ GF_Err truehd_process(GF_Filter *filter)
 			if (!ctx->truehd_buffer_size) {
 				if (ctx->opid)
 					gf_filter_pid_set_eos(ctx->opid);
+				if (ctx->aux_opid)
+					gf_filter_pid_set_eos(ctx->aux_opid);
 				if (ctx->src_pck) gf_filter_pck_unref(ctx->src_pck);
 				ctx->src_pck = NULL;
 				return GF_EOS;
@@ -498,6 +581,9 @@ GF_Err truehd_process(GF_Filter *filter)
 	//input pid sets some timescale - we flushed pending data , update cts
 	if (ctx->timescale && pck) {
 		cts = gf_filter_pck_get_cts(pck);
+		//init cts at first packet
+		if (!ctx->cts && (cts != GF_FILTER_NO_TS))
+			ctx->cts = cts;
 	}
 
 	if (cts == GF_FILTER_NO_TS) {
@@ -507,6 +593,7 @@ GF_Err truehd_process(GF_Filter *filter)
 
 	remain = ctx->truehd_buffer_size;
 	start = ctx->truehd_buffer;
+	ctx->src_current = pck;
 
 	if (ctx->resume_from) {
 		start += ctx->resume_from - 1;
@@ -522,10 +609,13 @@ GF_Err truehd_process(GF_Filter *filter)
 	while (remain > 8) {
 		u8 *frame;
 		TrueHDHdr hdr;
-		u32 frame_start, bytes_to_drop=0;
-
-		frame_start = (u32) gf_bs_get_position(ctx->bs);
-		e = truehd_parse_frame(ctx->bs, &hdr);
+		u32 bytes_to_drop=0;
+		u64 frame_start;
+		e = truehd_parse_frame(ctx, ctx->bs, &hdr, &frame_start);
+		if (e==GF_BUFFER_TOO_SMALL) {
+			e = GF_OK;
+			break;
+		}
 		if (e) {
 			remain = 0;
 			break;
@@ -547,7 +637,11 @@ GF_Err truehd_process(GF_Filter *filter)
 			return GF_OK;
 		}
 
-		frame = start + frame_start;
+		if (frame_start) {
+			start += frame_start;
+			remain -= (u32) frame_start;
+		}
+		frame = start;
 
 		if (ctx->in_seek) {
 			u64 nb_samples_at_seek = (u64) (ctx->start_range * hdr.sample_rate);
@@ -564,21 +658,29 @@ GF_Err truehd_process(GF_Filter *filter)
 				ctx->cts = cts;
 			cts = GF_FILTER_NO_TS;
 		}
+		if (!ctx->sample_rate) {
+			GF_LOG(ctx->is_sync ? GF_LOG_WARNING : GF_LOG_DEBUG, GF_LOG_MEDIA, ("[TrueHDDmx] Invalid TrueHD frame, no sample rate found\n"));
+			ctx->is_sync = GF_FALSE;
+			remain = 0;
+			break;
+		}
+		ctx->is_sync = GF_TRUE;
 
 		if (!ctx->in_seek) {
 			dst_pck = gf_filter_pck_new_alloc(ctx->opid, hdr.frame_size, &output);
 			if (!dst_pck) return GF_OUT_OF_MEM;
-			if (ctx->src_pck) gf_filter_pck_merge_properties(ctx->src_pck, dst_pck);
+			gf_filter_pck_merge_properties(ctx->src_pck ? ctx->src_pck : ctx->src_current, dst_pck);
 
 			memcpy(output, frame, hdr.frame_size);
 			gf_filter_pck_set_dts(dst_pck, ctx->cts);
 			gf_filter_pck_set_cts(dst_pck, ctx->cts);
-			if (hdr.sync)
-				gf_filter_pck_set_sap(dst_pck, GF_FILTER_SAP_1);
+			gf_filter_pck_set_sap(dst_pck, hdr.sync ? GF_FILTER_SAP_1 : GF_FILTER_SAP_NONE);
 			gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
 
 			if (ctx->byte_offset != GF_FILTER_NO_BO) {
-				gf_filter_pck_set_byte_offset(dst_pck, ctx->byte_offset + hdr.frame_size);
+				gf_filter_pck_set_byte_offset(dst_pck, ctx->byte_offset + hdr.frame_size + frame_start);
+			} else {
+				gf_filter_pck_set_byte_offset(dst_pck, GF_FILTER_NO_BO);
 			}
 
 			gf_filter_pck_send(dst_pck);
@@ -616,7 +718,7 @@ GF_Err truehd_process(GF_Filter *filter)
 		ctx->truehd_buffer_size = 0;
 		return truehd_process(filter);
 	} else {
-		if (remain) {
+		if (remain && (remain<ctx->truehd_buffer_size)) {
 			memmove(ctx->truehd_buffer, start, remain);
 		}
 		ctx->truehd_buffer_size = remain;
@@ -639,8 +741,8 @@ static const char *truehd_probe_data(const u8 *data, u32 size, GF_FilterProbeSco
 	GF_BitStream *bs = gf_bs_new(data, size, GF_BITSTREAM_READ);
 	while (gf_bs_available(bs) > 8 ) {
 		TrueHDHdr hdr;
-		u64 pos = gf_bs_get_position(bs);
-		GF_Err e = truehd_parse_frame(bs, &hdr);
+		u64 pos;
+		GF_Err e = truehd_parse_frame(NULL, bs, &hdr, &pos);
 		if (e || !hdr.frame_size) {
 			nb_frames = 0;
 			break;
@@ -674,6 +776,7 @@ static const GF_FilterCapability TrueHDDmxCaps[] =
 static const GF_FilterArgs TrueHDDmxArgs[] =
 {
 	{ OFFS(index), "indexing window length", GF_PROP_DOUBLE, "1.0", NULL, 0},
+	{ OFFS(auxac3), "expose auxiliary AC-3 stream if present", GF_PROP_BOOL, "false", NULL, 0},
 	{0}
 };
 
