@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2017-2020
+ *			Copyright (c) Telecom ParisTech 2017-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / NHML stream to file filter
@@ -115,6 +115,7 @@ GF_Err nhmldump_config_side_stream(GF_Filter *filter, GF_NHMLDumpCtx *ctx)
 
 	} else if (ctx->opid_info) {
 		gf_filter_pid_remove(ctx->opid_info);
+		ctx->opid_info = NULL;
 	}
 	if (ctx->info_file) gf_free(ctx->info_file);
 	ctx->info_file = NULL;
@@ -193,9 +194,18 @@ GF_Err nhmldump_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remo
 
 	if (is_remove) {
 		ctx->ipid = NULL;
-		gf_filter_pid_remove(ctx->opid_nhml);
-		gf_filter_pid_remove(ctx->opid_mdia);
-		if (ctx->opid_info) gf_filter_pid_remove(ctx->opid_info);
+		if (ctx->opid_nhml) {
+			gf_filter_pid_remove(ctx->opid_nhml);
+			ctx->opid_nhml = NULL;
+		}
+		if (ctx->opid_mdia) {
+			gf_filter_pid_remove(ctx->opid_mdia);
+			ctx->opid_mdia = NULL;
+		}
+		if (ctx->opid_info) {
+			gf_filter_pid_remove(ctx->opid_info);
+			ctx->opid_info = NULL;
+		}
 		return GF_OK;
 	}
 	if (! gf_filter_pid_check_caps(pid))
@@ -259,11 +269,11 @@ GF_Err nhmldump_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remo
 	name = (char*) gf_codecid_name(ctx->codecid);
 	if (ctx->exporter) {
 		if (ctx->w && ctx->h) {
-			GF_LOG(GF_LOG_INFO, GF_LOG_AUTHOR, ("Exporting %s - Size %dx%d\n", name, ctx->w, ctx->h));
+			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("Exporting %s - Size %dx%d\n", name, ctx->w, ctx->h));
 		} else if (ctx->sr && ctx->chan) {
-			GF_LOG(GF_LOG_INFO, GF_LOG_AUTHOR, ("Exporting %s - SampleRate %d %d channels %d bits per sample\n", name, ctx->sr, ctx->chan, ctx->bps));
+			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("Exporting %s - SampleRate %d %d channels %d bits per sample\n", name, ctx->sr, ctx->chan, ctx->bps));
 		} else {
-			GF_LOG(GF_LOG_INFO, GF_LOG_AUTHOR, ("Exporting %s\n", name));
+			GF_LOG(GF_LOG_INFO, GF_LOG_MEDIA, ("Exporting %s\n", name));
 		}
 	}
 
@@ -296,8 +306,7 @@ GF_Err nhmldump_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remo
 	ctx->is_stpp = (cid==GF_CODECID_SUBS_XML) ? GF_TRUE : GF_FALSE;
 
 	p = gf_filter_pid_get_property(pid, GF_PROP_PID_DURATION);
-	if (p) ctx->duration = p->value.lfrac;
-
+	if (p && (p->value.lfrac.num>0)) ctx->duration = p->value.lfrac;
 
 	if (ctx->opid_nhml)
 		gf_filter_pid_set_name(ctx->opid_nhml, "nhml");
@@ -333,7 +342,7 @@ GF_Err nhmldump_configure_pid(GF_Filter *filter, GF_FilterPid *pid, Bool is_remo
 
 
 
-static void nhmldump_send_header(GF_NHMLDumpCtx *ctx)
+static GF_Err nhmldump_send_header(GF_NHMLDumpCtx *ctx)
 {
 	GF_FilterPacket *dst_pck;
 	char nhml[1024];
@@ -393,14 +402,13 @@ static void nhmldump_send_header(GF_NHMLDumpCtx *ctx)
 		}
 	}
 	else if (ctx->sr && ctx->chan) {
-		sprintf(nhml, "sampleRate=\"%d\" numChannels=\"%d\" ", ctx->sr, ctx->chan);
-		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 		sprintf(nhml, "sampleRate=\"%d\" numChannels=\"%d\" ", ctx->sr, ctx->chan);
 		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 		p = gf_filter_pid_get_property(ctx->ipid, GF_PROP_PID_AUDIO_FORMAT);
-		if (p)
+		if (p) {
 			sprintf(nhml, "bitsPerSample=\"%d\" ", gf_audio_fmt_bit_depth(p->value.uint));
-		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
+			gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
+		}
 	}
 
 	NHML_PRINT_4CC(0, "codec_vendor", "codecVendor")
@@ -454,18 +462,20 @@ static void nhmldump_send_header(GF_NHMLDumpCtx *ctx)
 		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 
 		dst_pck = gf_filter_pck_new_shared(ctx->opid_info, ctx->dcfg, ctx->dcfg_size, NULL);
-		gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
-		gf_filter_pck_set_readonly(dst_pck);
-		gf_filter_pck_send(dst_pck);
+		if (dst_pck) {
+			gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_TRUE);
+			gf_filter_pck_set_readonly(dst_pck);
+			gf_filter_pck_send(dst_pck);
+		}
 	}
-
+	
 	NHML_PRINT_STRING(0, "meta:encoding", "encoding")
 	NHML_PRINT_STRING(0, "meta:contentEncoding", "content_encoding")
 	ctx->uncompress = GF_FALSE;
 	if (p) {
 		if (!strcmp(p->value.string, "deflate")) ctx->uncompress = GF_TRUE;
 		else {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_AUTHOR, ("[NHMLMx] content_encoding %s not supported\n", p->value.string ));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("[NHMLMx] content_encoding %s not supported\n", p->value.string ));
 		}
 	}
 
@@ -479,17 +489,19 @@ static void nhmldump_send_header(GF_NHMLDumpCtx *ctx)
 	gf_bs_get_content_no_truncate(ctx->bs_w, &ctx->nhml_buffer, &size, &ctx->nhml_buffer_size);
 
 	if (ctx->filep) {
-		gf_fwrite(ctx->nhml_buffer, size, ctx->filep);
-		return;
+		if (gf_fwrite(ctx->nhml_buffer, size, ctx->filep) != size) return GF_IO_ERR;
+		return GF_OK;
 	}
 
 	dst_pck = gf_filter_pck_new_alloc(ctx->opid_nhml, size, &output);
+	if (!dst_pck) return GF_OUT_OF_MEM;
+
 	memcpy(output, ctx->nhml_buffer, size);
 	gf_filter_pck_set_framing(dst_pck, GF_TRUE, GF_FALSE);
-	gf_filter_pck_send(dst_pck);
+	return gf_filter_pck_send(dst_pck);
 }
 
-static void nhmldump_send_dims(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, GF_FilterPacket *pck)
+static GF_Err nhmldump_send_dims(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, GF_FilterPacket *pck)
 {
 	char nhml[1024];
 	u32 size;
@@ -513,9 +525,12 @@ static void nhmldump_send_dims(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, G
 		if (pos+size+2 > data_size)
 			break;
 
-		prev = data[pos+2+size];
-		data[pos+2+size] = 0;
 
+		prev = 0;
+		if (pos+2+size<data_size) {
+			prev = data[pos+2+size];
+			data[pos+2+size] = 0;
+		}
 
 		sprintf(nhml, "<DIMSUnit time=\""LLU"\"", cts);
 		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
@@ -577,9 +592,11 @@ static void nhmldump_send_dims(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, G
 				inflateEnd(&d_stream);
 			}
 #else
-			GF_LOG(GF_LOG_ERROR, GF_LOG_AUTHOR, ("Error: your version of GPAC was compiled with no libz support."));
+			GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("Error: your version of GPAC was compiled with no libz support."));
 			gf_bs_del(ctx->bs_r);
-			return;
+			if (prev)
+				data[pos+2+size] = prev;
+			return GF_NOT_SUPPORTED;
 #endif
 		} else {
 			gf_bs_write_data(ctx->bs_w, data+pos+3, size-1);
@@ -587,21 +604,24 @@ static void nhmldump_send_dims(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, G
 		sprintf(nhml, "</DIMSUnit>\n");
 		gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 
-		data[pos+2+size] = prev;
+		if (prev)
+			data[pos+2+size] = prev;
 		gf_bs_skip_bytes(ctx->bs_r, size-1);
 	}
 
 	gf_bs_get_content_no_truncate(ctx->bs_w, &ctx->nhml_buffer, &size, &ctx->nhml_buffer_size);
 
 	if (ctx->filep) {
-		gf_fwrite(ctx->nhml_buffer, size, ctx->filep);
-		return;
+		if (gf_fwrite(ctx->nhml_buffer, size, ctx->filep) != size) return GF_IO_ERR;
+		return GF_OK;
 	}
 
 	dst_pck = gf_filter_pck_new_alloc(ctx->opid_nhml, size, &output);
+	if (!dst_pck) return GF_OUT_OF_MEM;
+
 	memcpy(output, ctx->nhml_buffer, size);
 	gf_filter_pck_set_framing(dst_pck, GF_FALSE, GF_FALSE);
-	gf_filter_pck_send(dst_pck);
+	return gf_filter_pck_send(dst_pck);
 }
 
 
@@ -628,7 +648,7 @@ static void nhmldump_pck_property(GF_NHMLDumpCtx *ctx, u32 p4cc, const char *pna
 		nhml[0] = 0;
 		break;
 	default:
-		sprintf(nhml, "%s", gf_props_dump_val(att, pval, GF_FALSE, NULL) );
+		sprintf(nhml, "%s", gf_props_dump_val(att, pval, GF_PROP_DUMP_DATA_NONE, NULL) );
 		break;
 	}
 	gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
@@ -636,7 +656,7 @@ static void nhmldump_pck_property(GF_NHMLDumpCtx *ctx, u32 p4cc, const char *pna
 	gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 }
 
-static void nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, GF_FilterPacket *pck)
+static GF_Err nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size, GF_FilterPacket *pck)
 {
 	GF_FilterPacket *dst_pck;
 	char nhml[1024];
@@ -745,7 +765,7 @@ static void nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size,
 
 
 			if (offset_in_sample + s_size > data_size) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_AUTHOR, ("Wrong subsample info: sample size %d vs subsample offset+size %dn", data_size, offset_in_sample + s_size));
+					GF_LOG(GF_LOG_ERROR, GF_LOG_MEDIA, ("Wrong subsample info: sample size %d vs subsample offset+size %dn", data_size, offset_in_sample + s_size));
 				break;
 			}
 
@@ -760,8 +780,8 @@ static void nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size,
 					gf_bs_write_data(ctx->bs_w, nhml, (u32) strlen(nhml));
 				} else {
 					u32 d_size;
-					if (ctx->b64_buffer_size<2*s_size) {
-						ctx->b64_buffer_size = 2 * s_size;
+					if (ctx->b64_buffer_size < 2*s_size + 3) {
+						ctx->b64_buffer_size = 2 * s_size + 3;
 						ctx->b64_buffer = gf_realloc(ctx->b64_buffer, ctx->b64_buffer_size);
 					}
 					d_size = gf_base64_encode(data + offset_in_sample, s_size, ctx->b64_buffer, ctx->b64_buffer_size);
@@ -791,20 +811,24 @@ static void nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size,
 	gf_bs_get_content_no_truncate(ctx->bs_w, &ctx->nhml_buffer, &size, &ctx->nhml_buffer_size);
 
 	if (ctx->filep) {
-		gf_fwrite(ctx->nhml_buffer, size, ctx->filep);
-		return;
+		if (gf_fwrite(ctx->nhml_buffer, size, ctx->filep) != size) return GF_IO_ERR;
+		return GF_OK;
 	}
 
 	dst_pck = gf_filter_pck_new_alloc(ctx->opid_nhml, size, &output);
-	memcpy(output, ctx->nhml_buffer, size);
-	gf_filter_pck_set_framing(dst_pck, GF_FALSE, GF_FALSE);
-	gf_filter_pck_send(dst_pck);
+	if (dst_pck) {
+		memcpy(output, ctx->nhml_buffer, size);
+		gf_filter_pck_set_framing(dst_pck, GF_FALSE, GF_FALSE);
+		gf_filter_pck_send(dst_pck);
+	}
 
 	ctx->mdia_pos += data_size;
 
 	if (ctx->opid_mdia) {
-		//send data packet
-		dst_pck = gf_filter_pck_new_ref(ctx->opid_mdia, data, data_size, pck);
+		//send the complete data packet
+		dst_pck = gf_filter_pck_new_ref(ctx->opid_mdia, 0, data_size, pck);
+		if (!dst_pck) return GF_OUT_OF_MEM;
+
 		gf_filter_pck_merge_properties(pck, dst_pck);
 		//keep byte offset ?
 //		gf_filter_pck_set_byte_offset(dst_pck, GF_FILTER_NO_BO);
@@ -812,6 +836,7 @@ static void nhmldump_send_frame(GF_NHMLDumpCtx *ctx, char *data, u32 data_size,
 		gf_filter_pck_set_framing(dst_pck, ctx->first, GF_FALSE);
 		gf_filter_pck_send(dst_pck);
 	}
+	return GF_OK;
 }
 
 
@@ -821,6 +846,7 @@ GF_Err nhmldump_process(GF_Filter *filter)
 	GF_FilterPacket *pck;
 	char *data;
 	u32 pck_size;
+	GF_Err e;
 
 	if (!ctx->side_streams_config) {
 		return nhmldump_config_side_stream(filter, ctx);
@@ -829,6 +855,7 @@ GF_Err nhmldump_process(GF_Filter *filter)
 	pck = gf_filter_pid_get_packet(ctx->ipid);
 	if (!pck) {
 		if (gf_filter_pid_is_eos(ctx->ipid)) {
+			e = GF_EOS;
 			if (ctx->bs_w && ctx->szRootName) {
 				char nhml[1024];
 				u32 size;
@@ -839,21 +866,26 @@ GF_Err nhmldump_process(GF_Filter *filter)
 				gf_bs_get_content_no_truncate(ctx->bs_w, &ctx->nhml_buffer, &size, &ctx->nhml_buffer_size);
 
 				if (ctx->filep) {
-					gf_fwrite(ctx->nhml_buffer, size, ctx->filep);
+					if (gf_fwrite(ctx->nhml_buffer, size, ctx->filep) != size)
+						e = GF_IO_ERR;
 				} else {
 					GF_FilterPacket *dst_pck;
 					u8 *output;
 					dst_pck = gf_filter_pck_new_alloc(ctx->opid_nhml, size, &output);
-					memcpy(output, ctx->nhml_buffer, size);
-					gf_filter_pck_set_framing(dst_pck, GF_FALSE, GF_TRUE);
-					gf_filter_pck_send(dst_pck);
+					if (dst_pck) {
+						memcpy(output, ctx->nhml_buffer, size);
+						gf_filter_pck_set_framing(dst_pck, GF_FALSE, GF_TRUE);
+						e = gf_filter_pck_send(dst_pck);
+					} else {
+						e = GF_OUT_OF_MEM;
+					}
 				}
 				ctx->szRootName = NULL;
 			}
 			if (ctx->opid_nhml) gf_filter_pid_set_eos(ctx->opid_nhml);
 			if (ctx->opid_mdia) gf_filter_pid_set_eos(ctx->opid_mdia);
 			if (ctx->opid_info) gf_filter_pid_set_eos(ctx->opid_info);
-			return GF_EOS;
+			return e;
 		}
 		return GF_OK;
 	}
@@ -862,7 +894,8 @@ GF_Err nhmldump_process(GF_Filter *filter)
 	else gf_bs_reassign_buffer(ctx->bs_w, ctx->nhml_buffer, ctx->nhml_buffer_size);
 
 	if (ctx->first) {
-		nhmldump_send_header(ctx);
+		e = nhmldump_send_header(ctx);
+		if (e) return e;
 		gf_bs_reassign_buffer(ctx->bs_w, ctx->nhml_buffer, ctx->nhml_buffer_size);
 	}
 
@@ -871,11 +904,12 @@ GF_Err nhmldump_process(GF_Filter *filter)
 
 	//send data
 	if (ctx->is_dims) {
-		nhmldump_send_dims(ctx, data, pck_size, pck);
+		e = nhmldump_send_dims(ctx, data, pck_size, pck);
 	} else {
-		nhmldump_send_frame(ctx, data, pck_size, pck);
+		e = nhmldump_send_frame(ctx, data, pck_size, pck);
 	}
 	ctx->first = GF_FALSE;
+	if (e) return e;
 
 
 	if (ctx->exporter) {
@@ -920,7 +954,7 @@ static const GF_FilterArgs NHMLDumpArgs[] =
 {
 	{ OFFS(exporter), "compatibility with old exporter, displays export results", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(dims), "use DIMS mode", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
-	{ OFFS(name), "set output name of files produced (needed media/info files refered to from XML", GF_PROP_STRING, NULL, NULL, 0},
+	{ OFFS(name), "set output name of media and info files produced", GF_PROP_STRING, NULL, NULL, 0},
 	{ OFFS(nhmlonly), "only dump NHML info, not media", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(pckp), "full NHML dump", GF_PROP_BOOL, "false", NULL, GF_FS_ARG_HINT_ADVANCED},
 	{ OFFS(chksum), "insert frame checksum\n"
