@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2020
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / RTP input module
@@ -1124,7 +1124,7 @@ static void gf_rtp_parse_3gpp_dims(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, c
 
 #ifndef GPAC_DISABLE_AV_PARSERS
 
-static void gf_rtp_parse_ac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *payload, u32 size)
+static void gf_rtp_parse_ac3_eac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *payload, u32 size, Bool is_eac3)
 {
 	u8 ft;
 
@@ -1142,14 +1142,25 @@ static void gf_rtp_parse_ac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *pay
 		rtp->sl_hdr.accessUnitStartFlag = rtp->sl_hdr.accessUnitEndFlag = 1;
 		while (size) {
 			u32 offset;
-			if (!gf_ac3_parser((u8*)payload, size, &offset, &ac3hdr, GF_FALSE)) {
-				return;
+			if (is_eac3) {
+				if (!gf_eac3_parser((u8*)payload, size, &offset, &ac3hdr, GF_FALSE)) {
+					return;
+				}
+			} else {
+				if (!gf_ac3_parser((u8*)payload, size, &offset, &ac3hdr, GF_FALSE)) {
+					return;
+				}
 			}
 			if (offset) {
 				if (offset>size) return;
 				payload+=offset;
 				size-=offset;
 			}
+			if (!rtp->sl_map.config) {
+				ac3hdr.is_ec3 = is_eac3;
+				gf_odf_ac3_cfg_write(&ac3hdr, &rtp->sl_map.config, &rtp->sl_map.configSize);
+				rtp->sl_map.config_updated = 1;
+			}
 			rtp->on_sl_packet(rtp->udta, payload, ac3hdr.framesize, &rtp->sl_hdr, GF_OK);
 			if (size < ac3hdr.framesize) return;
 			size -= ac3hdr.framesize;
@@ -1167,6 +1178,29 @@ static void gf_rtp_parse_ac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *pay
 		rtp->on_sl_packet(rtp->udta, payload, size, &rtp->sl_hdr, GF_OK);
 	}
 }
+static void gf_rtp_parse_ac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *payload, u32 size)
+{
+	gf_rtp_parse_ac3_eac3(rtp, hdr, payload, size, GF_FALSE);
+}
+static void gf_rtp_parse_eac3(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *payload, u32 size)
+{
+	gf_rtp_parse_ac3_eac3(rtp, hdr, payload, size, GF_TRUE);
+}
+
+
+static void gf_rtp_parse_opus(GF_RTPDepacketizer *rtp, GF_RTPHeader *hdr, u8 *payload, u32 size)
+{
+	rtp->sl_hdr.compositionTimeStampFlag = 1;
+	if (rtp->sl_hdr.compositionTimeStamp != hdr->TimeStamp) {
+		rtp->sl_hdr.accessUnitStartFlag = 1;
+		rtp->sl_hdr.compositionTimeStamp = hdr->TimeStamp;
+	}
+	rtp->sl_hdr.randomAccessPointFlag = 1;
+	rtp->on_sl_packet(rtp->udta, payload, size, &rtp->sl_hdr, GF_OK);
+	rtp->flags |= GF_RTP_NEW_AU;
+	rtp->sl_hdr.accessUnitStartFlag = 0;
+}
+
 #endif /*GPAC_DISABLE_AV_PARSERS*/
 
 static u32 gf_rtp_get_payload_type(GF_RTPMap *map, GF_SDPMedia *media)
@@ -1209,10 +1243,13 @@ static u32 gf_rtp_get_payload_type(GF_RTPMap *map, GF_SDPMedia *media)
 	else if (!stricmp(map->payload_name, "richmedia+xml")) return GF_RTP_PAYT_3GPP_DIMS;
 #endif
 	else if (!stricmp(map->payload_name, "ac3")) return GF_RTP_PAYT_AC3;
+	else if (!stricmp(map->payload_name, "eac3")) return GF_RTP_PAYT_EAC3;
 	else if (!stricmp(map->payload_name, "H264-SVC")) return GF_RTP_PAYT_H264_SVC;
 	else if (!stricmp(map->payload_name, "H265")) return GF_RTP_PAYT_HEVC;
 	else if (!stricmp(map->payload_name, "H265-SHVC")) return GF_RTP_PAYT_LHVC;
 	else if (!stricmp(map->payload_name, "H266")) return GF_RTP_PAYT_VVC;
+	else if (!stricmp(map->payload_name, "opus")) return GF_RTP_PAYT_OPUS;
+	else if (!stricmp(map->payload_name, "QCELP")) return GF_RTP_PAYT_QCELP;
 	else return 0;
 }
 
@@ -1812,6 +1849,32 @@ static GF_Err gf_rtp_payt_setup(GF_RTPDepacketizer *rtp, GF_RTPMap *map, GF_SDPM
 		rtp->sl_map.RandomAccessIndication = GF_TRUE;
 		/*assign depacketizer*/
 		rtp->depacketize = gf_rtp_parse_ac3;
+		break;
+	case GF_RTP_PAYT_EAC3:
+		rtp->sl_map.StreamType = GF_STREAM_AUDIO;
+		rtp->sl_map.CodecID = GF_CODECID_EAC3;
+		rtp->sl_map.RandomAccessIndication = GF_TRUE;
+		/*assign depacketizer*/
+		rtp->depacketize = gf_rtp_parse_eac3;
+		break;
+	case GF_RTP_PAYT_OPUS:
+		rtp->sl_map.StreamType = GF_STREAM_AUDIO;
+		rtp->sl_map.CodecID = GF_CODECID_OPUS;
+		rtp->sl_map.RandomAccessIndication = GF_TRUE;
+		/*assign depacketizer*/
+		rtp->depacketize = gf_rtp_parse_opus;
+
+		if (!rtp->sl_map.config) {
+			GF_OpusConfig opcfg;
+			opcfg.version = 1;
+			opcfg.OutputChannelCount = rtp->audio_channels;
+			opcfg.PreSkip = 0;
+			opcfg.InputSampleRate = rtp->clock_rate;
+			opcfg.OutputGain = 0;
+			opcfg.ChannelMappingFamily = 0;
+			gf_odf_opus_cfg_write(&opcfg, &rtp->sl_map.config, &rtp->sl_map.configSize);
+		}
+
 		break;
 #endif /*GPAC_DISABLE_AV_PARSERS*/
 	default:
@@ -1871,7 +1934,7 @@ GF_RTPDepacketizer *gf_rtp_depacketizer_new(GF_SDPMedia *media, u32 hdr_payt, gf
 	GF_RTPMap *map = NULL;
 	u32 payt;
 	GF_RTPDepacketizer *tmp;
-	u32 clock_rate;
+	u32 clock_rate, nb_chan=0;
 	const GF_RTPStaticMap *static_map = NULL;
 
 	/*check RTP map. For now we only support 1 RTPMap*/
@@ -1890,6 +1953,8 @@ GF_RTPDepacketizer *gf_rtp_depacketizer_new(GF_SDPMedia *media, u32 hdr_payt, gf
 			return NULL;
 		}
 		clock_rate = static_map->clock_rate;
+		if (static_map->stream_type==GF_STREAM_AUDIO)
+			nb_chan = 1;
 		payt = hdr_payt;
 	} else {
 		/*check payload type*/
@@ -1905,10 +1970,13 @@ GF_RTPDepacketizer *gf_rtp_depacketizer_new(GF_SDPMedia *media, u32 hdr_payt, gf
 				return NULL;
 			}
 			clock_rate = static_map->clock_rate;
+			if (static_map->stream_type==GF_STREAM_AUDIO)
+				nb_chan = 1;
 		} else {
 			payt = gf_rtp_get_payload_type(map, media);
 			if (!payt) return NULL;
 			clock_rate = map->ClockRate;
+			nb_chan = map->AudioChannels;
 		}
 	}
 
@@ -1916,6 +1984,8 @@ GF_RTPDepacketizer *gf_rtp_depacketizer_new(GF_SDPMedia *media, u32 hdr_payt, gf
 	if (!tmp) return NULL;
 	tmp->payt = payt;
 	tmp->static_map = static_map;
+	tmp->clock_rate = clock_rate;
+	tmp->audio_channels = nb_chan;
 
 	e = gf_rtp_payt_setup(tmp, map, media);
 	if (e) {
@@ -1924,7 +1994,6 @@ GF_RTPDepacketizer *gf_rtp_depacketizer_new(GF_SDPMedia *media, u32 hdr_payt, gf
 	}
 
 	assert(tmp->depacketize);
-	tmp->clock_rate = clock_rate;
 	tmp->on_sl_packet = sl_packet_cbk;
 	tmp->udta = udta;
 	return tmp;
