@@ -35,7 +35,7 @@ GF_Err gf_isom_parse_root_box(GF_Box **outBox, GF_BitStream *bs, u32 *box_type,
 	GF_Err ret;
 	u64 start;
 	start = gf_bs_get_position(bs);
-	ret = gf_isom_box_parse_ex(outBox, bs, 0, GF_TRUE);
+	ret = gf_isom_box_parse_ex(outBox, bs, 0, GF_TRUE, 0);
 	if (ret == GF_ISOM_INCOMPLETE_FILE) {
 		if (!*outBox) {
 			// We could not even read the box size, we at least need 8 bytes
@@ -90,7 +90,9 @@ static GF_Err gf_isom_full_box_read(GF_Box *ptr, GF_BitStream *bs);
 
 u64 unused_bytes = 0;
 
-GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type, Bool is_root_box)
+#define GF_SKIP_BOX 10
+
+GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type, Bool is_root_box, u64 parent_size)
 {
 	u32 type, uuid_type, hdr_size, restore_type;
 	u64 size, start, comp_start, end;
@@ -102,7 +104,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	GF_Box *newBox;
 	Bool skip_logs = (gf_bs_get_cookie(bs) & GF_ISOM_BS_COOKIE_NO_LOGS ) ? GF_TRUE : GF_FALSE;
 	Bool is_special = GF_TRUE;
-	
+
 	if ((bs == NULL) || (outBox == NULL) ) return GF_BAD_PARAM;
 	*outBox = NULL;
 	if (gf_bs_available(bs) < 8) {
@@ -132,8 +134,8 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 				size = gf_bs_available(bs) + 8;
 			} else {
 				if (!skip_logs) {
-					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Read Box type %s (0x%08X) at position "LLU" has size 0 but is not at root/file level !\n", gf_4cc_to_str(type), type, start));
-					return GF_ISOM_INVALID_FILE;
+					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Read Box type %s (0x%08X) at position "LLU" has size 0 but is not at root/file level. Forbidden, skipping end of parent box !\n", gf_4cc_to_str(type), type, start));
+					return GF_SKIP_BOX;
 				}
 				return GF_OK;
 			}
@@ -162,11 +164,13 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 
 			if (do_uncompress) {
 				compb = gf_malloc((u32) (size-8));
+				if (!compb) return GF_OUT_OF_MEM;
 
 				compressed_size = (u32) (size - 8);
 				gf_bs_read_data(bs, compb, compressed_size);
 				e = gf_gz_decompress_payload(compb, compressed_size, &uncomp_data, &osize);
 				if (e) {
+					gf_free(compb);
 					GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Failed to uncompress payload for box type %s (0x%08X)\n", gf_4cc_to_str(otype), otype));
 					return e;
 				}
@@ -180,11 +184,20 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 			}
 		}
 	}
+
+#define ERR_EXIT(_e) { \
+		if (uncomp_bs) {\
+			gf_free(uncomp_data);\
+			gf_bs_del(uncomp_bs); \
+		}\
+		return _e;\
+	}
+
 	/*handle uuid*/
 	memset(uuid, 0, 16);
 	if (type == GF_ISOM_BOX_TYPE_UUID ) {
 		if (gf_bs_available(bs) < 16) {
-			return GF_ISOM_INCOMPLETE_FILE;
+			ERR_EXIT(GF_ISOM_INCOMPLETE_FILE);
 		}
 		gf_bs_read_data(bs, uuid, 16);
 		hdr_size += 16;
@@ -194,7 +207,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	//handle large box
 	if (size == 1) {
 		if (gf_bs_available(bs) < 8) {
-			return GF_ISOM_INCOMPLETE_FILE;
+			ERR_EXIT(GF_ISOM_INCOMPLETE_FILE);
 		}
 		size = gf_bs_read_u64(bs);
 		hdr_size += 8;
@@ -203,8 +216,13 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[iso file] Read Box type %s size "LLD" start "LLD"\n", gf_4cc_to_str(type), size,  start));
 
 	if ( size < hdr_size ) {
-		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[iso file] Box size "LLD" less than box header size %d\n", size, hdr_size));
-		return GF_ISOM_INVALID_FILE;
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Box %s size "LLD" less than box header size %d\n", gf_4cc_to_str(type), size, hdr_size));
+		ERR_EXIT(GF_ISOM_INVALID_FILE);
+	}
+	//if parent size is given, make sure box fits within parent
+	if (parent_size && (parent_size<size)) {
+		GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Box %s size "LLU" is larger than remaining parent size "LLU"\n", gf_4cc_to_str(type), size, parent_size ));
+		ERR_EXIT(GF_ISOM_INVALID_FILE);
 	}
 	restore_type = 0;
 	if ((parent_type==GF_ISOM_BOX_TYPE_STSD) && (type==GF_QT_SUBTYPE_RAW) ) {
@@ -220,25 +238,25 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	//some special boxes (references and track groups) are handled by a single generic box with an associated ref/group type
 	if (parent_type && (parent_type == GF_ISOM_BOX_TYPE_TREF)) {
 		newBox = gf_isom_box_new(GF_ISOM_BOX_TYPE_REFT);
-		if (!newBox) return GF_OUT_OF_MEM;
+		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 		((GF_TrackReferenceTypeBox*)newBox)->reference_type = type;
 	} else if (parent_type && (parent_type == GF_ISOM_BOX_TYPE_IREF)) {
 		newBox = gf_isom_box_new(GF_ISOM_BOX_TYPE_REFI);
-		if (!newBox) return GF_OUT_OF_MEM;
+		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 		((GF_ItemReferenceTypeBox*)newBox)->reference_type = type;
 	} else if (parent_type && (parent_type == GF_ISOM_BOX_TYPE_TRGR)) {
 		newBox = gf_isom_box_new(GF_ISOM_BOX_TYPE_TRGT);
-		if (!newBox) return GF_OUT_OF_MEM;
+		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 		((GF_TrackGroupTypeBox*)newBox)->group_type = type;
 	} else if (parent_type && (parent_type == GF_ISOM_BOX_TYPE_GRPL)) {
 		newBox = gf_isom_box_new(GF_ISOM_BOX_TYPE_GRPT);
-		if (!newBox) return GF_OUT_OF_MEM;
+		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 		((GF_EntityToGroupTypeBox*)newBox)->grouping_type = type;
 	} else {
 		//OK, create the box based on the type
 		is_special = GF_FALSE;
 		newBox = gf_isom_box_new_ex(uuid_type ? uuid_type : type, parent_type, skip_logs, is_root_box);
-		if (!newBox) return GF_OUT_OF_MEM;
+		if (!newBox) ERR_EXIT(GF_OUT_OF_MEM);
 	}
 
 	//OK, init and read this box
@@ -255,7 +273,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	if (size - hdr_size > end ) {
 		newBox->size = size - hdr_size - end;
 		*outBox = newBox;
-		return GF_ISOM_INCOMPLETE_FILE;
+		ERR_EXIT(GF_ISOM_INCOMPLETE_FILE);
 	}
 
 	newBox->size = size - hdr_size;
@@ -303,8 +321,10 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 	if (e && (e != GF_ISOM_INCOMPLETE_FILE)) {
 		gf_isom_box_del(newBox);
 		*outBox = NULL;
+		if (is_root_box && (e==GF_SKIP_BOX))
+			e = GF_ISOM_INVALID_FILE;
 
-		if (!skip_logs) {
+		if (!skip_logs && (e!=GF_SKIP_BOX)) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[iso file] Read Box \"%s\" (start "LLU") failed (%s) - skipping\n", gf_4cc_to_str(type), start, gf_error_to_string(e)));
 		}
 		//we don't try to reparse known boxes that have been failing (too dangerous)
@@ -335,7 +355,7 @@ GF_Err gf_isom_box_parse_ex(GF_Box **outBox, GF_BitStream *bs, u32 parent_type,
 GF_EXPORT
 GF_Err gf_isom_box_parse(GF_Box **outBox, GF_BitStream *bs)
 {
-	return gf_isom_box_parse_ex(outBox, bs, 0, GF_FALSE);
+	return gf_isom_box_parse_ex(outBox, bs, 0, GF_FALSE, 0);
 }
 
 void gf_isom_box_array_reset(GF_List *boxlist)
@@ -374,11 +394,6 @@ void gf_isom_box_array_del_parent(GF_List **child_boxes, GF_List *boxlist)
 }
 
 
-GF_Err gf_isom_box_array_read(GF_Box *parent, GF_BitStream *bs)
-{
-	return gf_isom_box_array_read_ex(parent, bs, parent->type);
-}
-
 #ifndef GPAC_DISABLE_ISOM_WRITE
 
 GF_EXPORT
@@ -534,6 +549,7 @@ ISOM_BOX_IMPL_DECL(ireftype)
 ISOM_BOX_IMPL_DECL(free)
 ISOM_BOX_IMPL_DECL(wide)
 ISOM_BOX_IMPL_DECL(mdat)
+ISOM_BOX_IMPL_DECL(imda)
 ISOM_BOX_IMPL_DECL_CHILD(moov)
 ISOM_BOX_IMPL_DECL(mvhd)
 ISOM_BOX_IMPL_DECL(mdhd)
@@ -879,6 +895,7 @@ ISOM_BOX_IMPL_DECL(prhd)
 ISOM_BOX_IMPL_DECL(proj_type)
 //ISOM_BOX_IMPL_DECL(mesh)
 
+ISOM_BOX_IMPL_DECL(keys)
 
 #define BOX_DEFINE(__type, b_rad, __par) { __type, b_rad##_box_new, b_rad##_box_del, b_rad##_box_read, b_rad##_box_write, b_rad##_box_size, b_rad##_box_dump, 0, 0, 0, __par, "p12", GF_FALSE}
 
@@ -1016,6 +1033,7 @@ static struct box_registry_entry {
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_SKIP, free, "*"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_MDAT, mdat, "file"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_IDAT, mdat, "meta"),
+	BOX_DEFINE( GF_ISOM_BOX_TYPE_IMDA, mdat, "file"),
 	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_MOOV, moov, "file"),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_MVHD, mvhd, "moov", 1),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_MDHD, mdhd, "mdia", 1),
@@ -1073,7 +1091,7 @@ static struct box_registry_entry {
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_BTRT, btrt, "sample_entry"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_PASP, pasp, "video_sample_entry ipco"),
 	BOX_DEFINE( GF_ISOM_BOX_TYPE_CLAP, clap, "video_sample_entry ipco"),
-	FBOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_META, meta, "file moov trak moof traf udta", 0),	//apple uses meta in moov->udta
+	BOX_DEFINE_CHILD( GF_ISOM_BOX_TYPE_META, meta, "file moov trak moof traf udta"),	//apple uses meta in moov->udta
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_XML, xml, "meta", 0),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_BXML, bxml, "meta", 0),
 	FBOX_DEFINE( GF_ISOM_BOX_TYPE_ILOC, iloc, "meta", 2),
@@ -1424,12 +1442,35 @@ static struct box_registry_entry {
 	ITUNES_TAG(GF_ISOM_ITUNE_COMPOSER),
 	ITUNES_TAG(GF_ISOM_ITUNE_TRACK),
 	ITUNES_TAG(GF_ISOM_ITUNE_CONDUCTOR),
+	ITUNES_TAG(GF_ISOM_ITUNE_ART_DIRECTOR),
+	ITUNES_TAG(GF_ISOM_ITUNE_ARRANGER),
+	ITUNES_TAG(GF_ISOM_ITUNE_LYRICIST),
+	ITUNES_TAG(GF_ISOM_ITUNE_COPY_ACK),
+	ITUNES_TAG(GF_ISOM_ITUNE_SONG_DESC),
+	ITUNES_TAG(GF_ISOM_ITUNE_DIRECTOR),
+	ITUNES_TAG(GF_ISOM_ITUNE_EQ_PRESET),
+	ITUNES_TAG(GF_ISOM_ITUNE_LINER_NOTES),
+	ITUNES_TAG(GF_ISOM_ITUNE_REC_COMPANY),
+	ITUNES_TAG(GF_ISOM_ITUNE_ORIG_ARTIST),
+	ITUNES_TAG(GF_ISOM_ITUNE_PHONO_RIGHTS),
+	ITUNES_TAG(GF_ISOM_ITUNE_PRODUCER),
+	ITUNES_TAG(GF_ISOM_ITUNE_PERFORMER),
+	ITUNES_TAG(GF_ISOM_ITUNE_PUBLISHER),
+	ITUNES_TAG(GF_ISOM_ITUNE_SOUND_ENG),
+	ITUNES_TAG(GF_ISOM_ITUNE_SOLOIST),
+	ITUNES_TAG(GF_ISOM_ITUNE_CREDITS),
+	ITUNES_TAG(GF_ISOM_ITUNE_THANKS),
+	ITUNES_TAG(GF_ISOM_ITUNE_ONLINE),
+	ITUNES_TAG(GF_ISOM_ITUNE_EXEC_PRODUCER),
+	ITUNES_TAG(GF_ISOM_ITUNE_LOCATION),
+
 	BOX_DEFINE_S( GF_ISOM_BOX_TYPE_iTunesSpecificInfo, ilst_item, "ilst data", "apple"),
 	BOX_DEFINE_S(GF_ISOM_BOX_TYPE_GMHD, def_parent, "minf", "apple"),
 	BOX_DEFINE_S(GF_QT_BOX_TYPE_LOAD, load, "trak", "apple"),
 	BOX_DEFINE_S(GF_QT_BOX_TYPE_TAPT, def_parent, "trak", "apple"),
 	FBOX_DEFINE_S( GF_QT_BOX_TYPE_GMIN, gmin, "gmhd", 0, "apple"),
 	FBOX_DEFINE_FLAGS_S( GF_QT_BOX_TYPE_ALIS, alis, "dref", 0, 1, "apple"),
+	FBOX_DEFINE_FLAGS_S( GF_QT_BOX_TYPE_CIOS, alis, "dref", 0, 1, "apple"),
 	FBOX_DEFINE_S( GF_QT_BOX_TYPE_CLEF, clef, "tapt", 0, "apple"),
 	FBOX_DEFINE_S( GF_QT_BOX_TYPE_PROF, clef, "tapt", 0, "apple"),
 	FBOX_DEFINE_S( GF_QT_BOX_TYPE_ENOF, clef, "tapt", 0, "apple"),
@@ -1487,8 +1528,9 @@ static struct box_registry_entry {
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_YVYU, video_sample_entry, "stsd", "apple"),
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_RGBA, video_sample_entry, "stsd", "apple"),
 	BOX_DEFINE_S_CHILD( GF_QT_SUBTYPE_ABGR, video_sample_entry, "stsd", "apple"),
-	
-	
+
+	FBOX_DEFINE_S(GF_QT_BOX_TYPE_STPS, stss, "stbl", 0, "apple"),
+
 	//dolby boxes
 	BOX_DEFINE_S_CHILD( GF_ISOM_BOX_TYPE_AC3, audio_sample_entry, "stsd", "dolby"),
 	BOX_DEFINE_S_CHILD( GF_ISOM_BOX_TYPE_EC3, audio_sample_entry, "stsd", "dolby"),
@@ -1567,6 +1609,31 @@ static struct box_registry_entry {
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_EQUI, proj_type, "proj", 0, "youtube"),
 	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_MESH, proj_type, "proj", 0, "youtube"),
 
+	FBOX_DEFINE_S( GF_ISOM_BOX_TYPE_KEYS, keys, "meta", 0, "apple"),
+
+	BOX_DEFINE_S(GF_QT_SUBTYPE_ALAC, unkn, "stsd", "apple"),
+	BOX_DEFINE_S(GF_ISOM_SUBTYPE_FFV1, unkn, "stsd", "ffmpeg"),
+
+	BOX_DEFINE_S(GF_4CC('v','c','-','1'), unkn, "stsd", "smpte-RP2025"),
+	BOX_DEFINE_S(GF_4CC('d','v','c','1'), unkn, "vc-1 ipco", "smpte-RP2025"),
+
+	BOX_DEFINE_S(GF_4CC('G','M','C','W'), unkn, "stsd", "GPAC"),
+	BOX_DEFINE_S(GF_4CC('G','M','C','C'), unkn, "GMCW", "GPAC"),
+
+	/* for now we don't parse these*/
+	BOX_DEFINE_S(GF_4CC('u','n','c','C'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('c','m','p','d'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('c','p','a','l'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('c','p','a','t'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('c','l','e','v'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('s','p','l','z'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('s','n','u','c'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('s','b','p','m'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('c','l','o','c'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('f','p','c','k'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('d','i','s','i'), unkn, "video_sample_entry ipco", "rawff"),
+	BOX_DEFINE_S(GF_4CC('d','e','p','i'), unkn, "video_sample_entry ipco", "rawff"),
+
 /*
 	GF_ISOM_BOX_TYPE_CBMP	= GF_4CC( 'c', 'b', 'm', 'p' ),
 	GF_ISOM_BOX_TYPE_EQUI	= GF_4CC( 'e', 'q', 'u', 'i' ),
@@ -1603,8 +1670,6 @@ void gf_isom_registry_disable(u32 boxCode, Bool disable)
 static u32 get_box_reg_idx(u32 boxCode, u32 parent_type, u32 start_from)
 {
 	u32 i=0, count = gf_isom_get_num_supported_boxes();
-	const char *parent_name = parent_type ? gf_4cc_to_str(parent_type) : NULL;
-
 	if (!start_from) start_from = 1;
 
 	for (i=start_from; i<count; i++) {
@@ -1614,7 +1679,7 @@ static u32 get_box_reg_idx(u32 boxCode, u32 parent_type, u32 start_from)
 
 		if (!parent_type)
 			return i;
-		if (strstr(box_registry[i].parents_4cc, parent_name) != NULL)
+		if (strstr(box_registry[i].parents_4cc, gf_4cc_to_str(parent_type)) != NULL)
 			return i;
 		if (strstr(box_registry[i].parents_4cc, "*") != NULL)
 			return i;
@@ -1658,7 +1723,6 @@ GF_Box *gf_isom_box_new_ex(u32 boxType, u32 parentType, Bool skip_logs, Bool is_
 				break;
 			//some sample descritions are handled as generic ones but we know them, don't warn
 			case GF_ISOM_BOX_TYPE_STSD:
-				if (boxType==GF_ISOM_SUBTYPE_FFV1) break;
 				//fallthrough
 			default:
 				if (boxType==GF_ISOM_BOX_TYPE_GDAT) break;
@@ -1714,18 +1778,19 @@ GF_Box *gf_isom_box_new(u32 boxType)
 	return gf_isom_box_new_ex(boxType, 0, 0, GF_FALSE);
 }
 
-GF_Err gf_isom_box_array_read_ex(GF_Box *parent, GF_BitStream *bs, u32 parent_type)
+GF_Err gf_isom_box_array_read(GF_Box *parent, GF_BitStream *bs)
 {
 	GF_Err e;
+	u32 parent_type = parent->type;
 	GF_Box *a = NULL;
 	Bool skip_logs = (gf_bs_get_cookie(bs) & GF_ISOM_BS_COOKIE_NO_LOGS ) ? GF_TRUE : GF_FALSE;
 
 	//we may have terminators in some QT files (4 bytes set to 0 ...)
 	while (parent->size>=8) {
-		e = gf_isom_box_parse_ex(&a, bs, parent_type, GF_FALSE);
+		e = gf_isom_box_parse_ex(&a, bs, parent_type, GF_FALSE, skip_logs ? 0 : parent->size);
 		if (e) {
 			if (a) gf_isom_box_del(a);
-			return e;
+			return (e==GF_SKIP_BOX) ? GF_OK : e;
 		}
 		//sub box parsing aborted with no error
 		if (!a) return GF_OK;
@@ -1744,7 +1809,7 @@ GF_Err gf_isom_box_array_read_ex(GF_Box *parent, GF_BitStream *bs, u32 parent_ty
 		}
 
 		//check container validity
-		if (strlen(a->registry->parents_4cc)) {
+		if (parent_type && strlen(a->registry->parents_4cc)) {
 			Bool parent_OK = GF_FALSE;
 			const char *parent_code = gf_4cc_to_str(parent->type);
 			if (parent->type == GF_ISOM_BOX_TYPE_UNKNOWN)
@@ -1784,7 +1849,7 @@ GF_Err gf_isom_box_array_read_ex(GF_Box *parent, GF_BitStream *bs, u32 parent_ty
 		e = gf_list_add(parent->child_boxes, a);
 		if (e) return e;
 
-		if (parent->registry->add_rem_fn) {
+		if (parent->registry && parent->registry->add_rem_fn) {
 			e = parent->registry->add_rem_fn(parent, a, GF_FALSE);
 			if (e) {
 				if (e == GF_ISOM_INVALID_MEDIA) return GF_OK;
@@ -1853,7 +1918,7 @@ void gf_isom_check_position(GF_Box *s, GF_Box *child, u32 *pos)
 
 	s32 cur_pos = gf_list_find(s->child_boxes, child);
 
-	//happens when partially cloning boxes 
+	//happens when partially cloning boxes
 	if (cur_pos < 0) return;
 
 	if (cur_pos != (s32) *pos) {
@@ -1884,7 +1949,7 @@ GF_Err gf_isom_box_write(GF_Box *a, GF_BitStream *bs)
 	if (!a) return GF_BAD_PARAM;
 	//box has been disabled, do not write
 	if (!a->size) return GF_OK;
-	
+
 	if (a->registry->disabled) {
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_CONTAINER, ("[iso file] Box %s disabled registry, skip write\n", gf_4cc_to_str(a->type)));
 		return GF_OK;
@@ -2025,7 +2090,7 @@ u32 gf_isom_get_supported_box_type(u32 idx)
 
 #ifndef GPAC_DISABLE_ISOM_DUMP
 
-GF_Err gf_isom_box_dump_start(GF_Box *a, const char *name, FILE * trace)
+GF_Err gf_isom_box_dump_start_ex(GF_Box *a, const char *name, FILE * trace, Bool force_version)
 {
 	gf_fprintf(trace, "<%s ", name);
 	if (a->size > 0xFFFFFFFF) {
@@ -2049,7 +2114,7 @@ GF_Err gf_isom_box_dump_start(GF_Box *a, const char *name, FILE * trace)
 		gf_fprintf(trace, "}\" ");
 	}
 
-	if (a->registry->max_version_plus_one) {
+	if (a->registry->max_version_plus_one || force_version) {
 		gf_fprintf(trace, "Version=\"%d\" Flags=\"%d\" ", ((GF_FullBox*)a)->version,((GF_FullBox*)a)->flags);
 	}
 	gf_fprintf(trace, "Specification=\"%s\" ", a->registry->spec);
@@ -2060,6 +2125,10 @@ GF_Err gf_isom_box_dump_start(GF_Box *a, const char *name, FILE * trace)
 	}
 	return GF_OK;
 }
+GF_Err gf_isom_box_dump_start(GF_Box *a, const char *name, FILE * trace)
+{
+	return gf_isom_box_dump_start_ex(a, name, trace, GF_FALSE);
+}
 
 GF_Err gf_isom_box_dump(void *ptr, FILE * trace)
 {
