@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2019
+ *			Copyright (c) Telecom ParisTech 2000-2021
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -31,7 +31,8 @@ GF_Box *ghnt_box_new()
 {
 	GF_HintSampleEntryBox *tmp;
 	GF_SAFEALLOC(tmp, GF_HintSampleEntryBox);
-	if (tmp == NULL) return NULL;
+	if (!tmp) return NULL;
+	gf_isom_sample_entry_init((GF_SampleEntryBox *)tmp);
 
 	//this type is used internally for protocols that share the same base entry
 	//currently only RTP uses this, but a flexMux could use this entry too...
@@ -45,6 +46,7 @@ void ghnt_box_del(GF_Box *s)
 {
 	GF_HintSampleEntryBox *ptr;
 
+	gf_isom_sample_entry_predestroy((GF_SampleEntryBox *)s);
 	ptr = (GF_HintSampleEntryBox *)s;
 	if (ptr->hint_sample) gf_isom_hint_sample_del(ptr->hint_sample);
 	gf_free(ptr);
@@ -74,7 +76,7 @@ GF_Err ghnt_box_read(GF_Box *s, GF_BitStream *bs)
 		ptr->FEC_overhead = gf_bs_read_u16(bs);
 
 	}
-	return gf_isom_box_array_read(s, bs, NULL);
+	return gf_isom_box_array_read(s, bs);
 }
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
@@ -676,6 +678,7 @@ GF_Err gf_isom_hint_rtp_read(GF_RTPPacket *ptr, GF_BitStream *bs)
 		while (tempSize < TLVsize) {
 			e = gf_isom_box_parse(&a, bs);
 			if (e) return e;
+			if (!a) continue;
 			gf_list_add(ptr->TLV, a);
 			tempSize += (u32) a->size;
 		}
