@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2019
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / ISO Media File Format sub-project
@@ -344,23 +344,19 @@ GF_Err Media_GetESD(GF_MediaBox *mdia, u32 sampleDescIndex, GF_ESD **out_esd, Bo
 		if (entry->internal_type != GF_ISOM_SAMPLE_ENTRY_AUDIO)
 			return GF_ISOM_INVALID_MEDIA;
 	{
-		GF_OpusSpecificBox *e = ((GF_MPEGAudioSampleEntryBox*)entry)->cfg_opus;
-		GF_BitStream *bs_out;
-		if (!e) {
+		GF_OpusSpecificBox *opus_c;
+		if (true_desc_only)
+			return GF_ISOM_INVALID_MEDIA;
+
+		opus_c = ((GF_MPEGAudioSampleEntryBox*)entry)->cfg_opus;
+		if (!opus_c) {
 			GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("ESD not found for Opus\n)"));
 			break;
 		}
-
 		*out_esd = gf_odf_desc_esd_new(2);
 		(*out_esd)->decoderConfig->streamType = GF_STREAM_AUDIO;
 		(*out_esd)->decoderConfig->objectTypeIndication = GF_CODECID_OPUS;
-
-		//serialize box with header - compatibility with ffmpeg
-		bs_out = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-		gf_isom_box_size((GF_Box *) e);
-		gf_isom_box_write((GF_Box *) e, bs_out);
-		gf_bs_get_content(bs_out, & (*out_esd)->decoderConfig->decoderSpecificInfo->data, & (*out_esd)->decoderConfig->decoderSpecificInfo->dataLength);
-		gf_bs_del(bs_out);
+		gf_odf_opus_cfg_write(&opus_c->opcfg, & (*out_esd)->decoderConfig->decoderSpecificInfo->data, & (*out_esd)->decoderConfig->decoderSpecificInfo->dataLength);
 		break;
 	}
 	case GF_ISOM_SUBTYPE_3GP_H263:
@@ -648,6 +644,8 @@ GF_Err Media_GetSample(GF_MediaBox *mdia, u32 sampleNumber, GF_ISOSample **samp,
 			return GF_IO_ERR;
 		}
 		mdia->BytesMissing = 0;
+	} else {
+		(*samp)->dataLength = 0;
 	}
 
 	//finally rewrite the sample if this is an OD Access Unit or NAL-based one
@@ -696,8 +694,14 @@ GF_Err Media_CheckDataEntry(GF_MediaBox *mdia, u32 dataEntryIndex)
 
 	//ok, not self contained, let's go for it...
 	//we only support alias and URL boxes
-	if ((entry->type != GF_ISOM_BOX_TYPE_URL) && (entry->type != GF_QT_BOX_TYPE_ALIS) )
+	switch (entry->type) {
+	case GF_ISOM_BOX_TYPE_URL:
+	case GF_QT_BOX_TYPE_ALIS:
+	case GF_QT_BOX_TYPE_CIOS:
+		break;
+	default:
 		return GF_NOT_SUPPORTED;
+	}
 
 	if (mdia->mediaTrack->moov->mov->openMode == GF_ISOM_OPEN_WRITE) {
 		e = gf_isom_datamap_new(entry->location, NULL, GF_ISOM_DATA_MAP_READ, &map);
@@ -732,6 +736,7 @@ Bool Media_IsSelfContained(GF_MediaBox *mdia, u32 StreamDescIndex)
 	if (a->flags & 1) return 1;
 	/*QT specific*/
 	if (a->type == GF_QT_BOX_TYPE_ALIS) return 1;
+	if (a->type == GF_QT_BOX_TYPE_CIOS) return 1;
 	return 0;
 }
 
@@ -857,7 +862,6 @@ GF_Err Media_SetDuration(GF_TrackBox *trak)
 		if (Track_IsMPEG4Stream(trak->Media->handler->handlerType)) {
 			Media_GetESD(trak->Media, 1, &esd, 1);
 			if (esd && esd->URLString) trak->Media->mediaHeader->duration = (u64) -1;
-
 		}
 		return GF_OK;
 	}
@@ -866,13 +870,21 @@ GF_Err Media_SetDuration(GF_TrackBox *trak)
 	e = stbl_GetSampleDTS_and_Duration(trak->Media->information->sampleTable->TimeToSample, nbSamp, &DTS, &dur);
 	if (e < 0) return e;
 	DTS += dur;
-	trak->Media->mediaHeader->duration = DTS;
 
 	//do not do that for old arch compat which was not taking into account cts offset
-	if (gf_sys_old_arch_compat() || !trak->Media->information->sampleTable->CompositionOffset)
+	if (gf_sys_old_arch_compat() || !trak->Media->information->sampleTable->CompositionOffset) {
+		trak->Media->mediaHeader->duration = DTS;
 		return GF_OK;
-
+	}
 	//try to set duration according to spec: "should be the largest composition timestamp plus the duration of that sample"
+	s32 cts_o;
+	stbl_GetSampleCTS(trak->Media->information->sampleTable->CompositionOffset, nbSamp, &cts_o);
+	if (cts_o>0) DTS += cts_o;
+	if (DTS>trak->Media->mediaHeader->duration)
+		trak->Media->mediaHeader->duration = DTS;
+
+	//this can be more precise in some corner cases but takes way too long - we keep code for reference
+#if 0
 	//browse from sample_num_max_cts_delta (updated in read and edit to point to sample number with max cts offset)
 	u32 s_idx, min = trak->Media->information->sampleTable->CompositionOffset->sample_num_max_cts_delta;
 	if (!min) return GF_OK;
@@ -888,6 +900,7 @@ GF_Err Media_SetDuration(GF_TrackBox *trak)
 		}
 	}
 	trak->Media->mediaHeader->duration = DTS;
+#endif
 	return GF_OK;
 }
 
