@@ -445,20 +445,20 @@ GF_Err gf_isom_truehd_config_new(GF_ISOFile *the_file, u32 trackNumber, char *UR
 
 #ifndef GPAC_DISABLE_ISOM_WRITE
 GF_EXPORT
-GF_Err gf_isom_opus_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_OpusSpecificBox *cfg, char *URLname, char *URNname, u32 *outDescriptionIndex)
+GF_Err gf_isom_opus_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_OpusConfig *cfg, char *URLname, char *URNname, u32 *outDescriptionIndex)
 {
 	GF_TrackBox *trak;
 	GF_Err e;
 	u32 dataRefIndex;
 	GF_MPEGAudioSampleEntryBox *entry;
 	GF_SampleDescriptionBox *stsd;
-	ptrdiff_t offset;
 
+	if (!cfg) return GF_BAD_PARAM;
 	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
 
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
-	if (!trak || !trak->Media || !cfg) return GF_BAD_PARAM;
+	if (!trak || !trak->Media) return GF_BAD_PARAM;
 
 	//get or create the data ref
 	e = Media_FindDataRef(trak->Media->information->dataInformation->dref, URLname, URNname, &dataRefIndex);
@@ -476,9 +476,8 @@ GF_Err gf_isom_opus_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_OpusSpe
 	if (!entry) return GF_OUT_OF_MEM;
 	entry->cfg_opus = (GF_OpusSpecificBox*)gf_isom_box_new_parent(&entry->child_boxes, GF_ISOM_BOX_TYPE_DOPS);
 	if (!entry->cfg_opus) return GF_OUT_OF_MEM;
-	//skip box header
-	offset = (ptrdiff_t)&cfg->version - (ptrdiff_t)cfg;
-	memcpy((char*)entry->cfg_opus + offset, (char*)cfg + offset, sizeof(GF_OpusSpecificBox) - (size_t)offset);
+	entry->cfg_opus->opcfg = *cfg;
+	entry->cfg_opus->opcfg.version = 0;
 
 	entry->dataReferenceIndex = dataRefIndex;
 	*outDescriptionIndex = gf_list_count(stsd->child_boxes);
@@ -486,8 +485,7 @@ GF_Err gf_isom_opus_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_OpusSpe
 }
 #endif
 
-GF_EXPORT
-GF_Err gf_isom_opus_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 StreamDescriptionIndex, u8 **dsi, u32 *dsi_size)
+static GF_Err gf_isom_opus_config_get_internal(GF_ISOFile *the_file, u32 trackNumber, u32 StreamDescriptionIndex, u8 **dsi, u32 *dsi_size, GF_OpusConfig *ocfg)
 {
 	u32 type;
 	GF_TrackBox *trak;
@@ -495,6 +493,8 @@ GF_Err gf_isom_opus_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 Stream
 	trak = gf_isom_get_track_from_file(the_file, trackNumber);
 	if (dsi) *dsi = NULL;
 	if (dsi_size) *dsi_size = 0;
+	if (ocfg) memset(ocfg, 0, sizeof(GF_OpusConfig));
+
 	if (!trak || !StreamDescriptionIndex) return GF_BAD_PARAM;
 
 	entry = (GF_MPEGAudioSampleEntryBox *)gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, StreamDescriptionIndex-1);
@@ -512,15 +512,27 @@ GF_Err gf_isom_opus_config_get(GF_ISOFile *the_file, u32 trackNumber, u32 Stream
 		return GF_BAD_PARAM;
 
 	if (dsi && dsi_size) {
-		GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-		gf_isom_box_size((GF_Box *)entry->cfg_opus);
-		gf_isom_box_write((GF_Box *)entry->cfg_opus, bs);
-		gf_bs_get_content(bs, dsi, dsi_size);
-		gf_bs_del(bs);
+		gf_odf_opus_cfg_write(&entry->cfg_opus->opcfg, dsi, dsi_size);
 	}
+	if (ocfg)
+		*ocfg = entry->cfg_opus->opcfg;
 	return GF_OK;
 }
 
+GF_EXPORT
+GF_Err gf_isom_opus_config_get(GF_ISOFile *isom_file, u32 trackNumber, u32 sampleDescriptionIndex, u8 **dsi, u32 *dsi_size)
+{
+	return gf_isom_opus_config_get_internal(isom_file, trackNumber, sampleDescriptionIndex, dsi, dsi_size, NULL);
+}
+
+GF_EXPORT
+GF_Err gf_isom_opus_config_get_desc(GF_ISOFile *isom_file, u32 trackNumber, u32 sampleDescriptionIndex, GF_OpusConfig *opcfg)
+{
+	return gf_isom_opus_config_get_internal(isom_file, trackNumber, sampleDescriptionIndex, NULL, NULL, opcfg);
+
+}
+
+
 #ifndef GPAC_DISABLE_ISOM_WRITE
 
 GF_EXPORT
@@ -702,6 +714,35 @@ GF_Err gf_isom_ac3_config_new(GF_ISOFile *the_file, u32 trackNumber, GF_AC3Confi
 	return e;
 }
 
+GF_EXPORT
+GF_Err gf_isom_ac3_config_update(GF_ISOFile *the_file, u32 trackNumber, u32 sampleDescriptionIndex, GF_AC3Config *cfg)
+{
+	GF_TrackBox *trak;
+	GF_Err e;
+	GF_MPEGAudioSampleEntryBox *entry;
+
+	e = CanAccessMovie(the_file, GF_ISOM_OPEN_WRITE);
+	if (e) return e;
+
+	trak = gf_isom_get_track_from_file(the_file, trackNumber);
+	if (!trak || !trak->Media || !cfg || !sampleDescriptionIndex) return GF_BAD_PARAM;
+
+	if (!the_file->keep_utc)
+		trak->Media->mediaHeader->modificationTime = gf_isom_get_mp4time();
+
+	entry = gf_list_get(trak->Media->information->sampleTable->SampleDescription->child_boxes, sampleDescriptionIndex-1);
+	if (!entry) return GF_BAD_PARAM;
+	if (!entry->cfg_ac3) return GF_BAD_PARAM;
+
+	if (entry->cfg_ac3->cfg.is_ec3) {
+		if (!cfg->is_ec3) return GF_BAD_PARAM;
+	} else {
+		if (cfg->is_ec3) return GF_BAD_PARAM;
+	}
+	memcpy(&entry->cfg_ac3->cfg, cfg, sizeof(GF_AC3Config));
+	return GF_OK;
+}
+
 GF_EXPORT
 GF_Err gf_isom_flac_config_new(GF_ISOFile *the_file, u32 trackNumber, u8 *metadata, u32 metadata_size, const char *URLname, const char *URNname, u32 *outDescriptionIndex)
 {
@@ -1283,7 +1324,7 @@ GF_Err gf_isom_new_stxt_description(GF_ISOFile *movie, u32 trackNumber, u32 type
 	}
 
 	if (!mime) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_CONTAINER, ("Text (Metadata, Subtitle or SimpleText) SampleEntry: mime is mandatory. Using text/plain.\n"));
+		GF_LOG(GF_LOG_INFO, GF_LOG_CONTAINER, ("Text (Metadata, Subtitle or SimpleText) missing mime, using text/plain.\n"));
 		mime = "text/plain";
 	}
 
@@ -1659,7 +1700,7 @@ GF_Err gf_isom_tmcd_config_new(GF_ISOFile *the_file, u32 trackNumber, u32 fps_nu
 	return e;
 }
 
-GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const char *URLname, const char *URNname, u32 *outDescriptionIndex, u8 *dsi, u32 dsi_size)
+GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const char *URLname, const char *URNname, u32 *outDescriptionIndex, u8 *dsi, u32 dsi_size, u32 mha_subtype)
 {
 	GF_TrackBox *trak;
 	GF_Err e;
@@ -1668,7 +1709,6 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 
 	e = CanAccessMovie(movie, GF_ISOM_OPEN_WRITE);
 	if (e) return e;
-	if (dsi_size<6) return GF_BAD_PARAM;
 
 	trak = gf_isom_get_track_from_file(movie, trackNumber);
 	if (!trak || !trak->Media) return GF_BAD_PARAM;
@@ -1679,6 +1719,18 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 	default:
 		return GF_BAD_PARAM;
 	}
+	switch (mha_subtype) {
+	case GF_ISOM_BOX_TYPE_MHA1:
+	case GF_ISOM_BOX_TYPE_MHA2:
+		if (!dsi || (dsi_size<6)) return GF_BAD_PARAM;
+		break;
+	case GF_ISOM_BOX_TYPE_MHM1:
+	case GF_ISOM_BOX_TYPE_MHM2:
+		if (dsi_size && (dsi_size<6)) return GF_BAD_PARAM;
+		break;
+	default:
+		return GF_BAD_PARAM;
+	}
 
 	//get or create the data ref
 	e = Media_FindDataRef(trak->Media->information->dataInformation->dref, (char *)URLname, (char *)URNname, &dataRefIndex);
@@ -1690,23 +1742,25 @@ GF_Err gf_isom_new_mpha_description(GF_ISOFile *movie, u32 trackNumber, const ch
 	if (!movie->keep_utc)
 		trak->Media->mediaHeader->modificationTime = gf_isom_get_mp4time();
 
-	mpa = (GF_MPEGAudioSampleEntryBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_MHA1);
+	mpa = (GF_MPEGAudioSampleEntryBox *) gf_isom_box_new(mha_subtype);
 	if (!mpa) return GF_OUT_OF_MEM;
 	mpa->dataReferenceIndex = dataRefIndex;
 	gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, mpa);
 	if (outDescriptionIndex) *outDescriptionIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
 
-	mpa->cfg_mha = (GF_MHAConfigBox *) gf_isom_box_new_parent(&mpa->child_boxes, GF_ISOM_BOX_TYPE_MHAC);
-	if (!mpa->cfg_mha) return GF_OUT_OF_MEM;
-	mpa->cfg_mha->configuration_version = dsi[0];
-	mpa->cfg_mha->mha_pl_indication = dsi[1];
-	mpa->cfg_mha->reference_channel_layout = dsi[2];
-	mpa->cfg_mha->mha_config_size = dsi[3];
-	mpa->cfg_mha->mha_config_size <<= 8;
-	mpa->cfg_mha->mha_config_size |= dsi[4];
-	mpa->cfg_mha->mha_config = gf_malloc(sizeof(u8) * mpa->cfg_mha->mha_config_size);
-	if (!mpa->cfg_mha->mha_config) return GF_OUT_OF_MEM;
-	memcpy(mpa->cfg_mha->mha_config, dsi+5, dsi_size-5);
+	if (dsi) {
+		mpa->cfg_mha = (GF_MHAConfigBox *) gf_isom_box_new_parent(&mpa->child_boxes, GF_ISOM_BOX_TYPE_MHAC);
+		if (!mpa->cfg_mha) return GF_OUT_OF_MEM;
+		mpa->cfg_mha->configuration_version = dsi[0];
+		mpa->cfg_mha->mha_pl_indication = dsi[1];
+		mpa->cfg_mha->reference_channel_layout = dsi[2];
+		mpa->cfg_mha->mha_config_size = dsi[3];
+		mpa->cfg_mha->mha_config_size <<= 8;
+		mpa->cfg_mha->mha_config_size |= dsi[4];
+		mpa->cfg_mha->mha_config = gf_malloc(sizeof(u8) * mpa->cfg_mha->mha_config_size);
+		if (!mpa->cfg_mha->mha_config) return GF_OUT_OF_MEM;
+		memcpy(mpa->cfg_mha->mha_config, dsi+5, dsi_size-5);
+	}
 	return GF_OK;
 }
 
