@@ -211,7 +211,7 @@ default_sync:
 			} else {
 				esd->decoderConfig->rvc_config = (GF_DefaultDescriptor *) gf_odf_desc_new(GF_ODF_DSI_TAG);
 				if (mime_type && !strcmp(mime_type, "application/rvc-config+xml+gz") ) {
-#if !defined(GPAC_DISABLE_CORE_TOOLS) && !defined(GPAC_DISABLE_ZLIB)
+#if !defined(GPAC_DISABLE_ZLIB)
 					gf_gz_decompress_payload(rvc_cfg_data, rvc_cfg_size, &esd->decoderConfig->rvc_config->data, &esd->decoderConfig->rvc_config->dataLength);
 					gf_free(rvc_cfg_data);
 #endif
@@ -441,6 +441,17 @@ GF_TrunEntry *traf_get_sample_entry(GF_TrackFragmentBox *traf, u32 sample_index)
 }
 #endif
 
+static u32 saio_get_index(GF_TrackFragmentBox *traf, u32 sample_idx)
+{
+	u32 k, all_samples=0, saio_idx=0;
+	for (k=0; k<gf_list_count(traf->TrackRuns); k++) {
+		GF_TrackFragmentRunBox *trun = gf_list_get(traf->TrackRuns, k);
+		all_samples+=trun->nb_samples;
+		if (sample_idx<all_samples) break;
+		saio_idx++;
+	}
+	return saio_idx;
+}
 
 GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragmentBox *moof_box, u64 moof_offset, s32 compressed_diff, u64 *cumulated_offset)
 {
@@ -786,7 +797,7 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 
 			if (store_traf_map && first_samp_in_traf) {
 				first_samp_in_traf = GF_FALSE;
-				e = stbl_AppendTrafMap(trak->Media->information->sampleTable, is_seg_start, seg_start, frag_start, moof_template, moof_template_size, sidx_start, sidx_end, ent->nb_pack);
+				e = stbl_AppendTrafMap(trak->moov->mov, trak->Media->information->sampleTable, is_seg_start, seg_start, frag_start, moof_template, moof_template_size, sidx_start, sidx_end, ent->nb_pack);
 				if (e) return e;
 				//do not deallocate, the memory is now owned by traf map
 				moof_template = NULL;
@@ -1004,7 +1015,7 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 		|| traf->sample_encryption) {
 		/*Merge sample auxiliary encryption information*/
 		GF_SampleEncryptionBox *senc = NULL;
-		u32 scheme_type;
+		u32 scheme_type=0;
 		gf_isom_get_cenc_info(trak->moov->mov, track_num, DescIndex, NULL, &scheme_type, NULL);
 
 		if (traf->sample_encryption) {
@@ -1086,8 +1097,13 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 					const u8 *key_info=NULL;
 					u32 key_info_size, samp_num;
 					u64 cur_position;
-					if (nb_saio != 1)
-						offset = saio->offsets[i] + moof_offset;
+					if (nb_saio != 1) {
+						u32 saio_idx = saio_get_index(traf, i);
+						if (saio_idx>=saio->entry_count)
+							return GF_ISOM_INVALID_FILE;
+
+						offset = saio->offsets[saio_idx] + moof_offset;
+					}
 					size = saiz->default_sample_info_size ? saiz->default_sample_info_size : saiz->sample_info_size[i];
 
 					cur_position = gf_bs_get_position(trak->moov->mov->movieFileMap->bs);
@@ -1135,8 +1151,8 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 
 					e = gf_isom_cenc_merge_saiz_saio(senc, trak->Media->information->sampleTable, samp_num, offset, size);
 					if (e) return e;
-					if (nb_saio == 1)
-						offset += size;
+					//always increment size even for saio.nb_entries>1
+					offset += size;
 				}
 			}
 		} else if (traf->sample_encryption) {
@@ -1181,16 +1197,22 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 		}
 		u64 offset = saio->offsets[0] + moof_offset;
 		u32 nb_saio = saio->entry_count;
-		if ((nb_saio>1) && (saio->entry_count != saiz->sample_count)) {
-			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[isobmf] Number of size and offset mismatch in auxiliary type %s aux info type %d, cannot merge SAI\n", gf_4cc_to_str(saiz->aux_info_type), saiz->aux_info_type_parameter));
+		if ((nb_saio>1) && (saio->entry_count != gf_list_count(traf->TrackRuns))) {
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[isobmf] Number of SAI offset does not match number of fragments, cannot merge SAI %s aux info type %d\n", gf_4cc_to_str(saiz->aux_info_type), saiz->aux_info_type_parameter));
 			continue;
 		}
 
 		u32 sai_max_size=0;
 		u8 *sai = NULL;
 		for (j=0; j < saiz->sample_count; j++) {
-			if (nb_saio != 1)
+			if (nb_saio != 1) {
+				u32 saio_idx = saio_get_index(traf, i);
+				if (saio_idx>=saio->entry_count) {
+				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[isobmf] Number of offset less than number of fragments, cannot merge SAI %s aux info type %d\n", gf_4cc_to_str(saiz->aux_info_type), saiz->aux_info_type_parameter));
+					break;
+				}
 				offset = saio->offsets[j] + moof_offset;
+			}
 			size = saiz->default_sample_info_size ? saiz->default_sample_info_size : saiz->sample_info_size[j];
 
 			u64 cur_position = gf_bs_get_position(trak->moov->mov->movieFileMap->bs);
@@ -1210,8 +1232,8 @@ GF_Err MergeTrack(GF_TrackBox *trak, GF_TrackFragmentBox *traf, GF_MovieFragment
 				GF_LOG(GF_LOG_ERROR, GF_LOG_CONTAINER, ("[isobmf] Failed to merge sai data: %s\n", gf_error_to_string(e) ));
 			}
 
-			if (nb_saio == 1)
-				offset += size;
+			//always increment size even for saio.nb_entries>1
+			offset += size;
 		}
 		if (sai) gf_free(sai);
 	}
@@ -1573,7 +1595,7 @@ GF_Err Track_SetStreamDescriptor(GF_TrackBox *trak, u32 StreamDescriptionIndex,
 		entry_type = entry->type;
 		GF_ProtectionSchemeInfoBox *sinf = (GF_ProtectionSchemeInfoBox *) gf_isom_box_find_child(entry->child_boxes, GF_ISOM_BOX_TYPE_SINF);
 		if (sinf && sinf->original_format) entry_type = sinf->original_format->data_format;
-		
+
 		switch (entry_type) {
 		case GF_ISOM_BOX_TYPE_MP4S:
 			//OK, delete the previous ESD
@@ -1734,7 +1756,7 @@ GF_Err Track_SetStreamDescriptor(GF_TrackBox *trak, u32 StreamDescriptionIndex,
 		if (!trak->Media->information->sampleTable->SampleDescription->child_boxes)
 			trak->Media->information->sampleTable->SampleDescription->child_boxes = gf_list_new();
 		gf_list_add(trak->Media->information->sampleTable->SampleDescription->child_boxes, entry);
-		
+
 		e = stsd_on_child_box((GF_Box*)trak->Media->information->sampleTable->SampleDescription, (GF_Box *) entry, GF_FALSE);
 		if (e) return e;
 		if(outStreamIndex) *outStreamIndex = gf_list_count(trak->Media->information->sampleTable->SampleDescription->child_boxes);
