@@ -65,167 +65,6 @@ static GF_Err gf_export_message(GF_MediaExporter *dumper, GF_Err e, char *format
 	return e;
 }
 
-#ifndef GPAC_DISABLE_AV_PARSERS
-/*that's very very crude, we only support vorbis & theora in MP4 - this will need cleanup as soon as possible*/
-static GF_Err gf_dump_to_ogg(GF_MediaExporter *dumper, char *szName, u32 track)
-{
-#ifdef GPAC_DISABLE_OGG
-	return GF_NOT_SUPPORTED;
-#else
-	FILE *out;
-	ogg_stream_state os;
-	ogg_packet op;
-	ogg_page og;
-	u32 count, i, di, theora_kgs, nb_i, nb_p;
-	Bool flush_first = GF_TRUE;
-	GF_ISOSample *samp;
-	GF_ESD *esd = gf_isom_get_esd(dumper->file, track, 1);
-
-
-	memset(&os, 0, sizeof(ogg_stream_state));
-	memset(&og, 0, sizeof(ogg_page));
-	memset(&op, 0, sizeof(ogg_packet));
-
-	if (gf_sys_is_test_mode()) {
-		ogg_stream_init(&os, 1);
-	} else {
-		gf_rand_init(GF_TRUE);
-		ogg_stream_init(&os, gf_rand());
-	}
-
-	out = szName ? gf_fopen(szName, "wb") : stdout;
-	if (!out) return gf_export_message(dumper, GF_IO_ERR, "Error opening %s for writing - check disk access & permissions", szName);
-
-	theora_kgs = 0;
-	if (esd && esd->decoderConfig
-		&& esd->decoderConfig->decoderSpecificInfo
-		&& esd->decoderConfig->decoderSpecificInfo->data
-	) {
-		GF_BitStream *bs = gf_bs_new(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, GF_BITSTREAM_READ);
-		if (esd->decoderConfig->objectTypeIndication==GF_CODECID_OPUS) {
-			GF_BitStream *bs_out;
-			GF_OpusSpecificBox *dops = (GF_OpusSpecificBox *) gf_isom_box_new(GF_ISOM_BOX_TYPE_DOPS);
-			dops->size = gf_bs_read_u32(bs);
-			gf_bs_read_u32(bs);
-			gf_isom_box_read((GF_Box *)dops, bs);
-			bs_out = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
-			gf_bs_write_data(bs_out, "OpusHead", 8);
-			gf_bs_write_u8(bs_out, 1);//version
-			gf_bs_write_u8(bs_out, dops->OutputChannelCount);
-			gf_bs_write_u16_le(bs_out, dops->PreSkip);
-			gf_bs_write_u32_le(bs_out, dops->InputSampleRate);
-			gf_bs_write_u16_le(bs_out, dops->OutputGain);
-			gf_bs_write_u8(bs_out, dops->ChannelMappingFamily);
-			if (dops->ChannelMappingFamily) {
-				gf_bs_write_u8(bs_out, dops->StreamCount);
-				gf_bs_write_u8(bs_out, dops->CoupledCount);
-				gf_bs_write_data(bs, (char *) dops->ChannelMapping, dops->OutputChannelCount);
-			}
-			gf_isom_box_del((GF_Box*)dops);
-
-			gf_bs_get_content(bs_out, &op.packet, &op.bytes);
-			gf_bs_del(bs_out);
-			ogg_stream_packetin(&os, &op);
-			gf_free(op.packet);
-			op.packetno ++;
-
-		} else {
-			while (gf_bs_available(bs)) {
-				op.bytes = gf_bs_read_u16(bs);
-				op.packet = (unsigned char*)gf_malloc(sizeof(char) * op.bytes);
-				gf_bs_read_data(bs, (char*)op.packet, op.bytes);
-				ogg_stream_packetin(&os, &op);
-
-				if (flush_first) {
-					ogg_stream_pageout(&os, &og);
-					gf_fwrite(og.header, og.header_len, out);
-					gf_fwrite(og.body, og.body_len, out);
-					flush_first = 0;
-
-					if (esd->decoderConfig->objectTypeIndication==GF_CODECID_THEORA) {
-						u32 kff;
-						GF_BitStream *vbs = gf_bs_new((char*)op.packet, op.bytes, GF_BITSTREAM_READ);
-						gf_bs_skip_bytes(vbs, 40);
-						gf_bs_read_int(vbs, 6); /* quality */
-						kff = 1 << gf_bs_read_int(vbs, 5);
-						gf_bs_del(vbs);
-
-						theora_kgs = 0;
-						kff--;
-						while (kff) {
-							theora_kgs ++;
-							kff >>= 1;
-						}
-					}
-				}
-				gf_free(op.packet);
-				op.packetno ++;
-			}
-		}
-		gf_bs_del(bs);
-	}
-	gf_odf_desc_del((GF_Descriptor *)esd);
-
-	while (ogg_stream_pageout(&os, &og)>0) {
-		gf_fwrite(og.header, og.header_len, out);
-		gf_fwrite(og.body, og.body_len, out);
-	}
-
-	op.granulepos = -1;
-
-	count = gf_isom_get_sample_count(dumper->file, track);
-
-	nb_i = nb_p = 0;
-	samp = gf_isom_get_sample(dumper->file, track, 1, &di);
-	for (i=0; i<count; i++) {
-		GF_ISOSample *next_samp = gf_isom_get_sample(dumper->file, track, i+2, &di);
-		if (!samp) break;
-		op.bytes = samp->dataLength;
-		op.packet = (unsigned char*)samp->data;
-		op.packetno ++;
-
-		if (theora_kgs) {
-			if (samp->IsRAP) {
-				if (i) nb_i+=nb_p+1;
-				nb_p = 0;
-			} else {
-				nb_p++;
-			}
-			op.granulepos = nb_i;
-			op.granulepos <<= theora_kgs;
-			op.granulepos |= nb_p;
-		} else {
-			if (next_samp) op.granulepos = next_samp->DTS;
-		}
-		if (!next_samp) op.e_o_s = 1;
-
-		ogg_stream_packetin(&os, &op);
-
-		gf_isom_sample_del(&samp);
-		samp = next_samp;
-		next_samp = NULL;
-		gf_set_progress("OGG Export", i+1, count);
-		if (dumper->flags & GF_EXPORT_DO_ABORT) break;
-
-		while (ogg_stream_pageout(&os, &og)>0) {
-			gf_fwrite(og.header, og.header_len, out);
-			gf_fwrite(og.body, og.body_len, out);
-		}
-	}
-	if (samp) gf_isom_sample_del(&samp);
-
-	while (ogg_stream_flush(&os, &og)>0) {
-		gf_fwrite(og.header, og.header_len, out);
-		gf_fwrite(og.body, og.body_len, out);
-	}
-	ogg_stream_clear(&os);
-	if (szName) gf_fclose(out);
-	return GF_OK;
-#endif
-}
-#endif
-
-
 #ifndef GPAC_DISABLE_AV_PARSERS
 static GF_Err gf_dump_to_vobsub(GF_MediaExporter *dumper, char *szName, u32 track, char *dsi, u32 dsiSize)
 {
@@ -631,6 +470,7 @@ GF_Err gf_media_export_isom(GF_MediaExporter *dumper)
 GF_Err gf_media_export_webvtt_metadata(GF_MediaExporter *dumper)
 {
 	GF_ESD *esd;
+	GF_Err e = GF_OK;
 	char szName[1000], szMedia[1000];
 	FILE *med, *vtt;
 	u32 w, h;
@@ -827,21 +667,20 @@ GF_Err gf_media_export_webvtt_metadata(GF_MediaExporter *dumper)
 			gf_fprintf(vtt, "\n");
 		}
 		if (med) {
-			gf_fwrite(samp->data, samp->dataLength, med);
+			if (gf_fwrite(samp->data, samp->dataLength, med) != samp->dataLength)
+				e = GF_IO_ERR;
 		} else if (dumper->flags & GF_EXPORT_WEBVTT_META_EMBEDDED) {
 			if (isText) {
 				samp->data = (char *)gf_realloc(samp->data, samp->dataLength+1);
 				samp->data[samp->dataLength] = 0;
 				gf_fprintf(vtt, "%s\n", samp->data);
 			} else {
-				u32 b64_size;
+				u32 b64_size = samp->dataLength*2 + 3;
 				char *b64;
-				b64 = (char *)gf_malloc(samp->dataLength*3);
-				b64_size = gf_base64_encode(samp->data, samp->dataLength, b64, samp->dataLength*3);
-				if (b64_size != (u32)-1) {
-					b64[b64_size] = 0;
-					gf_fprintf(vtt, "%s\n", b64);
-				}
+				b64 = (char *)gf_malloc(sizeof(char)*b64_size);
+				b64_size = gf_base64_encode(samp->data, samp->dataLength, b64, b64_size);
+				b64[b64_size] = 0;
+				gf_fprintf(vtt, "%s\n", b64);
 				gf_free(b64);
 			}
 		}
@@ -854,7 +693,7 @@ GF_Err gf_media_export_webvtt_metadata(GF_MediaExporter *dumper)
 	}
 	if (med) gf_fclose(med);
 	gf_fclose(vtt);
-	return GF_OK;
+	return e;
 }
 
 #endif /*GPAC_DISABLE_VTT*/
@@ -863,6 +702,7 @@ GF_Err gf_media_export_webvtt_metadata(GF_MediaExporter *dumper)
 GF_Err gf_media_export_six(GF_MediaExporter *dumper)
 {
 	GF_ESD *esd;
+	GF_Err e=GF_OK;
 	char szName[1000], szMedia[1000];
 	FILE *media, *six;
 	u32 track, i, di, count, pos, header_size;
@@ -930,7 +770,8 @@ GF_Err gf_media_export_six(GF_MediaExporter *dumper)
 			} else
 #endif
 			{
-				gf_fwrite(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, media);
+				if (gf_fwrite(esd->decoderConfig->decoderSpecificInfo->data, esd->decoderConfig->decoderSpecificInfo->dataLength, media) != esd->decoderConfig->decoderSpecificInfo->dataLength)
+					e = GF_IO_ERR;
 				header_size = esd->decoderConfig->decoderSpecificInfo->dataLength;
 			}
 		}
@@ -951,7 +792,8 @@ GF_Err gf_media_export_six(GF_MediaExporter *dumper)
 		if (!samp) break;
 
 		if (media) {
-			gf_fwrite(samp->data, samp->dataLength, media);
+			if (gf_fwrite(samp->data, samp->dataLength, media)!=samp->dataLength)
+				e = GF_IO_ERR;
 		}
 
 		gf_fprintf(six, "<unit time=\""LLU"\" ", samp->DTS);
@@ -969,7 +811,7 @@ GF_Err gf_media_export_six(GF_MediaExporter *dumper)
 	gf_fprintf(six, "</%s>\n", szRootName);
 	if (media) gf_fclose(media);
 	gf_fclose(six);
-	return GF_OK;
+	return e;
 
 }
 
@@ -989,7 +831,7 @@ GF_Err gf_media_export_saf(GF_MediaExporter *dumper)
 	Bool is_stdout = 0;
 	FILE *saf_f;
 	SAFInfo safs[1024];
-
+	GF_Err e=GF_OK;
 	if (dumper->flags & GF_EXPORT_PROBE_ONLY) return GF_OK;
 
 	s_count = tot_samp = 0;
@@ -1032,13 +874,13 @@ GF_Err gf_media_export_saf(GF_MediaExporter *dumper)
 				mime = "audio/amr-wb";
 				break;
 			case GF_ISOM_SUBTYPE_3GP_EVRC:
-				mime = "audio/evrc";
+				mime = "audio/x-evrc-es";
 				break;
 			case GF_ISOM_SUBTYPE_3GP_QCELP:
-				mime = "audio/qcelp";
+				mime = "audio/x-qcelp-es";
 				break;
 			case GF_ISOM_SUBTYPE_3GP_SMV:
-				mime = "audio/smv";
+				mime = "audio/x-smv-es";
 				break;
 			}
 			if (!mime) continue;
@@ -1089,7 +931,7 @@ GF_Err gf_media_export_saf(GF_MediaExporter *dumper)
 		while (1) {
 			gf_saf_mux_for_time(mux, (u32) -1, 0, &data, &size);
 			if (!data) break;
-			gf_fwrite(data, size, saf_f);
+			if (gf_fwrite(data, size, saf_f) != size) e = GF_IO_ERR;
 			gf_free(data);
 		}
 		gf_set_progress("SAF Export", samp_done, tot_samp);
@@ -1097,14 +939,14 @@ GF_Err gf_media_export_saf(GF_MediaExporter *dumper)
 	}
 	gf_saf_mux_for_time(mux, (u32) -1, 1, &data, &size);
 	if (data) {
-		gf_fwrite(data, size, saf_f);
+		if (gf_fwrite(data, size, saf_f)!=size) e = GF_IO_ERR;
 		gf_free(data);
 	}
 	if (!is_stdout)
 		gf_fclose(saf_f);
 
 	gf_saf_mux_del(mux);
-	return GF_OK;
+	return e;
 #else
 	return GF_NOT_SUPPORTED;
 #endif
@@ -1220,18 +1062,12 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 			}
 			break;
 		}
-		//TODO, move these two to filters one of these days
-		if ((codec_id==GF_CODECID_VORBIS) || (codec_id==GF_CODECID_THEORA) || (codec_id==GF_CODECID_OPUS)) {
-			char *outname = dumper->out_name;
-			if (outname && !strcmp(outname, "std")) outname=NULL;
-			if (esd) gf_odf_desc_del((GF_Descriptor *) esd);
-#ifndef GPAC_DISABLE_AV_PARSERS
-			return gf_dump_to_ogg(dumper, outname, track_num);
-#else
-			return GF_NOT_SUPPORTED;
-#endif
 
-		}
+		if ((codec_id==GF_CODECID_VORBIS) || (codec_id==GF_CODECID_THEORA) || (codec_id==GF_CODECID_SPEEX))
+			strcpy(szExt, "ogg");
+		else if (codec_id==GF_CODECID_OPUS)
+			strcpy(szExt, "opus");
+
 		if (codec_id==GF_CODECID_SUBPIC) {
 #ifndef GPAC_DISABLE_AV_PARSERS
 			char *dsi = NULL;
@@ -1269,6 +1105,7 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 					} else if (dumper->track_type) {
 						if ((dumper->track_type==1) && (tki->stream_type!=GF_STREAM_VISUAL)) continue;
 						if ((dumper->track_type==2) && (tki->stream_type!=GF_STREAM_AUDIO)) continue;
+						if ((dumper->track_type==3) && (tki->stream_type!=GF_STREAM_TEXT)) continue;
 					}
 
 					found = GF_TRUE;
@@ -1303,6 +1140,7 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 	}
 	file_out = NULL;
 	args = NULL;
+	Bool load_dest = GF_FALSE;
 
 	if (dumper->flags & GF_EXPORT_REMUX) {
 		file_out = gf_fs_load_destination(fsess, dumper->out_name, NULL, NULL, &e);
@@ -1361,6 +1199,7 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 				e |= gf_dynstrcat(&args, ":ext=", NULL);
 				e |= gf_dynstrcat(&args, szExt, NULL);
 			}
+			load_dest = GF_TRUE;
 		} else if ((dumper->trackID || dumper->track_type) && use_dynext) {
 			e |= gf_dynstrcat(&args, ":dynext", NULL);
 		}
@@ -1371,7 +1210,12 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 			return e;
 		}
 
-		file_out = gf_fs_load_filter(fsess, args, &e);
+		if (load_dest) {
+			//skip fout:dst= whenever we have an extension specified, to allow using meta filters (ffmx) 
+			file_out = gf_fs_load_destination(fsess, args+9, NULL, NULL, &e);
+		} else {
+			file_out = gf_fs_load_filter(fsess, args, &e);
+		}
 		if (!file_out) {
 			gf_fs_del(fsess);
 			if (args) gf_free(args);
@@ -1488,7 +1332,7 @@ static GF_Err gf_media_export_filters(GF_MediaExporter *dumper)
 	}
 
 	if (dumper->track_type) {
-		const char *mtype = (dumper->track_type==1) ? "video" : "audio";
+		const char *mtype = (dumper->track_type==1) ? "video" : ((dumper->track_type==2) ? "audio" : "text");
 		if (dumper->trackID) {
 			sprintf(szSubArgs, "%s%d", mtype, dumper->trackID);
 		} else {
