@@ -605,17 +605,20 @@ GF_AVCConfig *gf_odf_avc_cfg_read(u8 *dsi, u32 dsi_size)
 		u32 size = gf_bs_read_int(bs, 16);
 		if ((size>gf_bs_available(bs)) || (size<2)) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
 		GF_SAFEALLOC(sl, GF_NALUFFParam );
 		if (!sl) {
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
 		sl->size = size;
 		sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
 		if (!sl->data) {
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
@@ -628,17 +631,20 @@ GF_AVCConfig *gf_odf_avc_cfg_read(u8 *dsi, u32 dsi_size)
 		u32 size = gf_bs_read_int(bs, 16);
 		if ((size>gf_bs_available(bs)) || (size<2)) {
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
 		GF_SAFEALLOC(sl, GF_NALUFFParam );
 		if (!sl) {
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
 		sl->size = size;
 		sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
 		if (!sl->data) {
+			gf_bs_del(bs);
 			gf_odf_avc_cfg_del(avcc);
 			return NULL;
 		}
@@ -661,17 +667,20 @@ GF_AVCConfig *gf_odf_avc_cfg_read(u8 *dsi, u32 dsi_size)
 				u32 size = gf_bs_read_int(bs, 16);
 				if ((size>gf_bs_available(bs)) || (size<2)) {
 					GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[AVC] Wrong param set size %d\n", size));
+					gf_bs_del(bs);
 					gf_odf_avc_cfg_del(avcc);
 					return NULL;
 				}
 				GF_SAFEALLOC(sl, GF_NALUFFParam );
 				if (!sl) {
+					gf_bs_del(bs);
 					gf_odf_avc_cfg_del(avcc);
 					return NULL;
 				}
 				sl->size = size;
 				sl->data = (char*)gf_malloc(sizeof(char)*sl->size);
 				if (!sl->data) {
+					gf_bs_del(bs);
 					gf_odf_avc_cfg_del(avcc);
 					return NULL;
 				}
@@ -708,6 +717,7 @@ GF_Err gf_odf_del_tx3g(GF_TextSampleDescriptor *sd)
 GF_EXPORT
 GF_TextSampleDescriptor *gf_odf_tx3g_read(u8 *dsi, u32 dsi_size)
 {
+#ifndef GPAC_DISABLE_ISOM
 	u32 i;
 	u32 gpp_read_rgba(GF_BitStream *bs);
 	void gpp_read_style(GF_BitStream *bs, GF_StyleRecord *rec);
@@ -733,10 +743,14 @@ GF_TextSampleDescriptor *gf_odf_tx3g_read(u8 *dsi, u32 dsi_size)
 	}
 	gf_bs_del(bs);
 	return txtc;
+#else
+	return NULL;
+#endif
 }
 
 GF_Err gf_odf_tx3g_write(GF_TextSampleDescriptor *a, u8 **outData, u32 *outSize)
 {
+#ifndef GPAC_DISABLE_ISOM
 	u32 j;
 	void gpp_write_rgba(GF_BitStream *bs, u32 col);
 	void gpp_write_box(GF_BitStream *bs, GF_BoxRecord *rec);
@@ -763,6 +777,9 @@ GF_Err gf_odf_tx3g_write(GF_TextSampleDescriptor *a, u8 **outData, u32 *outSize)
 	gf_bs_get_content(bs, outData, outSize);
 	gf_bs_del(bs);
 	return GF_OK;
+#else
+	return GF_NOT_SUPPORTED;
+#endif
 }
 
 /*TextConfig*/
@@ -1353,6 +1370,7 @@ GF_VVCConfig *gf_odf_vvc_cfg_read_bs(GF_BitStream *bs)
 	count = gf_bs_read_int(bs, 8);
 	for (i=0; i<count; i++) {
 		u32 nalucount, j;
+		Bool valid = GF_FALSE;
 		GF_NALUFFParamArray *ar;
 		GF_SAFEALLOC(ar, GF_NALUFFParamArray);
 		if (!ar) {
@@ -1360,14 +1378,29 @@ GF_VVCConfig *gf_odf_vvc_cfg_read_bs(GF_BitStream *bs)
 			GF_LOG(GF_LOG_ERROR, GF_LOG_CODING, ("[VVC] alloc failed while parsing vvc config\n"));
 			return NULL;
 		}
-		ar->nalus = gf_list_new();
-		gf_list_add(cfg->param_array, ar);
-
 		ar->array_completeness = gf_bs_read_int(bs, 1);
 		gf_bs_read_int(bs, 2);
 		ar->type = gf_bs_read_int(bs, 5);
 
-		if ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI))
+		switch (ar->type) {
+		case GF_VVC_NALU_DEC_PARAM:
+		case GF_VVC_NALU_OPI:
+		case GF_VVC_NALU_VID_PARAM:
+		case GF_VVC_NALU_SEQ_PARAM:
+		case GF_VVC_NALU_PIC_PARAM:
+		case GF_VVC_NALU_SEI_PREFIX:
+		case GF_VVC_NALU_SEI_SUFFIX:
+			valid = GF_TRUE;
+			ar->nalus = gf_list_new();
+			gf_list_add(cfg->param_array, ar);
+			break;
+		default:
+			GF_LOG(GF_LOG_WARNING, GF_LOG_CODING, ("[VVC] Invalid NALU type %d in vvcC - ignoring\n", ar->type));
+			gf_free(ar);
+			break;
+		}
+
+		if (!valid || ((ar->type != GF_VVC_NALU_DEC_PARAM) && (ar->type != GF_VVC_NALU_OPI)))
 			nalucount = gf_bs_read_int(bs, 16);
 		else
 			nalucount = 1;
@@ -1380,6 +1413,10 @@ GF_VVCConfig *gf_odf_vvc_cfg_read_bs(GF_BitStream *bs)
 				gf_odf_vvc_cfg_del(cfg);
 				return NULL;
 			}
+			if (!valid) {
+				gf_bs_skip_bytes(bs, size);
+				continue;
+			}
 			GF_SAFEALLOC(sl, GF_NALUFFParam );
 			if (!sl) {
 				gf_odf_vvc_cfg_del(cfg);
@@ -1768,7 +1805,13 @@ GF_Err gf_odf_ac3_cfg_write(GF_AC3Config *cfg, u8 **data, u32 *size)
 	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 	GF_Err e = gf_odf_ac3_cfg_write_bs(cfg, bs);
 
+	if (cfg->is_ec3 && (cfg->atmos_ec3_ext || cfg->complexity_index_type)) {
+		gf_bs_write_int(bs, 0, 7);
+		gf_bs_write_int(bs, cfg->atmos_ec3_ext, 1);
+		gf_bs_write_u8(bs, cfg->complexity_index_type);
+	}
 	gf_bs_get_content(bs, data, size);
+
 	gf_bs_del(bs);
 	return e;
 }
@@ -1819,6 +1862,66 @@ GF_Err gf_odf_ac3_config_parse(u8 *dsi, u32 dsi_len, Bool is_ec3, GF_AC3Config *
 	if (!cfg || !dsi) return GF_BAD_PARAM;
 	bs = gf_bs_new(dsi, dsi_len, GF_BITSTREAM_READ);
 	e = gf_odf_ac3_config_parse_bs(bs, is_ec3, cfg);
+	if (is_ec3 && gf_bs_available(bs)>=2) {
+		gf_bs_read_int(bs, 7);
+		cfg->atmos_ec3_ext = gf_bs_read_int(bs, 1);
+		cfg->complexity_index_type = gf_bs_read_u8(bs);
+	}
+	gf_bs_del(bs);
+	return e;
+}
+
+
+GF_Err gf_odf_opus_cfg_parse_bs(GF_BitStream *bs, GF_OpusConfig *cfg)
+{
+	memset(cfg, 0, sizeof(GF_OpusConfig));
+	cfg->version = gf_bs_read_u8(bs);
+	cfg->OutputChannelCount = gf_bs_read_u8(bs);
+	cfg->PreSkip = gf_bs_read_u16_le(bs);
+	cfg->InputSampleRate = gf_bs_read_u32_le(bs);
+	cfg->OutputGain = gf_bs_read_u16_le(bs);
+	cfg->ChannelMappingFamily = gf_bs_read_u8(bs);
+	if (cfg->ChannelMappingFamily) {
+		cfg->StreamCount = gf_bs_read_u8(bs);
+		cfg->CoupledCount = gf_bs_read_u8(bs);
+		gf_bs_read_data(bs, (char *) cfg->ChannelMapping, cfg->OutputChannelCount);
+	}
+	return GF_OK;
+}
+GF_Err gf_odf_opus_cfg_parse(u8 *dsi, u32 dsi_len, GF_OpusConfig *cfg)
+{
+	GF_BitStream *bs;
+	GF_Err e;
+	if (!cfg || !dsi) return GF_BAD_PARAM;
+	bs = gf_bs_new(dsi, dsi_len, GF_BITSTREAM_READ);
+	e = gf_odf_opus_cfg_parse_bs(bs, cfg);
+	gf_bs_del(bs);
+	return e;
+}
+
+GF_Err gf_odf_opus_cfg_write_bs(GF_OpusConfig *cfg, GF_BitStream *bs)
+{
+	if (!cfg || !bs) return GF_BAD_PARAM;
+	gf_bs_write_u8(bs, cfg->version);
+	gf_bs_write_u8(bs, cfg->OutputChannelCount);
+	gf_bs_write_u16_le(bs, cfg->PreSkip);
+	gf_bs_write_u32_le(bs, cfg->InputSampleRate);
+	gf_bs_write_u16_le(bs, cfg->OutputGain);
+	gf_bs_write_u8(bs, cfg->ChannelMappingFamily);
+	if (cfg->ChannelMappingFamily) {
+		gf_bs_write_u8(bs, cfg->StreamCount);
+		gf_bs_write_u8(bs, cfg->CoupledCount);
+		gf_bs_write_data(bs, (char *) cfg->ChannelMapping, cfg->OutputChannelCount);
+	}
+	return GF_OK;
+}
+
+GF_Err gf_odf_opus_cfg_write(GF_OpusConfig *cfg, u8 **data, u32 *size)
+{
+	GF_BitStream *bs = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
+	GF_Err e = gf_odf_opus_cfg_write_bs(cfg, bs);
+
+	gf_bs_get_content(bs, data, size);
 	gf_bs_del(bs);
 	return e;
 }
