@@ -232,7 +232,7 @@ struct _gf_vrml_script_priv
 struct __gf_download_manager *jsf_get_download_manager(JSContext *c);
 struct _gf_ft_mgr *jsf_get_font_manager(JSContext *c);
 GF_Err jsf_request_opengl(JSContext *c);
-GF_Err jsf_set_gl_active(JSContext *c);
+GF_Err jsf_set_gl_active(JSContext *c, Bool do_activate);
 GF_Err jsf_get_filter_packet_planes(JSContext *c, JSValue obj, u32 *width, u32 *height, u32 *pf, u32 *stride, u32 *stride_uv, const u8 **data, const u8 **p_u, const u8 **p_v, const u8 **p_a);
 
 Bool jsf_is_packet(JSContext *c, JSValue obj);
