@@ -841,25 +841,27 @@ void gf_sg_proto_del_instance(GF_ProtoInstance *inst)
 		GF_ProtoField *field = (GF_ProtoField *)gf_list_get(inst->fields, 0);
 		gf_list_rem(inst->fields, 0);
 
-		/*regular type*/
-		if ( (field->FieldType!=GF_SG_VRML_SFNODE) && (field->FieldType!=GF_SG_VRML_MFNODE)) {
-			gf_sg_vrml_field_pointer_del(field->field_pointer, field->FieldType);
-		}
-		/*node types: delete instances*/
-		else if (field->field_pointer) {
-			if (field->FieldType == GF_SG_VRML_SFNODE) {
-				gf_node_unregister((GF_Node *) field->field_pointer, (GF_Node *) inst);
-			} else {
-				GF_ChildNodeItem *list = (GF_ChildNodeItem *)field->field_pointer;
-				while (list) {
-					GF_ChildNodeItem *cur = list;
-					gf_node_unregister(list->node, (GF_Node *) inst);
-					list = list->next;
-					gf_free(cur);
+		if (field->field_pointer) {
+			/*regular type*/
+			if ( (field->FieldType!=GF_SG_VRML_SFNODE) && (field->FieldType!=GF_SG_VRML_MFNODE)) {
+				gf_sg_vrml_field_pointer_del(field->field_pointer, field->FieldType);
+			}
+			/*node types: delete instances*/
+			else {
+				if (field->FieldType == GF_SG_VRML_SFNODE) {
+					gf_node_unregister((GF_Node *) field->field_pointer, (GF_Node *) inst);
+				} else {
+					GF_ChildNodeItem *list = (GF_ChildNodeItem *)field->field_pointer;
+					while (list) {
+						GF_ChildNodeItem *cur = list;
+						gf_node_unregister(list->node, (GF_Node *) inst);
+						list = list->next;
+						gf_free(cur);
+					}
 				}
 			}
 		}
-
+		
 		gf_free(field);
 	}
 	gf_list_del(inst->fields);
@@ -990,6 +992,7 @@ GF_Err gf_sg_proto_instance_set_ised(GF_Node *protoinst, u32 protoFieldIndex, GF
 	GF_Err e;
 	GF_Route *r;
 	GF_FieldInfo field, nodeField;
+	if (!protoinst) return GF_BAD_PARAM;
 	if (protoinst->sgprivate->tag != TAG_ProtoNode) return GF_BAD_PARAM;
 
 	e = gf_node_get_field(protoinst, protoFieldIndex, &field);
@@ -1250,7 +1253,7 @@ GF_Proto *gf_node_get_proto(GF_Node *node)
 GF_EXPORT
 u32 gf_sg_proto_get_id(GF_Proto *proto)
 {
-	return proto->ID;
+	return proto ? proto->ID : 0;
 }
 
 GF_EXPORT
@@ -1289,7 +1292,10 @@ Bool gf_sg_proto_field_is_sftime_offset(GF_Node *node, GF_FieldInfo *field)
 
 		gf_node_get_field(r->ToNode, r->ToField.fieldIndex, &inf);
 		/*IS to another proto*/
-		if (r->ToNode->sgprivate->tag == TAG_ProtoNode) return gf_sg_proto_field_is_sftime_offset(r->ToNode, &inf);
+		if (r->ToNode->sgprivate->tag == TAG_ProtoNode) {
+			if (r->ToNode==node) continue;
+			return gf_sg_proto_field_is_sftime_offset(r->ToNode, &inf);
+		}
 		/*IS to a startTime/stopTime field*/
 		if (!stricmp(inf.name, "startTime") || !stricmp(inf.name, "stopTime")) return 1;
 	}
