@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2020
+ *			Copyright (c) Telecom ParisTech 2000-2022
  *					All rights reserved
  *
  *  This file is part of GPAC / common tools sub-project
@@ -61,7 +61,7 @@ struct __tag_bitstream
 	u32 cache_write_size, buffer_written;
 
 	Bool remove_emul_prevention_byte;
-	u32 nb_zeros;
+	u32 nb_zeros, nb_removed;
 
 	GF_Err (*on_block_out)(void *cbk, u8 *data, u32 block_size);
 	void *usr_data;
@@ -91,7 +91,7 @@ GF_Err gf_bs_reassign_buffer(GF_BitStream *bs, const u8 *buffer, u64 BufferSize)
 		bs->current = 0;
 		bs->nbBits = 8;
 		bs->current = 0;
-		bs->nb_zeros = 0;
+		bs->nb_zeros = bs->nb_removed = 0;
 		return GF_OK;
 	}
 	if (bs->bsmode==GF_BITSTREAM_WRITE) {
@@ -273,18 +273,21 @@ void gf_bs_prevent_dispatch(GF_BitStream *bs, Bool prevent_dispatch)
 	}
 }
 
-static void bs_flush_write_cache(GF_BitStream *bs)
+static Bool bs_flush_write_cache(GF_BitStream *bs)
 {
+	Bool res = GF_TRUE;
 	if (bs->buffer_written) {
 		u32 nb_write;
 		nb_write = (u32) gf_fwrite(bs->cache_write, bs->buffer_written, bs->stream);
-
+		if (nb_write != bs->buffer_written)
+			res = GF_FALSE;
 		//check we didn't rewind the bitstream
 		if (bs->size == bs->position)
 			bs->size += nb_write;
 		bs->position += nb_write;
 		bs->buffer_written = 0;
 	}
+	return res;
 }
 
 
@@ -312,10 +315,18 @@ void gf_bs_enable_emulation_byte_removal(GF_BitStream *bs, Bool do_remove)
 {
 	if (bs) {
 		bs->remove_emul_prevention_byte = do_remove;
-		bs->nb_zeros = 0;
+		bs->nb_zeros = bs->nb_removed = 0;
 	}
 }
 
+GF_EXPORT
+u32 gf_bs_get_emulation_byte_removed(GF_BitStream *bs)
+{
+	if (bs) return bs->nb_removed;
+	return 0;
+}
+
+
 /*returns 1 if aligned wrt current mode, 0 otherwise*/
 Bool gf_bs_is_align(GF_BitStream *bs)
 {
@@ -364,6 +375,7 @@ static u8 BS_ReadByte(GF_BitStream *bs)
 		if (bs->remove_emul_prevention_byte) {
 			if ((bs->nb_zeros==2) && (res==0x03) && (bs->position<bs->size) && (bs->original[bs->position]<0x04)) {
 				bs->nb_zeros = 0;
+				bs->nb_removed++;
 				res = bs->original[bs->position++];
 			}
 			if (!res) bs->nb_zeros++;
@@ -374,10 +386,13 @@ static u8 BS_ReadByte(GF_BitStream *bs)
 	if (bs->cache_write)
 		bs_flush_write_cache(bs);
 
-	is_eos = gf_feof(bs->stream);
+	is_eos = bs->stream ? gf_feof(bs->stream) : GF_TRUE;
+	//cache not fully read, reset EOS
+	if (bs->cache_read && (bs->cache_read_pos<bs->cache_read_size))
+		is_eos = GF_FALSE;
 
 	/*we are in FILE mode, test for end of file*/
-	if (!is_eos || bs->cache_read) {
+	if (!is_eos) {
 		u8 res;
 		Bool loc_eos=GF_FALSE;
 		assert(bs->position<=bs->size);
@@ -391,6 +406,7 @@ static u8 BS_ReadByte(GF_BitStream *bs)
 				u8 next = gf_bs_load_byte(bs, &loc_eos);
 				if (next < 0x04) {
 					bs->nb_zeros = 0;
+					bs->nb_removed++;
 					res = next;
 					bs->position++;
 				} else {
@@ -408,7 +424,10 @@ bs_eof:
 		bs->EndOfStream(bs->par);
 		if (!bs->overflow_state) bs->overflow_state = 1;
 	} else {
-		GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[BS] Attempt to overread bitstream\n"));
+		if (!bs->overflow_state) {
+			bs->overflow_state = 1;
+			GF_LOG(GF_LOG_ERROR, GF_LOG_CORE, ("[BS] Attempt to overread bitstream\n"));
+		}
 	}
 	assert(bs->position <= 1+bs->size);
 	return 0;
@@ -1055,7 +1074,8 @@ u32 gf_bs_write_data(GF_BitStream *bs, const u8 *data, u32 nbBytes)
 					return nbBytes;
 				}
 				//otherwise flush cache and use file write
-				bs_flush_write_cache(bs);
+				if (!bs_flush_write_cache(bs))
+					return 0;
 			}
 
 			if (gf_fwrite(data, nbBytes, bs->stream) != nbBytes) return 0;
@@ -1246,6 +1266,10 @@ void gf_bs_skip_bytes(GF_BitStream *bs, u64 nbBytes)
 			}
 		} else {
 			bs->position += nbBytes;
+			if (bs->position>bs->size) {
+				bs->position = bs->size;
+				bs->overflow_state = 1;
+			}
 		}
 		return;
 	}
@@ -1358,7 +1382,7 @@ GF_EXPORT
 u32 gf_bs_peek_bits(GF_BitStream *bs, u32 numBits, u64 byte_offset)
 {
 	u64 curPos;
-	u32 curBits, ret, current, nb_zeros;
+	u32 curBits, ret, current, nb_zeros, nb_removed;
 
 	if ( (bs->bsmode != GF_BITSTREAM_READ) && (bs->bsmode != GF_BITSTREAM_FILE_READ)) return 0;
 	if (!numBits || (bs->size < bs->position + byte_offset)) return 0;
@@ -1368,6 +1392,7 @@ u32 gf_bs_peek_bits(GF_BitStream *bs, u32 numBits, u64 byte_offset)
 	curBits = bs->nbBits;
 	current = bs->current;
 	nb_zeros = bs->nb_zeros;
+	nb_removed = bs->nb_removed;
 
 	if (byte_offset) {
 		if (bs->remove_emul_prevention_byte) {
@@ -1387,6 +1412,7 @@ u32 gf_bs_peek_bits(GF_BitStream *bs, u32 numBits, u64 byte_offset)
 	bs->nbBits = curBits;
 	bs->current = current;
 	bs->nb_zeros = nb_zeros;
+	bs->nb_removed = nb_removed;
 	return ret;
 }
 
@@ -1719,3 +1745,35 @@ u32 gf_bs_is_overflow(GF_BitStream *bs)
 {
 	return bs->overflow_state;
 }
+
+
+GF_EXPORT
+char *gf_bs_read_utf8(GF_BitStream *bs)
+{
+	char szC[2];
+	char *res = NULL;
+	if (!bs || !gf_bs_is_align(bs))
+		return NULL;
+
+	szC[1] = 0;
+	while (gf_bs_available(bs)) {
+		u8 c = gf_bs_read_u8(bs);
+		if (!c) break;
+		szC[0] = c;
+		gf_dynstrcat(&res, szC, NULL);
+	}
+	return res;
+}
+
+GF_EXPORT
+GF_Err gf_bs_write_utf8(GF_BitStream *bs, const char *str)
+{
+	if (!bs || !gf_bs_is_align(bs))
+		return GF_BAD_PARAM;
+
+	u32 i, len = str ? (u32) strlen(str) : 0;
+	for (i=0; i<len; i++)
+		gf_bs_write_u8(bs, str[i]);
+	gf_bs_write_u8(bs, 0);
+	return GF_OK;
+}
