@@ -2,7 +2,7 @@
  *					GPAC Multimedia Framework
  *
  *			Authors: Jean Le Feuvre, Pierre Souchay
- *			Copyright (c) Telecom ParisTech 2010-2020
+ *			Copyright (c) Telecom ParisTech 2010-2022
  *					All rights reserved
  *
  *   This file is part of GPAC / common tools sub-project
@@ -23,8 +23,6 @@
  *
  */
 
-#ifndef GPAC_DISABLE_CORE_TOOLS
-
 #include <gpac/cache.h>
 #include <gpac/network.h>
 #include <gpac/download.h>
@@ -490,7 +488,11 @@ DownloadedCacheEntry gf_cache_create_entry ( GF_DownloadManager * dm, const char
 		entry->cache_blob.mx = mx;
 		entry->cache_blob.data = entry->mem_storage;
 		entry->cache_blob.size = entry->contentLength;
-		sprintf(entry->cache_filename, "gmem://%p", &entry->cache_blob);
+		char *burl = gf_blob_register(&entry->cache_blob);
+		if (burl) {
+			strcpy(entry->cache_filename, burl);
+			gf_free(burl);
+		}
 		return entry;
 	}
 
@@ -664,7 +666,11 @@ GF_Err gf_cache_open_write_cache( const DownloadedCacheEntry entry, const GF_Dow
 		}
 		entry->cache_blob.data = entry->mem_storage;
 		entry->cache_blob.size = entry->contentLength;
-		sprintf(entry->cache_filename, "gmem://%p", &entry->cache_blob);
+		char *burl = gf_blob_register(&entry->cache_blob);
+		if (burl) {
+			strcpy(entry->cache_filename, burl);
+			gf_free(burl);
+		}
 		gf_mx_v(entry->cache_blob.mx);
 
 		if (!entry->mem_allocated) {
@@ -711,7 +717,11 @@ GF_Err gf_cache_write_to_cache( const DownloadedCacheEntry entry, const GF_Downl
 			entry->mem_allocated = new_size;
 			entry->cache_blob.data = entry->mem_storage;
 			entry->cache_blob.size = entry->contentLength;
-			sprintf(entry->cache_filename, "gmem://%p", &entry->cache_blob);
+			char *burl = gf_blob_register(&entry->cache_blob);
+			if (burl) {
+				strcpy(entry->cache_filename, burl);
+				gf_free(burl);
+			}
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Reallocating memory cache to %d bytes\n", new_size));
 		}
 		memcpy(entry->mem_storage + entry->written_in_cache, data, size);
@@ -812,6 +822,13 @@ GF_Err gf_cache_delete_entry ( const DownloadedCacheEntry entry )
 		gf_free ( entry->cache_filename );
 		entry->cache_filename = NULL;
 	}
+	gf_blob_unregister(&entry->cache_blob);
+
+	if (entry->external_blob) {
+		gf_blob_unregister(entry->external_blob);
+		entry->external_blob = NULL;
+	}
+
 	if ( entry->properties ) {
 		const char * propfile = NULL;
 		if (entry->deletableFilesOnDelete)
@@ -1021,6 +1038,10 @@ Bool gf_cache_set_content(const DownloadedCacheEntry entry, GF_Blob *blob, Bool
 
     if (!blob) {
         entry->flags = DELETED;
+        if (entry->external_blob) {
+			gf_blob_unregister(entry->external_blob);
+			entry->external_blob = NULL;
+		}
         return GF_TRUE;
     }
     if (blob->mx)
@@ -1029,8 +1050,14 @@ Bool gf_cache_set_content(const DownloadedCacheEntry entry, GF_Blob *blob, Bool
     if (!copy) {
         if (entry->mem_allocated) gf_free(entry->mem_storage);
 		entry->mem_storage = (u8 *) blob->data;
-        if (!entry->written_in_cache)
-            sprintf(entry->cache_filename, "gmem://%p", blob);
+        if (!entry->written_in_cache) {
+			char *burl = gf_blob_register(blob);
+			if (burl) {
+				strcpy(entry->cache_filename, burl);
+				gf_free(burl);
+			}
+		}
+
 		entry->written_in_cache = blob->size;
 		entry->mem_allocated = 0;
 		entry->cache_blob.data = NULL;
@@ -1049,8 +1076,13 @@ Bool gf_cache_set_content(const DownloadedCacheEntry entry, GF_Blob *blob, Bool
             entry->mem_allocated = new_size;
             entry->cache_blob.data = entry->mem_storage;
             entry->cache_blob.size = entry->contentLength;
-            if (!entry->written_in_cache)
-                sprintf(entry->cache_filename, "gmem://%p", &entry->cache_blob);
+            if (!entry->written_in_cache) {
+				char *burl = gf_blob_register(&entry->cache_blob);
+				if (burl) {
+					strcpy(entry->cache_filename, burl);
+					gf_free(burl);
+				}
+            }
             GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Reallocating memory cache to %d bytes\n", new_size));
         }
         memcpy(entry->mem_storage, blob->data, blob->size);
@@ -1073,4 +1105,3 @@ Bool gf_cache_set_content(const DownloadedCacheEntry entry, GF_Blob *blob, Bool
     return GF_TRUE;
 }
 
-#endif
